@@ -3,24 +3,25 @@
 Compute effect-matched-spatial filtering (EMS)
 ==============================================
 
-This example computes the EMS to reconstruct the time course of
-the experimental effect as described in:
+This example computes the EMS to reconstruct the time course of the
+experimental effect as described in [1]_.
 
-Aaron Schurger, Sebastien Marti, and Stanislas Dehaene, "Reducing multi-sensor
-data to a single time course that reveals experimental effects",
-BMC Neuroscience 2013, 14:122
-
-
-This technique is used to create spatial filters based on the
-difference between two conditions. By projecting the trial onto the
-corresponding spatial filters, surrogate single trials are created
-in which multi-sensor activity is reduced to one time series which
-exposes experimental effects, if present.
+This technique is used to create spatial filters based on the difference
+between two conditions. By projecting the trial onto the corresponding spatial
+filters, surrogate single trials are created in which multi-sensor activity is
+reduced to one time series which exposes experimental effects, if present.
 
 We will first plot a trials x times image of the single trials and order the
 trials by condition. A second plot shows the average time series for each
-condition. Finally a topographic plot is created which exhibits the
-temporal evolution of the spatial filters.
+condition. Finally a topographic plot is created which exhibits the temporal
+evolution of the spatial filters.
+
+References
+----------
+
+.. [1] Aaron Schurger, Sebastien Marti, and Stanislas Dehaene, "Reducing
+       multi-sensor data to a single time course that reveals experimental
+       effects", BMC Neuroscience 2013, 14:122.
 """
 # Author: Denis Engemann <denis.engemann@gmail.com>
 #         Jean-Remi King <jeanremi.king@gmail.com>
@@ -34,7 +35,7 @@ import mne
 from mne import io, EvokedArray
 from mne.datasets import sample
 from mne.decoding import EMS, compute_ems
-from sklearn.cross_validation import StratifiedKFold
+from sklearn.model_selection import StratifiedKFold
 
 print(__doc__)
 
@@ -47,8 +48,7 @@ event_ids = {'AudL': 1, 'VisL': 3}
 
 # Read data and create epochs
 raw = io.read_raw_fif(raw_fname, preload=True)
-raw.filter(0.5, 45, l_trans_bandwidth='auto', h_trans_bandwidth='auto',
-           filter_length='auto', phase='zero')
+raw.filter(0.5, 45, fir_design='firwin')
 events = mne.read_events(event_fname)
 
 picks = mne.pick_types(raw.info, meg='grad', eeg=False, stim=False, eog=True,
@@ -79,7 +79,7 @@ filters = list()  # Spatial filters at each time point
 # to overfit and cannot be used to estimate the variance of the
 # prediction within a given fold.
 
-for train, test in StratifiedKFold(y):
+for train, test in StratifiedKFold().split(X, y):
     # In the original paper, the z-scoring is applied outside the CV.
     # However, we recommend to apply this preprocessing inside the CV.
     # Note that such scaling should be done separately for each channels if the
