@@ -4,8 +4,8 @@ Compute MxNE with time-frequency sparse prior
 =============================================
 
 The TF-MxNE solver is a distributed inverse method (like dSPM or sLORETA)
-that promotes focal (sparse) sources (such as dipole fitting techniques).
-The benefit of this approach is that:
+that promotes focal (sparse) sources (such as dipole fitting techniques)
+[1]_ [2]_. The benefit of this approach is that:
 
   - it is spatio-temporal without assuming stationarity (sources properties
     can vary over time)
@@ -16,31 +16,34 @@ The benefit of this approach is that:
   - the solver solves a convex optimization problem, hence cannot be
     trapped in local minima.
 
-References:
+References
+----------
+.. [1] A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
+   "Time-Frequency Mixed-Norm Estimates: Sparse M/EEG imaging with
+   non-stationary source activations",
+   Neuroimage, Volume 70, pp. 410-422, 15 April 2013.
+   DOI: 10.1016/j.neuroimage.2012.12.051
 
-A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
-Time-Frequency Mixed-Norm Estimates: Sparse M/EEG imaging with
-non-stationary source activations
-Neuroimage, Volume 70, 15 April 2013, Pages 410-422, ISSN 1053-8119,
-DOI: 10.1016/j.neuroimage.2012.12.051.
-
-A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
-Functional Brain Imaging with M/EEG Using Structured Sparsity in
-Time-Frequency Dictionaries
-Proceedings Information Processing in Medical Imaging
-Lecture Notes in Computer Science, 2011, Volume 6801/2011,
-600-611, DOI: 10.1007/978-3-642-22092-0_49
-http://dx.doi.org/10.1007/978-3-642-22092-0_49
+.. [2] A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
+   "Functional Brain Imaging with M/EEG Using Structured Sparsity in
+   Time-Frequency Dictionaries",
+   Proceedings Information Processing in Medical Imaging
+   Lecture Notes in Computer Science, Volume 6801/2011, pp. 600-611, 2011.
+   DOI: 10.1007/978-3-642-22092-0_49
 """
 # Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+#         Daniel Strohmeier <daniel.strohmeier@tu-ilmenau.de>
 #
 # License: BSD (3-clause)
 
+import numpy as np
+
 import mne
 from mne.datasets import sample
 from mne.minimum_norm import make_inverse_operator, apply_inverse
-from mne.inverse_sparse import tf_mixed_norm
-from mne.viz import plot_sparse_source_estimates
+from mne.inverse_sparse import tf_mixed_norm, make_stc_from_dipoles
+from mne.viz import (plot_sparse_source_estimates,
+                     plot_dipole_locations, plot_dipole_amplitudes)
 
 print(__doc__)
 
@@ -62,14 +65,13 @@ evoked = mne.pick_channels_evoked(evoked)
 evoked.crop(tmin=-0.1, tmax=0.4)
 
 # Handling forward solution
-forward = mne.read_forward_solution(fwd_fname, force_fixed=False,
-                                    surf_ori=True)
+forward = mne.read_forward_solution(fwd_fname)
 
 ###############################################################################
 # Run solver
 
 # alpha_space regularization parameter is between 0 and 100 (100 is high)
-alpha_space = 50.  # spatial regularization parameter
+alpha_space = 30.  # spatial regularization parameter
 # alpha_time parameter promotes temporal smoothness
 # (0 means no temporal regularization)
 alpha_time = 1.  # temporal regularization parameter
@@ -82,18 +84,36 @@ inverse_operator = make_inverse_operator(evoked.info, forward, cov,
 stc_dspm = apply_inverse(evoked, inverse_operator, lambda2=1. / 9.,
                          method='dSPM')
 
-# Compute TF-MxNE inverse solution
-stc, residual = tf_mixed_norm(evoked, forward, cov, alpha_space, alpha_time,
-                              loose=loose, depth=depth, maxit=200, tol=1e-4,
-                              weights=stc_dspm, weights_min=8., debias=True,
-                              wsize=16, tstep=4, window=0.05,
-                              return_residual=True)
+# Compute TF-MxNE inverse solution with dipole output
+dipoles, residual = tf_mixed_norm(
+    evoked, forward, cov, alpha_space, alpha_time, loose=loose, depth=depth,
+    maxit=200, tol=1e-6, weights=stc_dspm, weights_min=8., debias=True,
+    wsize=16, tstep=4, window=0.05, return_as_dipoles=True,
+    return_residual=True)
 
 # Crop to remove edges
-stc.crop(tmin=-0.05, tmax=0.3)
+for dip in dipoles:
+    dip.crop(tmin=-0.05, tmax=0.3)
 evoked.crop(tmin=-0.05, tmax=0.3)
 residual.crop(tmin=-0.05, tmax=0.3)
 
+###############################################################################
+# Plot dipole activations
+plot_dipole_amplitudes(dipoles)
+
+# Plot dipole location of the strongest dipole with MRI slices
+idx = np.argmax([np.max(np.abs(dip.amplitude)) for dip in dipoles])
+plot_dipole_locations(dipoles[idx], forward['mri_head_t'], 'sample',
+                      subjects_dir=subjects_dir, mode='orthoview',
+                      idx='amplitude')
+
+# # Plot dipole locations of all dipoles with MRI slices
+# for dip in dipoles:
+#     plot_dipole_locations(dip, forward['mri_head_t'], 'sample',
+#                           subjects_dir=subjects_dir, mode='orthoview',
+#                           idx='amplitude')
+
+###############################################################################
 # Show the evoked response and the residual for gradiometers
 ylim = dict(grad=[-120, 120])
 evoked.pick_types(meg='grad', exclude='bads')
@@ -104,6 +124,10 @@ residual.pick_types(meg='grad', exclude='bads')
 residual.plot(titles=dict(grad='Residuals: Gradiometers'), ylim=ylim,
               proj=True)
 
+###############################################################################
+# Generate stc from dipoles
+stc = make_stc_from_dipoles(dipoles, forward['src'])
+
 ###############################################################################
 # View in 2D and 3D ("glass" brain like 3D plot)
 plot_sparse_source_estimates(forward['src'], stc, bgcolor=(1, 1, 1),
