@@ -37,11 +37,14 @@ ch_types = ['mag', 'mag', 'grad', 'grad']
 ch_names = ['sin', 'cos', 'sinX2', 'cosX2']
 
 ###############################################################################
-# Creation of info dictionary.
+# Create an :class:`info <mne.Info>` object.
 
 # It is also possible to use info from another raw object.
 info = mne.create_info(ch_names=ch_names, sfreq=sfreq, ch_types=ch_types)
 
+###############################################################################
+# Create a dummy :class:`mne.io.RawArray` object
+
 raw = mne.io.RawArray(data, info)
 
 # Scaling of the figure.
@@ -60,7 +63,8 @@ raw.plot(n_channels=4, scalings=scalings, title='Auto-scaled Data from arrays',
 ###############################################################################
 # EpochsArray
 
-event_id = 1
+event_id = 1  # This is used to identify the events.
+# First column is for the sample number.
 events = np.array([[200, 0, event_id],
                    [1200, 0, event_id],
                    [2000, 0, event_id]])  # List of three arbitrary events
@@ -95,6 +99,37 @@ evokeds = mne.EvokedArray(evoked_data, info=info, tmin=-0.2,
 evokeds.plot(picks=picks, show=True, units={'mag': '-'},
              titles={'mag': 'sin and cos averaged'})
 
+###############################################################################
+# Create epochs by windowing the raw data.
+
+# The events are spaced evenly every 1 second.
+duration = 1.
+
+# create a fixed size events array
+# start=0 and stop=None by default
+events = mne.make_fixed_length_events(raw, event_id, duration=duration)
+print(events)
+
+# for fixed size events no start time before and after event
+tmin = 0.
+tmax = 0.99  # inclusive tmax, 1 second epochs
+
+# create :class:`Epochs <mne.Epochs>` object
+epochs = mne.Epochs(raw, events=events, event_id=event_id, tmin=tmin,
+                    tmax=tmax, baseline=None, verbose=True)
+epochs.plot(scalings='auto', block=True)
+
+###############################################################################
+# Create overlapping epochs using :func:`mne.make_fixed_length_events` (50 %
+# overlap). This also roughly doubles the amount of events compared to the
+# previous event list.
+
+duration = 0.5
+events = mne.make_fixed_length_events(raw, event_id, duration=duration)
+print(events)
+epochs = mne.Epochs(raw, events=events, tmin=tmin, tmax=tmax, baseline=None,
+                    verbose=True)
+epochs.plot(scalings='auto', block=True)
 
 ###############################################################################
 # Extracting data from NEO file
@@ -110,11 +145,14 @@ title = seg.file_origin
 
 ch_names = list()
 data = list()
-for asig in seg.analogsignals:
+for ai, asig in enumerate(seg.analogsignals):
     # Since the data does not contain channel names, channel indices are used.
-    ch_names.append(str(asig.channel_index))
-    asig = asig.rescale('V').magnitude
-    data.append(asig)
+    ch_names.append('Neo %02d' % (ai + 1,))
+    # We need the ravel() here because Neo < 0.5 gave 1D, Neo 0.5 gives
+    # 2D (but still a single channel).
+    data.append(asig.rescale('V').magnitude.ravel())
+
+data = np.array(data, float)
 
 sfreq = int(seg.analogsignals[0].sampling_rate.magnitude)
 
