@@ -40,7 +40,8 @@ rt_client = MockRtClient(raw)
 
 # create the real-time epochs object
 rt_epochs = RtEpochs(rt_client, event_id, tmin, tmax, picks=picks, decim=1,
-                     reject=dict(grad=4000e-13, eog=150e-6))
+                     reject=dict(grad=4000e-13, eog=150e-6), baseline=None,
+                     isi_max=4.)
 
 # start the acquisition
 rt_epochs.start()
@@ -54,15 +55,17 @@ n_times = len(rt_epochs.times)
 from sklearn import preprocessing  # noqa
 from sklearn.svm import SVC  # noqa
 from sklearn.pipeline import Pipeline  # noqa
-from sklearn.cross_validation import cross_val_score, ShuffleSplit  # noqa
-from mne.decoding import EpochsVectorizer, FilterEstimator  # noqa
+from sklearn.model_selection import cross_val_score, ShuffleSplit  # noqa
+from mne.decoding import Vectorizer, FilterEstimator  # noqa
 
 
 scores_x, scores, std_scores = [], [], []
 
-filt = FilterEstimator(rt_epochs.info, 1, 40)
+# don't highpass filter because it's epoched data and the signal length
+# is small
+filt = FilterEstimator(rt_epochs.info, None, 40, fir_design='firwin')
 scaler = preprocessing.StandardScaler()
-vectorizer = EpochsVectorizer()
+vectorizer = Vectorizer()
 clf = SVC(C=1, kernel='linear')
 
 concat_classifier = Pipeline([('filter', filt), ('vector', vectorizer),
@@ -70,6 +73,14 @@ concat_classifier = Pipeline([('filter', filt), ('vector', vectorizer),
 
 data_picks = mne.pick_types(rt_epochs.info, meg='grad', eeg=False, eog=True,
                             stim=False, exclude=raw.info['bads'])
+ax = plt.subplot(111)
+ax.set_xlabel('Trials')
+ax.set_ylabel('Classification score (% correct)')
+ax.set_title('Real-time decoding')
+ax.set_xlim([min_trials, 50])
+ax.set_ylim([30, 105])
+plt.axhline(50, color='k', linestyle='--', label="Chance level")
+plt.show(block=False)
 
 for ev_num, ev in enumerate(rt_epochs.iter_evoked()):
 
@@ -84,7 +95,7 @@ for ev_num, ev in enumerate(rt_epochs.iter_evoked()):
 
     if ev_num >= min_trials:
 
-        cv = ShuffleSplit(len(y), 5, test_size=0.2, random_state=42)
+        cv = ShuffleSplit(5, test_size=0.2, random_state=42)
         scores_t = cross_val_score(concat_classifier, X, y, cv=cv,
                                    n_jobs=1) * 100
 
@@ -93,21 +104,19 @@ for ev_num, ev in enumerate(rt_epochs.iter_evoked()):
         scores_x.append(ev_num)
 
         # Plot accuracy
-        plt.clf()
 
-        plt.plot(scores_x, scores, '+', label="Classif. score")
-        plt.hold(True)
-        plt.plot(scores_x, scores)
-        plt.axhline(50, color='k', linestyle='--', label="Chance level")
+        plt.plot(scores_x[-2:], scores[-2:], '-x', color='b',
+                 label="Classif. score")
+        ax.plot(scores_x[-1], scores[-1])
+
         hyp_limits = (np.asarray(scores) - np.asarray(std_scores),
                       np.asarray(scores) + np.asarray(std_scores))
-        plt.fill_between(scores_x, hyp_limits[0], y2=hyp_limits[1],
-                         color='b', alpha=0.5)
-        plt.xlabel('Trials')
-        plt.ylabel('Classification score (% correct)')
-        plt.xlim([min_trials, 50])
-        plt.ylim([30, 105])
-        plt.title('Real-time decoding')
-        plt.show(block=False)
+        fill = plt.fill_between(scores_x, hyp_limits[0], y2=hyp_limits[1],
+                                color='b', alpha=0.5)
         plt.pause(0.01)
-plt.show()
+        plt.draw()
+        ax.collections.remove(fill)  # Remove old fill area
+
+plt.fill_between(scores_x, hyp_limits[0], y2=hyp_limits[1], color='b',
+                 alpha=0.5)
+plt.draw()  # Final figure
