@@ -34,13 +34,9 @@ tmax = 0.5
 raw = io.read_raw_fif(raw_fname)
 events = mne.read_events(event_fname)
 
-#   Set up pick list: MEG + STI 014 - bad channels (modify to your needs)
-include = []  # or stim channel ['STI 014']
-raw.info['bads'] += ['MEG 2443', 'EEG 053']  # bads + 2 more
-
 # pick MEG Gradiometers
 picks = mne.pick_types(raw.info, meg='grad', eeg=False, stim=False, eog=True,
-                       include=include, exclude='bads')
+                       exclude='bads')
 epochs = mne.Epochs(raw, events, event_id, tmin, tmax, picks=picks,
                     baseline=(None, 0), reject=dict(grad=4000e-13, eog=150e-6))
 data = epochs.get_data()
@@ -68,7 +64,7 @@ evoked = mne.EvokedArray(-np.log10(p_values)[:, np.newaxis],
 stats_picks = mne.pick_channels(evoked.ch_names, significant_sensors_names)
 mask = p_values[:, np.newaxis] <= 0.05
 
-evoked.plot_topomap(ch_type='grad', times=[0], scale=1,
+evoked.plot_topomap(ch_type='grad', times=[0], scalings=1,
                     time_format=None, cmap='Reds', vmin=0., vmax=np.max,
-                    unit='-log10(p)', cbar_fmt='-%0.1f', mask=mask,
+                    units='-log10(p)', cbar_fmt='-%0.1f', mask=mask,
                     size=3, show_names=lambda x: x[4:] + ' ' * 20)
