@@ -16,8 +16,9 @@ from mayavi import mlab
 
 import mne
 from mne.io import read_raw_fif, read_raw_ctf, read_raw_bti, read_raw_kit
-from mne.datasets import sample, spm_face
-from mne.viz import plot_trans
+from mne.io import read_raw_artemis123
+from mne.datasets import sample, spm_face, testing
+from mne.viz import plot_alignment
 
 print(__doc__)
 
@@ -32,11 +33,23 @@ raws = dict(
                                op.join(bti_path, 'test_config_linux'),
                                op.join(bti_path, 'test_hs_linux')),
     KIT=read_raw_kit(op.join(kit_path, 'test.sqd')),
+    Artemis123=read_raw_artemis123(op.join(
+        testing.data_path(), 'ARTEMIS123',
+        'Artemis_Data_2017-04-14-10h-38m-59s_Phantom_1k_HPI_1s.bin'))
 )
 
 for system, raw in raws.items():
+    meg = ['helmet', 'sensors']
     # We don't have coil definitions for KIT refs, so exclude them
-    ref_meg = False if system == 'KIT' else True
-    fig = plot_trans(raw.info, trans=None, dig=False, eeg_sensors=False,
-                     meg_sensors=True, coord_frame='meg', ref_meg=ref_meg)
-    mlab.title(system)
+    if system != 'KIT':
+        meg.append('ref')
+    fig = plot_alignment(raw.info, trans=None, dig=False, eeg=False,
+                         surfaces=[], meg=meg, coord_frame='meg')
+    text = mlab.title(system)
+    text.x_position = 0.5
+    text.y_position = 0.95
+    text.property.vertical_justification = 'top'
+    text.property.justification = 'center'
+    text.actor.text_scale_mode = 'none'
+    text.property.bold = True
+    mlab.draw(fig)
