@@ -3,11 +3,9 @@
 Compare evoked responses for different conditions
 =================================================
 
-In this example, an Epochs object for visual and
-auditory responses is created. Both conditions
-are then accessed by their respective names to
-create a sensor layout plot of the related
-evoked responses.
+In this example, an Epochs object for visual and auditory responses is created.
+Both conditions are then accessed by their respective names to create a sensor
+layout plot of the related evoked responses.
 
 """
 
@@ -62,14 +60,9 @@ evokeds = [epochs[name].average() for name in ('left', 'right')]
 ###############################################################################
 # Show topography for two different conditions
 
-colors = 'yellow', 'green'
-title = 'MNE sample data - left vs right (A/V combined)'
+colors = 'blue', 'red'
+title = 'MNE sample data\nleft vs right (A/V combined)'
 
-plot_evoked_topo(evokeds, color=colors, title=title)
-
-conditions = [e.comment for e in evokeds]
-for cond, col, pos in zip(conditions, colors, (0.025, 0.07)):
-    plt.figtext(0.99, pos, cond, color=col, fontsize=12,
-                horizontalalignment='right')
+plot_evoked_topo(evokeds, color=colors, title=title, background_color='w')
 
 plt.show()
