@@ -3,16 +3,16 @@ import warnings
 import os.path as op
 import copy as cp
 
-from nose.tools import assert_true, assert_raises
+from nose.tools import assert_true, assert_raises, assert_equal
 import numpy as np
 from numpy.testing import assert_array_equal, assert_array_almost_equal
 
 import mne
 from mne.datasets import testing
 from mne.beamformer import dics, dics_epochs, dics_source_power, tf_dics
-from mne.time_frequency import compute_epochs_csd
+from mne.time_frequency import csd_epochs
 from mne.externals.six import advance_iterator
-from mne.utils import run_tests_if_main, clean_warning_registry
+from mne.utils import run_tests_if_main
 
 # Note that this is the first test file, this will apply to all subsequent
 # tests in a full nosetest:
@@ -26,31 +26,29 @@ fname_fwd_vol = op.join(data_path, 'MEG', 'sample',
                         'sample_audvis_trunc-meg-vol-7-fwd.fif')
 fname_event = op.join(data_path, 'MEG', 'sample',
                       'sample_audvis_trunc_raw-eve.fif')
-label = 'Aud-lh'
-fname_label = op.join(data_path, 'MEG', 'sample', 'labels', '%s.label' % label)
-
-# bit of a hack to deal with old scipy/numpy throwing warnings in tests
-clean_warning_registry()
+fname_label = op.join(data_path, 'MEG', 'sample', 'labels', 'Aud-lh.label')
 
 
-def read_forward_solution_meg(*args, **kwargs):
-    fwd = mne.read_forward_solution(*args, **kwargs)
+def _read_forward_solution_meg(*args, **kwargs):
+    fwd = mne.read_forward_solution(*args)
+    fwd = mne.convert_forward_solution(fwd, **kwargs)
     return mne.pick_types_forward(fwd, meg=True, eeg=False)
 
 
 def _get_data(tmin=-0.11, tmax=0.15, read_all_forward=True, compute_csds=True):
-    """Read in data used in tests
-    """
+    """Read in data used in tests."""
     label = mne.read_label(fname_label)
     events = mne.read_events(fname_event)[:10]
     raw = mne.io.read_raw_fif(fname_raw, preload=False)
     raw.add_proj([], remove_existing=True)  # we'll subselect so remove proj
     forward = mne.read_forward_solution(fname_fwd)
     if read_all_forward:
-        forward_surf_ori = read_forward_solution_meg(fname_fwd, surf_ori=True)
-        forward_fixed = read_forward_solution_meg(fname_fwd, force_fixed=True,
-                                                  surf_ori=True)
-        forward_vol = mne.read_forward_solution(fname_fwd_vol, surf_ori=True)
+        forward_surf_ori = _read_forward_solution_meg(
+            fname_fwd, surf_ori=True)
+        forward_fixed = _read_forward_solution_meg(
+            fname_fwd, force_fixed=True, use_cps=False)
+        forward_vol = mne.read_forward_solution(fname_fwd_vol)
+        forward_vol = mne.convert_forward_solution(forward_vol, surf_ori=True)
     else:
         forward_surf_ori = None
         forward_fixed = None
@@ -72,16 +70,16 @@ def _get_data(tmin=-0.11, tmax=0.15, read_all_forward=True, compute_csds=True):
                         picks=picks, baseline=(None, 0), preload=True,
                         reject=dict(grad=4000e-13, mag=4e-12, eog=150e-6))
     epochs.resample(200, npad=0, n_jobs=2)
-    evoked = epochs.average()
+    evoked = epochs.average().crop(0, None)
 
     # Computing the data and noise cross-spectral density matrices
     if compute_csds:
-        data_csd = compute_epochs_csd(epochs, mode='multitaper', tmin=0.045,
-                                      tmax=None, fmin=8, fmax=12,
-                                      mt_bandwidth=72.72)
-        noise_csd = compute_epochs_csd(epochs, mode='multitaper', tmin=None,
-                                       tmax=0.0, fmin=8, fmax=12,
-                                       mt_bandwidth=72.72)
+        data_csd = csd_epochs(epochs, mode='multitaper', tmin=0.045,
+                              tmax=None, fmin=8, fmax=12,
+                              mt_bandwidth=72.72)
+        noise_csd = csd_epochs(epochs, mode='multitaper', tmin=None,
+                               tmax=0.0, fmin=8, fmax=12,
+                               mt_bandwidth=72.72)
     else:
         data_csd, noise_csd = None, None
 
@@ -91,28 +89,32 @@ def _get_data(tmin=-0.11, tmax=0.15, read_all_forward=True, compute_csds=True):
 
 @testing.requires_testing_data
 def test_dics():
-    """Test DICS with evoked data and single trials
-    """
+    """Test DICS with evoked data and single trials."""
     raw, epochs, evoked, data_csd, noise_csd, label, forward,\
         forward_surf_ori, forward_fixed, forward_vol = _get_data()
+    epochs.crop(0, None)
+    reg = 0.5  # Heavily regularize due to low SNR
 
-    stc = dics(evoked, forward, noise_csd=noise_csd, data_csd=data_csd,
-               label=label)
+    for real_filter in (True, False):
+        stc = dics(evoked, forward, noise_csd=noise_csd, data_csd=data_csd,
+                   label=label, real_filter=real_filter, reg=reg)
+        stc_pow = np.sum(stc.data, axis=1)
+        idx = np.argmax(stc_pow)
+        max_stc = stc.data[idx]
+        tmax = stc.times[np.argmax(max_stc)]
 
-    stc.crop(0, None)
-    stc_pow = np.sum(stc.data, axis=1)
-    idx = np.argmax(stc_pow)
-    max_stc = stc.data[idx]
-    tmax = stc.times[np.argmax(max_stc)]
-
-    # Incorrect due to limited number of epochs
-    assert_true(0.04 < tmax < 0.05)
-    assert_true(10 < np.max(max_stc) < 13)
+        # Incorrect due to limited number of epochs
+        assert_true(0.04 < tmax < 0.06, msg=tmax)
+        assert_true(3. < np.max(max_stc) < 6., msg=np.max(max_stc))
 
     # Test picking normal orientation
     stc_normal = dics(evoked, forward_surf_ori, noise_csd, data_csd,
-                      pick_ori="normal", label=label)
-    stc_normal.crop(0, None)
+                      pick_ori="normal", label=label, real_filter=True,
+                      reg=reg)
+    assert_true(stc_normal.data.min() < 0)  # this doesn't take abs
+    stc_normal = dics(evoked, forward_surf_ori, noise_csd, data_csd,
+                      pick_ori="normal", label=label, reg=reg)
+    assert_true(stc_normal.data.min() >= 0)  # this does take abs
 
     # The amplitude of normal orientation results should always be smaller than
     # free orientation results
@@ -135,11 +137,10 @@ def test_dics():
 
     # Now test single trial using fixed orientation forward solution
     # so we can compare it to the evoked solution
-    stcs = dics_epochs(epochs, forward_fixed, noise_csd, data_csd, reg=0.01,
-                       label=label)
+    stcs = dics_epochs(epochs, forward_fixed, noise_csd, data_csd, label=label)
 
     # Testing returning of generator
-    stcs_ = dics_epochs(epochs, forward_fixed, noise_csd, data_csd, reg=0.01,
+    stcs_ = dics_epochs(epochs, forward_fixed, noise_csd, data_csd,
                         return_generator=True, label=label)
     assert_array_equal(stcs[0].data, advance_iterator(stcs_).data)
 
@@ -150,38 +151,39 @@ def test_dics():
     # Average the single trial estimates
     stc_avg = np.zeros_like(stc.data)
     for this_stc in stcs:
-        stc_avg += this_stc.crop(0, None).data
+        stc_avg += this_stc.data
     stc_avg /= len(stcs)
 
     idx = np.argmax(np.max(stc_avg, axis=1))
     max_stc = stc_avg[idx]
     tmax = stc.times[np.argmax(max_stc)]
 
-    assert_true(0.045 < tmax < 0.06)  # incorrect due to limited # of epochs
+    assert_true(0.120 < tmax < 0.150, msg=tmax)  # incorrect due to limited #
     assert_true(12 < np.max(max_stc) < 18.5)
 
 
 @testing.requires_testing_data
 def test_dics_source_power():
-    """Test DICS source power computation
-    """
+    """Test DICS source power computation."""
     raw, epochs, evoked, data_csd, noise_csd, label, forward,\
         forward_surf_ori, forward_fixed, forward_vol = _get_data()
+    epochs.crop(0, None)
+    reg = 0.05
 
     stc_source_power = dics_source_power(epochs.info, forward, noise_csd,
-                                         data_csd, label=label)
+                                         data_csd, label=label, reg=reg)
 
     max_source_idx = np.argmax(stc_source_power.data)
     max_source_power = np.max(stc_source_power.data)
 
     # TODO: Maybe these could be more directly compared to dics() results?
-    assert_true(max_source_idx == 0)
-    assert_true(0.5 < max_source_power < 1.15)
+    assert_true(max_source_idx == 1)
+    assert_true(0.004 < max_source_power < 0.005, msg=max_source_power)
 
     # Test picking normal orientation and using a list of CSD matrices
     stc_normal = dics_source_power(epochs.info, forward_surf_ori,
                                    [noise_csd] * 2, [data_csd] * 2,
-                                   pick_ori="normal", label=label)
+                                   pick_ori="normal", label=label, reg=reg)
 
     assert_true(stc_normal.data.shape == (stc_source_power.data.shape[0], 2))
 
@@ -210,8 +212,8 @@ def test_dics_source_power():
                   [noise_csd] * 2, [data_csd] * 3)
 
     # Test detection of different frequencies in noise and data CSD objects
-    noise_csd.frequencies = [1, 2]
-    data_csd.frequencies = [1, 2, 3]
+    noise_csd.freqs = [1, 2]
+    data_csd.freqs = [1, 2, 3]
     assert_raises(ValueError, dics_source_power, epochs.info, forward,
                   noise_csd, data_csd)
 
@@ -219,31 +221,30 @@ def test_dics_source_power():
     data_csds = [cp.deepcopy(data_csd) for i in range(3)]
     frequencies = [1, 3, 4]
     for freq, data_csd in zip(frequencies, data_csds):
-        data_csd.frequencies = [freq]
+        data_csd.freqs = [freq]
     noise_csds = data_csds
     with warnings.catch_warnings(record=True) as w:
         dics_source_power(epochs.info, forward, noise_csds, data_csds)
-    assert len(w) == 1
+    assert_equal(len(w), 1)
 
 
 @testing.requires_testing_data
 def test_tf_dics():
-    """Test TF beamforming based on DICS
-    """
+    """Test TF beamforming based on DICS."""
     tmin, tmax, tstep = -0.2, 0.2, 0.1
     raw, epochs, _, _, _, label, forward, _, _, _ =\
         _get_data(tmin, tmax, read_all_forward=False, compute_csds=False)
 
     freq_bins = [(4, 20), (30, 55)]
     win_lengths = [0.2, 0.2]
-    reg = 0.001
+    reg = 0.05
 
     noise_csds = []
     for freq_bin, win_length in zip(freq_bins, win_lengths):
-        noise_csd = compute_epochs_csd(epochs, mode='fourier',
-                                       fmin=freq_bin[0], fmax=freq_bin[1],
-                                       fsum=True, tmin=tmin,
-                                       tmax=tmin + win_length)
+        noise_csd = csd_epochs(epochs, mode='fourier',
+                               fmin=freq_bin[0], fmax=freq_bin[1],
+                               fsum=True, tmin=tmin,
+                               tmax=tmin + win_length)
         noise_csds.append(noise_csd)
 
     stcs = tf_dics(epochs, forward, noise_csds, tmin, tmax, tstep, win_lengths,
@@ -251,20 +252,22 @@ def test_tf_dics():
 
     assert_true(len(stcs) == len(freq_bins))
     assert_true(stcs[0].shape[1] == 4)
+    assert_true(2.2 < stcs[0].data.max() < 2.3)
+    assert_true(0.94 < stcs[0].data.min() < 0.95)
 
     # Manually calculating source power in several time windows to compare
     # results and test overlapping
     source_power = []
     time_windows = [(-0.1, 0.1), (0.0, 0.2)]
     for time_window in time_windows:
-        data_csd = compute_epochs_csd(epochs, mode='fourier',
-                                      fmin=freq_bins[0][0],
-                                      fmax=freq_bins[0][1], fsum=True,
-                                      tmin=time_window[0], tmax=time_window[1])
-        noise_csd = compute_epochs_csd(epochs, mode='fourier',
-                                       fmin=freq_bins[0][0],
-                                       fmax=freq_bins[0][1], fsum=True,
-                                       tmin=-0.2, tmax=0.0)
+        data_csd = csd_epochs(epochs, mode='fourier',
+                              fmin=freq_bins[0][0],
+                              fmax=freq_bins[0][1], fsum=True,
+                              tmin=time_window[0], tmax=time_window[1])
+        noise_csd = csd_epochs(epochs, mode='fourier',
+                               fmin=freq_bins[0][0],
+                               fmax=freq_bins[0][1], fsum=True,
+                               tmin=-0.2, tmax=0.0)
         data_csd.data /= data_csd.n_fft
         noise_csd.data /= noise_csd.n_fft
         stc_source_power = dics_source_power(epochs.info, forward, noise_csd,
