@@ -6,7 +6,7 @@ import numpy as np
 from numpy.polynomial.legendre import legval
 from scipy import linalg
 
-from ..utils import logger, warn
+from ..utils import logger, warn, verbose
 from ..io.pick import pick_types, pick_channels, pick_info
 from ..surface import _normalize_vectors
 from ..bem import _fit_sphere
@@ -37,7 +37,7 @@ def _calc_g(cosang, stiffness=4, num_lterms=50):
 
 
 def _make_interpolation_matrix(pos_from, pos_to, alpha=1e-5):
-    """Compute interpolation matrix based on spherical splines
+    """Compute interpolation matrix based on spherical splines.
 
     Implementation based on [1]
 
@@ -62,7 +62,6 @@ def _make_interpolation_matrix(pos_from, pos_to, alpha=1e-5):
         Spherical splines for scalp potential and current density mapping.
         Electroencephalography Clinical Neurophysiology, Feb; 72(2):184-7.
     """
-
     pos_from = pos_from.copy()
     pos_to = pos_to.copy()
 
@@ -90,26 +89,24 @@ def _make_interpolation_matrix(pos_from, pos_to, alpha=1e-5):
 
 
 def _do_interp_dots(inst, interpolation, goods_idx, bads_idx):
-    """Dot product of channel mapping matrix to channel data
-    """
-    from ..io.base import _BaseRaw
-    from ..epochs import _BaseEpochs
+    """Dot product of channel mapping matrix to channel data."""
+    from ..io.base import BaseRaw
+    from ..epochs import BaseEpochs
     from ..evoked import Evoked
 
-    if isinstance(inst, _BaseRaw):
+    if isinstance(inst, (BaseRaw, Evoked)):
         inst._data[bads_idx] = interpolation.dot(inst._data[goods_idx])
-    elif isinstance(inst, _BaseEpochs):
+    elif isinstance(inst, BaseEpochs):
         inst._data[:, bads_idx, :] = np.einsum('ij,xjy->xiy', interpolation,
                                                inst._data[:, goods_idx, :])
-    elif isinstance(inst, Evoked):
-        inst.data[bads_idx] = interpolation.dot(inst.data[goods_idx])
     else:
         raise ValueError('Inputs of type {0} are not supported'
                          .format(type(inst)))
 
 
-def _interpolate_bads_eeg(inst):
-    """Interpolate bad EEG channels
+@verbose
+def _interpolate_bads_eeg(inst, verbose=None):
+    """Interpolate bad EEG channels.
 
     Operates in place.
 
@@ -157,6 +154,7 @@ def _interpolate_bads_eeg(inst):
     _do_interp_dots(inst, interpolation, goods_idx, bads_idx)
 
 
+@verbose
 def _interpolate_bads_meg(inst, mode='accurate', verbose=None):
     """Interpolate bad channels from data in good channels.
 
@@ -169,7 +167,8 @@ def _interpolate_bads_meg(inst, mode='accurate', verbose=None):
         Legendre polynomial expansion used for interpolation. `'fast'` should
         be sufficient for most applications.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
     """
     picks_meg = pick_types(inst.info, meg=True, eeg=False, exclude=[])
     picks_good = pick_types(inst.info, meg=True, eeg=False, exclude='bads')
