@@ -15,17 +15,18 @@ import os.path as op
 
 import numpy as np
 
-from ..transforms import _polar_to_cartesian, _cartesian_to_sphere
+from ..transforms import _pol_to_cart, _cart_to_sph
 from ..bem import fit_sphere_to_headshape
 from ..io.pick import pick_types
 from ..io.constants import FIFF
 from ..io.meas_info import Info
-from ..utils import _clean_names
+from ..utils import _clean_names, warn
 from ..externals.six.moves import map
+from .channels import _get_ch_info
 
 
 class Layout(object):
-    """Sensor layouts
+    """Sensor layouts.
 
     Layouts are typically loaded from a file using read_layout. Only use this
     class directly if you're constructing a new layout.
@@ -43,7 +44,8 @@ class Layout(object):
     kind : str
         The type of Layout (e.g. 'Vectorview-all').
     """
-    def __init__(self, box, pos, names, ids, kind):
+
+    def __init__(self, box, pos, names, ids, kind):  # noqa: D102
         self.box = box
         self.pos = pos
         self.names = names
@@ -51,7 +53,7 @@ class Layout(object):
         self.kind = kind
 
     def save(self, fname):
-        """Save Layout to disk
+        """Save Layout to disk.
 
         Parameters
         ----------
@@ -83,14 +85,18 @@ class Layout(object):
         f.close()
 
     def __repr__(self):
+        """Return the string representation."""
         return '<Layout | %s - Channels: %s ...>' % (self.kind,
                                                      ', '.join(self.names[:3]))
 
-    def plot(self, show=True):
+    def plot(self, picks=None, show=True):
         """Plot the sensor positions.
 
         Parameters
         ----------
+        picks : array-like
+            Indices of the channels to show. If None (default), all the
+            channels are shown.
         show : bool
             Show figure if True. Defaults to True.
 
@@ -101,16 +107,14 @@ class Layout(object):
 
         Notes
         -----
-
         .. versionadded:: 0.12.0
-
         """
         from ..viz.topomap import plot_layout
-        return plot_layout(self, show=show)
+        return plot_layout(self, picks=picks, show=show)
 
 
 def _read_lout(fname):
-    """Aux function"""
+    """Aux function."""
     with open(fname) as f:
         box_line = f.readline()  # first line contains box dimension
         box = tuple(map(float, box_line.split()))
@@ -132,7 +136,7 @@ def _read_lout(fname):
 
 
 def _read_lay(fname):
-    """Aux function"""
+    """Aux function."""
     with open(fname) as f:
         box = None
         names, pos, ids = [], [], []
@@ -153,7 +157,7 @@ def _read_lay(fname):
 
 
 def read_layout(kind, path=None, scale=True):
-    """Read layout from a file
+    """Read layout from a file.
 
     Parameters
     ----------
@@ -211,7 +215,7 @@ def read_layout(kind, path=None, scale=True):
 
 
 def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
-    """Create .lout file from EEG electrode digitization
+    """Create .lout file from EEG electrode digitization.
 
     Parameters
     ----------
@@ -286,7 +290,7 @@ def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads'):
 
 
 def make_grid_layout(info, picks=None, n_col=None):
-    """ Generate .lout file for custom data, i.e., ICA sources
+    """Generate .lout file for custom data, i.e., ICA sources.
 
     Parameters
     ----------
@@ -361,7 +365,7 @@ def make_grid_layout(info, picks=None, n_col=None):
 
 
 def find_layout(info, ch_type=None, exclude='bads'):
-    """Choose a layout based on the channels in the info 'chs' field
+    """Choose a layout based on the channels in the info 'chs' field.
 
     Parameters
     ----------
@@ -386,39 +390,12 @@ def find_layout(info, ch_type=None, exclude='bads'):
         raise ValueError('Invalid channel type (%s) requested '
                          '`ch_type` must be %s' % (ch_type, our_types))
 
-    chs = info['chs']
-    coil_types = set([ch['coil_type'] for ch in chs])
-    channel_types = set([ch['kind'] for ch in chs])
-
-    has_vv_mag = any(k in coil_types for k in
-                     [FIFF.FIFFV_COIL_VV_MAG_T1, FIFF.FIFFV_COIL_VV_MAG_T2,
-                      FIFF.FIFFV_COIL_VV_MAG_T3])
-    has_vv_grad = any(k in coil_types for k in [FIFF.FIFFV_COIL_VV_PLANAR_T1,
-                                                FIFF.FIFFV_COIL_VV_PLANAR_T2,
-                                                FIFF.FIFFV_COIL_VV_PLANAR_T3])
+    (has_vv_mag, has_vv_grad, is_old_vv, has_4D_mag, ctf_other_types,
+     has_CTF_grad, n_kit_grads, has_any_meg, has_eeg_coils,
+     has_eeg_coils_and_meg, has_eeg_coils_only) = _get_ch_info(info)
     has_vv_meg = has_vv_mag and has_vv_grad
     has_vv_only_mag = has_vv_mag and not has_vv_grad
     has_vv_only_grad = has_vv_grad and not has_vv_mag
-    is_old_vv = ' ' in chs[0]['ch_name']
-
-    has_4D_mag = FIFF.FIFFV_COIL_MAGNES_MAG in coil_types
-    ctf_other_types = (FIFF.FIFFV_COIL_CTF_REF_MAG,
-                       FIFF.FIFFV_COIL_CTF_REF_GRAD,
-                       FIFF.FIFFV_COIL_CTF_OFFDIAG_REF_GRAD)
-    has_CTF_grad = (FIFF.FIFFV_COIL_CTF_GRAD in coil_types or
-                    (FIFF.FIFFV_MEG_CH in channel_types and
-                     any(k in ctf_other_types for k in coil_types)))
-    # hack due to MNE-C bug in IO of CTF
-    n_kit_grads = sum(ch['coil_type'] == FIFF.FIFFV_COIL_KIT_GRAD
-                      for ch in chs)
-
-    has_any_meg = any([has_vv_mag, has_vv_grad, has_4D_mag, has_CTF_grad,
-                       n_kit_grads])
-    has_eeg_coils = (FIFF.FIFFV_COIL_EEG in coil_types and
-                     FIFF.FIFFV_EEG_CH in channel_types)
-    has_eeg_coils_and_meg = has_eeg_coils and has_any_meg
-    has_eeg_coils_only = has_eeg_coils and not has_any_meg
-
     if ch_type == "meg" and not has_any_meg:
         raise RuntimeError('No MEG channels present. Cannot find MEG layout.')
 
@@ -445,12 +422,13 @@ def find_layout(info, ch_type=None, exclude='bads'):
         layout_name = 'magnesWH3600'
     elif has_CTF_grad:
         layout_name = 'CTF-275'
-    elif n_kit_grads <= 157:
-        layout_name = 'KIT-157'
-    elif n_kit_grads > 157:
-        layout_name = 'KIT-AD'
+    elif n_kit_grads > 0:
+        layout_name = _find_kit_layout(info, n_kit_grads)
     else:
-        return None
+        xy = _auto_topomap_coords(info, picks=range(info['nchan']),
+                                  ignore_overlap=True, to_sphere=False)
+        return generate_2d_layout(xy, ch_names=info['ch_names'], name='custom',
+                                  normalize=False)
 
     layout = read_layout(layout_name)
     if not is_old_vv:
@@ -458,11 +436,69 @@ def find_layout(info, ch_type=None, exclude='bads'):
     if has_CTF_grad:
         layout.names = _clean_names(layout.names, before_dash=True)
 
+    # Apply mask for excluded channels.
+    if exclude == 'bads':
+        exclude = info['bads']
+    idx = [ii for ii, name in enumerate(layout.names) if name not in exclude]
+    layout.names = [layout.names[ii] for ii in idx]
+    layout.pos = layout.pos[idx]
+    layout.ids = [layout.ids[ii] for ii in idx]
+
     return layout
 
 
+def _find_kit_layout(info, n_grads):
+    """Determine the KIT layout.
+
+    Parameters
+    ----------
+    info : Info
+        Info object.
+    n_grads : int
+        Number of KIT-gradiometers in the info.
+
+    Returns
+    -------
+    kit_layout : str
+        One of 'KIT-AD', 'KIT-157' or 'KIT-UMD'.
+    """
+    if info['kit_system_id'] is not None:
+        # avoid circular import
+        from ..io.kit.constants import KIT_LAYOUT
+
+        if info['kit_system_id'] in KIT_LAYOUT:
+            kit_layout = KIT_LAYOUT[info['kit_system_id']]
+            if kit_layout is not None:
+                return kit_layout
+        raise NotImplementedError("The layout for the KIT system with ID %i "
+                                  "is missing. Please contact the developers "
+                                  "about adding it." % info['kit_system_id'])
+    elif n_grads > 157:
+        return 'KIT-AD'
+
+    # channels which are on the left hemisphere for NY and right for UMD
+    test_chs = ('MEG  13', 'MEG  14', 'MEG  15', 'MEG  16', 'MEG  25',
+                'MEG  26', 'MEG  27', 'MEG  28', 'MEG  29', 'MEG  30',
+                'MEG  31', 'MEG  32', 'MEG  57', 'MEG  60', 'MEG  61',
+                'MEG  62', 'MEG  63', 'MEG  64', 'MEG  73', 'MEG  90',
+                'MEG  93', 'MEG  95', 'MEG  96', 'MEG 105', 'MEG 112',
+                'MEG 120', 'MEG 121', 'MEG 122', 'MEG 123', 'MEG 124',
+                'MEG 125', 'MEG 126', 'MEG 142', 'MEG 144', 'MEG 153',
+                'MEG 154', 'MEG 155', 'MEG 156')
+    x = [ch['loc'][0] < 0 for ch in info['chs'] if ch['ch_name'] in test_chs]
+    if np.all(x):
+        return 'KIT-157'  # KIT-NY
+    elif np.all(np.invert(x)):
+        raise NotImplementedError("Guessing sensor layout for legacy UMD "
+                                  "files is not implemented. Please convert "
+                                  "your files using MNE-Python 0.13 or "
+                                  "higher.")
+    else:
+        raise RuntimeError("KIT system could not be determined for data")
+
+
 def _box_size(points, width=None, height=None, padding=0.0):
-    """ Given a series of points, calculate an appropriate box size.
+    """Given a series of points, calculate an appropriate box size.
 
     Parameters
     ----------
@@ -550,7 +586,7 @@ def _box_size(points, width=None, height=None, padding=0.0):
 
 
 def _find_topomap_coords(info, picks, layout=None):
-    """Try to guess the E/MEG layout and return appropriate topomap coordinates
+    """Guess the E/MEG layout and return appropriate topomap coordinates.
 
     Parameters
     ----------
@@ -581,8 +617,9 @@ def _find_topomap_coords(info, picks, layout=None):
     return pos
 
 
-def _auto_topomap_coords(info, picks, ignore_overlap=False):
+def _auto_topomap_coords(info, picks, ignore_overlap=False, to_sphere=True):
     """Make a 2 dimensional sensor map from sensor positions in an info dict.
+
     The default is to use the electrode locations. The fallback option is to
     attempt using digitization points of kind FIFFV_POINT_EEG. This only works
     with EEG and requires an equal number of digitization points and sensors.
@@ -593,6 +630,12 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False):
         The measurement info.
     picks : list of int
         The channel indices to generate topomap coords for.
+    ignore_overlap : bool
+        Whether to ignore overlapping positions in the layout. If False and
+        positions overlap, an error is thrown.
+    to_sphere : bool
+        If True, the radial distance of spherical coordinates is ignored, in
+        effect fitting the xyz-coordinates to a sphere. Defaults to True.
 
     Returns
     -------
@@ -656,7 +699,7 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False):
 
     # Duplicate points cause all kinds of trouble during visualization
     dist = pdist(locs3d)
-    if np.min(dist) < 1e-10 and not ignore_overlap:
+    if len(locs3d) > 1 and np.min(dist) < 1e-10 and not ignore_overlap:
         problematic_electrodes = [
             chs[elec_i]['ch_name']
             for elec_i in squareform(dist < 1e-10).any(axis=0).nonzero()[0]
@@ -666,14 +709,14 @@ def _auto_topomap_coords(info, picks, ignore_overlap=False):
                          '\n    ' + str(problematic_electrodes) + '\nThis '
                          'causes problems during visualization.')
 
-    x, y, z = locs3d.T
-    az, el, r = _cartesian_to_sphere(x, y, z)
-    locs2d = np.c_[_polar_to_cartesian(az, np.pi / 2 - el)]
-    return locs2d
+    if to_sphere:
+        # use spherical (theta, pol) as (r, theta) for polar->cartesian
+        return _pol_to_cart(_cart_to_sph(locs3d)[:, 1:][:, ::-1])
+    return _pol_to_cart(_cart_to_sph(locs3d))
 
 
 def _topo_to_sphere(pos, eegs):
-    """Helper function for transforming xy-coordinates to sphere.
+    """Transform xy-coordinates to sphere.
 
     Parameters
     ----------
@@ -705,8 +748,9 @@ def _topo_to_sphere(pos, eegs):
     return np.column_stack([xs, ys, zs])
 
 
-def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads'):
-    """Find the picks for pairing grad channels
+def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads',
+                       raise_error=True):
+    """Find the picks for pairing grad channels.
 
     Parameters
     ----------
@@ -720,6 +764,9 @@ def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads'):
     exclude : list of str | str
         List of channels to exclude. If empty do not exclude any (default).
         If 'bads', exclude channels in info['bads']. Defaults to 'bads'.
+    raise_error : bool
+        Whether to raise an error when no pairs are found. If False, raises a
+        warning.
 
     Returns
     -------
@@ -741,7 +788,11 @@ def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads'):
                 pairs[key].append(ch)
     pairs = [p for p in pairs.values() if len(p) == 2]
     if len(pairs) == 0:
-        raise ValueError("No 'grad' channel pairs found.")
+        if raise_error:
+            raise ValueError("No 'grad' channel pairs found.")
+        else:
+            warn("No 'grad' channel pairs found.")
+            return list()
 
     # find the picks corresponding to the grad channels
     grad_chs = sum(pairs, [])
@@ -760,7 +811,7 @@ def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads'):
 # this function is used to pair grad when info is not present
 # it is the case of Projection that don't have the info.
 def _pair_grad_sensors_from_ch_names(ch_names):
-    """Find the indexes for pairing grad channels
+    """Find the indexes for pairing grad channels.
 
     Parameters
     ----------
@@ -785,26 +836,34 @@ def _pair_grad_sensors_from_ch_names(ch_names):
     return grad_chs
 
 
-def _merge_grad_data(data):
-    """Merge data from channel pairs using the RMS
+def _merge_grad_data(data, method='rms'):
+    """Merge data from channel pairs using the RMS or mean.
 
     Parameters
     ----------
     data : array, shape = (n_channels, n_times)
         Data for channels, ordered in pairs.
+    method : str
+        Can be 'rms' or 'mean'.
 
     Returns
     -------
     data : array, shape = (n_channels / 2, n_times)
-        The root mean square for each pair.
+        The root mean square or mean for each pair.
     """
     data = data.reshape((len(data) // 2, 2, -1))
-    data = np.sqrt(np.sum(data ** 2, axis=1) / 2)
+    if method == 'mean':
+        data = np.mean(data, axis=1)
+    elif method == 'rms':
+        data = np.sqrt(np.sum(data ** 2, axis=1) / 2)
+    else:
+        raise ValueError('method must be "rms" or "mean, got %s.' % method)
     return data
 
 
 def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
-                       ch_indices=None, name='ecog', bg_image=None):
+                       ch_indices=None, name='ecog', bg_image=None,
+                       normalize=True):
     """Generate a custom 2D layout from xy points.
 
     Generates a 2-D layout for plotting with plot_topo methods and
@@ -836,6 +895,9 @@ def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
         image file, or an array that can be plotted with plt.imshow. If
         provided, xy points will be normalized by the width/height of this
         image. If not, xy points will be normalized by their own min/max.
+    normalize : bool
+        Whether to normalize the coordinates to run from 0 to 1. Defaults to
+        True.
 
     Returns
     -------
@@ -874,7 +936,7 @@ def generate_2d_layout(xy, w=.07, h=.05, pad=.02, ch_names=None,
             img = bg_image
         x /= img.shape[1]
         y /= img.shape[0]
-    else:
+    elif normalize:
         # Normalize x and y by their maxes
         for i_dim in [x, y]:
             i_dim -= i_dim.min(0)
