@@ -1,5 +1,4 @@
-""" Connectivity Analysis Tools
-"""
+"""Spectral and effective connectivity measures."""
 
 from .utils import seed_target_indices
 from .spectral import spectral_connectivity
