@@ -4,7 +4,7 @@
 #
 # License: BSD (3-clause)
 
-import copy as cp
+from copy import deepcopy
 from distutils.version import LooseVersion
 import itertools as itt
 from math import log
@@ -15,13 +15,14 @@ from scipy import linalg
 
 from .io.write import start_file, end_file
 from .io.proj import (make_projector, _proj_equal, activate_proj,
-                      _needs_eeg_average_ref_proj)
+                      _needs_eeg_average_ref_proj, _check_projs,
+                      _has_eeg_average_ref_proj)
 from .io import fiff_open
 from .io.pick import (pick_types, pick_channels_cov, pick_channels, pick_info,
                       _picks_by_type, _pick_data_channels)
 
 from .io.constants import FIFF
-from .io.meas_info import read_bad_channels
+from .io.meas_info import read_bad_channels, _simplify_info
 from .io.proj import _read_proj, _write_proj
 from .io.tag import find_tag
 from .io.tree import dir_tree_find
@@ -32,11 +33,12 @@ from .epochs import Epochs
 from .event import make_fixed_length_events
 from .utils import (check_fname, logger, verbose, estimate_rank,
                     _compute_row_norms, check_version, _time_mask, warn,
-                    copy_function_doc_to_method_doc)
+                    copy_function_doc_to_method_doc, _pl)
 from . import viz
 
 from .externals.six.moves import zip
 from .externals.six import string_types
+from .fixes import BaseEstimator
 
 
 def _check_covs_algebra(cov1, cov2):
@@ -51,7 +53,7 @@ def _check_covs_algebra(cov1, cov2):
 
 
 def _get_tslice(epochs, tmin, tmax):
-    """get the slice."""
+    """Get the slice."""
     mask = _time_mask(epochs.times, tmin, tmax, sfreq=epochs.info['sfreq'])
     tstart = np.where(mask)[0][0] if tmin is not None else None
     tend = np.where(mask)[0][-1] + 1 if tmax is not None else None
@@ -60,7 +62,6 @@ def _get_tslice(epochs, tmin, tmax):
 
 
 class Covariance(dict):
-
     """Noise covariance matrix.
 
     .. warning:: This class should not be instantiated directly, but
@@ -96,6 +97,8 @@ class Covariance(dict):
         List of channels' names.
     nfree : int
         Number of degrees of freedom i.e. number of time points used.
+    dim : int
+        The number of channels ``n_channels``.
 
     See Also
     --------
@@ -109,6 +112,7 @@ class Covariance(dict):
                  method=None, loglik=None):
         """Init of covariance."""
         diag = True if data.ndim == 1 else False
+        projs = _check_projs(projs)
         self.update(data=data, dim=len(data), names=names, bads=bads,
                     nfree=nfree, eig=eig, eigvec=eigvec, diag=diag,
                     projs=projs, kind=FIFF.FIFFV_MNE_NOISE_COV)
@@ -146,22 +150,22 @@ class Covariance(dict):
 
         try:
             _write_cov(fid, self)
-        except Exception as inst:
+        except Exception:
             fid.close()
             os.remove(fname)
-            raise inst
+            raise
 
         end_file(fid)
 
     def copy(self):
-        """Copy the Covariance object
+        """Copy the Covariance object.
 
         Returns
         -------
         cov : instance of Covariance
             The copied object.
         """
-        return cp.deepcopy(self)
+        return deepcopy(self)
 
     def as_diag(self):
         """Set covariance to be processed as being diagonal.
@@ -184,7 +188,7 @@ class Covariance(dict):
         self['eigvec'] = None
         return self
 
-    def __repr__(self):
+    def __repr__(self):  # noqa: D105
         if self.data.ndim == 2:
             s = 'size : %s x %s' % self.data.shape
         else:  # ndim == 1
@@ -196,7 +200,7 @@ class Covariance(dict):
     def __add__(self, cov):
         """Add Covariance taking into account number of degrees of freedom."""
         _check_covs_algebra(self, cov)
-        this_cov = cp.deepcopy(cov)
+        this_cov = cov.copy()
         this_cov['data'] = (((this_cov['data'] * this_cov['nfree']) +
                              (self['data'] * self['nfree'])) /
                             (self['nfree'] + this_cov['nfree']))
@@ -239,7 +243,8 @@ def read_cov(fname, verbose=None):
         The name of file containing the covariance matrix. It should end with
         -cov.fif or -cov.fif.gz.
     verbose : bool, str, int, or None (default None)
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -269,7 +274,8 @@ def make_ad_hoc_cov(info, verbose=None):
     info : instance of Info
         Measurement info.
     verbose : bool, str, int, or None (default None)
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -278,10 +284,12 @@ def make_ad_hoc_cov(info, verbose=None):
 
     Notes
     -----
+    This uses values of 5 fT/cm, 20 fT, and 0.2 uV for gradiometers,
+    magnetometers, and EEG channels, respectively.
+
     .. versionadded:: 0.9.0
     """
-    info = pick_info(info, pick_types(info, meg=True, eeg=True, exclude=[]))
-    info._check_consistency()
+    picks = pick_types(info, meg=True, eeg=True, exclude=())
 
     # Standard deviations to be used
     grad_std = 5e-13
@@ -291,12 +299,13 @@ def make_ad_hoc_cov(info, verbose=None):
                 '(MEG grad : %6.1f fT/cm MEG mag : %6.1f fT EEG : %6.1f uV)'
                 % (1e13 * grad_std, 1e15 * mag_std, 1e6 * eeg_std))
 
-    data = np.zeros(len(info['ch_names']))
+    data = np.zeros(len(picks))
     for meg, eeg, val in zip(('grad', 'mag', False), (False, False, True),
                              (grad_std, mag_std, eeg_std)):
-        data[pick_types(info, meg=meg, eeg=eeg)] = val * val
-    return Covariance(data, info['ch_names'], info['bads'], info['projs'],
-                      nfree=0)
+        these_picks = pick_types(info, meg=meg, eeg=eeg)
+        data[np.searchsorted(picks, these_picks)] = val * val
+    ch_names = [info['ch_names'][pick] for pick in picks]
+    return Covariance(data, ch_names, info['bads'], info['projs'], nfree=0)
 
 
 def _check_n_samples(n_samples, n_chan):
@@ -313,7 +322,8 @@ def _check_n_samples(n_samples, n_chan):
 def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
                            flat=None, picks=None, method='empirical',
                            method_params=None, cv=3, scalings=None, n_jobs=1,
-                           return_estimators=False, verbose=None):
+                           return_estimators=False, reject_by_annotation=True,
+                           verbose=None):
     """Estimate noise covariance matrix from a continuous segment of raw data.
 
     It is typically useful to estimate a noise covariance from empty room
@@ -380,7 +390,7 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
 
         .. versionadded:: 0.12
 
-    cv : int | sklearn cross_validation object (default 3)
+    cv : int | sklearn model_selection object (default 3)
         The cross validation method. Defaults to 3, which will
         internally trigger a default 3-fold shuffle split.
 
@@ -404,8 +414,16 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
 
         .. versionadded:: 0.12
 
+    reject_by_annotation : bool
+        Whether to reject based on annotations. If True (default), epochs
+        overlapping with segments whose description begins with ``'bad'`` are
+        rejected. If False, no rejection based on annotations is performed.
+
+        .. versionadded:: 0.14.0
+
     verbose : bool | str | int | None (default None)
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -424,8 +442,7 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
     tstep = tmax - tmin if tstep is None else float(tstep)
     tstep_m1 = tstep - 1. / raw.info['sfreq']  # inclusive!
     events = make_fixed_length_events(raw, 1, tmin, tmax, tstep)
-    pl = 's' if len(events) != 1 else ''
-    logger.info('Using up to %s segment%s' % (len(events), pl))
+    logger.info('Using up to %s segment%s' % (len(events), _pl(events)))
 
     # don't exclude any bad channels, inverses expect all channels present
     if picks is None:
@@ -437,7 +454,8 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
         pick_mask = slice(None)
     epochs = Epochs(raw, events, 1, 0, tstep_m1, baseline=None,
                     picks=picks, reject=reject, flat=flat, verbose=False,
-                    preload=False, proj=False, add_eeg_ref=False)
+                    preload=False, proj=False,
+                    reject_by_annotation=reject_by_annotation)
     if method is None:
         method = 'empirical'
     if isinstance(method, string_types) and method == 'empirical':
@@ -459,8 +477,8 @@ def compute_raw_covariance(raw, tmin=0, tmax=None, tstep=0.2, reject=None,
         logger.info('[done]')
         ch_names = [raw.info['ch_names'][k] for k in picks]
         bads = [b for b in raw.info['bads'] if b in ch_names]
-        projs = cp.deepcopy(raw.info['projs'])
-        return Covariance(data, ch_names, bads, projs, nfree=n_samples)
+        return Covariance(data, ch_names, bads, raw.info['projs'],
+                          nfree=n_samples)
     del picks, pick_mask
 
     # This makes it equivalent to what we used to do (and do above for
@@ -576,7 +594,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
             'pca': {'iter_n_components': None},
             'factor_analysis': {'iter_n_components': None}
 
-    cv : int | sklearn cross_validation object (default 3)
+    cv : int | sklearn model_selection object (default 3)
         The cross validation method. Defaults to 3, which will
         internally trigger a default 3-fold shuffle split.
     scalings : dict | None (default None)
@@ -597,7 +615,8 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
         have been processed with Maxwell filtering but not transformed
         to the same head position.
     verbose : bool | str | int | or None (default None)
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -631,13 +650,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
            'in a list) are "%s" or None.' % '" or "'.join(accepted_methods))
 
     # scale to natural unit for best stability with MEG/EEG
-    if isinstance(scalings, dict):
-        for k, v in scalings.items():
-            if k not in ('mag', 'grad', 'eeg'):
-                raise ValueError('The keys in `scalings` must be "mag" or'
-                                 '"grad" or "eeg". You gave me: %s' % k)
-    scalings = _handle_default('scalings', scalings)
-
+    scalings = _check_scalings_user(scalings)
     _method_params = {
         'empirical': {'store_precision': False, 'assume_centered': True},
         'diagonal_fixed': {'grad': 0.01, 'mag': 0.01, 'eeg': 0.0,
@@ -698,7 +711,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
 
     bads = epochs[0].info['bads']
     if projs is None:
-        projs = cp.deepcopy(epochs[0].info['projs'])
+        projs = epochs[0].info['projs']
         # make sure Epochs are compatible
         for epochs_t in epochs[1:]:
             if epochs_t.proj != epochs[0].proj:
@@ -706,8 +719,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
             for proj_a, proj_b in zip(epochs_t.info['projs'], projs):
                 if not _proj_equal(proj_a, proj_b):
                     raise ValueError('Epochs must have same projectors')
-    else:
-        projs = cp.deepcopy(projs)
+    projs = _check_projs(projs)
     ch_names = epochs[0].ch_names
 
     # make sure Epochs are compatible
@@ -821,7 +833,7 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
         cov.update(method=this_method, **data)
         covs.append(cov)
 
-    if ok_sklearn:
+    if ok_sklearn and len(covs) > 1:
         msg = ['log-likelihood on unseen data (descending order):']
         logliks = [(c['method'], c['loglik']) for c in covs]
         logliks.sort(reverse=True, key=lambda c: c[1])
@@ -829,11 +841,11 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
             msg.append('%s: %0.3f' % (k, v))
         logger.info('\n   '.join(msg))
 
-    if ok_sklearn and not return_estimators:
+    if ok_sklearn and not return_estimators and len(covs) > 1:
         keys, scores = zip(*[(c['method'], c['loglik']) for c in covs])
         out = covs[np.argmax(scores)]
         logger.info('selecting best estimator: {0}'.format(out['method']))
-    elif ok_sklearn:
+    elif ok_sklearn and len(covs) > 1:
         out = covs
         out.sort(key=lambda c: c['loglik'], reverse=True)
     else:
@@ -842,10 +854,23 @@ def compute_covariance(epochs, keep_sample_mean=True, tmin=None, tmax=None,
     return out
 
 
+def _check_scalings_user(scalings):
+    if isinstance(scalings, dict):
+        for k, v in scalings.items():
+            if k not in ('mag', 'grad', 'eeg'):
+                raise ValueError('The keys in `scalings` must be "mag" or'
+                                 '"grad" or "eeg". You gave me: %s' % k)
+    elif scalings is not None and not isinstance(scalings, np.ndarray):
+        raise TypeError('scalings must be a dict, ndarray, or None, got %s'
+                        % type(scalings))
+    scalings = _handle_default('scalings', scalings)
+    return scalings
+
+
 def _compute_covariance_auto(data, method, info, method_params, cv,
                              scalings, n_jobs, stop_early, picks_list,
                              verbose):
-    """docstring for _compute_covariance_auto."""
+    """Compute covariance auto mode."""
     try:
         from sklearn.model_selection import GridSearchCV
     except Exception:  # XXX support sklearn < 0.18
@@ -857,7 +882,6 @@ def _compute_covariance_auto(data, method, info, method_params, cv,
     _apply_scaling_array(data.T, picks_list=picks_list, scalings=scalings)
     estimator_cov_info = list()
     msg = 'Estimating covariance using %s'
-    _RegCovariance, _ShrunkCovariance = _get_covariance_classes()
     for this_method in method:
         data_ = data.copy()
         name = this_method.__name__ if callable(this_method) else this_method
@@ -933,9 +957,13 @@ def _compute_covariance_auto(data, method, info, method_params, cv,
                              ' a .fit method')
         logger.info('Done.')
 
-    logger.info('Using cross-validation to select the best estimator.')
     estimators, _, _ = zip(*estimator_cov_info)
-    logliks = np.array([_cross_val(data, e, cv, n_jobs) for e in estimators])
+    if len(method) > 1:
+        logger.info('Using cross-validation to select the best estimator.')
+        logliks = np.array(
+            [_cross_val(data, e, cv, n_jobs) for e in estimators])
+    else:
+        logliks = [None]
 
     # undo scaling
     for c in estimator_cov_info:
@@ -976,7 +1004,7 @@ def _gaussian_loglik_scorer(est, X, y=None):
 
 
 def _cross_val(data, est, cv, n_jobs):
-    """Helper to compute cross validation."""
+    """Compute cross validation."""
     try:
         from sklearn.model_selection import cross_val_score
     except ImportError:
@@ -988,8 +1016,8 @@ def _cross_val(data, est, cv, n_jobs):
 
 def _auto_low_rank_model(data, mode, n_jobs, method_params, cv,
                          stop_early=True, verbose=None):
-    """compute latent variable models."""
-    method_params = cp.deepcopy(method_params)
+    """Compute latent variable models."""
+    method_params = deepcopy(method_params)
     iter_n_components = method_params.pop('iter_n_components')
     if iter_n_components is None:
         iter_n_components = np.arange(5, data.shape[1], 5)
@@ -1007,7 +1035,7 @@ def _auto_low_rank_model(data, mode, n_jobs, method_params, cv,
     scores.fill(np.nan)
 
     # make sure we don't empty the thing if it's a generator
-    max_n = max(list(cp.deepcopy(iter_n_components)))
+    max_n = max(list(deepcopy(iter_n_components)))
     if max_n > data.shape[1]:
         warn('You are trying to estimate %i components on matrix '
              'with %i features.' % (max_n, data.shape[1]))
@@ -1047,116 +1075,119 @@ def _auto_low_rank_model(data, mode, n_jobs, method_params, cv,
     return est, runtime_info
 
 
-def _get_covariance_classes():
-    """Prepare special cov estimators."""
-    from sklearn.covariance import (EmpiricalCovariance, shrunk_covariance,
-                                    ShrunkCovariance)
+###############################################################################
+# Sklearn Estimators
 
-    class _RegCovariance(EmpiricalCovariance):
+class _RegCovariance(BaseEstimator):
+    """Aux class."""
 
-        """Aux class."""
+    def __init__(self, info, grad=0.01, mag=0.01, eeg=0.0,
+                 store_precision=False, assume_centered=False):
+        self.info = info
+        self.grad = grad
+        self.mag = mag
+        self.eeg = eeg
+        self.store_precision = store_precision
+        self.assume_centered = assume_centered
 
-        def __init__(self, info, grad=0.01, mag=0.01, eeg=0.0,
-                     store_precision=False, assume_centered=False):
-            self.info = info
-            self.grad = grad
-            self.mag = mag
-            self.eeg = eeg
-            self.store_precision = store_precision
-            self.assume_centered = assume_centered
+    def fit(self, X):
+        """Fit covariance model with classical diagonal regularization."""
+        from sklearn.covariance import EmpiricalCovariance
+        self.estimator_ = EmpiricalCovariance(
+            store_precision=self.store_precision,
+            assume_centered=self.assume_centered)
 
-        def fit(self, X):
-            EmpiricalCovariance.fit(self, X)
-            self.covariance_ = 0.5 * (self.covariance_ + self.covariance_.T)
-            cov_ = Covariance(
-                data=self.covariance_, names=self.info['ch_names'],
-                bads=self.info['bads'], projs=self.info['projs'],
-                nfree=len(self.covariance_))
-            cov_ = regularize(cov_, self.info, grad=self.grad, mag=self.mag,
-                              eeg=self.eeg, proj=False,
-                              exclude='bads')  # ~proj == important!!
-            self.covariance_ = cov_.data
-            return self
+        self.covariance_ = self.estimator_.fit(X).covariance_
+        self.covariance_ = 0.5 * (self.covariance_ + self.covariance_.T)
+        cov_ = Covariance(
+            data=self.covariance_, names=self.info['ch_names'],
+            bads=self.info['bads'], projs=self.info['projs'],
+            nfree=len(self.covariance_))
+        cov_ = regularize(cov_, self.info, grad=self.grad, mag=self.mag,
+                          eeg=self.eeg, proj=False,
+                          exclude='bads')  # ~proj == important!!
+        self.estimator_.covariance_ = self.covariance_ = cov_.data
+        return self
 
-    class _ShrunkCovariance(ShrunkCovariance):
+    def score(self, X_test, y=None):
+        """Delegate call to modified EmpiricalCovariance instance."""
+        return self.estimator_.score(X_test, y=y)
 
-        """Aux class."""
+    def get_precision(self):
+        """Delegate call to modified EmpiricalCovariance instance."""
+        return self.estimator_.get_precision()
 
-        def __init__(self, store_precision, assume_centered, shrinkage=0.1):
-            self.store_precision = store_precision
-            self.assume_centered = assume_centered
-            self.shrinkage = shrinkage
 
-        def fit(self, X):
-            EmpiricalCovariance.fit(self, X)
-            cov = self.covariance_
+class _ShrunkCovariance(BaseEstimator):
+    """Aux class."""
 
-            if not isinstance(self.shrinkage, (list, tuple)):
-                shrinkage = [('all', self.shrinkage, np.arange(len(cov)))]
-            else:
-                shrinkage = self.shrinkage
+    def __init__(self, store_precision, assume_centered,
+                 shrinkage=0.1):
 
-            zero_cross_cov = np.zeros_like(cov, dtype=bool)
-            for a, b in itt.combinations(shrinkage, 2):
-                picks_i, picks_j = a[2], b[2]
-                ch_ = a[0], b[0]
-                if 'eeg' in ch_:
-                    zero_cross_cov[np.ix_(picks_i, picks_j)] = True
-                    zero_cross_cov[np.ix_(picks_j, picks_i)] = True
+        self.store_precision = store_precision
+        self.assume_centered = assume_centered
+        self.shrinkage = shrinkage
 
-            self.zero_cross_cov_ = zero_cross_cov
+    def fit(self, X):
+        """Fit covariance model with oracle shrinkage regularization."""
+        from sklearn.covariance import shrunk_covariance
+        from sklearn.covariance import EmpiricalCovariance
+        self.estimator_ = EmpiricalCovariance(
+            store_precision=self.store_precision,
+            assume_centered=self.assume_centered)
 
-            # Apply shrinkage to blocks
-            for ch_type, c, picks in shrinkage:
-                sub_cov = cov[np.ix_(picks, picks)]
-                cov[np.ix_(picks, picks)] = shrunk_covariance(sub_cov,
-                                                              shrinkage=c)
+        cov = self.estimator_.fit(X).covariance_
 
-            # Apply shrinkage to cross-cov
-            for a, b in itt.combinations(shrinkage, 2):
-                shrinkage_i, shrinkage_j = a[1], b[1]
-                picks_i, picks_j = a[2], b[2]
-                c_ij = np.sqrt((1. - shrinkage_i) * (1. - shrinkage_j))
-                cov[np.ix_(picks_i, picks_j)] *= c_ij
-                cov[np.ix_(picks_j, picks_i)] *= c_ij
+        if not isinstance(self.shrinkage, (list, tuple)):
+            shrinkage = [('all', self.shrinkage, np.arange(len(cov)))]
+        else:
+            shrinkage = self.shrinkage
 
-            # Set to zero the necessary cross-cov
-            if np.any(zero_cross_cov):
-                cov[zero_cross_cov] = 0.0
+        zero_cross_cov = np.zeros_like(cov, dtype=bool)
+        for a, b in itt.combinations(shrinkage, 2):
+            picks_i, picks_j = a[2], b[2]
+            ch_ = a[0], b[0]
+            if 'eeg' in ch_:
+                zero_cross_cov[np.ix_(picks_i, picks_j)] = True
+                zero_cross_cov[np.ix_(picks_j, picks_i)] = True
 
-            self.covariance_ = cov
-            return self
+        self.zero_cross_cov_ = zero_cross_cov
 
-        def score(self, X_test, y=None):
-            """Compute the log-likelihood of a Gaussian data set with
-            `self.covariance_` as an estimator of its covariance matrix.
+        # Apply shrinkage to blocks
+        for ch_type, c, picks in shrinkage:
+            sub_cov = cov[np.ix_(picks, picks)]
+            cov[np.ix_(picks, picks)] = shrunk_covariance(sub_cov,
+                                                          shrinkage=c)
 
-            Parameters
-            ----------
-            X_test : array-like, shape = [n_samples, n_features]
-                Test data of which we compute the likelihood, where n_samples
-                is the number of samples and n_features is the number of
-                features. X_test is assumed to be drawn from the same
-                distribution as the data used in fit (including centering).
+        # Apply shrinkage to cross-cov
+        for a, b in itt.combinations(shrinkage, 2):
+            shrinkage_i, shrinkage_j = a[1], b[1]
+            picks_i, picks_j = a[2], b[2]
+            c_ij = np.sqrt((1. - shrinkage_i) * (1. - shrinkage_j))
+            cov[np.ix_(picks_i, picks_j)] *= c_ij
+            cov[np.ix_(picks_j, picks_i)] *= c_ij
 
-            y : not used, present for API consistence purpose.
+        # Set to zero the necessary cross-cov
+        if np.any(zero_cross_cov):
+            cov[zero_cross_cov] = 0.0
 
-            Returns
-            -------
-            res : float
-                The likelihood of the data set with `self.covariance_` as an
-                estimator of its covariance matrix.
-            """
-            from sklearn.covariance import empirical_covariance, log_likelihood
-            # compute empirical covariance of the test set
-            test_cov = empirical_covariance(X_test - self.location_,
-                                            assume_centered=True)
-            if np.any(self.zero_cross_cov_):
-                test_cov[self.zero_cross_cov_] = 0.
-            res = log_likelihood(test_cov, self.get_precision())
-            return res
+        self.estimator_.covariance_ = self.covariance_ = cov
+        return self
 
-    return _RegCovariance, _ShrunkCovariance
+    def score(self, X_test, y=None):
+        """Delegate to modified EmpiricalCovariance instance."""
+        from sklearn.covariance import empirical_covariance, log_likelihood
+        # compute empirical covariance of the test set
+        test_cov = empirical_covariance(X_test - self.estimator_.location_,
+                                        assume_centered=True)
+        if np.any(self.zero_cross_cov_):
+            test_cov[self.zero_cross_cov_] = 0.
+        res = log_likelihood(test_cov, self.estimator_.get_precision())
+        return res
+
+    def get_precision(self):
+        """Delegate to modified EmpiricalCovariance instance."""
+        return self.estimator_.get_precision()
 
 
 ###############################################################################
@@ -1193,7 +1224,7 @@ def _unpack_epochs(epochs):
 
 
 def _get_ch_whitener(A, pca, ch_type, rank):
-    """"Get whitener params for a set of channels."""
+    """Get whitener params for a set of channels."""
     # whitening operator
     eig, eigvec = linalg.eigh(A, overwrite_a=True)
     eigvec = eigvec.T
@@ -1210,6 +1241,33 @@ def _get_ch_whitener(A, pca, ch_type, rank):
     return eig, eigvec
 
 
+def _get_whitener(noise_cov, info, ch_names, rank, pca, scalings=None):
+    #
+    #   Handle noise cov
+    #
+    noise_cov = prepare_noise_cov(noise_cov, info, ch_names, rank)
+    n_chan = len(ch_names)
+
+    #   Omit the zeroes due to projection
+    eig = noise_cov['eig']
+    nzero = (eig > 0)
+    n_nzero = np.sum(nzero)
+
+    if pca:
+        #   Rows of eigvec are the eigenvectors
+        whitener = noise_cov['eigvec'][nzero] / np.sqrt(eig[nzero])[:, None]
+        logger.info('Reducing data rank to %d' % n_nzero)
+    else:
+        whitener = np.zeros((n_chan, n_chan), dtype=np.float)
+        whitener[nzero, nzero] = 1.0 / np.sqrt(eig[nzero])
+        #   Rows of eigvec are the eigenvectors
+        whitener = np.dot(whitener, noise_cov['eigvec'])
+
+    logger.info('Total rank is %d' % n_nzero)
+
+    return whitener, noise_cov, n_nzero
+
+
 @verbose
 def prepare_noise_cov(noise_cov, info, ch_names, rank=None,
                       scalings=None, verbose=None):
@@ -1235,7 +1293,8 @@ def prepare_noise_cov(noise_cov, info, ch_names, rank=None,
             dict(mag=1e12, grad=1e11, eeg=1e5)
 
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -1253,7 +1312,8 @@ def prepare_noise_cov(noise_cov, info, ch_names, rank=None,
     scalings = _handle_default('scalings_cov_rank', scalings)
 
     # Create the projection operator
-    proj, ncomp, _ = make_projector(info['projs'], ch_names)
+    proj, ncomp, _ = make_projector(info['projs'] + noise_cov['projs'],
+                                    ch_names)
     if ncomp > 0:
         logger.info('    Created an SSP operator (subspace dimension = %d)'
                     % ncomp)
@@ -1300,25 +1360,30 @@ def prepare_noise_cov(noise_cov, info, ch_names, rank=None,
 
     if has_meg:
         C_meg = C[np.ix_(out_meg_idx, out_meg_idx)]
-        this_info = pick_info(info, info_pick_meg)
+        this_info = pick_info(_simplify_info(info), info_pick_meg, copy=False)
         if rank_meg is None:
             rank_meg = _estimate_rank_meeg_cov(C_meg, this_info, scalings)
         eig[out_meg_idx], eigvec[np.ix_(out_meg_idx, out_meg_idx)] = \
             _get_ch_whitener(C_meg, False, 'MEG', rank_meg)
     if has_eeg:
         C_eeg = C[np.ix_(out_eeg_idx, out_eeg_idx)]
-        this_info = pick_info(info, info_pick_eeg)
+        this_info = pick_info(_simplify_info(info), info_pick_eeg, copy=False)
         if rank_eeg is None:
             rank_eeg = _estimate_rank_meeg_cov(C_eeg, this_info, scalings)
         eig[out_eeg_idx], eigvec[np.ix_(out_eeg_idx, out_eeg_idx)], = \
             _get_ch_whitener(C_eeg, False, 'EEG', rank_eeg)
-    if _needs_eeg_average_ref_proj(info):
-        warn('No average EEG reference present in info["projs"], covariance '
-             'may be adversely affected. Consider recomputing covariance using'
-             ' a raw file with an average eeg reference projector added.')
-    noise_cov = cp.deepcopy(noise_cov)
-    noise_cov.update(data=C, eig=eig, eigvec=eigvec, dim=len(ch_names),
-                     diag=False, names=ch_names)
+        if _needs_eeg_average_ref_proj(info) and not \
+                _has_eeg_average_ref_proj(noise_cov['projs']):
+            warn('No average EEG reference present in info["projs"], '
+                 'covariance may be adversely affected. Consider recomputing '
+                 'covariance using with an average eeg reference projector '
+                 'added.')
+    noise_cov = Covariance(
+        data=C, names=ch_names, bads=list(noise_cov['bads']),
+        projs=deepcopy(noise_cov['projs']),
+        nfree=noise_cov['nfree'], eig=eig, eigvec=eigvec,
+        method=noise_cov.get('method', None),
+        loglik=noise_cov.get('loglik', None))
     return noise_cov
 
 
@@ -1330,12 +1395,12 @@ def regularize(cov, info, mag=0.1, grad=0.1, eeg=0.1, exclude='bads',
     channel type separately. Special care is taken to keep the
     rank of the data constant.
 
-    **Note:** This function is kept for reasons of backward-compatibility.
-    Please consider explicitly using the ``method`` parameter in
-    `compute_covariance` to directly combine estimation with regularization
-    in a data-driven fashion see the
-    `faq <http://martinos.org/mne/dev/faq.html#how-should-i-regularize-the-covariance-matrix>`_
-    for more information.
+    .. note:: This function is kept for reasons of backward-compatibility.
+              Please consider explicitly using the ``method`` parameter in
+              :func:`mne.compute_covariance` to directly combine estimation
+              with regularization in a data-driven fashion.
+              See the `faq <http://martinos.org/mne/dev/faq.html#how-should-i-regularize-the-covariance-matrix>`_
+              for more information.
 
     Parameters
     ----------
@@ -1355,7 +1420,7 @@ def regularize(cov, info, mag=0.1, grad=0.1, eeg=0.1, exclude='bads',
     proj : bool (default true)
         Apply or not projections to keep rank of data.
     verbose : bool | str | int | None (default None)
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`).
 
     Returns
     -------
@@ -1364,9 +1429,9 @@ def regularize(cov, info, mag=0.1, grad=0.1, eeg=0.1, exclude='bads',
 
     See Also
     --------
-    compute_covariance
-    """  # noqa
-    cov = cp.deepcopy(cov)
+    mne.compute_covariance
+    """  # noqa: E501
+    cov = cov.copy()
     info._check_consistency()
 
     if exclude is None:
@@ -1386,6 +1451,7 @@ def regularize(cov, info, mag=0.1, grad=0.1, eeg=0.1, exclude='bads',
     ch_names_eeg = [info_ch_names[i] for i in sel_eeg]
     ch_names_mag = [info_ch_names[i] for i in sel_mag]
     ch_names_grad = [info_ch_names[i] for i in sel_grad]
+    del sel_eeg, sel_mag, sel_grad
 
     # This actually removes bad channels from the cov, which is not backward
     # compatible, so let's leave all channels in
@@ -1518,7 +1584,8 @@ def _regularized_covariance(data, reg=None):
 
 @verbose
 def compute_whitener(noise_cov, info, picks=None, rank=None,
-                     scalings=None, verbose=None):
+                     scalings=None, return_rank=False,
+                     verbose=None):
     """Compute whitening matrix.
 
     Parameters
@@ -1538,8 +1605,13 @@ def compute_whitener(noise_cov, info, picks=None, rank=None,
     scalings : dict | None
         The rescaling method to be applied. See documentation of
         ``prepare_noise_cov`` for details.
+    return_rank : bool
+        If True, return the rank used to compute the whitener.
+
+        .. versionadded:: 0.15
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -1547,35 +1619,37 @@ def compute_whitener(noise_cov, info, picks=None, rank=None,
         The whitening matrix.
     ch_names : list
         The channel names.
+    rank : int
+        Rank reduction of the whitener. Returned only if return_rank is True.
     """
     if picks is None:
         picks = pick_types(info, meg=True, eeg=True, ref_meg=False,
                            exclude='bads')
 
-    ch_names = [info['chs'][k]['ch_name'] for k in picks]
+    ch_names = [info['ch_names'][k] for k in picks]
 
-    noise_cov = cp.deepcopy(noise_cov)
-    noise_cov = prepare_noise_cov(noise_cov, info, ch_names,
-                                  rank=rank, scalings=scalings)
-    n_chan = len(ch_names)
+    # XXX this relies on pick_channels, which does not respect order,
+    # so this could create problems if users have reordered their data
+    noise_cov = pick_channels_cov(noise_cov, include=ch_names, exclude=[])
+    if len(noise_cov['data']) != len(ch_names):
+        missing = list(set(ch_names) - set(noise_cov['names']))
+        raise RuntimeError('Not all channels present in noise covariance:\n%s'
+                           % missing)
 
-    W = np.zeros((n_chan, n_chan), dtype=np.float)
-    #
-    #   Omit the zeroes due to projection
-    #
-    eig = noise_cov['eig']
-    nzero = (eig > 0)
-    W[nzero, nzero] = 1.0 / np.sqrt(eig[nzero])
-    #
-    #   Rows of eigvec are the eigenvectors
-    #
-    W = np.dot(W, noise_cov['eigvec'])
+    scalings = _handle_default('scalings_cov_rank', scalings)
+    W, noise_cov, n_nzero = _get_whitener(noise_cov, info, ch_names, rank,
+                                          pca=False, scalings=scalings)
+
+    # Do the back projection
     W = np.dot(noise_cov['eigvec'].T, W)
-    return W, ch_names
+    out = W, ch_names
+    if return_rank:
+        out += (n_nzero,)
+    return out
 
 
 @verbose
-def whiten_evoked(evoked, noise_cov, picks=None, diag=False, rank=None,
+def whiten_evoked(evoked, noise_cov, picks=None, diag=None, rank=None,
                   scalings=None, verbose=None):
     """Whiten evoked data using given noise covariance.
 
@@ -1604,42 +1678,28 @@ def whiten_evoked(evoked, noise_cov, picks=None, diag=False, rank=None,
             dict(mag=1e12, grad=1e11, eeg=1e5)
 
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
     evoked_white : instance of Evoked
         The whitened evoked data.
     """
-    evoked = cp.deepcopy(evoked)
+    evoked = evoked.copy()
     if picks is None:
         picks = pick_types(evoked.info, meg=True, eeg=True)
-    W = _get_whitener_data(evoked.info, noise_cov, picks,
-                           diag=diag, rank=rank, scalings=scalings)
+
+    if diag:
+        noise_cov = noise_cov.as_diag()
+
+    W, rank = compute_whitener(noise_cov, evoked.info, picks=picks,
+                               rank=rank, scalings=scalings)
+
     evoked.data[picks] = np.sqrt(evoked.nave) * np.dot(W, evoked.data[picks])
     return evoked
 
 
-@verbose
-def _get_whitener_data(info, noise_cov, picks, diag=False, rank=None,
-                       scalings=None, verbose=None):
-    """Get whitening matrix for a set of data."""
-    ch_names = [info['ch_names'][k] for k in picks]
-    noise_cov = pick_channels_cov(noise_cov, include=ch_names, exclude=[])
-    if len(noise_cov['data']) != len(ch_names):
-        missing = list(set(ch_names) - set(noise_cov['names']))
-        raise RuntimeError('Not all channels present in noise covariance:\n%s'
-                           % missing)
-    if diag:
-        noise_cov = cp.deepcopy(noise_cov)
-        noise_cov['data'] = np.diag(np.diag(noise_cov['data']))
-
-    scalings = _handle_default('scalings_cov_rank', scalings)
-    W = compute_whitener(noise_cov, info, picks=picks, rank=rank,
-                         scalings=scalings)[0]
-    return W
-
-
 @verbose
 def _read_cov(fid, node, cov_kind, limited=False, verbose=None):
     """Read a noise covariance matrix."""
@@ -1820,12 +1880,17 @@ def _apply_scaling_array(data, picks_list, scalings):
         data *= scalings[:, np.newaxis]  # F - order
 
 
-def _undo_scaling_array(data, picks_list, scalings):
-    scalings = _check_scaling_inputs(data, picks_list, scalings)
+def _invert_scalings(scalings):
     if isinstance(scalings, dict):
         scalings = dict((k, 1. / v) for k, v in scalings.items())
     elif isinstance(scalings, np.ndarray):
         scalings = 1. / scalings
+    return scalings
+
+
+def _undo_scaling_array(data, picks_list, scalings):
+    scalings = _invert_scalings(_check_scaling_inputs(data, picks_list,
+                                                      scalings))
     return _apply_scaling_array(data, picks_list, scalings)
 
 
@@ -1856,11 +1921,8 @@ def _apply_scaling_cov(data, picks_list, scalings):
 
 
 def _undo_scaling_cov(data, picks_list, scalings):
-    scalings = _check_scaling_inputs(data, picks_list, scalings)
-    if isinstance(scalings, dict):
-        scalings = dict((k, 1. / v) for k, v in scalings.items())
-    elif isinstance(scalings, np.ndarray):
-        scalings = 1. / scalings
+    scalings = _invert_scalings(_check_scaling_inputs(data, picks_list,
+                                                      scalings))
     return _apply_scaling_cov(data, picks_list, scalings)
 
 
@@ -1932,7 +1994,7 @@ def _estimate_rank_meeg_signals(data, info, scalings, tol='auto',
 
 def _estimate_rank_meeg_cov(data, info, scalings, tol='auto',
                             return_singular=False):
-    """Estimate rank of M/EEG covariance data, given the covariance
+    """Estimate rank of M/EEG covariance data, given the covariance.
 
     Parameters
     ----------
