@@ -14,7 +14,7 @@ _multiply_inplace_c128 = _halve_c128 = _real_c128 = None
 
 
 def _get_cudafft():
-    """Helper to deal with scikit-cuda namespace change"""
+    """Deal with scikit-cuda namespace change."""
     try:
         from skcuda import fft
     except ImportError:
@@ -26,7 +26,7 @@ def _get_cudafft():
 
 
 def get_cuda_memory():
-    """Get the amount of free memory for CUDA operations
+    """Get the amount of free memory for CUDA operations.
 
     Returns
     -------
@@ -43,7 +43,7 @@ def get_cuda_memory():
 
 
 def init_cuda(ignore_config=False):
-    """Initialize CUDA functionality
+    """Initialize CUDA functionality.
 
     This function attempts to load the necessary interfaces
     (hardware connectivity) to run CUDA-based filtering. This
@@ -64,14 +64,14 @@ def init_cuda(ignore_config=False):
     # Triage possible errors for informative messaging
     _cuda_capable = False
     try:
-        from pycuda import gpuarray, driver  # noqa
+        from pycuda import gpuarray, driver  # noqa: F401
         from pycuda.elementwise import ElementwiseKernel
     except ImportError:
         warn('module pycuda not found, CUDA not enabled')
         return
     try:
         # Initialize CUDA; happens with importing autoinit
-        import pycuda.autoinit  # noqa
+        import pycuda.autoinit  # noqa: F401
     except ImportError:
         warn('pycuda.autoinit could not be imported, likely a hardware error, '
              'CUDA not enabled%s' % _explain_exception())
@@ -107,7 +107,7 @@ def init_cuda(ignore_config=False):
 # Repeated FFT multiplication
 
 def setup_cuda_fft_multiply_repeated(n_jobs, h_fft):
-    """Set up repeated CUDA FFT multiplication with a given filter
+    """Set up repeated CUDA FFT multiplication with a given filter.
 
     Parameters
     ----------
@@ -181,7 +181,7 @@ def setup_cuda_fft_multiply_repeated(n_jobs, h_fft):
 
 
 def fft_multiply_repeated(h_fft, x, cuda_dict=dict(use_cuda=False)):
-    """Do FFT multiplication by a filter function (possibly using CUDA)
+    """Do FFT multiplication by a filter function (possibly using CUDA).
 
     Parameters
     ----------
@@ -219,7 +219,7 @@ def fft_multiply_repeated(h_fft, x, cuda_dict=dict(use_cuda=False)):
 # FFT Resampling
 
 def setup_cuda_fft_resample(n_jobs, W, new_len):
-    """Set up CUDA FFT resampling
+    """Set up CUDA FFT resampling.
 
     Parameters
     ----------
@@ -296,9 +296,9 @@ def setup_cuda_fft_resample(n_jobs, W, new_len):
     return n_jobs, cuda_dict, W
 
 
-def fft_resample(x, W, new_len, npads, to_removes,
-                 cuda_dict=dict(use_cuda=False)):
-    """Do FFT resampling with a filter function (possibly using CUDA)
+def fft_resample(x, W, new_len, npads, to_removes, cuda_dict=None,
+                 pad='reflect_limited'):
+    """Do FFT resampling with a filter function (possibly using CUDA).
 
     Parameters
     ----------
@@ -315,16 +315,24 @@ def fft_resample(x, W, new_len, npads, to_removes,
         Number of samples to remove after resampling.
     cuda_dict : dict
         Dictionary constructed using setup_cuda_multiply_repeated().
+    pad : str
+        The type of padding to use. Supports all :func:`np.pad` ``mode``
+        options. Can also be "reflect_limited" (default), which pads with a
+        reflected version of each vector mirrored on the first and last values
+        of the vector, followed by zeros.
+
+        .. versionadded:: 0.15
 
     Returns
     -------
     x : 1-d array
         Filtered version of x.
     """
+    cuda_dict = dict(use_cuda=False) if cuda_dict is None else cuda_dict
     # add some padding at beginning and end to make this work a little cleaner
     if x.dtype != np.float64:
         x = x.astype(np.float64)
-    x = _smart_pad(x, npads)
+    x = _smart_pad(x, npads, pad)
     old_len = len(x)
     shorter = new_len < old_len
     if not cuda_dict['use_cuda']:
@@ -381,15 +389,19 @@ def fft_resample(x, W, new_len, npads, to_removes,
 # Misc
 
 # this has to go in mne.cuda instead of mne.filter to avoid import errors
-def _smart_pad(x, n_pad):
-    """Pad vector x
-    """
+def _smart_pad(x, n_pad, pad='reflect_limited'):
+    """Pad vector x."""
+    n_pad = np.asarray(n_pad)
+    assert n_pad.shape == (2,)
     if (n_pad == 0).all():
         return x
     elif (n_pad < 0).any():
         raise RuntimeError('n_pad must be non-negative')
-    # need to pad with zeros if len(x) <= npad
-    l_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
-    r_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
-    return np.concatenate([l_z_pad, 2 * x[0] - x[n_pad[0]:0:-1], x,
-                           2 * x[-1] - x[-2:-n_pad[1] - 2:-1], r_z_pad])
+    if pad == 'reflect_limited':
+        # need to pad with zeros if len(x) <= npad
+        l_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
+        r_z_pad = np.zeros(max(n_pad[0] - len(x) + 1, 0), dtype=x.dtype)
+        return np.concatenate([l_z_pad, 2 * x[0] - x[n_pad[0]:0:-1], x,
+                               2 * x[-1] - x[-2:-n_pad[1] - 2:-1], r_z_pad])
+    else:
+        return np.pad(x, (tuple(n_pad),), pad)
