@@ -2,9 +2,9 @@
 #
 # License: BSD (3-clause)
 
-import os.path as op
+from functools import partial
+
 from ...utils import verbose
-from ...fixes import partial
 from ..utils import (has_dataset, _data_path, _get_version, _version_doc,
                      _data_path_doc)
 
@@ -20,15 +20,11 @@ URL: http://neuroimage.usc.edu/brainstorm/DatasetResting
 
 @verbose
 def data_path(path=None, force_update=False, update_path=True, download=True,
-              verbose=None):
-    archive_name = dict(brainstorm='bst_resting.tar.gz')
-    data_path = _data_path(path=path, force_update=force_update,
-                           update_path=update_path, name='brainstorm',
-                           download=download, archive_name=archive_name)
-    if data_path != '':
-        return op.join(data_path, 'bst_resting')
-    else:
-        return data_path
+              verbose=None):  # noqa: D103
+    return _data_path(path=path, force_update=force_update,
+                      update_path=update_path, name='brainstorm',
+                      download=download, archive_name='bst_resting.tar.gz')
+
 
 _data_path_doc = _data_path_doc.format(name='brainstorm',
                                        conf='MNE_DATASETS_BRAINSTORM_DATA'
@@ -38,14 +34,14 @@ _data_path_doc = _data_path_doc.replace('brainstorm dataset',
 data_path.__doc__ = _data_path_doc
 
 
-def get_version():
+def get_version():  # noqa: D103
     return _get_version('brainstorm')
 
+
 get_version.__doc__ = _version_doc.format(name='brainstorm')
 
 
 def description():
-    """Get description of brainstorm (bst_resting) dataset
-    """
+    """Get description of brainstorm (bst_resting) dataset."""
     for desc in _description.splitlines():
         print(desc)
