@@ -15,10 +15,10 @@ from .urls import (url_match, valid_data_types, valid_data_formats,
 @verbose
 def data_path(url, path=None, force_update=False, update_path=None,
               verbose=None):
-    """Get path to local copy of MEGSIM dataset URL
+    """Get path to local copy of MEGSIM dataset URL.
 
     This is a low-level function useful for getting a local copy of a
-    remote MEGSIM dataset.
+    remote MEGSIM dataset [1]_.
 
     Parameters
     ----------
@@ -27,10 +27,9 @@ def data_path(url, path=None, force_update=False, update_path=None,
     path : None | str
         Location of where to look for the MEGSIM data storing location.
         If None, the environment variable or config parameter
-        MNE_DATASETS_MEGSIM_PATH is used. If it doesn't exist, the
-        "mne-python/examples" directory is used. If the MEGSIM dataset
-        is not found under the given path (e.g., as
-        "mne-python/examples/MEGSIM"), the data
+        ``MNE_DATASETS_MEGSIM_PATH`` is used. If it doesn't exist, the
+        "~/mne_data" directory is used. If the MEGSIM dataset
+        is not found under the given path, the data
         will be automatically downloaded to the specified folder.
     force_update : bool
         Force update of the dataset even if a local copy exists.
@@ -38,7 +37,7 @@ def data_path(url, path=None, force_update=False, update_path=None,
         If True, set the MNE_DATASETS_MEGSIM_PATH in mne-python
         config to the given path. If None, the user is prompted.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`).
 
     Returns
     -------
@@ -59,12 +58,13 @@ def data_path(url, path=None, force_update=False, update_path=None,
     folder, and prompt the user to save the 'datasets' path to the mne-python
     config, if it isn't there already.
 
-    The MEGSIM dataset is documented in the following publication:
-        Aine CJ, Sanfratello L, Ranken D, Best E, MacArthur JA, Wallace T,
-        Gilliam K, Donahue CH, Montano R, Bryant JE, Scott A, Stephen JM
-        (2012) MEG-SIM: A Web Portal for Testing MEG Analysis Methods using
-        Realistic Simulated and Empirical Data. Neuroinform 10:141-158
-    """  # noqa
+    References
+    ----------
+    .. [1] Aine CJ, Sanfratello L, Ranken D, Best E, MacArthur JA, Wallace T,
+           Gilliam K, Donahue CH, Montano R, Bryant JE, Scott A, Stephen JM
+           (2012) MEG-SIM: A Web Portal for Testing MEG Analysis Methods using
+           Realistic Simulated and Empirical Data. Neuroinform 10:141-158
+    """  # noqa: E501
     key = 'MNE_DATASETS_MEGSIM_PATH'
     name = 'MEGSIM'
     path = _get_path(path, key, name)
@@ -103,7 +103,9 @@ def data_path(url, path=None, force_update=False, update_path=None,
 @verbose
 def load_data(condition='visual', data_format='raw', data_type='experimental',
               path=None, force_update=False, update_path=None, verbose=None):
-    """Get path to local copy of MEGSIM dataset type
+    """Get path to local copy of MEGSIM dataset type.
+
+    The MEGSIM dataset is described in [1]_.
 
     Parameters
     ----------
@@ -116,18 +118,17 @@ def load_data(condition='visual', data_format='raw', data_type='experimental',
     path : None | str
         Location of where to look for the MEGSIM data storing location.
         If None, the environment variable or config parameter
-        MNE_DATASETS_MEGSIM_PATH is used. If it doesn't exist, the
-        "mne-python/examples" directory is used. If the MEGSIM dataset
-        is not found under the given path (e.g., as
-        "mne-python/examples/MEGSIM"), the data
+        ``MNE_DATASETS_MEGSIM_PATH`` is used. If it doesn't exist, the
+        "~/mne_data" directory is used. If the MEGSIM dataset
+        is not found under the given path, the data
         will be automatically downloaded to the specified folder.
     force_update : bool
         Force update of the dataset even if a local copy exists.
     update_path : bool | None
-        If True, set the MNE_DATASETS_MEGSIM_PATH in mne-python
+        If True, set the ``MNE_DATASETS_MEGSIM_PATH`` in mne-python
         config to the given path. If None, the user is prompted.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`).
 
     Returns
     -------
@@ -145,13 +146,13 @@ def load_data(condition='visual', data_format='raw', data_type='experimental',
     'datasets' folder, and prompt the user to save the 'datasets' path to the
     mne-python config, if it isn't there already.
 
-    The MEGSIM dataset is documented in the following publication:
-        Aine CJ, Sanfratello L, Ranken D, Best E, MacArthur JA, Wallace T,
-        Gilliam K, Donahue CH, Montano R, Bryant JE, Scott A, Stephen JM
-        (2012) MEG-SIM: A Web Portal for Testing MEG Analysis Methods using
-        Realistic Simulated and Empirical Data. Neuroinform 10:141-158
-    """  # noqa
-
+    References
+    ----------
+    .. [1] Aine CJ, Sanfratello L, Ranken D, Best E, MacArthur JA, Wallace T,
+           Gilliam K, Donahue CH, Montano R, Bryant JE, Scott A, Stephen JM
+           (2012) MEG-SIM: A Web Portal for Testing MEG Analysis Methods using
+           Realistic Simulated and Empirical Data. Neuroinform 10:141-158
+    """  # noqa: E501
     if not condition.lower() in valid_conditions:
         raise ValueError('Unknown condition "%s"' % condition)
     if data_format not in valid_data_formats:
