@@ -16,7 +16,7 @@ has_spm_data = partial(has_dataset, name='spm')
 
 @verbose
 def data_path(path=None, force_update=False, update_path=True, download=True,
-              verbose=None):
+              verbose=None):  # noqa: D103
     return _data_path(path=path, force_update=force_update,
                       update_path=update_path, name='spm',
                       download=download)
@@ -25,7 +25,7 @@ data_path.__doc__ = _data_path_doc.format(name='spm',
                                           conf='MNE_DATASETS_SPM_DATA_PATH')
 
 
-def get_version():
+def get_version():  # noqa: D103
     return _get_version('spm')
 
 get_version.__doc__ = _version_doc.format(name='spm')
