@@ -1,4 +1,3 @@
-"""MNE testing dataset
-"""
+"""MNE testing dataset."""
 
 from ._testing import data_path, requires_testing_data, get_version
