@@ -1,5 +1,4 @@
-"""IO with fif files containing events
-"""
+"""IO with fif files containing events."""
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
@@ -19,10 +18,11 @@ from .io.tag import read_tag
 from .io.open import fiff_open
 from .io.write import write_int, start_block, start_file, end_block, end_file
 from .io.pick import pick_channels
+from .externals.six import string_types
 
 
 def pick_events(events, include=None, exclude=None, step=False):
-    """Select some events
+    """Select some events.
 
     Parameters
     ----------
@@ -75,7 +75,7 @@ def pick_events(events, include=None, exclude=None, step=False):
 
 def define_target_events(events, reference_id, target_id, sfreq, tmin, tmax,
                          new_id=None, fill_na=None):
-    """Define new events by co-occurrence of existing events
+    """Define new events by co-occurrence of existing events.
 
     This function can be used to evaluate events depending on the
     temporal lag to another event. For example, this can be used to
@@ -110,7 +110,6 @@ def define_target_events(events, reference_id, target_id, sfreq, tmin, tmax,
     lag : ndarray
         time lag between reference and target in milliseconds.
     """
-
     if new_id is None:
         new_id = reference_id
 
@@ -148,7 +147,7 @@ def define_target_events(events, reference_id, target_id, sfreq, tmin, tmax,
 
 
 def _read_events_fif(fid, tree):
-    """Aux function"""
+    """Aux function."""
     #   Find the desired block
     events = dir_tree_find(tree, FIFF.FIFFB_MNE_EVENTS)
 
@@ -191,7 +190,10 @@ def _read_events_fif(fid, tree):
 
 def read_events(filename, include=None, exclude=None, mask=None,
                 mask_type=None):
-    """Reads events from fif or text file
+    """Read events from fif or text file.
+
+    See :ref:`tut_epoching_and_averaging` as well as :ref:`ex_read_events`
+    for more information about events.
 
     Parameters
     ----------
@@ -214,7 +216,8 @@ def read_events(filename, include=None, exclude=None, mask=None,
         If None (default), no masking is performed.
     mask_type: 'and' | 'not_and'
         The type of operation between the mask and the trigger.
-        Choose 'and' for MNE-C masking behavior.
+        Choose 'and' for MNE-C masking behavior. The default ('not_and')
+        will change to 'and' in 0.16.
 
         .. versionadded:: 0.13
 
@@ -241,14 +244,14 @@ def read_events(filename, include=None, exclude=None, mask=None,
     ext = splitext(filename)[1].lower()
     if ext == '.fif' or ext == '.gz':
         fid, tree, _ = fiff_open(filename)
-        try:
-            event_list, _ = _read_events_fif(fid, tree)
-        finally:
-            fid.close()
+        with fid as f:
+            event_list, _ = _read_events_fif(f, tree)
+        # hack fix for windows to avoid bincount problems
+        event_list = event_list.astype(int)
     else:
         #  Have to read this in as float64 then convert because old style
         #  eve/lst files had a second float column that will raise errors
-        lines = np.loadtxt(filename, dtype=np.float64).astype(np.uint32)
+        lines = np.loadtxt(filename, dtype=np.float64).astype(int)
         if len(lines) == 0:
             raise ValueError('No text lines found')
 
@@ -280,7 +283,7 @@ def read_events(filename, include=None, exclude=None, mask=None,
 
 
 def write_events(filename, event_list):
-    """Write events to file
+    """Write events to file.
 
     Parameters
     ----------
@@ -366,7 +369,7 @@ def _find_stim_steps(data, first_samp, pad_start=None, pad_stop=None, merge=0):
 
 def find_stim_steps(raw, pad_start=None, pad_stop=None, merge=0,
                     stim_channel=None):
-    """Find all steps in data from a stim channel
+    """Find all steps in data from a stim channel.
 
     Parameters
     ----------
@@ -402,7 +405,6 @@ def find_stim_steps(raw, pad_start=None, pad_stop=None, merge=0,
     --------
     find_events : More sophisticated options for finding events in a Raw file.
     """
-
     # pull stim channel from config if necessary
     stim_channel = _get_stim_channel(stim_channel, raw.info)
 
@@ -420,9 +422,11 @@ def find_stim_steps(raw, pad_start=None, pad_stop=None, merge=0,
 
 
 def _find_events(data, first_samp, verbose=None, output='onset',
-                 consecutive='increasing', min_samples=0, mask=0,
+                 consecutive='increasing', min_samples=0, mask=None,
                  uint_cast=False, mask_type=None):
-    """Helper function for find events"""
+    """Help find events."""
+    assert data.shape[0] == 1  # data should be only a row vector
+
     if min_samples > 0:
         merge = int(min_samples // 1)
         if merge == min_samples:
@@ -482,7 +486,7 @@ def _find_events(data, first_samp, verbose=None, output='onset',
         events[:, 2] = event_id
         events[:, 0] -= 1
     else:
-        raise Exception("Invalid output parameter %r" % output)
+        raise ValueError("Invalid output parameter %r" % output)
 
     logger.info("%s events found" % len(events))
     logger.info("Events id: %s" % np.unique(events[:, 2]))
@@ -490,12 +494,27 @@ def _find_events(data, first_samp, verbose=None, output='onset',
     return events
 
 
+def _find_unique_events(events):
+    """Uniquify events (ie remove duplicated rows."""
+    e = np.ascontiguousarray(events).view(
+        np.dtype((np.void, events.dtype.itemsize * events.shape[1])))
+    _, idx = np.unique(e, return_index=True)
+    n_dupes = len(events) - len(idx)
+    if n_dupes > 0:
+        warn("Some events are duplicated in your different stim channels."
+             " %d events were ignored during deduplication." % n_dupes)
+    return events[idx]
+
+
 @verbose
 def find_events(raw, stim_channel=None, output='onset',
                 consecutive='increasing', min_duration=0,
                 shortest_event=2, mask=None, uint_cast=False,
                 mask_type=None, verbose=None):
-    """Find events from raw file
+    """Find events from raw file.
+
+    See :ref:`tut_epoching_and_averaging` as well as :ref:`ex_read_events`
+    for more information about events.
 
     Parameters
     ----------
@@ -503,11 +522,13 @@ def find_events(raw, stim_channel=None, output='onset',
         The raw data.
     stim_channel : None | string | list of string
         Name of the stim channel or all the stim channels
-        affected by the trigger. If None, the config variables
+        affected by triggers. If None, the config variables
         'MNE_STIM_CHANNEL', 'MNE_STIM_CHANNEL_1', 'MNE_STIM_CHANNEL_2',
         etc. are read. If these are not found, it will fall back to
         'STI 014' if present, then fall back to the first channel of type
-        'stim', if present.
+        'stim', if present. If multiple channels are provided
+        then the returned events are the union of all the events
+        extracted from individual stim channels.
     output : 'onset' | 'offset' | 'step'
         Whether to report when events start, when events end, or both.
     consecutive : bool | 'increasing'
@@ -537,12 +558,14 @@ def find_events(raw, stim_channel=None, output='onset',
 
     mask_type: 'and' | 'not_and'
         The type of operation between the mask and the trigger.
-        Choose 'and' for MNE-C masking behavior.
+        Choose 'and' for MNE-C masking behavior. The default ('not_and')
+        will change to 'and' in 0.16.
 
         .. versionadded:: 0.13
 
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -645,30 +668,49 @@ def find_events(raw, stim_channel=None, output='onset',
     # pull stim channel from config if necessary
     stim_channel = _get_stim_channel(stim_channel, raw.info)
 
-    pick = pick_channels(raw.info['ch_names'], include=stim_channel)
-    if len(pick) == 0:
+    picks = pick_channels(raw.info['ch_names'], include=stim_channel)
+    if len(picks) == 0:
         raise ValueError('No stim channel found to extract event triggers.')
-    data, _ = raw[pick, :]
+    data, _ = raw[picks, :]
 
-    events = _find_events(data, raw.first_samp, verbose=verbose, output=output,
-                          consecutive=consecutive, min_samples=min_samples,
-                          mask=mask, uint_cast=uint_cast, mask_type=mask_type)
+    events_list = []
+    for d in data:
+        events = _find_events(d[np.newaxis, :], raw.first_samp,
+                              verbose=verbose, output=output,
+                              consecutive=consecutive, min_samples=min_samples,
+                              mask=mask, uint_cast=uint_cast,
+                              mask_type=mask_type)
+        # add safety check for spurious events (for ex. from neuromag syst.) by
+        # checking the number of low sample events
+        n_short_events = np.sum(np.diff(events[:, 0]) < shortest_event)
+        if n_short_events > 0:
+            raise ValueError("You have %i events shorter than the "
+                             "shortest_event. These are very unusual and you "
+                             "may want to set min_duration to a larger value "
+                             "e.g. x / raw.info['sfreq']. Where x = 1 sample "
+                             "shorter than the shortest event "
+                             "length." % (n_short_events))
 
-    # add safety check for spurious events (for ex. from neuromag syst.) by
-    # checking the number of low sample events
-    n_short_events = np.sum(np.diff(events[:, 0]) < shortest_event)
-    if n_short_events > 0:
-        raise ValueError("You have %i events shorter than the "
-                         "shortest_event. These are very unusual and you "
-                         "may want to set min_duration to a larger value e.g."
-                         " x / raw.info['sfreq']. Where x = 1 sample shorter "
-                         "than the shortest event length." % (n_short_events))
+        events_list.append(events)
 
+    events = np.concatenate(events_list, axis=0)
+    events = _find_unique_events(events)
+    events = events[np.argsort(events[:, 0])]
     return events
 
 
 def _mask_trigs(events, mask, mask_type):
-    """Helper function for masking digital trigger values"""
+    """Mask digital trigger values."""
+    if mask_type is None:
+        mask_type = 'not_and'
+        if mask is not None:
+            warn('The default mask type "not_and" will change to "and" in '
+                 '0.16, set it explicitly to avoid this warning.',
+                 DeprecationWarning)
+    if not isinstance(mask_type, string_types) or \
+            mask_type not in ('not_and', 'and'):
+        raise ValueError('mask_type must be "not_and" or "and", got %s'
+                         % (mask_type,))
     if mask is not None:
         if not isinstance(mask, int):
             raise TypeError('You provided a(n) %s.' % type(mask) +
@@ -678,10 +720,6 @@ def _mask_trigs(events, mask, mask_type):
         return events.copy()
 
     if mask is not None:
-        if mask_type is None:
-            warn("The default setting for mask_type will change from "
-                 "'not and' to 'and' in v0.14.", DeprecationWarning)
-            mask_type = 'not_and'
         if mask_type == 'not_and':
             mask = np.bitwise_not(mask)
         elif mask_type != 'and':
@@ -695,7 +733,7 @@ def _mask_trigs(events, mask, mask_type):
 
 
 def merge_events(events, ids, new_id, replace_events=True):
-    """Merge a set of events
+    """Merge a set of events.
 
     Parameters
     ----------
@@ -756,7 +794,7 @@ def merge_events(events, ids, new_id, replace_events=True):
 
 
 def shift_time_events(events, ids, tshift, sfreq):
-    """Shift an event
+    """Shift an event.
 
     Parameters
     ----------
@@ -783,7 +821,7 @@ def shift_time_events(events, ids, tshift, sfreq):
 
 def make_fixed_length_events(raw, id, start=0, stop=None, duration=1.,
                              first_samp=True):
-    """Make a set of events separated by a fixed duration
+    """Make a set of events separated by a fixed duration.
 
     Parameters
     ----------
@@ -810,8 +848,8 @@ def make_fixed_length_events(raw, id, start=0, stop=None, duration=1.,
     new_events : array
         The new events.
     """
-    from .io.base import _BaseRaw
-    if not isinstance(raw, _BaseRaw):
+    from .io.base import BaseRaw
+    if not isinstance(raw, BaseRaw):
         raise ValueError('Input data must be an instance of Raw, got'
                          ' %s instead.' % (type(raw)))
     if not isinstance(id, int):
@@ -843,22 +881,19 @@ def make_fixed_length_events(raw, id, start=0, stop=None, duration=1.,
 
 
 def concatenate_events(events, first_samps, last_samps):
-    """Concatenate event lists in a manner compatible with
-    concatenate_raws
+    """Concatenate event lists to be compatible with concatenate_raws.
 
     This is useful, for example, if you processed and/or changed
     events in raw files separately before combining them using
-    concatenate_raws.
+    :func:`mne.concatenate_raws`.
 
     Parameters
     ----------
     events : list of arrays
         List of event arrays, typically each extracted from a
         corresponding raw file that is being concatenated.
-
     first_samps : list or array of int
         First sample numbers of the raw files concatenated.
-
     last_samps : list or array of int
         Last sample numbers of the raw files concatenated.
 
@@ -866,6 +901,10 @@ def concatenate_events(events, first_samps, last_samps):
     -------
     events : array
         The concatenated events.
+
+    See Also
+    --------
+    mne.concatenate_raws
     """
     if not isinstance(events, list):
         raise ValueError('events must be a list of arrays')
@@ -889,7 +928,7 @@ def concatenate_events(events, first_samps, last_samps):
 
 
 class AcqParserFIF(object):
-    """ Parser for Elekta data acquisition settings.
+    """Parser for Elekta data acquisition settings.
 
     This class parses parameters (e.g. events and averaging categories) that
     are defined in the Elekta TRIUX/VectorView data acquisition software (DACQ)
@@ -917,6 +956,10 @@ class AcqParserFIF(object):
     acq_dict : dict
         All DACQ parameters.
 
+    See Also
+    --------
+    mne.io.Raw.acqparser : access the parser through a Raw attribute
+
     Notes
     -----
     Any averaging category (also non-active ones) can be accessed by indexing
@@ -937,7 +980,7 @@ class AcqParserFIF(object):
     _event_vars_compat = ('Comment', 'Delay')
 
     _cat_vars = ('Comment', 'Display', 'Start', 'State', 'End', 'Event',
-                 'Nave', 'ReqEvent', 'ReqWhen', 'ReqWithin',  'SubAve')
+                 'Nave', 'ReqEvent', 'ReqWhen', 'ReqWithin', 'SubAve')
 
     # new versions only (DACQ >= 3.4)
     _dacq_vars = _dacq_vars_compat + ('magMax', 'magMin', 'magNoise',
@@ -946,11 +989,11 @@ class AcqParserFIF(object):
     _event_vars = _event_vars_compat + ('Name', 'Channel', 'NewBits',
                                         'OldBits', 'NewMask', 'OldMask')
 
-    def __init__(self, info):
+    def __init__(self, info):  # noqa: D102
         acq_pars = info['acq_pars']
         if not acq_pars:
             raise ValueError('No acquisition parameters')
-        self.acq_dict = self._acqpars_dict(acq_pars)
+        self.acq_dict = dict(self._acqpars_gen(acq_pars))
         if 'ERFversion' in self.acq_dict:
             self.compat = False  # DACQ ver >= 3.4
         elif 'ERFncateg' in self.acq_dict:  # probably DACQ < 3.4
@@ -990,7 +1033,7 @@ class AcqParserFIF(object):
         self.flat = {k: float(v) for k, v in self.flat.items()
                      if float(v) > 0}
 
-    def __repr__(self):
+    def __repr__(self):  # noqa: D105
         s = '<AcqParserFIF | '
         s += 'categories: %d ' % self.ncateg
         cats_in_use = len(self._categories_in_use)
@@ -1006,7 +1049,7 @@ class AcqParserFIF(object):
         return s
 
     def __getitem__(self, item):
-        """ Return an averaging category, or list of categories.
+        """Return an averaging category, or list of categories.
 
         Parameters
         ----------
@@ -1064,15 +1107,16 @@ class AcqParserFIF(object):
         return cats[0] if len(cats) == 1 else cats
 
     def __len__(self):
-        """ Return number of averaging categories marked active in DACQ. """
+        """Return number of averaging categories marked active in DACQ."""
         return len(self.categories)
 
     def _events_from_acq_pars(self):
-        """ Collect DACQ events into a dict.
+        """Collect DACQ events into a dict.
 
         Events are keyed by number starting from 1 (DACQ index of event).
         Each event is itself represented by a dict containing the event
-        parameters. """
+        parameters.
+        """
         # lookup table for event number -> bits for old DACQ versions
         _compat_event_lookup = {1: 1, 2: 2, 3: 4, 4: 8, 5: 16, 6: 32, 7: 3,
                                 8: 5, 9: 6, 10: 7, 11: 9, 12: 10, 13: 11,
@@ -1106,13 +1150,8 @@ class AcqParserFIF(object):
             events[evnum] = evdi
         return events
 
-    def _acqpars_dict(self, acq_pars):
-        """ Parse `` info['acq_pars']`` into a dict. """
-        return dict(self._acqpars_gen(acq_pars))
-
     def _acqpars_gen(self, acq_pars):
-        """ Yields key/value pairs from ``info['acq_pars'])``. """
-        # DACQ variable names always start with one of these
+        """Yield key/value pairs from ``info['acq_pars'])``."""
         key, val = '', ''
         for line in acq_pars.split():
             if any([line.startswith(x) for x in self._acq_var_magic]):
@@ -1126,10 +1165,11 @@ class AcqParserFIF(object):
             yield key, val
 
     def _categories_from_acq_pars(self):
-        """ Collect DACQ averaging categories into a dict.
+        """Collect DACQ averaging categories into a dict.
 
         Categories are keyed by the comment field in DACQ. Each category is
-        itself represented a dict containing the category parameters. """
+        itself represented a dict containing the category parameters.
+        """
         cats = dict()
         for catnum in [str(x).zfill(2) for x in range(1, self.nevent + 1)]:
             catdi = dict()
@@ -1152,14 +1192,15 @@ class AcqParserFIF(object):
         return cats
 
     def _events_mne_to_dacq(self, mne_events):
-        """ Creates list of DACQ events based on mne trigger transitions list.
+        """Create list of DACQ events based on mne trigger transitions list.
 
         mne_events is typically given by mne.find_events (use consecutive=True
         to get all transitions). Output consists of rows in the form
         [t, 0, event_codes] where t is time in samples and event_codes is all
         DACQ events compatible with the transition, bitwise ORed together:
         e.g. [t1, 0, 5] means that events 1 and 3 occurred at time t1,
-        as 2**(1 - 1) + 2**(3 - 1) = 5. """
+        as 2**(1 - 1) + 2**(3 - 1) = 5.
+        """
         events_ = mne_events.copy()
         events_[:, 1:3] = 0
         for n, ev in self._events.items():
@@ -1175,13 +1216,12 @@ class AcqParserFIF(object):
         return events_
 
     def _mne_events_to_category_t0(self, cat, mne_events, sfreq):
-        """ Translate mne_events to epoch zero times (t0).
+        """Translate mne_events to epoch zero times (t0).
 
         First mne events (trigger transitions) are converted into DACQ events.
         Then the zero times for the epochs are obtained by considering the
         reference and conditional (required) events and the delay to stimulus.
         """
-
         cat_ev = cat['event']
         cat_reqev = cat['reqevent']
         # first convert mne events to dacq event list
@@ -1216,7 +1256,7 @@ class AcqParserFIF(object):
 
     @property
     def categories(self):
-        """ Return list of averaging categories ordered by DACQ index.
+        """Return list of averaging categories ordered by DACQ index.
 
         Only returns categories marked active in DACQ.
         """
@@ -1226,7 +1266,7 @@ class AcqParserFIF(object):
 
     @property
     def events(self):
-        """ Return events ordered by DACQ index.
+        """Return events ordered by DACQ index.
 
         Only returns events that are in use (referred to by a category).
         """
@@ -1243,7 +1283,7 @@ class AcqParserFIF(object):
 
     def get_condition(self, raw, condition=None, stim_channel=None, mask=None,
                       uint_cast=None, mask_type='and', delayed_lookup=True):
-        """ Get averaging parameters for a condition (averaging category).
+        """Get averaging parameters for a condition (averaging category).
 
         Output is designed to be used with the Epochs class to extract the
         corresponding epochs.
@@ -1287,7 +1327,6 @@ class AcqParserFIF(object):
             transitions with a resolution of one sample, use
             delayed_lookup=False.
 
-
         Returns
         -------
         conds_data : dict or list of dict, each with following keys:
