@@ -14,9 +14,10 @@ from .baseline import rescale
 from .channels.channels import (ContainsMixin, UpdateChannelsMixin,
                                 SetChannelsMixin, InterpolationMixin,
                                 equalize_channels)
-from .filter import resample, detrend, FilterMixin
+from .channels.layout import _merge_grad_data, _pair_grad_sensors
+from .filter import detrend, FilterMixin
 from .utils import (check_fname, logger, verbose, _time_mask, warn, sizeof_fmt,
-                    deprecated, SizeMixin, copy_function_doc_to_method_doc)
+                    SizeMixin, copy_function_doc_to_method_doc)
 from .viz import (plot_evoked, plot_evoked_topomap, plot_evoked_field,
                   plot_evoked_image, plot_evoked_topo)
 from .viz.evoked import (_plot_evoked_white, plot_evoked_joint,
@@ -45,7 +46,7 @@ _aspect_rev = {str(FIFF.FIFFV_ASPECT_AVERAGE): 'average',
 class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
              SetChannelsMixin, InterpolationMixin, FilterMixin,
              ToDataFrameMixin, TimeMixin, SizeMixin):
-    """Evoked data
+    """Evoked data.
 
     Parameters
     ----------
@@ -55,16 +56,6 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
     condition : int, or str
         Dataset ID number (int) or comment/name (str). Optional if there is
         only one data set in file.
-    baseline : tuple or list of length 2, or None
-        This parameter has been deprecated and will be removed in 0.14
-        Use inst.apply_baseline(baseline) instead.
-        The time interval to apply rescaling / baseline correction.
-        If None do not apply it. If baseline is (a, b)
-        the interval is between "a (s)" and "b (s)".
-        If a is None the beginning of the data is used
-        and if b is None then b is set to the end of the interval.
-        If baseline is equal to (None, None) all the time
-        interval is used. If None, no correction is applied.
     proj : bool, optional
         Apply SSP projection vectors
     kind : str
@@ -77,7 +68,8 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         SSS/tSSS to remove the compensation signals that may also affect brain
         activity. Can also be "yes" to load without eliciting a warning.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Attributes
     ----------
@@ -105,11 +97,12 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
     Notes
     -----
     Evoked objects contain a single condition only.
-
     """
+
     @verbose
-    def __init__(self, fname, condition=None, baseline=None, proj=True,
-                 kind='average', allow_maxshield=False, verbose=None):
+    def __init__(self, fname, condition=None, proj=True,
+                 kind='average', allow_maxshield=False,
+                 verbose=None):  # noqa: D102
         if not isinstance(proj, bool):
             raise ValueError(r"'proj' must be 'True' or 'False'")
         # Read the requested data
@@ -118,14 +111,24 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
                 fname, condition, kind, allow_maxshield)
         self.kind = _aspect_rev.get(str(self._aspect_kind), 'Unknown')
         self.verbose = verbose
+        self.preload = True
         # project and baseline correct
         if proj:
             self.apply_proj()
-        self.apply_baseline(baseline, self.verbose)
+
+    @property
+    def data(self):
+        """The data matrix."""
+        return self._data
+
+    @data.setter
+    def data(self, data):
+        """Set the data matrix."""
+        self._data = data
 
     @verbose
     def apply_baseline(self, baseline=(None, 0), verbose=None):
-        """Baseline correct evoked data
+        """Baseline correct evoked data.
 
         Parameters
         ----------
@@ -139,7 +142,9 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
             from the data. The baseline (a, b) includes both endpoints, i.e.
             all timepoints t such that a <= t <= b.
         verbose : bool, str, int, or None
-            If not None, override default verbose level (see mne.verbose).
+            If not None, override default verbose level (see
+            :func:`mne.verbose` and :ref:`Logging documentation <tut_logging>`
+            for more).
 
         Returns
         -------
@@ -170,7 +175,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         """
         write_evokeds(fname, self)
 
-    def __repr__(self):
+    def __repr__(self):  # noqa: D105
         s = "comment : '%s'" % self.comment
         s += ', kind : %s' % self.kind
         s += ", time : [%f, %f]" % (self.times[0], self.times[-1])
@@ -181,11 +186,11 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
 
     @property
     def ch_names(self):
-        """Channel names"""
+        """Channel names."""
         return self.info['ch_names']
 
     def crop(self, tmin=None, tmax=None):
-        """Crop data to a given time interval
+        """Crop data to a given time interval.
 
         Parameters
         ----------
@@ -212,7 +217,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         return self
 
     def decimate(self, decim, offset=0):
-        """Decimate the evoked data
+        """Decimate the evoked data.
 
         .. note:: No filtering is performed. To avoid aliasing, ensure
                   your data are properly lowpassed.
@@ -235,7 +240,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         --------
         Epochs.decimate
         Epochs.resample
-        Raw.resample
+        mne.io.Raw.resample
 
         Notes
         -----
@@ -255,7 +260,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         return self
 
     def shift_time(self, tshift, relative=True):
-        """Shift time scale in evoked data
+        """Shift time scale in evoked data.
 
         Parameters
         ----------
@@ -286,13 +291,14 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
     def plot(self, picks=None, exclude='bads', unit=True, show=True, ylim=None,
              xlim='tight', proj=False, hline=None, units=None, scalings=None,
              titles=None, axes=None, gfp=False, window_title=None,
-             spatial_colors=False, zorder='unsorted', selectable=True):
+             spatial_colors=False, zorder='unsorted', selectable=True,
+             verbose=None):
         return plot_evoked(
             self, picks=picks, exclude=exclude, unit=unit, show=show,
             ylim=ylim, proj=proj, xlim=xlim, hline=hline, units=units,
             scalings=scalings, titles=titles, axes=axes, gfp=gfp,
             window_title=window_title, spatial_colors=spatial_colors,
-            zorder=zorder, selectable=selectable)
+            zorder=zorder, selectable=selectable, verbose=verbose)
 
     @copy_function_doc_to_method_doc(plot_evoked_image)
     def plot_image(self, picks=None, exclude='bads', unit=True, show=True,
@@ -306,9 +312,10 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
     @copy_function_doc_to_method_doc(plot_evoked_topo)
     def plot_topo(self, layout=None, layout_scale=0.945, color=None,
                   border='none', ylim=None, scalings=None, title=None,
-                  proj=False, vline=[0.0], fig_facecolor='k',
-                  fig_background=None, axis_facecolor='k', font_color='w',
-                  merge_grads=False, show=True):
+                  proj=False, vline=[0.0], fig_facecolor=None,
+                  fig_background=None, axis_facecolor=None, font_color=None,
+                  merge_grads=False, legend=True, axes=None,
+                  background_color=None, show=True):
         """
 
         Notes
@@ -322,29 +329,28 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
                                 fig_background=fig_background,
                                 axis_facecolor=axis_facecolor,
                                 font_color=font_color, merge_grads=merge_grads,
-                                show=show)
+                                legend=legend, axes=axes,
+                                background_color=background_color, show=show)
 
     @copy_function_doc_to_method_doc(plot_evoked_topomap)
     def plot_topomap(self, times="auto", ch_type=None, layout=None, vmin=None,
                      vmax=None, cmap=None, sensors=True, colorbar=True,
-                     scale=None, scale_time=1e3, unit=None, res=64, size=1,
-                     cbar_fmt="%3.1f", time_format='%01d ms', proj=False,
-                     show=True, show_names=False, title=None, mask=None,
-                     mask_params=None, outlines='head', contours=6,
-                     image_interp='bilinear', average=None, head_pos=None,
-                     axes=None):
-        return plot_evoked_topomap(self, times=times, ch_type=ch_type,
-                                   layout=layout, vmin=vmin, vmax=vmax,
-                                   cmap=cmap, sensors=sensors,
-                                   colorbar=colorbar, scale=scale,
-                                   scale_time=scale_time, unit=unit, res=res,
-                                   proj=proj, size=size, cbar_fmt=cbar_fmt,
-                                   time_format=time_format, show=show,
-                                   show_names=show_names, title=title,
-                                   mask=mask, mask_params=mask_params,
-                                   outlines=outlines, contours=contours,
-                                   image_interp=image_interp, average=average,
-                                   head_pos=head_pos, axes=axes)
+                     scalings=None, scaling_time=1e3, units=None, res=64,
+                     size=1, cbar_fmt="%3.1f", time_format='%01d ms',
+                     proj=False, show=True, show_names=False, title=None,
+                     mask=None, mask_params=None, outlines='head',
+                     contours=6, image_interp='bilinear', average=None,
+                     head_pos=None, axes=None, scale=None, scale_time=None,
+                     unit=None):
+        return plot_evoked_topomap(
+            self, times=times, ch_type=ch_type, layout=layout, vmin=vmin,
+            vmax=vmax, cmap=cmap, sensors=sensors, colorbar=colorbar,
+            scalings=scalings, scaling_time=scaling_time, units=units, res=res,
+            size=size, cbar_fmt=cbar_fmt, time_format=time_format,
+            proj=proj, show=show, show_names=show_names, title=title,
+            mask=mask, mask_params=mask_params, outlines=outlines, unit=unit,
+            contours=contours, image_interp=image_interp, average=average,
+            head_pos=head_pos, axes=axes, scale=scale, scale_time=scale_time)
 
     @copy_function_doc_to_method_doc(plot_evoked_field)
     def plot_field(self, surf_maps, time=None, time_label='t = %0.0f ms',
@@ -352,8 +358,8 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         return plot_evoked_field(self, surf_maps, time=time,
                                  time_label=time_label, n_jobs=n_jobs)
 
-    def plot_white(self, noise_cov, show=True):
-        """Plot whitened evoked response
+    def plot_white(self, noise_cov, show=True, rank=None):
+        """Plot whitened evoked response.
 
         Plots the whitened evoked response and the whitened GFP as described in
         [1]_. If one single covariance object is passed, the GFP panel (bottom)
@@ -373,6 +379,14 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
             The noise covariance as computed by ``mne.cov.compute_covariance``.
         show : bool
             Whether to show the figure or not. Defaults to True.
+        rank : dict of int | None
+            Dict of ints where keys are 'eeg', 'meg', mag' or 'grad'. If None,
+            the rank is detected automatically. Defaults to None. 'mag' or
+            'grad' cannot be specified jointly with 'meg'. For SSS'd data,
+            only 'meg' is valid. For non-SSS'd data, 'mag' and/or 'grad' must
+            be specified separately. If only one is specified, the other one
+            gets estimated. Note. The rank estimation will be printed by the
+            logger for each noise covariance estimator that is passed.
 
         Returns
         -------
@@ -390,7 +404,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         .. versionadded:: 0.9.0
         """
         return _plot_evoked_white(self, noise_cov=noise_cov, scalings=None,
-                                  rank=None, show=show)
+                                  rank=rank, show=show)
 
     @copy_function_doc_to_method_doc(plot_evoked_joint)
     def plot_joint(self, times="peaks", title='', picks=None,
@@ -402,9 +416,11 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
 
     def animate_topomap(self, ch_type='mag', times=None, frame_rate=None,
                         butterfly=False, blit=True, show=True):
-        """Make animation of evoked data as topomap timeseries. Animation can
-        be paused/resumed with left mouse button. Left and right arrow keys can
-        be used to move backward or forward in time
+        """Make animation of evoked data as topomap timeseries.
+
+        The animation can be paused/resumed with left mouse button.
+        Left and right arrow keys can be used to move backward or forward
+        in time.
 
         Parameters
         ----------
@@ -446,11 +462,10 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
                                        show=show)
 
     def as_type(self, ch_type='grad', mode='fast'):
-        """Compute virtual evoked using interpolated fields in mag/grad
-        channels.
+        """Compute virtual evoked using interpolated fields.
 
         .. Warning:: Using virtual evoked to compute inverse can yield
-            unexpected results. The virtual channels have `'_virtual'` appended
+            unexpected results. The virtual channels have `'_v'` appended
             at the end of the names to emphasize that the data contained in
             them are interpolated.
 
@@ -475,40 +490,8 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         from .forward import _as_meg_type_evoked
         return _as_meg_type_evoked(self, ch_type=ch_type, mode=mode)
 
-    def resample(self, sfreq, npad='auto', window='boxcar'):
-        """Resample data
-
-        This function operates in-place.
-
-        Parameters
-        ----------
-        sfreq : float
-            New sample rate to use
-        npad : int | str
-            Amount to pad the start and end of the data.
-            Can also be "auto" to use a padding that will result in
-            a power-of-two size (can be much faster).
-        window : string or tuple
-            Window to use in resampling. See scipy.signal.resample.
-
-        Returns
-        -------
-        evoked : instance of mne.Evoked
-            The resampled evoked object.
-        """
-        sfreq = float(sfreq)
-        o_sfreq = self.info['sfreq']
-        self.data = resample(self.data, sfreq, o_sfreq, npad, -1, window)
-        # adjust indirectly affected variables
-        self.info['sfreq'] = sfreq
-        self.times = (np.arange(self.data.shape[1], dtype=np.float) / sfreq +
-                      self.times[0])
-        self.first = int(self.times[0] * self.info['sfreq'])
-        self.last = len(self.times) + self.first - 1
-        return self
-
     def detrend(self, order=1, picks=None):
-        """Detrend data
+        """Detrend data.
 
         This function operates in-place.
 
@@ -531,7 +514,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         return self
 
     def copy(self):
-        """Copy the instance of evoked
+        """Copy the instance of evoked.
 
         Returns
         -------
@@ -541,7 +524,7 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         return evoked
 
     def __neg__(self):
-        """Negate channel responses
+        """Negate channel responses.
 
         Returns
         -------
@@ -554,47 +537,9 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
         out.comment = '-' + (out.comment or 'unknown')
         return out
 
-    @deprecated('ev1 + ev2 weighted summation has been deprecated and will be '
-                'removed in 0.14, use combine_evoked([ev1, ev2],'
-                'weights="nave") instead')
-    def __add__(self, evoked):
-        """Add evoked taking into account number of epochs
-
-        The addition will be performed by weighting each Evoked
-        instance by the number of averages.
-
-        See Also
-        --------
-        mne.combine_evoked
-        """
-        out = combine_evoked([self, evoked], weights='nave')
-        out.comment = self.comment + " + " + evoked.comment
-        return out
-
-    @deprecated('ev1 - ev2 weighted subtraction has been deprecated and will '
-                'be removed in 0.14, use combine_evoked([ev1, -ev2], '
-                'weights="nave") instead')
-    def __sub__(self, evoked):
-        """Subtract evoked taking into account number of epochs
-
-        The subtraction will be performed by weighting each Evoked
-        instance by the number of averages.
-
-        See Also
-        --------
-        mne.combine_evoked
-        """
-        out = combine_evoked([self, -evoked], weights='nave')
-        if self.comment is None or evoked.comment is None:
-            warn('evoked.comment expects a string but is None')
-            out.comment = 'unknown'
-        else:
-            out.comment = self.comment + " - " + evoked.comment
-        return out
-
-    def get_peak(self, ch_type=None, tmin=None, tmax=None, mode='abs',
-                 time_as_index=False):
-        """Get location and latency of peak amplitude
+    def get_peak(self, ch_type=None, tmin=None, tmax=None,
+                 mode='abs', time_as_index=False, merge_grads=False):
+        """Get location and latency of peak amplitude.
 
         Parameters
         ----------
@@ -615,6 +560,8 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
             Defaults to 'abs'.
         time_as_index : bool
             Whether to return the time index instead of the latency in seconds.
+        merge_grads : bool
+            If True, compute peak from merged gradiometer data.
 
         Returns
         -------
@@ -644,6 +591,13 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
                                'must not be `None`, pass a sensor type '
                                'value instead')
 
+        if merge_grads:
+            if ch_type != 'grad':
+                raise ValueError('Channel type must be grad for merge_grads')
+            elif mode == 'neg':
+                raise ValueError('Negative mode (mode=neg) does not make '
+                                 'sense with merge_grads=True')
+
         meg = eeg = misc = seeg = ecog = fnirs = False
         picks = None
         if ch_type in ('mag', 'grad'):
@@ -660,24 +614,31 @@ class Evoked(ProjMixin, ContainsMixin, UpdateChannelsMixin,
             fnirs = ch_type
 
         if ch_type is not None:
-            picks = pick_types(self.info, meg=meg, eeg=eeg, misc=misc,
-                               seeg=seeg, ecog=ecog, ref_meg=False,
-                               fnirs=fnirs)
-
+            if merge_grads:
+                picks = _pair_grad_sensors(self.info, topomap_coords=False)
+            else:
+                picks = pick_types(self.info, meg=meg, eeg=eeg, misc=misc,
+                                   seeg=seeg, ecog=ecog, ref_meg=False,
+                                   fnirs=fnirs)
         data = self.data
         ch_names = self.ch_names
+
         if picks is not None:
             data = data[picks]
             ch_names = [ch_names[k] for k in picks]
+
+        if merge_grads:
+            data = _merge_grad_data(data)
+            ch_names = [ch_name[:-1] + 'X' for ch_name in ch_names[::2]]
+
         ch_idx, time_idx = _get_peak(data, self.times, tmin,
                                      tmax, mode)
-
         return (ch_names[ch_idx],
                 time_idx if time_as_index else self.times[time_idx])
 
 
 def _check_decim(info, decim, offset):
-    """Helper to check decimation parameters"""
+    """Check decimation parameters."""
     if decim < 1 or decim != int(decim):
         raise ValueError('decim must be an integer > 0')
     decim = int(decim)
@@ -701,17 +662,17 @@ def _check_decim(info, decim, offset):
 
 
 class EvokedArray(Evoked):
-    """Evoked object from numpy array
+    """Evoked object from numpy array.
 
     Parameters
     ----------
     data : array of shape (n_channels, n_times)
-        The channels' evoked response.
+        The channels' evoked response. See notes for proper units of measure.
     info : instance of Info
         Info dictionary. Consider using ``create_info`` to populate
         this structure.
     tmin : float
-        Start time before event.
+        Start time before event. Defaults to 0.
     comment : string
         Comment on dataset. Can be the condition. Defaults to ''.
     nave : int
@@ -719,8 +680,18 @@ class EvokedArray(Evoked):
     kind : str
         Type of data, either average or standard_error. Defaults to 'average'.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
-        Defaults to raw.verbose.
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
+
+    Notes
+    -----
+    Proper units of measure:
+    * V: eeg, eog, seeg, emg, ecg, bio, ecog
+    * T: mag
+    * T/m: grad
+    * M: hbo, hbr
+    * Am: dipole
+    * AU: misc
 
     See Also
     --------
@@ -728,9 +699,8 @@ class EvokedArray(Evoked):
     """
 
     @verbose
-    def __init__(self, data, info, tmin, comment='', nave=1, kind='average',
-                 verbose=None):
-
+    def __init__(self, data, info, tmin=0., comment='', nave=1, kind='average',
+                 verbose=None):  # noqa: D102
         dtype = np.complex128 if np.any(np.iscomplex(data)) else np.float64
         data = np.asanyarray(data, dtype=dtype)
 
@@ -750,12 +720,13 @@ class EvokedArray(Evoked):
         self.last = self.first + np.shape(data)[-1] - 1
         self.times = np.arange(self.first, self.last + 1,
                                dtype=np.float) / info['sfreq']
-        self.info = info
+        self.info = info.copy()  # do not modify original info
         self.nave = nave
         self.kind = kind
         self.comment = comment
         self.picks = None
         self.verbose = verbose
+        self.preload = True
         self._projector = None
         if not isinstance(self.kind, string_types):
             raise TypeError('kind must be a string, not "%s"' % (type(kind),))
@@ -766,7 +737,7 @@ class EvokedArray(Evoked):
 
 
 def _get_entries(fid, evoked_node, allow_maxshield=False):
-    """Helper to get all evoked entries"""
+    """Get all evoked entries."""
     comments = list()
     aspect_kinds = list()
     for ev in evoked_node:
@@ -809,7 +780,7 @@ def _get_aspect(evoked, allow_maxshield):
 
 
 def _get_evoked_node(fname):
-    """Helper to get info in evoked file"""
+    """Get info in evoked file."""
     f, tree, _ = fiff_open(fname)
     with f as fid:
         _, meas = read_meas_info(fid, tree, verbose=False)
@@ -818,7 +789,7 @@ def _get_evoked_node(fname):
 
 
 def grand_average(all_evoked, interpolate_bads=True):
-    """Make grand average of a list evoked data
+    """Make grand average of a list evoked data.
 
     The function interpolates bad channels based on `interpolate_bads`
     parameter. If `interpolate_bads` is True, the grand average
@@ -868,8 +839,8 @@ def grand_average(all_evoked, interpolate_bads=True):
     return grand_average
 
 
-def combine_evoked(all_evoked, weights=None):
-    """Merge evoked data by weighted addition or subtraction
+def combine_evoked(all_evoked, weights):
+    """Merge evoked data by weighted addition or subtraction.
 
     Data should have the same channels and the same time instants.
     Subtraction can be performed by passing negative weights (e.g., [1, -1]).
@@ -892,11 +863,6 @@ def combine_evoked(all_evoked, weights=None):
     -----
     .. versionadded:: 0.9.0
     """
-    if weights is None:
-        weights = 'nave'
-        warn('In 0.13 the default is weights="nave", but in 0.14 the default '
-             'will be removed and it will have to be explicitly set',
-             DeprecationWarning)
     evoked = all_evoked[0].copy()
     if isinstance(weights, string_types):
         if weights not in ('nave', 'equal'):
@@ -950,7 +916,7 @@ def combine_evoked(all_evoked, weights=None):
 @verbose
 def read_evokeds(fname, condition=None, baseline=None, kind='average',
                  proj=True, allow_maxshield=False, verbose=None):
-    """Read evoked dataset(s)
+    """Read evoked dataset(s).
 
     Parameters
     ----------
@@ -980,7 +946,8 @@ def read_evokeds(fname, condition=None, baseline=None, kind='average',
         SSS/tSSS to remove the compensation signals that may also affect brain
         activity. Can also be "yes" to load without eliciting a warning.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -1011,7 +978,7 @@ def read_evokeds(fname, condition=None, baseline=None, kind='average',
 
 
 def _read_evoked(fname, condition=None, kind='average', allow_maxshield=False):
-    """Read evoked data from a FIF file"""
+    """Read evoked data from a FIF file."""
     if fname is None:
         raise ValueError('No evoked filename specified')
 
@@ -1019,7 +986,6 @@ def _read_evoked(fname, condition=None, kind='average', allow_maxshield=False):
     with f as fid:
         #   Read the measurement info
         info, meas = read_meas_info(fid, tree, clean_bads=True)
-        info['filename'] = fname
 
         #   Locate the data of interest
         processed = dir_tree_find(meas, FIFF.FIFFB_PROCESSED_DATA)
@@ -1185,7 +1151,7 @@ def _read_evoked(fname, condition=None, kind='average', allow_maxshield=False):
 
 
 def write_evokeds(fname, evoked):
-    """Write an evoked dataset to a file
+    """Write an evoked dataset to a file.
 
     Parameters
     ----------
@@ -1204,7 +1170,7 @@ def write_evokeds(fname, evoked):
 
 
 def _write_evokeds(fname, evoked, check=True):
-    """Helper to write evoked data"""
+    """Write evoked data."""
     if check:
         check_fname(fname, 'evoked', ('-ave.fif', '-ave.fif.gz'))
 
@@ -1260,7 +1226,7 @@ def _write_evokeds(fname, evoked, check=True):
 
 
 def _get_peak(data, times, tmin=None, tmax=None, mode='abs'):
-    """Get feature-index and time of maximum signal from 2D array
+    """Get feature-index and time of maximum signal from 2D array.
 
     Note. This is a 'getter', not a 'finder'. For non-evoked type
     data and continuous signals, please use proper peak detection algorithms.
