@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 
+from copy import deepcopy
+
 import numpy as np
 from scipy import linalg
-from copy import deepcopy
 
 from ..bem import _check_origin
 from ..io.constants import FIFF
@@ -10,17 +11,18 @@ from ..io.pick import pick_types, pick_info
 from ..surface import get_head_surf, get_meg_helmet_surf
 
 from ..io.proj import _has_eeg_average_ref_proj, make_projector
-from ..transforms import transform_surface_to, read_trans, _find_trans
+from ..transforms import (transform_surface_to, read_trans, _find_trans,
+                          _ensure_trans)
 from ._make_forward import _create_meg_coils, _create_eeg_els, _read_coil_defs
 from ._lead_dots import (_do_self_dots, _do_surface_dots, _get_legen_table,
-                         _get_legen_lut_fast, _get_legen_lut_accurate,
                          _do_cross_dots)
 from ..parallel import check_n_jobs
 from ..utils import logger, verbose
-from ..fixes import partial
+from ..externals.six import string_types
 
 
 def _is_axial_coil(coil):
+    """Determine if the coil is axial."""
     is_ax = coil['coil_class'] in (FIFF.FWD_COILC_MAG,
                                    FIFF.FWD_COILC_AXIAL_GRAD,
                                    FIFF.FWD_COILC_AXIAL_GRAD2)
@@ -28,6 +30,8 @@ def _is_axial_coil(coil):
 
 
 def _ad_hoc_noise(coils, ch_type='meg'):
+    """Create ad-hoc noise covariance."""
+    # XXX should de-duplicate with make_ad_hoc_cov
     v = np.empty(len(coils))
     if ch_type == 'meg':
         axs = np.array([_is_axial_coil(coil) for coil in coils], dtype=bool)
@@ -40,24 +44,18 @@ def _ad_hoc_noise(coils, ch_type='meg'):
 
 
 def _setup_dots(mode, coils, ch_type):
-    """Setup dot products"""
+    """Set up dot products."""
+    from scipy.interpolate import interp1d
     int_rad = 0.06
     noise = _ad_hoc_noise(coils, ch_type)
-    if mode == 'fast':
-        # Use 50 coefficients with nearest-neighbor interpolation
-        n_coeff = 50
-        lut_fun = _get_legen_lut_fast
-    else:  # 'accurate'
-        # Use 100 coefficients with linear interpolation
-        n_coeff = 100
-        lut_fun = _get_legen_lut_accurate
+    n_coeff, interp = (50, 'nearest') if mode == 'fast' else (100, 'linear')
     lut, n_fact = _get_legen_table(ch_type, False, n_coeff, verbose=False)
-    lut_fun = partial(lut_fun, lut=lut)
+    lut_fun = interp1d(np.linspace(-1, 1, lut.shape[0]), lut, interp, axis=0)
     return int_rad, noise, lut_fun, n_fact
 
 
 def _compute_mapping_matrix(fmd, info):
-    """Do the hairy computations"""
+    """Do the hairy computations."""
     logger.info('    Preparing the mapping matrix...')
     # assemble a projector and apply it to the data
     ch_names = fmd['ch_names']
@@ -80,8 +78,10 @@ def _compute_mapping_matrix(fmd, info):
     # Eigenvalue truncation
     sumk = np.cumsum(sing)
     sumk /= sumk[-1]
-    fmd['nest'] = np.where(sumk > (1.0 - fmd['miss']))[0][0]
-    logger.info('    [Truncate at %d missing %g]' % (fmd['nest'], fmd['miss']))
+    fmd['nest'] = np.where(sumk > (1.0 - fmd['miss']))[0][0] + 1
+    logger.info('    Truncating at %d/%d components to omit less than %g '
+                '(%0.2g)' % (fmd['nest'], len(sing), fmd['miss'],
+                             1. - sumk[fmd['nest'] - 1]))
     sing = 1.0 / sing[:fmd['nest']]
 
     # Put the inverse together
@@ -210,7 +210,7 @@ def _as_meg_type_evoked(evoked, ch_type='grad', mode='fast'):
 
     # change channel names to emphasize they contain interpolated data
     for ch in evoked.info['chs']:
-        ch['ch_name'] += '_virtual'
+        ch['ch_name'] += '_v'
     evoked.info._update_redundant()
     evoked.info._check_consistency()
     return evoked
@@ -219,7 +219,7 @@ def _as_meg_type_evoked(evoked, ch_type='grad', mode='fast'):
 @verbose
 def _make_surface_mapping(info, surf, ch_type='meg', trans=None, mode='fast',
                           n_jobs=1, origin=(0., 0., 0.04), verbose=None):
-    """Re-map M/EEG data to a surface
+    """Re-map M/EEG data to a surface.
 
     Parameters
     ----------
@@ -244,7 +244,8 @@ def _make_surface_mapping(info, surf, ch_type='meg', trans=None, mode='fast',
         coords and in meters. The default is ``'auto'``, which means
         a head-digitization-based origin fit.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -332,7 +333,7 @@ def _make_surface_mapping(info, surf, ch_type='meg', trans=None, mode='fast',
 def make_field_map(evoked, trans='auto', subject=None, subjects_dir=None,
                    ch_type=None, mode='fast', meg_surf='helmet',
                    origin=(0., 0., 0.04), n_jobs=1, verbose=None):
-    """Compute surface maps used for field display in 3D
+    """Compute surface maps used for field display in 3D.
 
     Parameters
     ----------
@@ -361,19 +362,19 @@ def make_field_map(evoked, trans='auto', subject=None, subjects_dir=None,
         to compute the MEG field map. The default value is ``'helmet'``
     origin : array-like, shape (3,) | str
         Origin of internal and external multipolar moment space in head
-        coords and in meters. The default is ``'auto'``, which means
-        a head-digitization-based origin fit.
+        coords and in meters. Can be ``'auto'``, which means
+        a head-digitization-based origin fit. Default is ``(0., 0., 0.04)``.
 
         .. versionadded:: 0.11
 
     n_jobs : int
         The number of jobs to run in parallel.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
         .. versionadded:: 0.11
 
-
     Returns
     -------
     surf_maps : list
@@ -402,7 +403,9 @@ def make_field_map(evoked, trans='auto', subject=None, subjects_dir=None,
         raise RuntimeError('No data available for mapping.')
 
     if trans is not None:
-        trans = read_trans(trans)
+        if isinstance(trans, string_types):
+            trans = read_trans(trans)
+        trans = _ensure_trans(trans, 'head', 'mri')
 
     if meg_surf not in ['helmet', 'head']:
         raise ValueError('Surface to plot MEG fields must be '
