@@ -6,6 +6,7 @@ import os.path as op
 import warnings
 
 from nose.tools import assert_raises, assert_true
+import pytest
 import numpy as np
 from numpy.testing import (assert_equal, assert_allclose)
 
@@ -18,14 +19,14 @@ from mne import (read_forward_solution, make_forward_solution,
                  pick_types_forward, pick_info, pick_types, Transform,
                  read_evokeds, read_cov, read_dipole)
 from mne.utils import (requires_mne, requires_nibabel, _TempDir,
-                       run_tests_if_main, slow_test, run_subprocess)
+                       run_tests_if_main, run_subprocess)
 from mne.forward._make_forward import _create_meg_coils, make_forward_dipole
 from mne.forward._compute_forward import _magnetic_dipole_field_vec
 from mne.forward import Forward, _do_forward_solution
 from mne.dipole import Dipole, fit_dipole
 from mne.simulation import simulate_evoked
 from mne.source_estimate import VolSourceEstimate
-from mne.source_space import (get_volume_labels_from_aseg,
+from mne.source_space import (get_volume_labels_from_aseg, write_source_spaces,
                               _compare_source_spaces, setup_source_space)
 
 data_path = testing.data_path(download=False)
@@ -50,16 +51,16 @@ fname_bem_meg = op.join(subjects_dir, 'sample', 'bem',
 def _compare_forwards(fwd, fwd_py, n_sensors, n_src,
                       meg_rtol=1e-4, meg_atol=1e-9,
                       eeg_rtol=1e-3, eeg_atol=1e-3):
-    """Helper to test forwards"""
+    """Test forwards."""
     # check source spaces
     assert_equal(len(fwd['src']), len(fwd_py['src']))
     _compare_source_spaces(fwd['src'], fwd_py['src'], mode='approx')
     for surf_ori, force_fixed in product([False, True], [False, True]):
         # use copy here to leave our originals unmodified
-        fwd = convert_forward_solution(fwd, surf_ori, force_fixed,
-                                       copy=True)
+        fwd = convert_forward_solution(fwd, surf_ori, force_fixed, copy=True,
+                                       use_cps=True)
         fwd_py = convert_forward_solution(fwd_py, surf_ori, force_fixed,
-                                          copy=True)
+                                          copy=True, use_cps=True)
         check_src = n_src // 3 if force_fixed else n_src
 
         for key in ('nchan', 'source_rr', 'source_ori',
@@ -94,17 +95,16 @@ def _compare_forwards(fwd, fwd_py, n_sensors, n_src,
 
 
 def test_magnetic_dipole():
-    """Test basic magnetic dipole forward calculation
-    """
-    trans = Transform('mri', 'head', np.eye(4))
+    """Test basic magnetic dipole forward calculation."""
+    trans = Transform('mri', 'head')
     info = read_info(fname_raw)
     picks = pick_types(info, meg=True, eeg=False, exclude=[])
     info = pick_info(info, picks[:12])
     coils = _create_meg_coils(info['chs'], 'normal', trans)
-    # magnetic dipole at device origin
+    # magnetic dipole far (meters!) from device origin
     r0 = np.array([0., 13., -6.])
     for ch, coil in zip(info['chs'], coils):
-        rr = (ch['loc'][:3] + r0) / 2.
+        rr = (ch['loc'][:3] + r0) / 2.  # get halfway closer
         far_fwd = _magnetic_dipole_field_vec(r0[np.newaxis, :], [coil])
         near_fwd = _magnetic_dipole_field_vec(rr[np.newaxis, :], [coil])
         ratio = 8. if ch['ch_name'][-1] == '1' else 16.  # grad vs mag
@@ -114,8 +114,7 @@ def test_magnetic_dipole():
 @testing.requires_testing_data
 @requires_mne
 def test_make_forward_solution_kit():
-    """Test making fwd using KIT, BTI, and CTF (compensated) files
-    """
+    """Test making fwd using KIT, BTI, and CTF (compensated) files."""
     kit_dir = op.join(op.dirname(__file__), '..', '..', 'io', 'kit',
                       'tests', 'data')
     sqd_path = op.join(kit_dir, 'test.sqd')
@@ -138,8 +137,9 @@ def test_make_forward_solution_kit():
     # first set up a small testing source space
     temp_dir = _TempDir()
     fname_src_small = op.join(temp_dir, 'sample-oct-2-src.fif')
-    src = setup_source_space('sample', fname_src_small, 'oct2',
-                             subjects_dir=subjects_dir, add_dist=False)
+    src = setup_source_space('sample', 'oct2', subjects_dir=subjects_dir,
+                             add_dist=False)
+    write_source_spaces(fname_src_small, src)  # to enable working with MNE-C
     n_src = 108  # this is the resulting # of verts in fwd
 
     # first use mne-C: convert file, make forward solution
@@ -190,7 +190,7 @@ def test_make_forward_solution_kit():
     _compare_forwards(fwd, fwd_py, 274, n_src)
 
     # CTF with compensation changed in python
-    ctf_raw = read_raw_fif(fname_ctf_raw, add_eeg_ref=False)
+    ctf_raw = read_raw_fif(fname_ctf_raw)
     ctf_raw.apply_gradient_compensation(2)
 
     fwd_py = make_forward_solution(ctf_raw.info, fname_trans, src,
@@ -203,13 +203,13 @@ def test_make_forward_solution_kit():
     _compare_forwards(fwd, fwd_py, 274, n_src)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_make_forward_solution():
-    """Test making M-EEG forward solution from python
-    """
+    """Test making M-EEG forward solution from python."""
     fwd_py = make_forward_solution(fname_raw, fname_trans, fname_src,
-                                   fname_bem, mindist=5.0, eeg=True, meg=True)
+                                   fname_bem, mindist=5.0, eeg=True, meg=True,
+                                   n_jobs=-1)
     assert_true(isinstance(fwd_py, Forward))
     fwd = read_forward_solution(fname_meeg)
     assert_true(isinstance(fwd, Forward))
@@ -219,11 +219,12 @@ def test_make_forward_solution():
 @testing.requires_testing_data
 @requires_mne
 def test_make_forward_solution_sphere():
-    """Test making a forward solution with a sphere model"""
+    """Test making a forward solution with a sphere model."""
     temp_dir = _TempDir()
     fname_src_small = op.join(temp_dir, 'sample-oct-2-src.fif')
-    src = setup_source_space('sample', fname_src_small, 'oct2',
-                             subjects_dir=subjects_dir, add_dist=False)
+    src = setup_source_space('sample', 'oct2', subjects_dir=subjects_dir,
+                             add_dist=False)
+    write_source_spaces(fname_src_small, src)  # to enable working with MNE-C
     out_name = op.join(temp_dir, 'tmp-fwd.fif')
     run_subprocess(['mne_forward_solution', '--meg', '--eeg',
                     '--meas', fname_raw, '--src', fname_src_small,
@@ -244,12 +245,11 @@ def test_make_forward_solution_sphere():
                         1.0, rtol=1e-3)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_nibabel(False)
 def test_forward_mixed_source_space():
-    """Test making the forward solution for a mixed source space
-    """
+    """Test making the forward solution for a mixed source space."""
     temp_dir = _TempDir()
     # get the surface source space
     surf = read_source_spaces(fname_src)
@@ -258,12 +258,10 @@ def test_forward_mixed_source_space():
     label_names = get_volume_labels_from_aseg(fname_aseg)
     vol_labels = [label_names[int(np.random.rand() * len(label_names))]
                   for _ in range(2)]
-    vol1 = setup_volume_source_space('sample', fname=None, pos=20.,
-                                     mri=fname_aseg,
+    vol1 = setup_volume_source_space('sample', pos=20., mri=fname_aseg,
                                      volume_label=vol_labels[0],
                                      add_interpolator=False)
-    vol2 = setup_volume_source_space('sample', fname=None, pos=20.,
-                                     mri=fname_aseg,
+    vol2 = setup_volume_source_space('sample', pos=20., mri=fname_aseg,
                                      volume_label=vol_labels[1],
                                      add_interpolator=False)
 
@@ -296,10 +294,10 @@ def test_forward_mixed_source_space():
                   mri_resolution=True, trans=vox_mri_t)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_make_forward_dipole():
-    """Test forward-projecting dipoles"""
+    """Test forward-projecting dipoles."""
     rng = np.random.RandomState(0)
 
     evoked = read_evokeds(fname_evo)[0]
@@ -337,10 +335,10 @@ def test_make_forward_dipole():
     # Now simulate evoked responses for each of the test dipoles,
     # and fit dipoles to them (sphere model, MEG and EEG)
     times, pos, amplitude, ori, gof = [], [], [], [], []
-    snr = 20.  # add a tiny amount of noise to the simulated evokeds
+    nave = 100  # add a tiny amount of noise to the simulated evokeds
     for s in stc:
         evo_test = simulate_evoked(fwd, s, info, cov,
-                                   snr=snr, random_state=rng)
+                                   nave=nave, random_state=rng)
         # evo_test.add_proj(make_eeg_average_ref_proj(evo_test.info))
         dfit, resid = fit_dipole(evo_test, cov, sphere, None)
         times += dfit.times.tolist()
