@@ -10,8 +10,9 @@ from nose.tools import assert_equal, assert_false, assert_raises, assert_true
 
 from mne.datasets import testing
 from mne.io.tests import data_dir as fiff_data_dir
-from mne.utils import (_TempDir, requires_mne, requires_freesurfer,
-                       requires_traits)
+from mne.utils import (_TempDir, requires_freesurfer, requires_mayavi,
+                       run_tests_if_main)
+from mne.channels import read_dig_montage
 
 data_path = testing.data_path(download=False)
 subjects_dir = os.path.join(data_path, 'subjects')
@@ -22,9 +23,9 @@ fid_path = os.path.join(fiff_data_dir, 'fsaverage-fiducials.fif')
 
 
 @testing.requires_testing_data
-@requires_traits
+@requires_mayavi
 def test_bem_source():
-    """Test SurfaceSource"""
+    """Test SurfaceSource."""
     from mne.gui._file_traits import SurfaceSource
 
     bem = SurfaceSource()
@@ -37,9 +38,9 @@ def test_bem_source():
 
 
 @testing.requires_testing_data
-@requires_traits
+@requires_mayavi
 def test_fiducials_source():
-    """Test FiducialsSource"""
+    """Test FiducialsSource."""
     from mne.gui._file_traits import FiducialsSource
 
     fid = FiducialsSource()
@@ -55,12 +56,13 @@ def test_fiducials_source():
 
 
 @testing.requires_testing_data
-@requires_traits
+@requires_mayavi
 def test_inst_source():
-    """Test InstSource"""
-    from mne.gui._file_traits import InstSource
+    """Test DigSource."""
+    from mne.gui._file_traits import DigSource
+    tempdir = _TempDir()
 
-    inst = InstSource()
+    inst = DigSource()
     assert_equal(inst.inst_fname, '-')
 
     inst.file = inst_path
@@ -73,11 +75,19 @@ def test_inst_source():
     assert_allclose(inst.nasion, nasion)
     assert_allclose(inst.rpa, rpa)
 
+    montage = read_dig_montage(fif=inst_path)  # test reading DigMontage
+    montage_path = os.path.join(tempdir, 'temp_montage.fif')
+    montage.save(montage_path)
+    inst.file = montage_path
+    assert_allclose(inst.lpa, lpa)
+    assert_allclose(inst.nasion, nasion)
+    assert_allclose(inst.rpa, rpa)
+
 
 @testing.requires_testing_data
-@requires_traits
+@requires_mayavi
 def test_subject_source():
-    """Test SubjectSelector"""
+    """Test SubjectSelector."""
     from mne.gui._file_traits import MRISubjectSource
 
     mri = MRISubjectSource()
@@ -87,11 +97,10 @@ def test_subject_source():
 
 
 @testing.requires_testing_data
-@requires_traits
-@requires_mne
+@requires_mayavi
 @requires_freesurfer
 def test_subject_source_with_fsaverage():
-    """Test SubjectSelector"""
+    """Test SubjectSelector."""
     from mne.gui._file_traits import MRISubjectSource
     tempdir = _TempDir()
 
@@ -102,3 +111,6 @@ def test_subject_source_with_fsaverage():
     mri.subjects_dir = tempdir
     assert_true(mri.can_create_fsaverage)
     mri.create_fsaverage()
+
+
+run_tests_if_main()
