@@ -1,24 +1,24 @@
 # Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 # License: Simplified BSD
-from copy import deepcopy
 
 import numpy as np
 from scipy import linalg
 
-from ..forward import is_fixed_orient, _to_fixed_ori
+from ..forward import is_fixed_orient, convert_forward_solution
 
 from ..minimum_norm.inverse import _check_reference
 from ..utils import logger, verbose, warn
 from ..externals.six.moves import xrange as range
 from .mxne_inverse import (_make_sparse_stc, _prepare_gain,
-                           _reapply_source_weighting, _compute_residual)
+                           _reapply_source_weighting, _compute_residual,
+                           _make_dipoles_sparse, _check_loose_forward)
 
 
 @verbose
 def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
                    group_size=1, gammas=None, verbose=None):
-    """Hierarchical Bayes (Gamma-MAP)
+    """Hierarchical Bayes (Gamma-MAP).
 
     Parameters
     ----------
@@ -40,7 +40,8 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
         Initial values for posterior variances (gammas). If None, a
         variance of 1.0 is used.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -48,12 +49,6 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
         Estimated source time courses.
     active_set : array, shape=(n_active,)
         Indices of active sources.
-
-    References
-    ----------
-    [1] Wipf et al. Analysis of Empirical Bayesian Methods for
-    Neuroelectromagnetic Source Localization, Advances in Neural Information
-    Processing Systems (2007).
     """
     G = G.copy()
     M = M.copy()
@@ -89,6 +84,7 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
         def denom_fun(x):
             return x
 
+    last_size = -1
     for itno in range(maxit):
         gammas[np.isnan(gammas)] = 0.0
 
@@ -125,7 +121,8 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
             if denom is None:
                 gammas = numer
             else:
-                gammas = numer / denom_fun(denom)
+                gammas = numer / np.maximum(denom_fun(denom),
+                                            np.finfo('float').eps)
         else:
             numer_comb = np.sum(numer.reshape(-1, group_size), axis=1)
             if denom is None:
@@ -145,13 +142,13 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
 
         gammas_full_old = gammas_full
 
-        logger.info('Iteration: %d\t active set size: %d\t convergence: %0.3e'
-                    % (itno, len(gammas), err))
+        breaking = (err < tol or n_active == 0)
+        if len(gammas) != last_size or breaking:
+            logger.info('Iteration: %d\t active set size: %d\t convergence: '
+                        '%0.3e' % (itno, len(gammas), err))
+            last_size = len(gammas)
 
-        if err < tol:
-            break
-
-        if n_active == 0:
+        if breaking:
             break
 
     if itno < maxit - 1:
@@ -167,15 +164,16 @@ def _gamma_map_opt(M, G, alpha, maxit=10000, tol=1e-6, update_mode=1,
 
 
 @verbose
-def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
+def gamma_map(evoked, forward, noise_cov, alpha, loose="auto", depth=0.8,
               xyz_same_gamma=True, maxit=10000, tol=1e-6, update_mode=1,
               gammas=None, pca=True, return_residual=False,
-              verbose=None):
-    """Hierarchical Bayes (Gamma-MAP) sparse source localization method
+              return_as_dipoles=False, verbose=None):
+    """Hierarchical Bayes (Gamma-MAP) sparse source localization method.
 
     Models each source time course using a zero-mean Gaussian prior with an
     unknown variance (gamma) parameter. During estimation, most gammas are
-    driven to zero, resulting in a sparse source estimate.
+    driven to zero, resulting in a sparse source estimate, as in
+    [1]_ and [2]_.
 
     For fixed-orientation forward operators, a separate gamma is used for each
     source time course, while for free-orientation forward operators, the same
@@ -192,11 +190,13 @@ def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
         Noise covariance to compute whitener.
     alpha : float
         Regularization parameter (noise variance).
-    loose : float in [0, 1]
+    loose : float in [0, 1] | 'auto'
         Value that weights the source variances of the dipole components
         that are parallel (tangential) to the cortical surface. If loose
-        is 0 or None then the solution is computed with fixed orientation.
+        is 0 then the solution is computed with fixed orientation.
         If loose is 1, it corresponds to free orientations.
+        The default value ('auto') is set to 0.2 for surface-oriented source
+        space and set to 1.0 for volumic or discrete source space.
     depth: None | float in [0, 1]
         Depth weighting coefficients. If None, no depth weighting is performed.
     xyz_same_gamma : bool
@@ -215,8 +215,11 @@ def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
         If True the rank of the data is reduced to the true dimension.
     return_residual : bool
         If True, the residual is returned as an Evoked instance.
+    return_as_dipoles : bool
+        If True, the sources are returned as a list of Dipole instances.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -228,18 +231,23 @@ def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
 
     References
     ----------
-    Wipf et al. Analysis of Empirical Bayesian Methods for Neuroelectromagnetic
-    Source Localization, Advances in Neural Information Process. Systems (2007)
+    .. [1] Wipf et al. Analysis of Empirical Bayesian Methods for
+           Neuroelectromagnetic Source Localization, Advances in Neural
+           Information Process. Systems (2007)
 
-    Wipf et al. A unified Bayesian framework for MEG/EEG source imaging,
-    NeuroImage, vol. 44, no. 3, pp. 947-66, Mar. 2009.
+    .. [2] D. Wipf, S. Nagarajan
+           "A unified Bayesian framework for MEG/EEG source imaging",
+           Neuroimage, Volume 44, Number 3, pp. 947-966, Feb. 2009.
+           DOI: 10.1016/j.neuroimage.2008.02.059
     """
     _check_reference(evoked)
 
+    loose, forward = _check_loose_forward(loose, forward)
+
     # make forward solution in fixed orientation if necessary
-    if loose is None and not is_fixed_orient(forward):
-        forward = deepcopy(forward)
-        _to_fixed_ori(forward)
+    if loose == 0. and not is_fixed_orient(forward):
+        forward = convert_forward_solution(
+            forward, surf_ori=True, force_fixed=True, copy=True, use_cps=True)
 
     if is_fixed_orient(forward) or not xyz_same_gamma:
         group_size = 1
@@ -265,6 +273,9 @@ def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
     if len(active_set) == 0:
         raise Exception("No active dipoles found. alpha is too big.")
 
+    # Compute estimated whitened sensor data
+    M_estimated = np.dot(gain[:, active_set], X)
+
     # Reapply weights to have correct unit
     n_dip_per_pos = 1 if is_fixed_orient(forward) else 3
     X = _reapply_source_weighting(X, source_weighting,
@@ -291,10 +302,17 @@ def gamma_map(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
 
     tmin = evoked.times[0]
     tstep = 1.0 / evoked.info['sfreq']
-    stc = _make_sparse_stc(X, active_set, forward, tmin, tstep,
-                           active_is_idx=True, verbose=verbose)
+
+    if return_as_dipoles:
+        out = _make_dipoles_sparse(X, active_set, forward, tmin, tstep, M,
+                                   M_estimated, active_is_idx=True)
+    else:
+        out = _make_sparse_stc(X, active_set, forward, tmin, tstep,
+                               active_is_idx=True, verbose=verbose)
+
+    logger.info('[done]')
 
     if return_residual:
-        return stc, residual
-    else:
-        return stc
+        out = out, residual
+
+    return out
