@@ -3,17 +3,19 @@
 #
 # License: Simplified BSD
 
-from copy import deepcopy
 import numpy as np
 from scipy import linalg, signal
 
-from ..source_estimate import SourceEstimate
-from ..minimum_norm.inverse import combine_xyz, _prepare_forward
-from ..minimum_norm.inverse import _check_reference
-from ..forward import compute_orient_prior, is_fixed_orient, _to_fixed_ori
+from ..source_estimate import (SourceEstimate, VolSourceEstimate,
+                               _BaseSourceEstimate)
+from ..minimum_norm.inverse import (combine_xyz, _prepare_forward,
+                                    _check_reference, _check_loose_forward)
+from ..forward import (compute_orient_prior, is_fixed_orient,
+                       convert_forward_solution)
 from ..io.pick import pick_channels_evoked
 from ..io.proj import deactivate_proj
 from ..utils import logger, verbose
+from ..dipole import Dipole
 from ..externals.six.moves import xrange as range
 
 from .mxne_optim import (mixed_norm_solver, iterative_mixed_norm_solver,
@@ -23,8 +25,7 @@ from .mxne_optim import (mixed_norm_solver, iterative_mixed_norm_solver,
 @verbose
 def _prepare_weights(forward, gain, source_weighting, weights, weights_min):
     mask = None
-    if isinstance(weights, SourceEstimate):
-        # weights = np.sqrt(np.sum(weights.data ** 2, axis=1))
+    if isinstance(weights, _BaseSourceEstimate):
         weights = np.max(np.abs(weights.data), axis=1)
     weights_max = np.max(weights)
     if weights_min > weights_max:
@@ -60,14 +61,24 @@ def _prepare_gain_column(forward, info, noise_cov, pca, depth, loose, weights,
 
     logger.info('Whitening lead field matrix.')
     gain = np.dot(whitener, gain)
+    is_fixed_ori = is_fixed_orient(forward)
 
     if depth is not None:
-        depth_prior = np.sum(gain ** 2, axis=0) ** depth
-        source_weighting = np.sqrt(depth_prior ** -1.)
+        depth_prior = np.sum(gain ** 2, axis=0)
+        if not is_fixed_ori:
+            depth_prior = depth_prior.reshape(-1, 3).sum(axis=1)
+        # Spherical leadfield can be zero at the center
+        depth_prior[depth_prior == 0.] = np.min(
+            depth_prior[depth_prior != 0.])
+        depth_prior **= depth
+        if not is_fixed_ori:
+            depth_prior = np.repeat(depth_prior, 3)
+        source_weighting = np.sqrt(1. / depth_prior)
     else:
         source_weighting = np.ones(gain.shape[1], dtype=gain.dtype)
 
-    if loose is not None and loose != 1.0:
+    assert (is_fixed_ori or (0 <= loose <= 1))
+    if loose is not None and loose < 1.:
         source_weighting *= np.sqrt(compute_orient_prior(forward, loose))
 
     gain *= source_weighting[None, :]
@@ -151,36 +162,132 @@ def _make_sparse_stc(X, active_set, forward, tmin, tstep,
 
     src = forward['src']
 
-    n_lh_points = len(src[0]['vertno'])
-    lh_vertno = src[0]['vertno'][active_idx[active_idx < n_lh_points]]
-    rh_vertno = src[1]['vertno'][active_idx[active_idx >= n_lh_points] -
-                                 n_lh_points]
-    vertices = [lh_vertno, rh_vertno]
-    stc = SourceEstimate(X, vertices=vertices, tmin=tmin, tstep=tstep)
+    if src.kind != 'surface':
+        vertices = src[0]['vertno'][active_idx]
+        stc = VolSourceEstimate(X, vertices=vertices, tmin=tmin, tstep=tstep)
+    else:
+        vertices = []
+        n_points_so_far = 0
+        for this_src in src:
+            this_n_points_so_far = n_points_so_far + len(this_src['vertno'])
+            this_active_idx = active_idx[(n_points_so_far <= active_idx) &
+                                         (active_idx < this_n_points_so_far)]
+            this_active_idx -= n_points_so_far
+            this_vertno = this_src['vertno'][this_active_idx]
+            n_points_so_far = this_n_points_so_far
+            vertices.append(this_vertno)
+
+        stc = SourceEstimate(X, vertices=vertices, tmin=tmin, tstep=tstep)
+
     return stc
 
 
 @verbose
-def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
+def _make_dipoles_sparse(X, active_set, forward, tmin, tstep, M, M_est,
+                         active_is_idx=False, verbose=None):
+    times = tmin + tstep * np.arange(X.shape[1])
+
+    if not active_is_idx:
+        active_idx = np.where(active_set)[0]
+    else:
+        active_idx = active_set
+
+    n_dip_per_pos = 1 if is_fixed_orient(forward) else 3
+    if n_dip_per_pos > 1:
+        active_idx = np.unique(active_idx // n_dip_per_pos)
+
+    gof = np.zeros(M_est.shape[1])
+    M_norm2 = np.sum(M ** 2, axis=0)
+    R_norm2 = np.sum((M - M_est) ** 2, axis=0)
+    gof[M_norm2 > 0.0] = 1. - R_norm2[M_norm2 > 0.0] / M_norm2[M_norm2 > 0.0]
+    gof *= 100.
+
+    dipoles = []
+    for k, i_dip in enumerate(active_idx):
+        i_pos = forward['source_rr'][i_dip][np.newaxis, :]
+        i_pos = i_pos.repeat(len(times), axis=0)
+        X_ = X[k * n_dip_per_pos: (k + 1) * n_dip_per_pos]
+        if n_dip_per_pos == 1:
+            amplitude = X_[0]
+            i_ori = forward['source_nn'][i_dip][np.newaxis, :]
+            i_ori = i_ori.repeat(len(times), axis=0)
+        else:
+            if forward['surf_ori']:
+                X_ = np.dot(forward['source_nn'][i_dip *
+                            n_dip_per_pos:(i_dip + 1) * n_dip_per_pos].T, X_)
+
+            amplitude = np.sqrt(np.sum(X_ ** 2, axis=0))
+            i_ori = np.zeros((len(times), 3))
+            i_ori[amplitude > 0.] = (X_[:, amplitude > 0.] /
+                                     amplitude[amplitude > 0.]).T
+
+        dipoles.append(Dipole(times, i_pos, amplitude, i_ori, gof))
+
+    return dipoles
+
+
+@verbose
+def make_stc_from_dipoles(dipoles, src, verbose=None):
+    """Convert a list of spatio-temporal dipoles into a SourceEstimate.
+
+    Parameters
+    ----------
+    dipoles : Dipole | list of instances of Dipole
+        The dipoles to convert.
+    src : instance of SourceSpaces
+        The source space used to generate the forward operator.
+    verbose : bool, str, int, or None
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
+
+    Returns
+    -------
+    stc : SourceEstimate
+        The source estimate.
+    """
+    logger.info('Converting dipoles into a SourceEstimate.')
+    if isinstance(dipoles, Dipole):
+        dipoles = [dipoles]
+    if not isinstance(dipoles, list):
+        raise ValueError('Dipoles must be an instance of Dipole or '
+                         'a list of instances of Dipole. '
+                         'Got %s!' % type(dipoles))
+    tmin = dipoles[0].times[0]
+    tstep = dipoles[0].times[1] - tmin
+    X = np.zeros((len(dipoles), len(dipoles[0].times)))
+    source_rr = np.concatenate([_src['rr'][_src['vertno'], :] for _src in src],
+                               axis=0)
+    n_lh_points = len(src[0]['vertno'])
+    lh_vertno = list()
+    rh_vertno = list()
+    for i in range(len(dipoles)):
+        if not np.all(dipoles[i].pos == dipoles[i].pos[0]):
+            raise ValueError('Only dipoles with fixed position over time '
+                             'are supported!')
+        X[i] = dipoles[i].amplitude
+        idx = np.all(source_rr == dipoles[i].pos[0], axis=1)
+        idx = np.where(idx)[0][0]
+        if idx < n_lh_points:
+            lh_vertno.append(src[0]['vertno'][idx])
+        else:
+            rh_vertno.append(src[1]['vertno'][idx - n_lh_points])
+    vertices = [np.array(lh_vertno).astype(int),
+                np.array(rh_vertno).astype(int)]
+    stc = SourceEstimate(X, vertices=vertices, tmin=tmin, tstep=tstep)
+    logger.info('[done]')
+    return stc
+
+
+@verbose
+def mixed_norm(evoked, forward, noise_cov, alpha, loose='auto', depth=0.8,
                maxit=3000, tol=1e-4, active_set_size=10, pca=True,
                debias=True, time_pca=True, weights=None, weights_min=None,
                solver='auto', n_mxne_iter=1, return_residual=False,
-               verbose=None):
-    """Mixed-norm estimate (MxNE) and iterative reweighted MxNE (irMxNE)
+               return_as_dipoles=False, verbose=None):
+    """Mixed-norm estimate (MxNE) and iterative reweighted MxNE (irMxNE).
 
-    Compute L1/L2 mixed-norm solution or L0.5/L2 mixed-norm solution
-    on evoked data.
-
-    References:
-    Gramfort A., Kowalski M. and Hamalainen, M.,
-    Mixed-norm estimates for the M/EEG inverse problem using accelerated
-    gradient methods, Physics in Medicine and Biology, 2012
-    http://dx.doi.org/10.1088/0031-9155/57/7/1937
-
-    Strohmeier D., Haueisen J., and Gramfort A.,
-    Improved MEG/EEG source localization with reweighted mixed-norms,
-    4th International Workshop on Pattern Recognition in Neuroimaging,
-    Tuebingen, 2014
+    Compute L1/L2 mixed-norm solution [1]_ or L0.5/L2 [2]_ mixed-norm
+    solution on evoked data.
 
     Parameters
     ----------
@@ -192,11 +299,13 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
         Noise covariance to compute whitener.
     alpha : float
         Regularization parameter.
-    loose : float in [0, 1]
+    loose : float in [0, 1] | 'auto'
         Value that weights the source variances of the dipole components
         that are parallel (tangential) to the cortical surface. If loose
-        is 0 or None then the solution is computed with fixed orientation.
+        is 0 then the solution is computed with fixed orientation.
         If loose is 1, it corresponds to free orientations.
+        The default value ('auto') is set to 0.2 for surface-oriented source
+        space and set to 1.0 for volumic or discrete source space.
     depth: None | float in [0, 1]
         Depth weighting coefficients. If None, no depth weighting is performed.
     maxit : int
@@ -213,9 +322,9 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
         If True the rank of the concatenated epochs is reduced to
         its true dimension. If is 'int' the rank is limited to this value.
     weights : None | array | SourceEstimate
-        Weight for penalty in mixed_norm. Can be None or
-        1d array of length n_sources or a SourceEstimate e.g. obtained
-        with wMNE or dSPM or fMRI.
+        Weight for penalty in mixed_norm. Can be None, a
+        1d array with shape (n_sources,), or a SourceEstimate (e.g. obtained
+        with wMNE, dSPM, or fMRI).
     weights_min : float
         Do not consider in the estimation sources for which weights
         is less than weights_min.
@@ -229,8 +338,11 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
         is applied.
     return_residual : bool
         If True, the residual is returned as an Evoked instance.
+    return_as_dipoles : bool
+        If True, the sources are returned as a list of Dipole instances.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -243,11 +355,22 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
     See Also
     --------
     tf_mixed_norm
+
+    References
+    ----------
+    .. [1] A. Gramfort, M. Kowalski, M. Hamalainen,
+       "Mixed-norm estimates for the M/EEG inverse problem using accelerated
+       gradient methods", Physics in Medicine and Biology, 2012.
+       http://dx.doi.org/10.1088/0031-9155/57/7/1937
+
+    .. [2] D. Strohmeier, Y. Bekhti, J. Haueisen, A. Gramfort,
+       "The Iterative Reweighted Mixed-Norm Estimate for Spatio-Temporal
+       MEG/EEG Source Reconstruction", IEEE Transactions of Medical Imaging,
+       Volume 35 (10), pp. 2218-2228, 2016.
     """
     if n_mxne_iter < 1:
         raise ValueError('MxNE has to be computed at least 1 time. '
-                         'Requires n_mxne_iter > 0. '
-                         'Got n_mxne_iter = %d.' % n_mxne_iter)
+                         'Requires n_mxne_iter >= 1, got %d' % n_mxne_iter)
 
     if not isinstance(evoked, list):
         evoked = [evoked]
@@ -259,10 +382,12 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
                for i in range(1, len(evoked))):
         raise Exception('All the datasets must have the same good channels.')
 
+    loose, forward = _check_loose_forward(loose, forward)
+
     # put the forward solution in fixed orientation if it's not already
-    if loose is None and not is_fixed_orient(forward):
-        forward = deepcopy(forward)
-        _to_fixed_ori(forward)
+    if loose == 0. and not is_fixed_orient(forward):
+        forward = convert_forward_solution(
+            forward, surf_ori=True, force_fixed=True, copy=True, use_cps=True)
 
     gain, gain_info, whitener, source_weighting, mask = _prepare_gain(
         forward, evoked[0].info, noise_cov, pca, depth, loose, weights,
@@ -301,15 +426,19 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
             n_orient=n_dip_per_pos, active_set_size=active_set_size,
             debias=debias, solver=solver, verbose=verbose)
 
+    if time_pca:
+        X = np.dot(X, Vh)
+        M = np.dot(M, Vh)
+
+    # Compute estimated whitened sensor data
+    M_estimated = np.dot(gain[:, active_set], X)
+
     if mask is not None:
         active_set_tmp = np.zeros(len(mask), dtype=np.bool)
         active_set_tmp[mask] = active_set
         active_set = active_set_tmp
         del active_set_tmp
 
-    if time_pca:
-        X = np.dot(X, Vh)
-
     if active_set.sum() == 0:
         raise Exception("No active dipoles found. alpha is too big.")
 
@@ -317,15 +446,21 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
     X = _reapply_source_weighting(X, source_weighting,
                                   active_set, n_dip_per_pos)
 
-    stcs = list()
+    outs = list()
     residual = list()
     cnt = 0
     for e in evoked:
         tmin = e.times[0]
         tstep = 1.0 / e.info['sfreq']
         Xe = X[:, cnt:(cnt + len(e.times))]
-        stc = _make_sparse_stc(Xe, active_set, forward, tmin, tstep)
-        stcs.append(stc)
+        if return_as_dipoles:
+            out = _make_dipoles_sparse(
+                Xe, active_set, forward, tmin, tstep,
+                M[:, cnt:(cnt + len(e.times))],
+                M_estimated[:, cnt:(cnt + len(e.times))], verbose=None)
+        else:
+            out = _make_sparse_stc(Xe, active_set, forward, tmin, tstep)
+        outs.append(out)
         cnt += len(e.times)
 
         if return_residual:
@@ -334,12 +469,12 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
 
     logger.info('[done]')
 
-    if len(stcs) == 1:
-        out = stcs[0]
+    if len(outs) == 1:
+        out = outs[0]
         if return_residual:
             residual = residual[0]
     else:
-        out = stcs
+        out = outs
 
     if return_residual:
         out = out, residual
@@ -348,7 +483,7 @@ def mixed_norm(evoked, forward, noise_cov, alpha, loose=0.2, depth=0.8,
 
 
 def _window_evoked(evoked, size):
-    """Window evoked (size in seconds)"""
+    """Window evoked (size in seconds)."""
     if isinstance(size, (float, int)):
         lsize = rsize = float(size)
     else:
@@ -368,30 +503,14 @@ def _window_evoked(evoked, size):
 
 @verbose
 def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
-                  loose=0.2, depth=0.8, maxit=3000, tol=1e-4,
+                  loose='auto', depth=0.8, maxit=3000, tol=1e-4,
                   weights=None, weights_min=None, pca=True, debias=True,
                   wsize=64, tstep=4, window=0.02, return_residual=False,
-                  verbose=None):
-    """Time-Frequency Mixed-norm estimate (TF-MxNE)
+                  return_as_dipoles=False, verbose=None):
+    """Time-Frequency Mixed-norm estimate (TF-MxNE).
 
     Compute L1/L2 + L1 mixed-norm solution on time-frequency
-    dictionary. Works with evoked data.
-
-    References:
-
-    A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
-    Time-Frequency Mixed-Norm Estimates: Sparse M/EEG imaging with
-    non-stationary source activations
-    Neuroimage, Volume 70, 15 April 2013, Pages 410-422, ISSN 1053-8119,
-    DOI: 10.1016/j.neuroimage.2012.12.051.
-
-    A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
-    Functional Brain Imaging with M/EEG Using Structured Sparsity in
-    Time-Frequency Dictionaries
-    Proceedings Information Processing in Medical Imaging
-    Lecture Notes in Computer Science, 2011, Volume 6801/2011,
-    600-611, DOI: 10.1007/978-3-642-22092-0_49
-    http://dx.doi.org/10.1007/978-3-642-22092-0_49
+    dictionary. Works with evoked data [1]_ [2]_.
 
     Parameters
     ----------
@@ -408,11 +527,13 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
         Regularization parameter for temporal sparsity. It set to 0,
         no temporal regularization is applied. It this case, TF-MxNE is
         equivalent to MxNE with L21 norm.
-    loose : float in [0, 1]
+    loose : float in [0, 1] | 'auto'
         Value that weights the source variances of the dipole components
         that are parallel (tangential) to the cortical surface. If loose
-        is 0 or None then the solution is computed with fixed orientation.
+        is 0 then the solution is computed with fixed orientation.
         If loose is 1, it corresponds to free orientations.
+        The default value ('auto') is set to 0.2 for surface-oriented source
+        space and set to 1.0 for volumic or discrete source space.
     depth: None | float in [0, 1]
         Depth weighting coefficients. If None, no depth weighting is performed.
     maxit : int
@@ -441,8 +562,11 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
         and right window length.
     return_residual : bool
         If True, the residual is returned as an Evoked instance.
+    return_as_dipoles : bool
+        If True, the sources are returned as a list of Dipole instances.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -455,6 +579,21 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
     See Also
     --------
     mixed_norm
+
+    References
+    ----------
+    .. [1] A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
+       "Time-Frequency Mixed-Norm Estimates: Sparse M/EEG imaging with
+       non-stationary source activations",
+       Neuroimage, Volume 70, pp. 410-422, 15 April 2013.
+       DOI: 10.1016/j.neuroimage.2012.12.051
+
+    .. [2] A. Gramfort, D. Strohmeier, J. Haueisen, M. Hamalainen, M. Kowalski
+       "Functional Brain Imaging with M/EEG Using Structured Sparsity in
+       Time-Frequency Dictionaries",
+       Proceedings Information Processing in Medical Imaging
+       Lecture Notes in Computer Science, Volume 6801/2011, pp. 600-611, 2011.
+       DOI: 10.1007/978-3-642-22092-0_49
     """
     _check_reference(evoked)
 
@@ -469,10 +608,12 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
         raise Exception('alpha_time must be in range [0, 100].'
                         ' Got alpha_time = %f' % alpha_time)
 
+    loose, forward = _check_loose_forward(loose, forward)
+
     # put the forward solution in fixed orientation if it's not already
-    if loose is None and not is_fixed_orient(forward):
-        forward = deepcopy(forward)
-        _to_fixed_ori(forward)
+    if loose == 0. and not is_fixed_orient(forward):
+        forward = convert_forward_solution(
+            forward, surf_ori=True, force_fixed=True, copy=True, use_cps=True)
 
     n_dip_per_pos = 1 if is_fixed_orient(forward) else 3
 
@@ -499,12 +640,15 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
     X, active_set, E = tf_mixed_norm_solver(
         M, gain, alpha_space, alpha_time, wsize=wsize, tstep=tstep,
         maxit=maxit, tol=tol, verbose=verbose, n_orient=n_dip_per_pos,
-        log_objective=False, debias=debias)
+        log_objective=True, debias=debias)
 
     if active_set.sum() == 0:
         raise Exception("No active dipoles found. "
                         "alpha_space/alpha_time are too big.")
 
+    # Compute estimated whitened sensor data
+    M_estimated = np.dot(gain[:, active_set], X)
+
     if mask is not None:
         active_set_tmp = np.zeros(len(mask), dtype=np.bool)
         active_set_tmp[mask] = active_set
@@ -518,14 +662,17 @@ def tf_mixed_norm(evoked, forward, noise_cov, alpha_space, alpha_time,
         residual = _compute_residual(
             forward, evoked, X, active_set, gain_info)
 
-    stc = _make_sparse_stc(
-        X, active_set, forward, evoked.times[0], 1.0 / info['sfreq'])
+    if return_as_dipoles:
+        out = _make_dipoles_sparse(
+            X, active_set, forward, evoked.times[0], 1.0 / info['sfreq'],
+            M, M_estimated, verbose=None)
+    else:
+        out = _make_sparse_stc(
+            X, active_set, forward, evoked.times[0], 1.0 / info['sfreq'])
 
     logger.info('[done]')
 
     if return_residual:
-        out = stc, residual
-    else:
-        out = stc
+        out = out, residual
 
     return out
