@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
-"""Conversion tool from Brain Vision EEG to FIF"""
+"""Conversion tool from Brain Vision EEG to FIF."""
 
 # Authors: Teon Brooks <teon.brooks@gmail.com>
 #          Christian Brodbeck <christianbrodbeck@nyu.edu>
 #          Eric Larson <larson.eric.d@gmail.com>
 #          Jona Sassenhagen <jona.sassenhagen@gmail.com>
 #          Phillip Alday <phillip.alday@unisa.edu.au>
+#          Okba Bekhelifi <okba.bekhelifi@gmail.com>
 #
 # License: BSD (3-clause)
 
@@ -18,24 +19,26 @@ import numpy as np
 from ...utils import verbose, logger, warn
 from ..constants import FIFF
 from ..meas_info import _empty_info
-from ..base import _BaseRaw, _check_update_montage
-from ..utils import _read_segments_file, _synthesize_stim_channel
+from ..base import BaseRaw, _check_update_montage
+from ..utils import (_read_segments_file, _synthesize_stim_channel,
+                     _mult_cal_one)
 
-from ...externals.six import StringIO
+from ...externals.six import StringIO, string_types
 from ...externals.six.moves import configparser
 
 
-class RawBrainVision(_BaseRaw):
-    """Raw object from Brain Vision EEG file
+class RawBrainVision(BaseRaw):
+    """Raw object from Brain Vision EEG file.
 
     Parameters
     ----------
     vhdr_fname : str
         Path to the EEG header file.
     montage : str | None | instance of Montage
-        Path or instance of montage containing electrode positions.
-        If None, sensor locations are (0,0,0). See the documentation of
-        :func:`mne.channels.read_montage` for more information.
+        Path or instance of montage containing electrode positions. If None,
+        read sensor locations from header file if present, otherwise (0, 0, 0).
+        See the documentation of :func:`mne.channels.read_montage` for more
+        information.
     eog : list or tuple
         Names of channels or list of indices that should be designated
         EOG channels. Values should correspond to the vhdr file.
@@ -64,46 +67,73 @@ class RawBrainVision(_BaseRaw):
         or an empty dict (default), only stimulus events are added to the
         stimulus channel. Keys are case sensitive.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     See Also
     --------
     mne.io.Raw : Documentation of attribute and methods.
     """
+
     @verbose
     def __init__(self, vhdr_fname, montage=None,
                  eog=('HEOGL', 'HEOGR', 'VEOGb'), misc='auto',
                  scale=1., preload=False, response_trig_shift=0,
-                 event_id=None, verbose=None):
+                 event_id=None, verbose=None):  # noqa: D102
         # Channel info and events
         logger.info('Extracting parameters from %s...' % vhdr_fname)
         vhdr_fname = os.path.abspath(vhdr_fname)
-        info, fmt, self._order, mrk_fname, montage = _get_vhdr_info(
-            vhdr_fname, eog, misc, scale, montage)
+        info, data_filename, fmt, order, mrk_fname, montage, n_samples = \
+            _get_vhdr_info(vhdr_fname, eog, misc, scale, montage)
+        self._order = order
+        self._n_samples = n_samples
         events = _read_vmrk_events(mrk_fname, event_id, response_trig_shift)
         _check_update_montage(info, montage)
-        with open(info['filename'], 'rb') as f:
-            f.seek(0, os.SEEK_END)
-            n_samples = f.tell()
-        dtype_bytes = _fmt_byte_dict[fmt]
+        with open(data_filename, 'rb') as f:
+            if isinstance(fmt, dict):  # ASCII, this will be slow :(
+                n_skip = 0
+                for ii in range(int(fmt['skiplines'])):
+                    n_skip += len(f.readline())
+                offsets = np.cumsum([n_skip] + [len(line) for line in f])
+                n_samples = len(offsets) - 1
+            else:
+                f.seek(0, os.SEEK_END)
+                n_samples = f.tell()
+                dtype_bytes = _fmt_byte_dict[fmt]
+                offsets = None
+                n_samples = n_samples // (dtype_bytes * (info['nchan'] - 1))
         self.preload = False  # so the event-setting works
-        last_samps = [(n_samples // (dtype_bytes * (info['nchan'] - 1))) - 1]
-        self._create_event_ch(events, last_samps[0] + 1)
+        self._create_event_ch(events, n_samples)
         super(RawBrainVision, self).__init__(
-            info, last_samps=last_samps, filenames=[info['filename']],
-            orig_format=fmt, preload=preload, verbose=verbose)
+            info, last_samps=[n_samples - 1], filenames=[data_filename],
+            orig_format=fmt, preload=preload, verbose=verbose,
+            raw_extras=[offsets])
 
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
-        """Read a chunk of raw data"""
+        """Read a chunk of raw data."""
         # read data
-        dtype = _fmt_dtype_dict[self.orig_format]
-        n_data_ch = len(self.ch_names) - 1
-        _read_segments_file(self, data, idx, fi, start, stop, cals, mult,
-                            dtype=dtype, n_channels=n_data_ch,
-                            trigger_ch=self._event_ch)
+        if self._order == 'C':
+            _read_segments_c(self, data, idx, fi, start, stop, cals, mult)
+        elif isinstance(self.orig_format, string_types):
+            dtype = _fmt_dtype_dict[self.orig_format]
+            n_data_ch = len(self.ch_names) - 1
+            _read_segments_file(self, data, idx, fi, start, stop, cals, mult,
+                                dtype=dtype, n_channels=n_data_ch,
+                                trigger_ch=self._event_ch)
+        else:
+            offsets = self._raw_extras[fi]
+            with open(self._filenames[fi], 'rb') as fid:
+                fid.seek(offsets[start])
+                block = np.empty((len(self.ch_names), stop - start))
+                for ii in range(stop - start):
+                    line = fid.readline().decode('ASCII')
+                    line = line.strip().replace(',', '.').split()
+                    block[:-1, ii] = list(map(float, line))
+            block[-1] = self._event_ch[start:stop]
+            _mult_cal_one(data, block, idx, cals, mult)
 
     def _get_brainvision_events(self):
-        """Retrieve the events associated with the Brain Vision Raw object
+        """Retrieve the events associated with the Brain Vision Raw object.
 
         Returns
         -------
@@ -114,7 +144,7 @@ class RawBrainVision(_BaseRaw):
         return self._events.copy()
 
     def _set_brainvision_events(self, events):
-        """Set the events and update the synthesized stim channel
+        """Set the events and update the synthesized stim channel.
 
         Parameters
         ----------
@@ -125,7 +155,7 @@ class RawBrainVision(_BaseRaw):
         self._create_event_ch(events)
 
     def _create_event_ch(self, events, n_samp=None):
-        """Create the event channel"""
+        """Create the event channel."""
         if n_samp is None:
             n_samp = self.last_samp - self.first_samp + 1
         events = np.array(events, int)
@@ -138,8 +168,28 @@ class RawBrainVision(_BaseRaw):
             self._data[-1] = self._event_ch
 
 
+def _read_segments_c(raw, data, idx, fi, start, stop, cals, mult):
+    """Read chunk of vectorized raw data."""
+    n_samples = raw._n_samples
+    dtype = _fmt_dtype_dict[raw.orig_format]
+    n_bytes = _fmt_byte_dict[raw.orig_format]
+    n_channels = len(raw.ch_names)
+    trigger_ch = raw._event_ch
+    block = np.zeros((n_channels, stop - start))
+    with open(raw._filenames[fi], 'rb', buffering=0) as fid:
+        for ch_id in np.arange(n_channels)[idx]:
+            if ch_id == n_channels - 1:  # stim channel
+                stim_ch = trigger_ch[start:stop]
+                block[ch_id] = stim_ch
+                continue
+            fid.seek(start * n_bytes + ch_id * n_bytes * n_samples)
+            block[ch_id] = np.fromfile(fid, dtype, stop - start)
+
+        _mult_cal_one(data, block, idx, cals, mult)
+
+
 def _read_vmrk_events(fname, event_id=None, response_trig_shift=0):
-    """Read events from a vmrk file
+    """Read events from a vmrk file.
 
     Parameters
     ----------
@@ -182,15 +232,20 @@ def _read_vmrk_events(fname, event_id=None, response_trig_shift=0):
     # blocks, such as that the filename are specifying are not
     # guaranteed to be ASCII.
 
-    codepage = 'utf-8'
     try:
         # if there is an explicit codepage set, use it
         # we pretend like it's ascii when searching for the codepage
         cp_setting = re.search('Codepage=(.+)',
                                txt.decode('ascii', 'ignore'),
                                re.IGNORECASE & re.MULTILINE)
+        codepage = 'utf-8'
         if cp_setting:
             codepage = cp_setting.group(1).strip()
+        # BrainAmp Recorder also uses ANSI codepage
+        # an ANSI codepage raises a LookupError exception
+        # python recognize ANSI decoding as cp1252
+        if codepage == 'ANSI':
+            codepage = 'cp1252'
         txt = txt.decode(codepage)
     except UnicodeDecodeError:
         # if UTF-8 (new standard) or explicit codepage setting fails,
@@ -199,16 +254,16 @@ def _read_vmrk_events(fname, event_id=None, response_trig_shift=0):
         txt = txt.decode('latin-1')
 
     # extract Marker Infos block
-    m = re.search("\[Marker Infos\]", txt)
+    m = re.search(r"\[Marker Infos\]", txt)
     if not m:
         return np.zeros(0)
     mk_txt = txt[m.end():]
-    m = re.search("\[.*\]", mk_txt)
+    m = re.search(r"\[.*\]", mk_txt)
     if m:
         mk_txt = mk_txt[:m.start()]
 
     # extract event information
-    items = re.findall("^Mk\d+=(.*)", mk_txt, re.MULTILINE)
+    items = re.findall(r"^Mk\d+=(.*)", mk_txt, re.MULTILINE)
     events, dropped = list(), list()
     for info in items:
         mtype, mdesc, onset, duration = info.split(',')[:4]
@@ -218,7 +273,7 @@ def _read_vmrk_events(fname, event_id=None, response_trig_shift=0):
             trigger = event_id[mdesc]
         else:
             try:
-                trigger = int(re.findall('[A-Za-z]*\s*?(\d+)', mdesc)[0])
+                trigger = int(re.findall(r'[A-Za-z]*\s*?(\d+)', mdesc)[0])
             except IndexError:
                 trigger = None
             if mtype.lower().startswith('response'):
@@ -247,15 +302,18 @@ def _read_vmrk_events(fname, event_id=None, response_trig_shift=0):
 
 
 def _check_hdr_version(header):
-    tags = ['Brain Vision Data Exchange Header File Version 1.0',
-            'Brain Vision Data Exchange Header File Version 2.0']
-    if header not in tags:
-        raise ValueError("Currently only support %r, not %r"
-                         "Contact MNE-Developers for support."
-                         % (str(tags), header))
+    """Check the header version."""
+    if header == 'Brain Vision Data Exchange Header File Version 1.0':
+        return 1
+    elif header == 'Brain Vision Data Exchange Header File Version 2.0':
+        return 2
+    else:
+        raise ValueError("Currently only support versions 1.0 and 2.0, not %r "
+                         "Contact MNE-Developers for support." % header)
 
 
 def _check_mrk_version(header):
+    """Check the marker version."""
     tags = ['Brain Vision Data Exchange Marker File, Version 1.0',
             'Brain Vision Data Exchange Marker File, Version 2.0']
     if header not in tags:
@@ -280,7 +338,7 @@ _unit_dict = {'V': 1.,  # V stands for Volt
 
 
 def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
-    """Extracts all the information from the header file.
+    """Extract all the information from the header file.
 
     Parameters
     ----------
@@ -297,10 +355,11 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
         header file, units are in microvolts. Default scale factor is 1.
-    montage : str | True | None | instance of Montage
-        Path or instance of montage containing electrode positions.
-        If None, sensor locations are (0,0,0). See the documentation of
-        :func:`mne.channels.read_montage` for more information.
+    montage : str | None | instance of Montage
+        Path or instance of montage containing electrode positions. If None,
+        read sensor locations from header file if present, otherwise (0, 0, 0).
+        See the documentation of :func:`mne.channels.read_montage` for more
+        information.
 
     Returns
     -------
@@ -314,11 +373,10 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
         Events from the corresponding vmrk file.
     """
     scale = float(scale)
-
     ext = os.path.splitext(vhdr_fname)[-1]
     if ext != '.vhdr':
         raise IOError("The header file must be given to read the data, "
-                      "not the '%s' file." % ext)
+                      "not a file with extension '%s'." % ext)
     with open(vhdr_fname, 'rb') as f:
         # extract the first section to resemble a cfg
         header = f.readline()
@@ -338,6 +396,11 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
                                    re.IGNORECASE & re.MULTILINE)
             if cp_setting:
                 codepage = cp_setting.group(1).strip()
+            # BrainAmp Recorder also uses ANSI codepage
+            # an ANSI codepage raises a LookupError exception
+            # python recognize ANSI decoding as cp1252
+            if codepage == 'ANSI':
+                codepage = 'cp1252'
             settings = settings.decode(codepage)
         except UnicodeDecodeError:
             # if UTF-8 (new standard) or explicit codepage setting fails,
@@ -360,21 +423,42 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     sfreq = 1e6 / cfg.getfloat('Common Infos', 'SamplingInterval')
     info = _empty_info(sfreq)
 
-    # check binary format
-    assert cfg.get('Common Infos', 'DataFormat') == 'BINARY'
     order = cfg.get('Common Infos', 'DataOrientation')
     if order not in _orientation_dict:
         raise NotImplementedError('Data Orientation %s is not supported'
                                   % order)
     order = _orientation_dict[order]
 
-    fmt = cfg.get('Binary Infos', 'BinaryFormat')
-    if fmt not in _fmt_dict:
-        raise NotImplementedError('Datatype %s is not supported' % fmt)
-    fmt = _fmt_dict[fmt]
+    data_format = cfg.get('Common Infos', 'DataFormat')
+    if data_format == 'BINARY':
+        fmt = cfg.get('Binary Infos', 'BinaryFormat')
+        if fmt not in _fmt_dict:
+            raise NotImplementedError('Datatype %s is not supported' % fmt)
+        fmt = _fmt_dict[fmt]
+    else:
+        fmt = dict((key, cfg.get('ASCII Infos', key))
+                   for key in cfg.options('ASCII Infos'))
+
+    # locate EEG and marker files
+    path = os.path.dirname(vhdr_fname)
+    data_filename = os.path.join(path, cfg.get('Common Infos', 'DataFile'))
+    info['meas_date'] = int(time.time())
+    info['buffer_size_sec'] = 1.  # reasonable default
 
     # load channel labels
     nchan = cfg.getint('Common Infos', 'NumberOfChannels') + 1
+    n_samples = None
+    if order == 'C':
+        try:
+            n_samples = cfg.getint('Common Infos', 'DataPoints')
+        except configparser.NoOptionError:
+            logger.warning('No info on DataPoints found. Inferring number of '
+                           'samples from the data file size.')
+            with open(data_filename, 'rb') as fid:
+                fid.seek(0, 2)
+                n_bytes = fid.tell()
+                n_samples = n_bytes // _fmt_byte_dict[fmt] // (nchan - 1)
+
     ch_names = [''] * nchan
     cals = np.empty(nchan)
     ranges = np.empty(nchan)
@@ -406,20 +490,31 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     misc = list(misc_chs.keys()) if misc == 'auto' else misc
 
     # create montage
-    if montage is True:
-        from ...transforms import _sphere_to_cartesian
+    if cfg.has_section('Coordinates') and montage is None:
+        from ...transforms import _sph_to_cart
         from ...channels.montage import Montage
         montage_pos = list()
         montage_names = list()
+        to_misc = list()
         for ch in cfg.items('Coordinates'):
-            montage_names.append(ch_dict[ch[0]])
+            ch_name = ch_dict[ch[0]]
+            montage_names.append(ch_name)
             radius, theta, phi = map(float, ch[1].split(','))
             # 1: radius, 2: theta, 3: phi
-            pos = _sphere_to_cartesian(r=radius, theta=theta, phi=phi)
+            pol = np.deg2rad(theta)
+            az = np.deg2rad(phi)
+            pos = _sph_to_cart(np.array([[radius * 85., az, pol]]))[0]
+            if (pos == 0).all() and ch_name not in list(eog) + misc:
+                to_misc.append(ch_name)
             montage_pos.append(pos)
         montage_sel = np.arange(len(montage_pos))
         montage = Montage(montage_pos, montage_names, 'Brainvision',
                           montage_sel)
+        if len(to_misc) > 0:
+            misc += to_misc
+            warn('No coordinate information found for channels {}. '
+                 'Setting channel types to misc. To avoid this warning, set '
+                 'channel types explicitly.'.format(to_misc))
 
     ch_names[-1] = 'STI 014'
     cals[-1] = 1.
@@ -437,7 +532,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
         settings = settings[idx + 1:]
         hp_col, lp_col = 4, 5
         for idx, setting in enumerate(settings):
-            if re.match('#\s+Name', setting):
+            if re.match(r'#\s+Name', setting):
                 break
             else:
                 idx = None
@@ -450,7 +545,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     if 'S o f t w a r e  F i l t e r s' in settings:
         idx = settings.index('S o f t w a r e  F i l t e r s')
         for idx, setting in enumerate(settings[idx + 1:], idx + 1):
-            if re.match('#\s+Low Cutoff', setting):
+            if re.match(r'#\s+Low Cutoff', setting):
                 hp_col, lp_col = 1, 2
                 warn('Online software filter detected. Using software '
                      'filter settings and ignoring hardware values')
@@ -462,28 +557,34 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
         lowpass = []
         highpass = []
 
+        # for newer BV files, the unit is specified for every channel
+        # separated by a single space, while for older files, the unit is
+        # specified in the column headers
+        divider = r'\s+'
+        if 'Resolution / Unit' in settings[idx]:
+            shift = 1  # shift for unit
+        else:
+            shift = 0
+
         # extract filter units and convert s to Hz if necessary
         # this cannot be done as post-processing as the inverse t-f
         # relationship means that the min/max comparisons don't make sense
         # unless we know the units
-        header = re.split('\s\s+', settings[idx])
+        header = re.split(r'\s\s+', settings[idx])
         hp_s = '[s]' in header[hp_col]
         lp_s = '[s]' in header[lp_col]
 
         for i, ch in enumerate(ch_names[:-1], 1):
-            line = re.split('\s\s+', settings[idx + i])
+            line = re.split(divider, settings[idx + i])
             # double check alignment with channel by using the hw settings
-            # the actual divider is multiple spaces -- for newer BV
-            # files, the unit is specified for every channel separated
-            # by a single space, while for older files, the unit is
-            # specified in the column headers
             if idx == idx_amp:
                 line_amp = line
             else:
-                line_amp = re.split('\s\s+', settings[idx_amp + i])
+                line_amp = re.split(divider, settings[idx_amp + i])
             assert ch in line_amp
-            highpass.append(line[hp_col])
-            lowpass.append(line[lp_col])
+
+            highpass.append(line[hp_col + shift])
+            lowpass.append(line[lp_col + shift])
         if len(highpass) == 0:
             pass
         elif len(set(highpass)) == 1:
@@ -587,12 +688,6 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
                      'Highest (weakest) filter setting (%0.2f Hz%s) '
                      'will be stored.' % (info['lowpass'], nyquist))
 
-    # locate EEG and marker files
-    path = os.path.dirname(vhdr_fname)
-    info['filename'] = os.path.join(path, cfg.get('Common Infos', 'DataFile'))
-    info['meas_date'] = int(time.time())
-    info['buffer_size_sec'] = 1.  # reasonable default
-
     # Creates a list of dicts of eeg channels for raw.info
     logger.info('Setting channel info structure...')
     info['chs'] = []
@@ -626,14 +721,14 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale, montage):
     mrk_fname = os.path.join(path, cfg.get('Common Infos', 'MarkerFile'))
     info._update_redundant()
     info._check_consistency()
-    return info, fmt, order, mrk_fname, montage
+    return info, data_filename, fmt, order, mrk_fname, montage, n_samples
 
 
 def read_raw_brainvision(vhdr_fname, montage=None,
                          eog=('HEOGL', 'HEOGR', 'VEOGb'), misc='auto',
                          scale=1., preload=False, response_trig_shift=0,
                          event_id=None, verbose=None):
-    """Reader for Brain Vision EEG file
+    """Reader for Brain Vision EEG file.
 
     Parameters
     ----------
@@ -671,7 +766,8 @@ def read_raw_brainvision(vhdr_fname, montage=None,
         or an empty dict (default), only stimulus events are added to the
         stimulus channel. Keys are case sensitive.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
