@@ -5,18 +5,17 @@
 
 
 class Bunch(dict):
-    """ Container object for datasets: dictionnary-like object that
-        exposes its keys as attributes.
-    """
+    """Dictionnary-like object thatexposes its keys as attributes."""
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs):  # noqa: D102
         dict.__init__(self, kwargs)
         self.__dict__ = self
 
 
 class BunchConst(Bunch):
-    """Class to prevent us from re-defining constants (DRY)"""
-    def __setattr__(self, attr, val):
+    """Class to prevent us from re-defining constants (DRY)."""
+
+    def __setattr__(self, attr, val):  # noqa: D105
         if attr != '__dict__' and hasattr(self, attr):
             raise AttributeError('Attribute "%s" already set' % attr)
         super(BunchConst, self).__setattr__(attr, val)
@@ -42,9 +41,9 @@ FIFF.FIFFB_EVOKED             = 104
 FIFF.FIFFB_ASPECT             = 105
 FIFF.FIFFB_SUBJECT            = 106
 FIFF.FIFFB_ISOTRAK            = 107
-FIFF.FIFFB_HPI_MEAS           = 108
-FIFF.FIFFB_HPI_RESULT         = 109
-FIFF.FIFFB_HPI_COIL           = 110
+FIFF.FIFFB_HPI_MEAS           = 108  # HPI measurement
+FIFF.FIFFB_HPI_RESULT         = 109  # Result of a HPI fitting procedure
+FIFF.FIFFB_HPI_COIL           = 110  # Data acquired from one HPI coil
 FIFF.FIFFB_PROJECT            = 111
 FIFF.FIFFB_CONTINUOUS_DATA    = 112
 FIFF.FIFFB_VOID               = 114
@@ -121,7 +120,7 @@ FIFF.FIFF_REF_EVENT    = 211
 FIFF.FIFF_EXPERIMENTER = 212
 FIFF.FIFF_DIG_POINT   = 213
 FIFF.FIFF_CH_POS      = 214
-FIFF.FIFF_HPI_SLOPES  = 215
+FIFF.FIFF_HPI_SLOPES  = 215           # HPI data
 FIFF.FIFF_HPI_NCOIL   = 216
 FIFF.FIFF_REQ_EVENT   = 217
 FIFF.FIFF_REQ_LIMIT   = 218
@@ -132,7 +131,7 @@ FIFF.FIFF_COORD_TRANS = 222
 FIFF.FIFF_HIGHPASS    = 223
 FIFF.FIFF_CH_CALS        = 22     # This will not occur in new files
 FIFF.FIFF_HPI_BAD_CHS    = 225    # List of channels considered to be bad in hpi
-FIFF.FIFF_HPI_CORR_COEFF = 226    # Hpi curve fit correlations
+FIFF.FIFF_HPI_CORR_COEFF = 226    # HPI curve fit correlations
 FIFF.FIFF_EVENT_COMMENT  = 227    # Comment about the events used in averaging
 FIFF.FIFF_NO_SAMPLES     = 228    # Number of samples in an epoch
 FIFF.FIFF_FIRST_TIME     = 229    # Time scale minimum
@@ -294,6 +293,9 @@ FIFF.FIFFV_BEM_SURF_ID_BRAIN      = 1
 FIFF.FIFFV_BEM_SURF_ID_SKULL      = 3
 FIFF.FIFFV_BEM_SURF_ID_HEAD       = 4
 
+FIFF.FIFF_SPHERE_ORIGIN          = 3001
+FIFF.FIFF_SPHERE_RADIUS          = 3002
+
 FIFF.FIFF_BEM_SURF_ID           = 3101  # int    surface number
 FIFF.FIFF_BEM_SURF_NAME         = 3102  # string surface name
 FIFF.FIFF_BEM_SURF_NNODE        = 3103  # int    number of nodes on a surface
@@ -796,15 +798,16 @@ FIFF.FIFFV_COIL_VV_MAG_T3          = 3024  # Vectorview SQ20950N magnetometer
 
 FIFF.FIFFV_COIL_MAGNES_MAG         = 4001  # Magnes WH magnetometer
 FIFF.FIFFV_COIL_MAGNES_GRAD        = 4002  # Magnes WH gradiometer
-FIFF.FIFFV_COIL_MAGNES_R_MAG       = 4003  # Magnes WH reference magnetometer
-FIFF.FIFFV_COIL_MAGNES_R_GRAD_DIA  = 4004  # Magnes WH reference diagonal gradioometer
-FIFF.FIFFV_COIL_MAGNES_R_GRAD_OFF  = 4005  # Magnes WH reference off-diagonal gradiometer
 #
 # Magnes reference sensors
 #
 FIFF.FIFFV_COIL_MAGNES_REF_MAG          = 4003
 FIFF.FIFFV_COIL_MAGNES_REF_GRAD         = 4004
 FIFF.FIFFV_COIL_MAGNES_OFFDIAG_REF_GRAD = 4005
+FIFF.FIFFV_COIL_MAGNES_R_MAG = FIFF.FIFFV_COIL_MAGNES_REF_MAG
+FIFF.FIFFV_COIL_MAGNES_R_GRAD = FIFF.FIFFV_COIL_MAGNES_REF_GRAD
+FIFF.FIFFV_COIL_MAGNES_R_GRAD_OFF = FIFF.FIFFV_COIL_MAGNES_OFFDIAG_REF_GRAD
+
 #
 # CTF coil and channel types
 #
@@ -820,9 +823,24 @@ FIFF.FIFFV_COIL_KIT_REF_MAG      = 6002
 #
 # BabySQUID sensors
 #
-FIFF.FIFFV_COIL_BABY_GRAD               = 7001
-FIFF.FIFFV_COIL_BABY_MAG                = 7002
-FIFF.FIFFV_COIL_BABY_REF_MAG            = 7003
+FIFF.FIFFV_COIL_BABY_GRAD          = 7001
+FIFF.FIFFV_COIL_BABY_MAG           = 7002
+FIFF.FIFFV_COIL_BABY_REF_MAG       = 7003
+FIFF.FIFFV_COIL_BABY_REF_MAG2      = 7004
+#
+# Artemis123 sensors
+#
+FIFF.FIFFV_COIL_ARTEMIS123_GRAD         = 7501
+FIFF.FIFFV_COIL_ARTEMIS123_REF_MAG      = 7502
+FIFF.FIFFV_COIL_ARTEMIS123_REF_GRAD     = 7503
+#
+# KRISS sensors
+#
+FIFF.FIFFV_COIL_KRISS_GRAD         = 8001
+#
+# Sample TMS sensors
+#
+FIFF.FIFFV_COIL_SAMPLE_TMS_PLANAR  = 9001
 
 # MNE RealTime
 FIFF.FIFF_MNE_RT_COMMAND           = 3700  # realtime command
