@@ -1,5 +1,4 @@
-"""Conversion tool from CTF to FIF
-"""
+"""Conversion tool from CTF to FIF."""
 
 # Author: Eric Larson <larson.eric.d<gmail.com>
 #
@@ -13,7 +12,7 @@ import numpy as np
 from ...utils import verbose, logger
 from ...externals.six import string_types
 
-from ..base import _BaseRaw
+from ..base import BaseRaw
 from ..utils import _mult_cal_one, _blk_read_lims
 
 from .res4 import _read_res4, _make_ctf_name
@@ -26,12 +25,12 @@ from .constants import CTF
 
 def read_raw_ctf(directory, system_clock='truncate', preload=False,
                  verbose=None):
-    """Raw object from CTF directory
+    """Raw object from CTF directory.
 
     Parameters
     ----------
     directory : str
-        Path to the KIT data (ending in ``'.ds'``).
+        Path to the CTF data (ending in ``'.ds'``).
     system_clock : str
         How to treat the system clock. Use "truncate" (default) to truncate
         the data file when the system clock drops to zero, and use "ignore"
@@ -44,7 +43,8 @@ def read_raw_ctf(directory, system_clock='truncate', preload=False,
         file name of a memory-mapped file which is used to store the data
         on the hard drive (slower, requires less memory).
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -62,13 +62,13 @@ def read_raw_ctf(directory, system_clock='truncate', preload=False,
     return RawCTF(directory, system_clock, preload=preload, verbose=verbose)
 
 
-class RawCTF(_BaseRaw):
-    """Raw object from CTF directory
+class RawCTF(BaseRaw):
+    """Raw object from CTF directory.
 
     Parameters
     ----------
     directory : str
-        Path to the KIT data (ending in ``'.ds'``).
+        Path to the CTF data (ending in ``'.ds'``).
     system_clock : str
         How to treat the system clock. Use "truncate" (default) to truncate
         the data file when the system clock drops to zero, and use "ignore"
@@ -81,15 +81,17 @@ class RawCTF(_BaseRaw):
         file name of a memory-mapped file which is used to store the data
         on the hard drive (slower, requires less memory).
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     See Also
     --------
     mne.io.Raw : Documentation of attribute and methods.
     """
+
     @verbose
     def __init__(self, directory, system_clock='truncate', preload=False,
-                 verbose=None):
+                 verbose=None):  # noqa: D102
         # adapted from mne_ctf2fiff.c
         if not isinstance(directory, string_types) or \
                 not directory.endswith('.ds'):
@@ -131,17 +133,18 @@ class RawCTF(_BaseRaw):
             if len(fnames) == 0:
                 info['buffer_size_sec'] = \
                     sample_info['block_size'] / info['sfreq']
-                info['filename'] = directory
             fnames.append(meg4_name)
             last_samps.append(sample_info['n_samp'] - 1)
             raw_extras.append(sample_info)
+            first_samps = [0] * len(last_samps)
         super(RawCTF, self).__init__(
-            info, preload, last_samps=last_samps, filenames=fnames,
+            info, preload, first_samps=first_samps,
+            last_samps=last_samps, filenames=fnames,
             raw_extras=raw_extras, orig_format='int', verbose=verbose)
 
     @verbose
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
-        """Read a chunk of raw data"""
+        """Read a chunk of raw data."""
         si = self._raw_extras[fi]
         offset = 0
         trial_start_idx, r_lims, d_lims = _blk_read_lims(start, stop,
@@ -164,7 +167,7 @@ class RawCTF(_BaseRaw):
 
 
 def _get_sample_info(fname, res4, system_clock):
-    """Helper to determine the number of valid samples"""
+    """Determine the number of valid samples."""
     logger.info('Finding samples for %s: ' % (fname,))
     if CTF.SYSTEM_CLOCK_CH in res4['ch_names']:
         clock_ch = res4['ch_names'].index(CTF.SYSTEM_CLOCK_CH)
@@ -214,11 +217,10 @@ def _get_sample_info(fname, res4, system_clock):
     else:
         n_trial = n_samp // res4['nsamp']
         n_omit = n_samp_tot - n_samp
-        n_samp = n_trial * res4['nsamp']
         logger.info('    %d x %d = %d samples from %d chs'
                     % (n_trial, res4['nsamp'], n_samp, res4['nchan']))
         if n_omit != 0:
             logger.info('    %d samples omitted at the end' % n_omit)
+
     return dict(n_samp=n_samp, n_samp_tot=n_samp_tot, block_size=res4['nsamp'],
-                n_trial=n_trial, res4_nsamp=res4['nsamp'],
-                n_chan=res4['nchan'])
+                res4_nsamp=res4['nsamp'], n_chan=res4['nchan'])
