@@ -11,10 +11,10 @@ from ..utils import (_read_segments_file, _find_channels,
                      _synthesize_stim_channel)
 from ..constants import FIFF
 from ..meas_info import _empty_info, create_info
-from ..base import _BaseRaw, _check_update_montage
+from ..base import BaseRaw, _check_update_montage
 from ...utils import logger, verbose, check_version, warn
 from ...channels.montage import Montage
-from ...epochs import _BaseEpochs
+from ...epochs import BaseEpochs
 from ...event import read_events
 from ...externals.six import string_types
 
@@ -55,12 +55,14 @@ def _to_loc(ll):
     if isinstance(ll, (int, float)) or len(ll) > 0:
         return ll
     else:
-        return 0.
+        return np.nan
 
 
 def _get_info(eeg, montage, eog=()):
     """Get measurement info."""
+    from scipy import io
     info = _empty_info(sfreq=eeg.srate)
+    update_ch_names = True
 
     # add the ch_names and info['chs'][idx]['loc']
     path = None
@@ -68,21 +70,33 @@ def _get_info(eeg, montage, eog=()):
             eeg.chanlocs = [eeg.chanlocs]
 
     if len(eeg.chanlocs) > 0:
-        ch_names, pos = list(), list()
+        pos_fields = ['X', 'Y', 'Z']
+        if (isinstance(eeg.chanlocs, np.ndarray) and not isinstance(
+                eeg.chanlocs[0], io.matlab.mio5_params.mat_struct)):
+            has_pos = all(fld in eeg.chanlocs[0].dtype.names
+                          for fld in pos_fields)
+        else:
+            has_pos = all(hasattr(eeg.chanlocs[0], fld)
+                          for fld in pos_fields)
+        get_pos = has_pos and montage is None
+        pos_ch_names, ch_names, pos = list(), list(), list()
         kind = 'user_defined'
-        selection = np.arange(len(eeg.chanlocs))
-        locs_available = True
+        update_ch_names = False
         for chanloc in eeg.chanlocs:
             ch_names.append(chanloc.labels)
-            loc_x = _to_loc(chanloc.X)
-            loc_y = _to_loc(chanloc.Y)
-            loc_z = _to_loc(chanloc.Z)
-            locs = np.r_[-loc_y, loc_x, loc_z]
-            if np.unique(locs).size == 1:
-                locs_available = False
-            pos.append(locs)
-        if locs_available:
-            montage = Montage(np.array(pos), ch_names, kind, selection)
+            if get_pos:
+                loc_x = _to_loc(chanloc.X)
+                loc_y = _to_loc(chanloc.Y)
+                loc_z = _to_loc(chanloc.Z)
+                locs = np.r_[-loc_y, loc_x, loc_z]
+                if not np.any(np.isnan(locs)):
+                    pos_ch_names.append(chanloc.labels)
+                    pos.append(locs)
+        n_channels_with_pos = len(pos_ch_names)
+        info = create_info(ch_names, eeg.srate, ch_types='eeg')
+        if n_channels_with_pos > 0:
+            selection = np.arange(n_channels_with_pos)
+            montage = Montage(np.array(pos), pos_ch_names, kind, selection)
     elif isinstance(montage, string_types):
         path = op.dirname(montage)
     else:  # if eeg.chanlocs is empty, we still need default chan names
@@ -92,7 +106,7 @@ def _get_info(eeg, montage, eog=()):
         info = create_info(ch_names, eeg.srate, ch_types='eeg')
     else:
         _check_update_montage(info, montage, path=path,
-                              update_ch_names=True)
+                              update_ch_names=update_ch_names)
 
     info['buffer_size_sec'] = 1.  # reasonable default
     # update the info dict
@@ -111,7 +125,7 @@ def _get_info(eeg, montage, eog=()):
 def read_raw_eeglab(input_fname, montage=None, eog=(), event_id=None,
                     event_id_func='strip_to_integer', preload=False,
                     verbose=None, uint16_codec=None):
-    """Read an EEGLAB .set file
+    r"""Read an EEGLAB .set file.
 
     Parameters
     ----------
@@ -153,7 +167,8 @@ def read_raw_eeglab(input_fname, montage=None, eog=(), event_id=None,
         preload=False will be effective only if the data is stored in a
         separate binary file.
     verbose : bool | str | int | None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
     uint16_codec : str | None
         If your \*.set file contains non-ascii characters, sometimes reading
         it may fail and give rise to error message stating that "buffer is
@@ -181,7 +196,7 @@ def read_raw_eeglab(input_fname, montage=None, eog=(), event_id=None,
 
 def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
                        eog=(), verbose=None, uint16_codec=None):
-    """Reader function for EEGLAB epochs files
+    r"""Reader function for EEGLAB epochs files.
 
     Parameters
     ----------
@@ -214,7 +229,8 @@ def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
         If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
         Defaults to empty tuple.
     verbose : bool | str | int | None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
     uint16_codec : str | None
         If your \*.set file contains non-ascii characters, sometimes reading
         it may fail and give rise to error message stating that "buffer is
@@ -242,8 +258,8 @@ def read_epochs_eeglab(input_fname, events=None, event_id=None, montage=None,
     return epochs
 
 
-class RawEEGLAB(_BaseRaw):
-    """Raw object from EEGLAB .set file.
+class RawEEGLAB(BaseRaw):
+    r"""Raw object from EEGLAB .set file.
 
     Parameters
     ----------
@@ -283,7 +299,8 @@ class RawEEGLAB(_BaseRaw):
         a memory-mapped file which is used to store the data on the hard
         drive (slower, requires less memory).
     verbose : bool | str | int | None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
     uint16_codec : str | None
         If your \*.set file contains non-ascii characters, sometimes reading
         it may fail and give rise to error message stating that "buffer is
@@ -304,12 +321,11 @@ class RawEEGLAB(_BaseRaw):
     --------
     mne.io.Raw : Documentation of attribute and methods.
     """
+
     @verbose
     def __init__(self, input_fname, montage, eog=(), event_id=None,
                  event_id_func='strip_to_integer', preload=False,
-                 verbose=None, uint16_codec=None):
-        """Read EEGLAB .set file.
-        """
+                 verbose=None, uint16_codec=None):  # noqa: D102
         from scipy import io
         basedir = op.dirname(input_fname)
         _check_mat_struct(input_fname)
@@ -331,8 +347,8 @@ class RawEEGLAB(_BaseRaw):
         info['chs'].append(stim_chan)
         info._update_redundant()
 
-        events = _read_eeglab_events(eeg, event_id=event_id,
-                                     event_id_func=event_id_func)
+        events = read_events_eeglab(eeg, event_id=event_id,
+                                    event_id_func=event_id_func)
         self._create_event_ch(events, n_samples=eeg.pnts)
 
         # read the data
@@ -364,7 +380,16 @@ class RawEEGLAB(_BaseRaw):
                 verbose=verbose)
 
     def _create_event_ch(self, events, n_samples=None):
-        """Create the event channel"""
+        """Create the event channel."""
+        n_dropped = len(events[:, 0]) - len(set(events[:, 0]))
+        if n_dropped > 0:
+            warn(str(n_dropped) + " events will be dropped because they "
+                 "occur on the same time sample as another event. "
+                 "`mne.io.Raw` objects store events on an event channel, "
+                 "which cannot represent two events on the same sample. You "
+                 "can extract the original event structure using "
+                 "`mne.io.eeglab.read_events_eeglab`. Then, you can e.g. "
+                 "subset the extracted events for constructing epochs.")
         if n_samples is None:
             n_samples = self.last_samp - self.first_samp + 1
         events = np.array(events, int)
@@ -374,14 +399,14 @@ class RawEEGLAB(_BaseRaw):
         self._event_ch = _synthesize_stim_channel(events, n_samples)
 
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
-        """Read a chunk of raw data"""
+        """Read a chunk of raw data."""
         _read_segments_file(self, data, idx, fi, start, stop, cals, mult,
                             dtype=np.float32, trigger_ch=self._event_ch,
                             n_channels=self.info['nchan'] - 1)
 
 
-class EpochsEEGLAB(_BaseEpochs):
-    """Epochs from EEGLAB .set file
+class EpochsEEGLAB(BaseEpochs):
+    r"""Epochs from EEGLAB .set file.
 
     Parameters
     ----------
@@ -443,7 +468,8 @@ class EpochsEEGLAB(_BaseEpochs):
         If 'auto', the channel names containing ``EOG`` or ``EYE`` are used.
         Defaults to empty tuple.
     verbose : bool | str | int | None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
     uint16_codec : str | None
         If your \*.set file contains non-ascii characters, sometimes reading
         it may fail and give rise to error message stating that "buffer is
@@ -459,11 +485,12 @@ class EpochsEEGLAB(_BaseEpochs):
     --------
     mne.Epochs : Documentation of attribute and methods.
     """
+
     @verbose
     def __init__(self, input_fname, events=None, event_id=None, tmin=0,
-                 baseline=None,  reject=None, flat=None, reject_tmin=None,
+                 baseline=None, reject=None, flat=None, reject_tmin=None,
                  reject_tmax=None, montage=None, eog=(), verbose=None,
-                 uint16_codec=None):
+                 uint16_codec=None):  # noqa: D102
         from scipy import io
         _check_mat_struct(input_fname)
         eeg = io.loadmat(input_fname, struct_as_record=False,
@@ -480,8 +507,11 @@ class EpochsEEGLAB(_BaseEpochs):
             ev_idx = 0
             warn_multiple_events = False
             for ep in eeg.epoch:
+                if isinstance(ep.eventtype, int):
+                    ep.eventtype = str(ep.eventtype)
                 if not isinstance(ep.eventtype, string_types):
-                    event_type = '/'.join(ep.eventtype.tolist())
+                    event_type = '/'.join([str(et) for et
+                                           in ep.eventtype.tolist()])
                     event_name.append(event_type)
                     # store latency of only first event
                     event_latencies.append(eeg.event[ev_idx].latency)
@@ -528,7 +558,6 @@ class EpochsEEGLAB(_BaseEpochs):
                 raise ValueError('No matching events found for %s '
                                  '(event id %i)' % (key, val))
 
-        self._filename = input_fname
         if isinstance(eeg.data, string_types):
             basedir = op.dirname(input_fname)
             data_fname = op.join(basedir, eeg.data)
@@ -550,22 +579,82 @@ class EpochsEEGLAB(_BaseEpochs):
         super(EpochsEEGLAB, self).__init__(
             info, data, events, event_id, tmin, tmax, baseline,
             reject=reject, flat=flat, reject_tmin=reject_tmin,
-            reject_tmax=reject_tmax, add_eeg_ref=False, verbose=verbose)
+            reject_tmax=reject_tmax, filename=input_fname, verbose=verbose)
+
+        # data are preloaded but _bad_dropped is not set so we do it here:
+        self._bad_dropped = True
         logger.info('Ready.')
 
 
-def _read_eeglab_events(eeg, event_id=None, event_id_func='strip_to_integer'):
-    """Create events array from EEGLAB structure
+def read_events_eeglab(eeg, event_id=None, event_id_func='strip_to_integer',
+                       uint16_codec=None):
+    r"""Create events array from EEGLAB structure.
 
     An event array is constructed by looking up events in the
     event_id, trying to reduce them to their integer part otherwise, and
     entirely dropping them (with a warning) if this is impossible.
-    Returns a 1x3 array of zeros if no events are found."""
+    Returns a 1x3 array of zeros if no events are found.
+
+    Usually, the EEGLAB readers will automatically construct event information
+    for you. However, the reader for continuous data stores event information
+    in the stimulus channel, which can only code one event per time sample.
+    Use this function if your EEGLAB file has events happening at the
+    same time (sample) point to manually create an events array.
+
+    Parameters
+    ----------
+    eeg : str | object
+        The EEGLAB object from which events are read in.
+        If str, path to the (EEGLAB) .set file.
+        Else, the "EEG" field of a MATLAB EEGLAB structure as read in by
+        scipy.io.loadmat.
+    event_id : dict | None
+        The ids of the events to consider. If None (default), an empty dict is
+        used and ``event_id_func`` (see below) is called on every event value.
+        If dict, the keys will be mapped to trigger values on the stimulus
+        channel and only keys not in ``event_id`` will be handled by
+        ``event_id_func``. Keys are case-sensitive.
+        Example::
+
+            {'SyncStatus': 1; 'Pulse Artifact': 3}
+
+    event_id_func : None | str | callable
+        What to do for events not found in ``event_id``. Must take one ``str``
+        argument and return an ``int``. If string, must be 'strip-to-integer',
+        in which case it defaults to stripping event codes such as "D128" or
+        "S  1" of their non-integer parts and returns the integer.
+        If the event is not in the ``event_id`` and calling ``event_id_func``
+        on it results in a ``TypeError`` (e.g. if ``event_id_func`` is
+        ``None``) or a ``ValueError``, the event is dropped.
+    uint16_codec : str | None
+        If your \*.set file contains non-ascii characters, sometimes reading
+        it may fail and give rise to error message stating that "buffer is
+        too small". ``uint16_codec`` allows to specify what codec (for example:
+        'latin1' or 'utf-8') should be used when reading character arrays and
+        can therefore help you solve this problem.
+
+    Returns
+    -------
+    events : array, shape = (n_events, 3)
+        All events that were found. The first column contains the event time
+        in samples and the third column contains the event id. The center
+        column is zero.
+
+    See Also
+    --------
+    mne.find_events : Extract events from a stim channel. Note that stim
+        channels can only code for one event per time point.
+    """
     if event_id_func is 'strip_to_integer':
         event_id_func = _strip_to_integer
     if event_id is None:
         event_id = dict()
 
+    if isinstance(eeg, string_types):
+        from scipy import io
+        eeg = io.loadmat(eeg, struct_as_record=False, squeeze_me=True,
+                         uint16_codec=uint16_codec)['EEG']
+
     if isinstance(eeg.event, np.ndarray):
         types = [str(event.type) for event in eeg.event]
         latencies = [event.latency for event in eeg.event]
