@@ -5,22 +5,29 @@ from __future__ import print_function
 #
 # License: BSD (3-clause)
 
-import os.path as op
 import inspect
+import os.path as op
+import warnings
+
 import numpy as np
 from numpy.testing import assert_array_almost_equal, assert_array_equal
-from nose.tools import assert_equal, assert_raises, assert_true
+from nose.tools import (assert_equal, assert_almost_equal, assert_raises,
+                        assert_true)
 from scipy import linalg
 import scipy.io
 
+import mne
 from mne import pick_types, Epochs, find_events, read_events
+from mne.datasets.testing import requires_testing_data
 from mne.transforms import apply_trans
 from mne.tests.common import assert_dig_allclose
-from mne.utils import run_tests_if_main
+from mne.utils import run_tests_if_main, _TempDir
 from mne.io import read_raw_fif, read_raw_kit, read_epochs_kit
+from mne.io.constants import FIFF
 from mne.io.kit.coreg import read_sns
-from mne.io.kit.constants import KIT, KIT_CONSTANTS, KIT_NY, KIT_UMD_2014
+from mne.io.kit.constants import KIT
 from mne.io.tests.test_raw import _test_raw_reader
+from mne.surface import _get_ico_surface
 
 FILE = inspect.getfile(inspect.currentframe())
 parent_dir = op.dirname(op.abspath(FILE))
@@ -37,7 +44,11 @@ hsp_txt_path = op.join(data_dir, 'test_hsp.txt')
 elp_path = op.join(data_dir, 'test.elp')
 hsp_path = op.join(data_dir, 'test.hsp')
 
+data_path = mne.datasets.testing.data_path(download=False)
+sqd_as_path = op.join(data_path, 'KIT', 'test_as-raw.con')
 
+
+@requires_testing_data
 def test_data():
     """Test reading raw kit files."""
     assert_raises(TypeError, read_raw_kit, epochs_path)
@@ -56,7 +67,18 @@ def test_data():
                               stimthresh=1)
     assert_true('RawKIT' in repr(raw_py))
     assert_equal(raw_mrk.info['kit_system_id'], KIT.SYSTEM_NYU_2010)
-    assert_true(KIT_CONSTANTS[raw_mrk.info['kit_system_id']] is KIT_NY)
+
+    # check number/kind of channels
+    assert_equal(len(raw_py.info['chs']), 193)
+    kit_channels = (('kind', {FIFF.FIFFV_MEG_CH: 157, FIFF.FIFFV_REF_MEG_CH: 3,
+                              FIFF.FIFFV_MISC_CH: 32, FIFF.FIFFV_STIM_CH: 1}),
+                    ('coil_type', {FIFF.FIFFV_COIL_KIT_GRAD: 157,
+                                   FIFF.FIFFV_COIL_KIT_REF_MAG: 3,
+                                   FIFF.FIFFV_COIL_NONE: 33}))
+    for label, target in kit_channels:
+        actual = {id_: sum(ch[label] == id_ for ch in raw_py.info['chs']) for
+                  id_ in target.keys()}
+        assert_equal(actual, target)
 
     # Test stim channel
     raw_stim = read_raw_kit(sqd_path, mrk_path, elp_txt_path, hsp_txt_path,
@@ -71,7 +93,7 @@ def test_data():
     # Binary file only stores the sensor channels
     py_picks = pick_types(raw_py.info, exclude='bads')
     raw_bin = op.join(data_dir, 'test_bin_raw.fif')
-    raw_bin = read_raw_fif(raw_bin, preload=True, add_eeg_ref=False)
+    raw_bin = read_raw_fif(raw_bin, preload=True)
     bin_picks = pick_types(raw_bin.info, stim=True, exclude='bads')
     data_bin, _ = raw_bin[bin_picks]
     data_py, _ = raw_py[py_picks]
@@ -92,15 +114,34 @@ def test_data():
     _test_raw_reader(read_raw_kit, input_fname=sqd_umd_path)
     raw = read_raw_kit(sqd_umd_path)
     assert_equal(raw.info['kit_system_id'], KIT.SYSTEM_UMD_2014_12)
-    assert_true(KIT_CONSTANTS[raw.info['kit_system_id']] is KIT_UMD_2014)
+    # check number/kind of channels
+    assert_equal(len(raw.info['chs']), 193)
+    for label, target in kit_channels:
+        actual = {id_: sum(ch[label] == id_ for ch in raw.info['chs']) for
+                  id_ in target.keys()}
+        assert_equal(actual, target)
+
+    # KIT Academia Sinica
+    raw = read_raw_kit(sqd_as_path, slope='+')
+    assert_equal(raw.info['kit_system_id'], KIT.SYSTEM_AS_2008)
+    assert_equal(raw.info['chs'][100]['ch_name'], 'MEG 101')
+    assert_equal(raw.info['chs'][100]['kind'], FIFF.FIFFV_MEG_CH)
+    assert_equal(raw.info['chs'][100]['coil_type'], FIFF.FIFFV_COIL_KIT_GRAD)
+    assert_equal(raw.info['chs'][157]['ch_name'], 'MEG 158')
+    assert_equal(raw.info['chs'][157]['kind'], FIFF.FIFFV_REF_MEG_CH)
+    assert_equal(raw.info['chs'][157]['coil_type'],
+                 FIFF.FIFFV_COIL_KIT_REF_MAG)
+    assert_equal(raw.info['chs'][160]['ch_name'], 'EEG 001')
+    assert_equal(raw.info['chs'][160]['kind'], FIFF.FIFFV_EEG_CH)
+    assert_equal(raw.info['chs'][160]['coil_type'], FIFF.FIFFV_COIL_EEG)
+    assert_array_equal(find_events(raw), [[91, 0, 2]])
 
 
 def test_epochs():
     """Test reading epoched SQD file."""
     raw = read_raw_kit(sqd_path, stim=None)
     events = read_events(events_path)
-    raw_epochs = Epochs(raw, events, None, tmin=0, tmax=.099, baseline=None,
-                        add_eeg_ref=False)
+    raw_epochs = Epochs(raw, events, None, tmin=0, tmax=.099, baseline=None)
     data1 = raw_epochs.get_data()
     epochs = read_epochs_kit(epochs_path, events_path)
     data11 = epochs.get_data()
@@ -141,10 +182,10 @@ def test_ch_loc():
     """Test raw kit loc."""
     raw_py = read_raw_kit(sqd_path, mrk_path, elp_txt_path, hsp_txt_path,
                           stim='<')
-    raw_bin = read_raw_fif(op.join(data_dir, 'test_bin_raw.fif'),
-                           add_eeg_ref=False)
+    raw_bin = read_raw_fif(op.join(data_dir, 'test_bin_raw.fif'))
 
-    ch_py = raw_py._raw_extras[0]['sensor_locs'][:, :5]
+    ch_py = np.array([ch['loc'] for ch in
+                      raw_py._raw_extras[0]['channels'][:160]])
     # ch locs stored as m, not mm
     ch_py[:, :3] *= 1e3
     ch_sns = read_sns(op.join(data_dir, 'sns.txt'))
@@ -187,4 +228,34 @@ def test_hsp_elp():
     assert_array_almost_equal(pts_elp_in_dev, pts_txt_in_dev, decimal=5)
 
 
+def test_decimate():
+    """Test decimation of digitizer headshapes with too many points."""
+    # load headshape and convert to meters
+    hsp_mm = _get_ico_surface(5)['rr'] * 100
+    hsp_m = hsp_mm / 1000.
+
+    # save headshape to a file in mm in temporary directory
+    tempdir = _TempDir()
+    sphere_hsp_path = op.join(tempdir, 'test_sphere.txt')
+    np.savetxt(sphere_hsp_path, hsp_mm)
+
+    # read in raw data using spherical hsp, and extract new hsp
+    with warnings.catch_warnings(record=True) as w:
+        raw = read_raw_kit(sqd_path, mrk_path, elp_txt_path, sphere_hsp_path)
+    assert_true(any('more than' in str(ww.message) for ww in w))
+    # collect headshape from raw (should now be in m)
+    hsp_dec = np.array([dig['r'] for dig in raw.info['dig']])[8:]
+
+    # with 10242 points and _decimate_points set to resolution of 5 mm, hsp_dec
+    # should be a bit over 5000 points. If not, something is wrong or
+    # decimation resolution has been purposefully changed
+    assert_true(len(hsp_dec) > 5000)
+
+    # should have similar size, distance from center
+    dist = np.sqrt(np.sum((hsp_m - np.mean(hsp_m, axis=0))**2, axis=1))
+    dist_dec = np.sqrt(np.sum((hsp_dec - np.mean(hsp_dec, axis=0))**2, axis=1))
+    hsp_rad = np.mean(dist)
+    hsp_dec_rad = np.mean(dist_dec)
+    assert_almost_equal(hsp_rad, hsp_dec_rad, places=3)
+
 run_tests_if_main()
