@@ -15,7 +15,7 @@ from ..externals.six import string_types
 
 
 def channel_type(info, idx):
-    """Get channel type
+    """Get channel type.
 
     Parameters
     ----------
@@ -83,7 +83,7 @@ def channel_type(info, idx):
 
 
 def pick_channels(ch_names, include, exclude=[]):
-    """Pick channels by names
+    """Pick channels by names.
 
     Returns the indices of the good channels in ch_names.
 
@@ -93,6 +93,10 @@ def pick_channels(ch_names, include, exclude=[]):
         List of channels.
     include : list of string
         List of channels to include (if empty include all available).
+
+        .. note:: This is to be treated as a set. The order of this list
+           is not used or maintained in ``sel``.
+
     exclude : list of string
         List of channels to exclude (if empty do not exclude any channel).
         Defaults to [].
@@ -123,7 +127,7 @@ def pick_channels(ch_names, include, exclude=[]):
 
 
 def pick_channels_regexp(ch_names, regexp):
-    """Pick channels using regular expression
+    """Pick channels using regular expression.
 
     Returns the indices of the good channels in ch_names.
 
@@ -157,7 +161,7 @@ def pick_channels_regexp(ch_names, regexp):
 
 
 def _triage_meg_pick(ch, meg):
-    """Helper to triage an MEG pick type"""
+    """Triage an MEG pick type."""
     if meg is True:
         return True
     elif ch['unit'] == FIFF.FIFF_UNIT_T_M:
@@ -173,7 +177,7 @@ def _triage_meg_pick(ch, meg):
 
 
 def _triage_fnirs_pick(ch, fnirs):
-    """Helper to triage an fNIRS pick type."""
+    """Triage an fNIRS pick type."""
     if fnirs is True:
         return True
     elif ch['coil_type'] == FIFF.FIFFV_COIL_FNIRS_HBO and fnirs == 'hbo':
@@ -184,7 +188,7 @@ def _triage_fnirs_pick(ch, fnirs):
 
 
 def _check_meg_type(meg, allow_auto=False):
-    """Helper to ensure a valid meg type"""
+    """Ensure a valid meg type."""
     if isinstance(meg, string_types):
         allowed_types = ['grad', 'mag', 'planar1', 'planar2']
         allowed_types += ['auto'] if allow_auto else []
@@ -196,9 +200,9 @@ def _check_meg_type(meg, allow_auto=False):
 def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
                emg=False, ref_meg='auto', misc=False, resp=False, chpi=False,
                exci=False, ias=False, syst=False, seeg=False, dipole=False,
-               gof=False, bio=False, ecog=False, fnirs=False, include=[],
+               gof=False, bio=False, ecog=False, fnirs=False, include=(),
                exclude='bads', selection=None):
-    """Pick channels by type and names
+    """Pick channels by type and names.
 
     Parameters
     ----------
@@ -300,7 +304,7 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
     for k in range(nchan):
         kind = info['chs'][k]['kind']
         # XXX eventually we should de-duplicate this with channel_type!
-        if kind == FIFF.FIFFV_MEG_CH:
+        if kind == FIFF.FIFFV_MEG_CH and meg:
             pick[k] = _triage_meg_pick(info['chs'][k], meg)
         elif kind == FIFF.FIFFV_EEG_CH and eeg:
             pick[k] = True
@@ -347,7 +351,7 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
         # the selection only restricts these types of channels
         sel_kind = [FIFF.FIFFV_MEG_CH, FIFF.FIFFV_REF_MEG_CH,
                     FIFF.FIFFV_EEG_CH]
-        for k in np.where(pick == True)[0]:  # noqa
+        for k in np.where(pick)[0]:
             if (info['chs'][k]['kind'] in sel_kind and
                     info['ch_names'][k] not in selection):
                 pick[k] = False
@@ -364,7 +368,7 @@ def pick_types(info, meg=True, eeg=False, stim=False, eog=False, ecg=False,
 
 
 def pick_info(info, sel=(), copy=True):
-    """Restrict an info structure to a selection of channels
+    """Restrict an info structure to a selection of channels.
 
     Parameters
     ----------
@@ -381,8 +385,7 @@ def pick_info(info, sel=(), copy=True):
         Info structure restricted to a selection of channels.
     """
     info._check_consistency()
-    if copy:
-        info = deepcopy(info)
+    info = info.copy() if copy else info
     if sel is None:
         return info
     elif len(sel) == 0:
@@ -392,23 +395,24 @@ def pick_info(info, sel=(), copy=True):
     info._update_redundant()
     info['bads'] = [ch for ch in info['bads'] if ch in info['ch_names']]
 
-    comps = deepcopy(info['comps'])
-    for c in comps:
-        row_idx = [k for k, n in enumerate(c['data']['row_names'])
-                   if n in info['ch_names']]
-        row_names = [c['data']['row_names'][i] for i in row_idx]
-        rowcals = c['rowcals'][row_idx]
-        c['rowcals'] = rowcals
-        c['data']['nrow'] = len(row_names)
-        c['data']['row_names'] = row_names
-        c['data']['data'] = c['data']['data'][row_idx]
-    info['comps'] = comps
+    if 'comps' in info:
+        comps = deepcopy(info['comps'])
+        for c in comps:
+            row_idx = [k for k, n in enumerate(c['data']['row_names'])
+                       if n in info['ch_names']]
+            row_names = [c['data']['row_names'][i] for i in row_idx]
+            rowcals = c['rowcals'][row_idx]
+            c['rowcals'] = rowcals
+            c['data']['nrow'] = len(row_names)
+            c['data']['row_names'] = row_names
+            c['data']['data'] = c['data']['data'][row_idx]
+        info['comps'] = comps
     info._check_consistency()
     return info
 
 
 def _has_kit_refs(info, picks):
-    """Helper to determine if KIT ref channels are chosen
+    """Determine if KIT ref channels are chosen.
 
     This is currently only used by make_forward_solution, which cannot
     run when KIT reference channels are included.
@@ -420,7 +424,7 @@ def _has_kit_refs(info, picks):
 
 
 def pick_channels_evoked(orig, include=[], exclude='bads'):
-    """Pick channels from evoked data
+    """Pick channels from evoked data.
 
     Parameters
     ----------
@@ -464,7 +468,7 @@ def pick_channels_evoked(orig, include=[], exclude='bads'):
 
 @verbose
 def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
-    """Pick channels from forward operator
+    """Pick channels from forward operator.
 
     Parameters
     ----------
@@ -477,7 +481,8 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
         Channels to exclude (if empty, do not exclude any). Defaults to [].
         If 'bads', then exclude bad channels in orig.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -539,7 +544,7 @@ def pick_channels_forward(orig, include=[], exclude=[], verbose=None):
 
 def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, seeg=False,
                        ecog=False, include=[], exclude=[]):
-    """Pick by channel type and names from a forward operator
+    """Pick by channel type and names from a forward operator.
 
     Parameters
     ----------
@@ -579,8 +584,7 @@ def pick_types_forward(orig, meg=True, eeg=False, ref_meg=True, seeg=False,
 
 
 def channel_indices_by_type(info):
-    """Get indices of channels by type
-    """
+    """Get indices of channels by type."""
     idx = dict((key, list()) for key in _PICK_TYPES_KEYS if
                key not in ('meg', 'fnirs'))
     idx.update(mag=list(), grad=list(), hbo=list(), hbr=list())
@@ -593,7 +597,7 @@ def channel_indices_by_type(info):
 
 
 def pick_channels_cov(orig, include=[], exclude='bads'):
-    """Pick channels from covariance matrix
+    """Pick channels from covariance matrix.
 
     Parameters
     ----------
@@ -609,23 +613,22 @@ def pick_channels_cov(orig, include=[], exclude='bads'):
     res : dict
         Covariance solution restricted to selected channels.
     """
+    from ..cov import Covariance
     exclude = orig['bads'] if exclude == 'bads' else exclude
     sel = pick_channels(orig['names'], include=include, exclude=exclude)
-    res = deepcopy(orig)
-    res['dim'] = len(sel)
-    if not res['diag']:
-        res['data'] = orig['data'][sel][:, sel]
-    else:
-        res['data'] = orig['data'][sel]
-    res['names'] = [orig['names'][k] for k in sel]
-    res['bads'] = [name for name in orig['bads'] if name in res['names']]
-    res['eig'] = None
-    res['eigvec'] = None
+    data = orig['data'][sel][:, sel] if not orig['diag'] else orig['data'][sel]
+    names = [orig['names'][k] for k in sel]
+    bads = [name for name in orig['bads'] if name in orig['names']]
+    res = Covariance(
+        data=data, names=names, bads=bads, projs=deepcopy(orig['projs']),
+        nfree=orig['nfree'], eig=None, eigvec=None,
+        method=orig.get('method', None), loglik=orig.get('loglik', None))
     return res
 
 
-def _picks_by_type(info, meg_combined=False, ref_meg=False):
-    """Get data channel indices as separate list of tuples
+def _picks_by_type(info, meg_combined=False, ref_meg=False, exclude='bads'):
+    """Get data channel indices as separate list of tuples.
+
     Parameters
     ----------
     info : instance of mne.measuerment_info.Info
@@ -634,6 +637,9 @@ def _picks_by_type(info, meg_combined=False, ref_meg=False):
         Whether to return combined picks for grad and mag.
     ref_meg : bool
         If True include CTF / 4D reference channels
+    exclude : list of string | str
+        List of channels to exclude. If 'bads' (default), exclude channels
+        in info['bads'].
 
     Returns
     -------
@@ -647,22 +653,22 @@ def _picks_by_type(info, meg_combined=False, ref_meg=False):
     if has_mag and (meg_combined is not True or not has_grad):
         picks_list.append(
             ('mag', pick_types(info, meg='mag', eeg=False, stim=False,
-             ref_meg=ref_meg))
+             ref_meg=ref_meg, exclude=exclude))
         )
     if has_grad and (meg_combined is not True or not has_mag):
         picks_list.append(
             ('grad', pick_types(info, meg='grad', eeg=False, stim=False,
-             ref_meg=ref_meg))
+             ref_meg=ref_meg, exclude=exclude))
         )
     if has_mag and has_grad and meg_combined is True:
         picks_list.append(
             ('meg', pick_types(info, meg=True, eeg=False, stim=False,
-             ref_meg=ref_meg))
+             ref_meg=ref_meg, exclude=exclude))
         )
     if has_eeg:
         picks_list.append(
             ('eeg', pick_types(info, meg=False, eeg=True, stim=False,
-             ref_meg=ref_meg))
+             ref_meg=ref_meg, exclude=exclude))
         )
     return picks_list
 
@@ -706,15 +712,20 @@ _PICK_TYPES_DATA_DICT = dict(
 _PICK_TYPES_KEYS = tuple(list(_PICK_TYPES_DATA_DICT.keys()) + ['ref_meg'])
 _DATA_CH_TYPES_SPLIT = ['mag', 'grad', 'eeg', 'seeg', 'ecog', 'hbo', 'hbr']
 
+# Valid data types, ordered for consistency, used in viz/evoked.
+_VALID_CHANNEL_TYPES = ['eeg', 'grad', 'mag', 'seeg', 'eog', 'ecg', 'emg',
+                        'dipole', 'gof', 'bio', 'ecog', 'hbo', 'hbr',
+                        'misc']
+
 
 def _pick_data_channels(info, exclude='bads', with_ref_meg=True):
-    """Convenience function for picking only data channels."""
+    """Pick only data channels."""
     return pick_types(info, ref_meg=with_ref_meg, include=[], exclude=exclude,
                       selection=None, **_PICK_TYPES_DATA_DICT)
 
 
 def _pick_aux_channels(info, exclude='bads'):
-    """Convenience function for picking only auxiliary channels
+    """Pick only auxiliary channels.
 
     Corresponds to EOG, ECG, EMG and BIO
     """
@@ -723,11 +734,9 @@ def _pick_aux_channels(info, exclude='bads'):
 
 
 def _pick_data_or_ica(info):
-    """Convenience function for picking only data or ICA channels."""
-    ch_names = [c['ch_name'] for c in info['chs']]
-    if 'ICA ' in ','.join(ch_names):
+    """Pick only data or ICA channels."""
+    if any(ch_name.startswith('ICA') for ch_name in info['ch_names']):
         picks = pick_types(info, exclude=[], misc=True)
     else:
-        picks = _pick_data_channels(info, exclude=[],
-                                    with_ref_meg=False)
+        picks = _pick_data_channels(info, exclude=[], with_ref_meg=True)
     return picks
