@@ -18,24 +18,103 @@ from .constants import FIFF
 from .pick import pick_types
 from .write import (write_int, write_float, write_string, write_name_list,
                     write_float_matrix, end_block, start_block)
-from ..utils import logger, verbose, warn, deprecated
+from ..utils import logger, verbose, warn
 from ..externals.six import string_types
 
 
 class Projection(dict):
-    """Projection vector
+    """Projection vector.
 
     A basic class to proj a meaningful print for projection vectors.
     """
-    def __repr__(self):
+
+    def __repr__(self):  # noqa: D105
         s = "%s" % self['desc']
         s += ", active : %s" % self['active']
         s += ", n_channels : %s" % self['data']['ncol']
         return "<Projection  |  %s>" % s
 
+    # Can't use copy_ function here b/c of circular import
+    def plot_topomap(self, layout=None, cmap=None, sensors=True,
+                     colorbar=False, res=64, size=1, show=True,
+                     outlines='head', contours=6, image_interp='bilinear',
+                     axes=None, info=None):
+        """Plot topographic maps of SSP projections.
+
+        Parameters
+        ----------
+        layout : None | Layout | list of Layout
+            Layout instance specifying sensor positions (does not need to be
+            specified for Neuromag data). Or a list of Layout if projections
+            are from different sensor types.
+        cmap : matplotlib colormap | (colormap, bool) | 'interactive' | None
+            Colormap to use. If tuple, the first value indicates the colormap to
+            use and the second value is a boolean defining interactivity. In
+            interactive mode (only works if ``colorbar=True``) the colors are
+            adjustable by clicking and dragging the colorbar with left and right
+            mouse button. Left mouse button moves the scale up and down and right
+            mouse button adjusts the range. Hitting space bar resets the range. Up
+            and down arrows can be used to change the colormap. If None (default),
+            'Reds' is used for all positive data, otherwise defaults to 'RdBu_r'.
+            If 'interactive', translates to (None, True).
+        sensors : bool | str
+            Add markers for sensor locations to the plot. Accepts matplotlib plot
+            format string (e.g., 'r+' for red plusses). If True, a circle will be
+            used (via .add_artist). Defaults to True.
+        colorbar : bool
+            Plot a colorbar.
+        res : int
+            The resolution of the topomap image (n pixels along each side).
+        size : scalar
+            Side length of the topomaps in inches (only applies when plotting
+            multiple topomaps at a time).
+        show : bool
+            Show figure if True.
+        outlines : 'head' | 'skirt' | dict | None
+            The outlines to be drawn. If 'head', the default head scheme will be
+            drawn. If 'skirt' the head scheme will be drawn, but sensors are
+            allowed to be plotted outside of the head circle. If dict, each key
+            refers to a tuple of x and y positions, the values in 'mask_pos' will
+            serve as image mask, and the 'autoshrink' (bool) field will trigger
+            automated shrinking of the positions due to points outside the outline.
+            Alternatively, a matplotlib patch object can be passed for advanced
+            masking options, either directly or as a function that returns patches
+            (required for multi-axis plots). If None, nothing will be drawn.
+            Defaults to 'head'.
+        contours : int | array of float
+            The number of contour lines to draw. If 0, no contours will be drawn.
+            When an integer, matplotlib ticker locator is used to find suitable
+            values for the contour thresholds (may sometimes be inaccurate, use
+            array for accuracy). If an array, the values represent the levels for
+            the contours. Defaults to 6.
+        image_interp : str
+            The image interpolation to be used. All matplotlib options are
+            accepted.
+        axes : instance of Axes | list | None
+            The axes to plot to. If list, the list must be a list of Axes of
+            the same length as the number of projectors. If instance of Axes,
+            there must be only one projector. Defaults to None.
+        info : instance of Info | None
+            The measurement information to use to determine the layout.
+            If not None, ``layout`` must be None.
+
+        Returns
+        -------
+        fig : instance of matplotlib figure
+            Figure distributing one image per channel across sensor topography.
+
+        Notes
+        -----
+        .. versionadded:: 0.15.0
+        """  # noqa: E501
+        from ..viz.topomap import plot_projs_topomap
+        return plot_projs_topomap([self], layout, cmap, sensors, colorbar,
+                                  res, size, show, outlines,
+                                  contours, image_interp, axes, info)
+
 
 class ProjMixin(object):
-    """Mixin class for Raw, Evoked, Epochs
+    """Mixin class for Raw, Evoked, Epochs.
 
     Notes
     -----
@@ -59,14 +138,16 @@ class ProjMixin(object):
     apply_proj(). The reason is that you want to reject with projs although
     it's not stored in proj mode.
     """
+
     @property
     def proj(self):
+        """Whether or not projections are active."""
         return (len(self.info['projs']) > 0 and
                 all(p['active'] for p in self.info['projs']))
 
     @verbose
     def add_proj(self, projs, remove_existing=False, verbose=None):
-        """Add SSP projection vectors
+        """Add SSP projection vectors.
 
         Parameters
         ----------
@@ -75,7 +156,9 @@ class ProjMixin(object):
         remove_existing : bool
             Remove the projection vectors currently in the file.
         verbose : bool, str, int, or None
-            If not None, override default verbose level (see mne.verbose).
+            If not None, override default verbose level (see
+            :func:`mne.verbose` and :ref:`Logging documentation <tut_logging>`
+            for more).
 
         Returns
         -------
@@ -105,21 +188,6 @@ class ProjMixin(object):
                                              check_active=False, sort=False)
         return self
 
-    @deprecated('This function is deprecated and will be removed in 0.14. '
-                'Use set_eeg_reference() instead.')
-    def add_eeg_average_proj(self):
-        """Add an average EEG reference projector if one does not exist."""
-        if _needs_eeg_average_ref_proj(self.info):
-            # Don't set as active, since we haven't applied it
-            eeg_proj = make_eeg_average_ref_proj(self.info, activate=False)
-            self.add_proj(eeg_proj)
-        elif self.info.get('custom_ref_applied', False):
-            raise RuntimeError('Cannot add an average EEG reference '
-                               'projection since a custom reference has been '
-                               'applied to the data earlier.')
-
-        return self
-
     def apply_proj(self):
         """Apply the signal space projection (SSP) operators to the data.
 
@@ -148,15 +216,16 @@ class ProjMixin(object):
         self : instance of Raw | Epochs | Evoked
             The instance.
         """
-        from ..epochs import _BaseEpochs
-        from .base import _BaseRaw
+        from ..epochs import BaseEpochs
+        from ..evoked import Evoked
+        from .base import BaseRaw
         if self.info['projs'] is None or len(self.info['projs']) == 0:
             logger.info('No projector specified for this dataset. '
                         'Please consider the method self.add_proj.')
             return self
 
         # Exit delayed mode if you apply proj
-        if isinstance(self, _BaseEpochs) and self._do_delayed_proj:
+        if isinstance(self, BaseEpochs) and self._do_delayed_proj:
             logger.info('Leaving delayed SSP mode.')
             self._do_delayed_proj = False
 
@@ -173,45 +242,46 @@ class ProjMixin(object):
                         ' Doing nothing.')
             return self
         self._projector, self.info = _projector, info
-        if isinstance(self, _BaseRaw):
+        if isinstance(self, (BaseRaw, Evoked)):
             if self.preload:
                 self._data = np.dot(self._projector, self._data)
-        elif isinstance(self, _BaseEpochs):
+        else:  # BaseEpochs
             if self.preload:
                 for ii, e in enumerate(self._data):
                     self._data[ii] = self._project_epoch(e)
             else:
                 self.load_data()  # will automatically apply
-        else:  # Evoked
-            self.data = np.dot(self._projector, self.data)
         logger.info('SSP projectors applied...')
         return self
 
-    def del_proj(self, idx):
-        """Remove SSP projection vector
+    def del_proj(self, idx='all'):
+        """Remove SSP projection vector.
 
         Note: The projection vector can only be removed if it is inactive
               (has not been applied to the data).
 
         Parameters
         ----------
-        idx : int
-            Index of the projector to remove.
+        idx : int | list of int | str
+            Index of the projector to remove. Can also be "all" (default)
+            to remove all projectors.
 
         Returns
         -------
         self : instance of Raw | Epochs | Evoked
         """
-        if self.info['projs'][idx]['active']:
+        if isinstance(idx, string_types) and idx == 'all':
+            idx = list(range(len(self.info['projs'])))
+        idx = np.atleast_1d(np.array(idx, int)).ravel()
+        if any(self.info['projs'][ii]['active'] for ii in idx):
             raise ValueError('Cannot remove projectors that have already '
                              'been applied')
-
-        self.info['projs'].pop(idx)
-
+        self.info['projs'] = [p for pi, p in enumerate(self.info['projs'])
+                              if pi not in idx]
         return self
 
     def plot_projs_topomap(self, ch_type=None, layout=None, axes=None):
-        """Plot SSP vector
+        """Plot SSP vector.
 
         Parameters
         ----------
@@ -259,8 +329,7 @@ class ProjMixin(object):
 
 
 def _proj_equal(a, b, check_active=True):
-    """ Test if two projectors are equal """
-
+    """Test if two projectors are equal."""
     equal = ((a['active'] == b['active'] or not check_active) and
              a['kind'] == b['kind'] and
              a['desc'] == b['desc'] and
@@ -283,11 +352,12 @@ def _read_proj(fid, node, verbose=None):
     node : tree node
         The node of the tree where to look.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
-    projs: dict
+    projs : list of Projection
         The list of projections.
     """
     projs = list()
@@ -434,8 +504,22 @@ def _write_proj(fid, projs):
 
 ###############################################################################
 # Utils
+
+def _check_projs(projs, copy=True):
+    """Check that projs is a list of Projection."""
+    if not isinstance(projs, (list, tuple)):
+        raise TypeError('projs must be a list or tuple, got %s'
+                        % (type(projs),))
+    for pi, p in enumerate(projs):
+        if not isinstance(p, Projection):
+            raise TypeError('All entries in projs list must be Projection '
+                            'instances, but projs[%d] is type %s'
+                            % (pi, type(p)))
+    return deepcopy(projs) if copy else projs
+
+
 def make_projector(projs, ch_names, bads=(), include_active=True):
-    """Create an SSP operator from SSP projection vectors
+    """Create an SSP operator from SSP projection vectors.
 
     Parameters
     ----------
@@ -465,7 +549,7 @@ def make_projector(projs, ch_names, bads=(), include_active=True):
 
 def _make_projector(projs, ch_names, bads=(), include_active=True,
                     inplace=False):
-    """Helper to subselect projs based on ch_names and bads
+    """Subselect projs based on ch_names and bads.
 
     Use inplace=True mode to modify ``projs`` inplace so that no
     warning will be raised next time projectors are constructed with
@@ -495,6 +579,7 @@ def _make_projector(projs, ch_names, bads=(), include_active=True,
     vecs = np.zeros((nchan, nvec))
     nvec = 0
     nonzero = 0
+    bads = set(bads)
     for k, p in enumerate(projs):
         if not p['active'] or include_active:
             if (len(p['data']['col_names']) !=
@@ -506,8 +591,9 @@ def _make_projector(projs, ch_names, bads=(), include_active=True,
             # the projection vectors omitting bad channels
             sel = []
             vecsel = []
+            p_set = set(p['data']['col_names'])  # faster membership access
             for c, name in enumerate(ch_names):
-                if name in p['data']['col_names'] and name not in bads:
+                if name not in bads and name in p_set:
                     sel.append(c)
                     vecsel.append(p['data']['col_names'].index(name))
 
@@ -521,8 +607,11 @@ def _make_projector(projs, ch_names, bads=(), include_active=True,
             for v in range(p['data']['nrow']):
                 psize = sqrt(np.sum(this_vecs[:, v] * this_vecs[:, v]))
                 if psize > 0:
-                    orig_n = p['data']['data'].shape[1]
-                    if len(vecsel) < 0.9 * orig_n and not inplace:
+                    orig_n = p['data']['data'].any(axis=0).sum()
+                    # Average ref still works if channels are removed
+                    if len(vecsel) < 0.9 * orig_n and not inplace and \
+                            (p['kind'] != FIFF.FIFFV_MNE_PROJ_ITEM_EEG_AVREF or
+                             len(vecsel) == 1):
                         warn('Projection vector "%s" has magnitude %0.2f '
                              '(should be unity), applying projector with '
                              '%s/%s of the original channels available may '
@@ -559,7 +648,7 @@ def _make_projector(projs, ch_names, bads=(), include_active=True,
 
 
 def _normalize_proj(info):
-    """Helper to normalize proj after subselection to avoid warnings
+    """Normalize proj after subselection to avoid warnings.
 
     This is really only useful for tests, and might not be needed
     eventually if we change or improve our handling of projectors
@@ -571,7 +660,7 @@ def _normalize_proj(info):
 
 
 def make_projector_info(info, include_active=True):
-    """Make an SSP operator using the measurement info
+    """Make an SSP operator using the measurement info.
 
     Calls make_projector on good channels.
 
@@ -596,7 +685,7 @@ def make_projector_info(info, include_active=True):
 
 @verbose
 def activate_proj(projs, copy=True, verbose=None):
-    """Set all projections to active
+    """Set all projections to active.
 
     Useful before passing them to make_projector.
 
@@ -607,7 +696,8 @@ def activate_proj(projs, copy=True, verbose=None):
     copy : bool
         Modify projs in place or operate on a copy.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -628,7 +718,7 @@ def activate_proj(projs, copy=True, verbose=None):
 
 @verbose
 def deactivate_proj(projs, copy=True, verbose=None):
-    """Set all projections to inactive
+    """Set all projections to inactive.
 
     Useful before saving raw data without projectors applied.
 
@@ -639,7 +729,8 @@ def deactivate_proj(projs, copy=True, verbose=None):
     copy : bool
         Modify projs in place or operate on a copy.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -660,7 +751,7 @@ def deactivate_proj(projs, copy=True, verbose=None):
 
 @verbose
 def make_eeg_average_ref_proj(info, activate=True, verbose=None):
-    """Create an EEG average reference SSP projection vector
+    """Create an EEG average reference SSP projection vector.
 
     Parameters
     ----------
@@ -669,7 +760,8 @@ def make_eeg_average_ref_proj(info, activate=True, verbose=None):
     activate : bool
         If True projections are activated.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -677,9 +769,10 @@ def make_eeg_average_ref_proj(info, activate=True, verbose=None):
         The SSP/PCA projector.
     """
     if info.get('custom_ref_applied', False):
-        raise RuntimeError('Cannot add an average EEG reference projection '
-                           'since a custom reference has been applied to the '
-                           'data earlier.')
+        raise RuntimeError('A custom reference has been applied to the '
+                           'data earlier. Please use the '
+                           'mne.io.set_eeg_reference function to move from '
+                           'one EEG reference to another.')
 
     logger.info("Adding average EEG reference projection.")
     eeg_sel = pick_types(info, meg=False, eeg=True, ref_meg=False,
@@ -703,7 +796,7 @@ def make_eeg_average_ref_proj(info, activate=True, verbose=None):
 
 
 def _has_eeg_average_ref_proj(projs, check_active=False):
-    """Determine if a list of projectors has an average EEG ref
+    """Determine if a list of projectors has an average EEG ref.
 
     Optionally, set check_active=True to additionally check if the CAR
     has already been applied.
@@ -717,7 +810,7 @@ def _has_eeg_average_ref_proj(projs, check_active=False):
 
 
 def _needs_eeg_average_ref_proj(info):
-    """Determine if the EEG needs an averge EEG reference
+    """Determine if the EEG needs an averge EEG reference.
 
     This returns True if no custom reference has been applied and no average
     reference projection is present in the list of projections.
@@ -731,7 +824,7 @@ def _needs_eeg_average_ref_proj(info):
 
 @verbose
 def setup_proj(info, add_eeg_ref=True, activate=True, verbose=None):
-    """Set up projection for Raw and Epochs
+    """Set up projection for Raw and Epochs.
 
     Parameters
     ----------
@@ -743,7 +836,8 @@ def setup_proj(info, add_eeg_ref=True, activate=True, verbose=None):
     activate : bool
         If True projections are activated.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -776,7 +870,7 @@ def setup_proj(info, add_eeg_ref=True, activate=True, verbose=None):
 
 
 def _uniquify_projs(projs, check_active=True, sort=True):
-    """Aux function"""
+    """Make unique projs."""
     final_projs = []
     for proj in projs:  # flatten
         if not any(_proj_equal(p, proj, check_active) for p in final_projs):
@@ -785,7 +879,7 @@ def _uniquify_projs(projs, check_active=True, sort=True):
     my_count = count(len(final_projs))
 
     def sorter(x):
-        """sort in a nice way"""
+        """Sort in a nice way."""
         digits = [s for s in x['desc'] if s.isdigit()]
         if digits:
             sort_idx = int(digits[-1])
