@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-
+import warnings
 import os.path as op
 
 from nose.tools import assert_false, assert_equal, assert_raises, assert_true
@@ -17,6 +17,8 @@ from mne.io.meas_info import (Info, create_info, _write_dig_points,
 from mne.utils import _TempDir, run_tests_if_main
 from mne.channels.montage import read_montage, read_dig_montage
 
+warnings.simplefilter("always")  # ensure we can verify expected warnings
+
 base_dir = op.join(op.dirname(__file__), 'data')
 fiducials_fname = op.join(base_dir, 'fsaverage-fiducials.fif')
 raw_fname = op.join(base_dir, 'test_raw.fif')
@@ -84,6 +86,7 @@ def test_make_info():
     assert_array_equal(ch_pos, m.pos)
     idents = [p['ident'] for p in info['dig']]
     assert_true(FIFF.FIFFV_POINT_NASION in idents)
+    assert_array_equal(info['meas_date'], [0, 0])
 
 
 def test_fiducials_io():
@@ -110,17 +113,15 @@ def test_fiducials_io():
 
 def test_info():
     """Test info object."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     event_id, tmin, tmax = 1, -0.2, 0.5
     events = read_events(event_name)
     event_id = int(events[0, 2])
     epochs = Epochs(raw, events[:1], event_id, tmin, tmax, picks=None,
-                    baseline=(None, 0), add_eeg_ref=False)
+                    baseline=(None, 0))
 
     evoked = epochs.average()
 
-    events = read_events(event_name)
-
     # Test subclassing was successful.
     info = Info(a=7, b='aaaaa')
     assert_true('a' in info)
@@ -134,6 +135,7 @@ def test_info():
         info_str = '%s' % obj.info
         assert_equal(len(info_str.split('\n')), len(obj.info.keys()) + 2)
         assert_true(all(k in info_str for k in obj.info.keys()))
+        assert '2002-12-03 19:01:10 GMT' in repr(obj.info), repr(obj.info)
 
     # Test read-only fields
     info = raw.info.copy()
@@ -209,20 +211,20 @@ def test_io_dig_points():
 
 def test_make_dig_points():
     """Test application of Polhemus HSP to info."""
-    dig_points = _read_dig_points(hsp_fname)
+    extra_points = _read_dig_points(hsp_fname)
     info = create_info(ch_names=['Test Ch'], sfreq=1000., ch_types=None)
     assert_false(info['dig'])
 
-    info['dig'] = _make_dig_points(dig_points=dig_points)
+    info['dig'] = _make_dig_points(extra_points=extra_points)
     assert_true(info['dig'])
     assert_allclose(info['dig'][0]['r'], [-.10693, .09980, .06881])
 
-    dig_points = _read_dig_points(elp_fname)
-    nasion, lpa, rpa = dig_points[:3]
+    elp_points = _read_dig_points(elp_fname)
+    nasion, lpa, rpa = elp_points[:3]
     info = create_info(ch_names=['Test Ch'], sfreq=1000., ch_types=None)
     assert_false(info['dig'])
 
-    info['dig'] = _make_dig_points(nasion, lpa, rpa, dig_points[3:], None)
+    info['dig'] = _make_dig_points(nasion, lpa, rpa, elp_points[3:], None)
     assert_true(info['dig'])
     idx = [d['ident'] for d in info['dig']].index(FIFF.FIFFV_POINT_NASION)
     assert_array_equal(info['dig'][idx]['r'],
@@ -231,9 +233,9 @@ def test_make_dig_points():
     assert_raises(ValueError, _make_dig_points, None, lpa[:2])
     assert_raises(ValueError, _make_dig_points, None, None, rpa[:2])
     assert_raises(ValueError, _make_dig_points, None, None, None,
-                  dig_points[:, :2])
+                  elp_points[:, :2])
     assert_raises(ValueError, _make_dig_points, None, None, None, None,
-                  dig_points[:, :2])
+                  elp_points[:, :2])
 
 
 def test_redundant():
@@ -315,6 +317,13 @@ def test_check_consistency():
     info2['lowpass'] = 'foo'
     assert_raises(ValueError, info2._check_consistency)
 
+    info2 = info.copy()
+    info2['filename'] = 'foo'
+    with warnings.catch_warnings(record=True) as w:
+        info2._check_consistency()
+    assert_equal(len(w), 1)
+    assert_true(all('filename' in str(ww.message) for ww in w))
+
     # Silent type conversion to float
     info2 = info.copy()
     info2['sfreq'] = 1
@@ -330,13 +339,20 @@ def test_check_consistency():
     info2['chs'][2]['ch_name'] = 'b'
     assert_raises(RuntimeError, info2._check_consistency)
 
+    # Duplicates appended with running numbers
+    with warnings.catch_warnings(record=True) as w:
+        info3 = create_info(ch_names=['a', 'b', 'b', 'c', 'b'], sfreq=1000.)
+    assert_equal(len(w), 1)
+    assert_true(all('Channel names are not' in '%s' % ww.message for ww in w))
+    assert_array_equal(info3['ch_names'], ['a', 'b-0', 'b-1', 'c', 'b-2'])
+
 
 def test_anonymize():
-    """Checks that sensitive information can be anonymized."""
+    """Test that sensitive information can be anonymized."""
     assert_raises(ValueError, anonymize_info, 'foo')
 
     # Fake some subject data
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     raw.info['subject_info'] = dict(id=1, his_id='foobar', last_name='bar',
                                     first_name='bar', birthday=(1987, 4, 8),
                                     sex=0, hand=1)
@@ -345,7 +361,7 @@ def test_anonymize():
     orig_meas_id = raw.info['meas_id']['secs']
     # Test instance method
     events = read_events(event_name)
-    epochs = Epochs(raw, events[:1], 2, 0., 0.1, add_eeg_ref=False)
+    epochs = Epochs(raw, events[:1], 2, 0., 0.1)
     for inst in [raw, epochs]:
         assert_true('subject_info' in inst.info.keys())
         assert_true(inst.info['subject_info'] is not None)
@@ -356,19 +372,24 @@ def test_anonymize():
         assert_true('subject_info' not in inst.info.keys())
         assert_equal(inst.info['file_id']['secs'], 0)
         assert_equal(inst.info['meas_id']['secs'], 0)
-        assert_equal(inst.info['meas_date'], [0, 0])
+        assert_array_equal(inst.info['meas_date'], [0, 0])
 
     # When we write out with raw.save, these get overwritten with the
     # new save time
     tempdir = _TempDir()
     out_fname = op.join(tempdir, 'test_subj_info_raw.fif')
     raw.save(out_fname, overwrite=True)
-    raw = read_raw_fif(out_fname, add_eeg_ref=False)
+    raw = read_raw_fif(out_fname)
     assert_true(raw.info.get('subject_info') is None)
     assert_array_equal(raw.info['meas_date'], [0, 0])
     # XXX mne.io.write.write_id necessarily writes secs
     assert_true(raw.info['file_id']['secs'] != orig_file_id)
     assert_true(raw.info['meas_id']['secs'] != orig_meas_id)
 
+    # Test no error for incomplete info
+    info = raw.info.copy()
+    info.pop('file_id')
+    anonymize_info(info)
+
 
 run_tests_if_main()
