@@ -49,8 +49,8 @@ def _test_raw_reader(reader, test_preloading=True, **kwargs):
         other_raws = [reader(preload=buffer_fname, **kwargs),
                       reader(preload=False, **kwargs)]
         for sl_time in slices:
+            data1, times1 = raw[picks, sl_time]
             for other_raw in other_raws:
-                data1, times1 = raw[picks, sl_time]
                 data2, times2 = other_raw[picks, sl_time]
                 assert_allclose(data1, data2)
                 assert_allclose(times1, times2)
@@ -63,8 +63,9 @@ def _test_raw_reader(reader, test_preloading=True, **kwargs):
 
     # Test saving and reading
     out_fname = op.join(tempdir, 'test_raw.fif')
+    raw = concatenate_raws([raw])
     raw.save(out_fname, tmax=raw.times[-1], overwrite=True, buffer_size_sec=1)
-    raw3 = read_raw_fif(out_fname, add_eeg_ref=False)
+    raw3 = read_raw_fif(out_fname)
     assert_equal(set(raw.info.keys()), set(raw3.info.keys()))
     assert_allclose(raw3[0:20][0], full_data[0:20], rtol=1e-6,
                     atol=1e-20)  # atol is very small but > 0
@@ -84,6 +85,9 @@ def _test_raw_reader(reader, test_preloading=True, **kwargs):
     assert_equal(concat_raw.n_times, 2 * raw.n_times)
     assert_equal(concat_raw.first_samp, first_samp)
     assert_equal(concat_raw.last_samp - last_samp + first_samp, last_samp + 1)
+    idx = np.where(concat_raw.annotations.description == 'BAD boundary')[0]
+    assert_array_almost_equal([(last_samp - first_samp) / raw.info['sfreq']],
+                              concat_raw.annotations.onset[idx], decimal=2)
     return raw
 
 
@@ -107,10 +111,10 @@ def _test_concat(reader, *args):
             for last_preload in (True, False):
                 t_crops = raw.times[np.argmin(np.abs(raw.times - 0.5)) +
                                     [0, 1]]
-                raw1 = raw.copy().crop(0, t_crops[0], copy=False)
+                raw1 = raw.copy().crop(0, t_crops[0])
                 if preloads[0]:
                     raw1.load_data()
-                raw2 = raw.copy().crop(t_crops[1], None, copy=False)
+                raw2 = raw.copy().crop(t_crops[1], None)
                 if preloads[1]:
                     raw2.load_data()
                 raw1.append(raw2)
@@ -124,7 +128,7 @@ def test_time_index():
     """Test indexing of raw times."""
     raw_fname = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
                         'data', 'test_raw.fif')
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
 
     # Test original (non-rounding) indexing behavior
     orig_inds = raw.time_as_index(raw.times)
