@@ -11,13 +11,13 @@ import numpy as np
 from nose.tools import assert_true, assert_equal, assert_raises
 from numpy.testing import assert_array_equal, assert_allclose
 
-from mne import (pick_channels, pick_types, Evoked, Epochs, read_events,
+from mne import (pick_channels, pick_types, Epochs, read_events,
                  set_eeg_reference, set_bipolar_reference,
                  add_reference_channels)
-from mne.epochs import _BaseEpochs
+from mne.epochs import BaseEpochs
 from mne.io import read_raw_fif
 from mne.io.constants import FIFF
-from mne.io.proj import _has_eeg_average_ref_proj
+from mne.io.proj import _has_eeg_average_ref_proj, Projection
 from mne.io.reference import _apply_reference
 from mne.datasets import testing
 from mne.utils import run_tests_if_main
@@ -41,15 +41,12 @@ def _test_reference(raw, reref, ref_data, ref_from):
     picks_ref = [raw.ch_names.index(ch) for ch in ref_from]
 
     # Get data
-    if isinstance(raw, Evoked):
-        _data = raw.data
-        _reref = reref.data
-    else:
-        _data = raw._data
-        _reref = reref._data
+    _data = raw._data
+    _reref = reref._data
 
     # Check that the ref has been properly computed
-    assert_array_equal(ref_data, _data[..., picks_ref, :].mean(-2))
+    if ref_data is not None:
+        assert_array_equal(ref_data, _data[..., picks_ref, :].mean(-2))
 
     # Get the raw EEG data and other channel data
     raw_eeg_data = _data[..., picks_eeg, :]
@@ -59,21 +56,22 @@ def _test_reference(raw, reref, ref_data, ref_from):
     reref_eeg_data = _reref[..., picks_eeg, :]
     reref_other_data = _reref[..., picks_other, :]
 
-    # Undo rereferencing of EEG channels
-    if isinstance(raw, _BaseEpochs):
-        unref_eeg_data = reref_eeg_data + ref_data[:, np.newaxis, :]
-    else:
-        unref_eeg_data = reref_eeg_data + ref_data
-
-    # Check that both EEG data and other data is the same
-    assert_allclose(raw_eeg_data, unref_eeg_data, 1e-6, atol=1e-15)
+    # Check that non-EEG channels are untouched
     assert_allclose(raw_other_data, reref_other_data, 1e-6, atol=1e-15)
 
+    # Undo rereferencing of EEG channels if possible
+    if ref_data is not None:
+        if isinstance(raw, BaseEpochs):
+            unref_eeg_data = reref_eeg_data + ref_data[:, np.newaxis, :]
+        else:
+            unref_eeg_data = reref_eeg_data + ref_data
+        assert_allclose(raw_eeg_data, unref_eeg_data, 1e-6, atol=1e-15)
+
 
 @testing.requires_testing_data
 def test_apply_reference():
     """Test base function for rereferencing."""
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
 
     # Rereference raw data by creating a copy of original data
     reref, ref_data = _apply_reference(
@@ -84,20 +82,20 @@ def test_apply_reference():
     # The CAR reference projection should have been removed by the function
     assert_true(not _has_eeg_average_ref_proj(reref.info['projs']))
 
-    # Test that disabling the reference does not break anything
-    reref, ref_data = _apply_reference(raw, [])
-    assert_array_equal(raw._data, reref._data)
-
     # Test that data is modified in place when copy=False
     reref, ref_data = _apply_reference(raw, ['EEG 001', 'EEG 002'])
     assert_true(raw is reref)
 
+    # Test that disabling the reference does not change anything
+    reref, ref_data = _apply_reference(raw.copy(), [])
+    assert_array_equal(raw._data, reref._data)
+
     # Test re-referencing Epochs object
-    raw = read_raw_fif(fif_fname, preload=False, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=False)
     events = read_events(eve_fname)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, add_eeg_ref=False)
+                    picks=picks_eeg, preload=True)
     reref, ref_data = _apply_reference(
         epochs.copy(), ref_from=['EEG 001', 'EEG 002'])
     assert_true(reref.info['custom_ref_applied'])
@@ -110,28 +108,64 @@ def test_apply_reference():
     assert_true(reref.info['custom_ref_applied'])
     _test_reference(evoked, reref, ref_data, ['EEG 001', 'EEG 002'])
 
-    # Test invalid input
-    raw_np = read_raw_fif(fif_fname, preload=False, add_eeg_ref=False)
+    # Referencing needs data to be preloaded
+    raw_np = read_raw_fif(fif_fname, preload=False)
     assert_raises(RuntimeError, _apply_reference, raw_np, ['EEG 001'])
 
+    # Test having inactive SSP projections that deal with channels involved
+    # during re-referencing
+    raw = read_raw_fif(fif_fname, preload=True)
+    raw.add_proj(
+        Projection(
+            active=False,
+            data=dict(
+                col_names=['EEG 001', 'EEG 002'],
+                row_names=None,
+                data=np.array([[1, 1]]),
+                ncol=2,
+                nrow=1
+            ),
+            desc='test',
+            kind=1,
+        )
+    )
+    # Projection concerns channels mentioned in projector
+    assert_raises(RuntimeError, _apply_reference, raw, ['EEG 001'])
+
+    # Projection does not concern channels mentioned in projector, no error
+    _apply_reference(raw, ['EEG 003'], ['EEG 004'])
+
 
 @testing.requires_testing_data
 def test_set_eeg_reference():
     """Test rereference eeg data."""
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     raw.info['projs'] = []
 
-    # Test setting an average reference
+    # Test setting an average reference projection
     assert_true(not _has_eeg_average_ref_proj(raw.info['projs']))
-    reref, ref_data = set_eeg_reference(raw)
+    reref, ref_data = set_eeg_reference(raw, projection=True)
     assert_true(_has_eeg_average_ref_proj(reref.info['projs']))
+    assert_true(not reref.info['projs'][0]['active'])
     assert_true(ref_data is None)
+    reref.apply_proj()
+    eeg_chans = [raw.ch_names[ch]
+                 for ch in pick_types(raw.info, meg=False, eeg=True)]
+    _test_reference(raw, reref, ref_data,
+                    [ch for ch in eeg_chans if ch not in raw.info['bads']])
 
     # Test setting an average reference when one was already present
-    with warnings.catch_warnings(record=True):  # weight tables
-        reref, ref_data = set_eeg_reference(raw, copy=False)
+    with warnings.catch_warnings(record=True):
+        reref, ref_data = set_eeg_reference(raw, copy=False, projection=True)
     assert_true(ref_data is None)
 
+    # Test setting an average reference on non-preloaded data
+    raw_nopreload = read_raw_fif(fif_fname, preload=False)
+    raw_nopreload.info['projs'] = []
+    reref, ref_data = set_eeg_reference(raw_nopreload, projection=True)
+    assert_true(_has_eeg_average_ref_proj(reref.info['projs']))
+    assert_true(not reref.info['projs'][0]['active'])
+
     # Rereference raw data by creating a copy of original data
     reref, ref_data = set_eeg_reference(raw, ['EEG 001', 'EEG 002'], copy=True)
     assert_true(reref.info['custom_ref_applied'])
@@ -142,11 +176,60 @@ def test_set_eeg_reference():
                                         copy=False)
     assert_true(raw is reref)
 
+    # Test moving from custom to average reference
+    reref, ref_data = set_eeg_reference(raw, ['EEG 001', 'EEG 002'])
+    reref, _ = set_eeg_reference(reref, projection=True)
+    assert_true(_has_eeg_average_ref_proj(reref.info['projs']))
+    assert_equal(reref.info['custom_ref_applied'], False)
+
+    # When creating an average reference fails, make sure the
+    # custom_ref_applied flag remains untouched.
+    reref = raw.copy()
+    reref.info['custom_ref_applied'] = True
+    reref.pick_types(eeg=False)  # Cause making average ref fail
+    assert_raises(ValueError, set_eeg_reference, reref, projection=True)
+    assert_true(reref.info['custom_ref_applied'])
+
+    # Test moving from average to custom reference
+    reref, ref_data = set_eeg_reference(raw, projection=True)
+    reref, _ = set_eeg_reference(reref, ['EEG 001', 'EEG 002'])
+    assert_true(not _has_eeg_average_ref_proj(reref.info['projs']))
+    assert_equal(reref.info['custom_ref_applied'], True)
+
+    # Test that disabling the reference does not change anything
+    reref, _ = set_eeg_reference(raw, [])
+    assert_array_equal(raw._data, reref._data)
+
+    # make sure ref_channels=[] removes average reference projectors
+    reref, _ = set_eeg_reference(raw, 'average', projection=True)
+    assert_true(_has_eeg_average_ref_proj(reref.info['projs']))
+    reref, _ = set_eeg_reference(reref, [])
+    assert_true(not _has_eeg_average_ref_proj(reref.info['projs']))
+
+    # Test that average reference gives identical results when calculated
+    # via SSP projection (projection=True) or directly (projection=False)
+    raw.info['projs'] = []
+    reref_1, _ = set_eeg_reference(raw.copy(), projection=True)
+    reref_1.apply_proj()
+    reref_2, _ = set_eeg_reference(raw.copy(), projection=False)
+    assert_allclose(reref_1._data, reref_2._data, rtol=1e-6, atol=1e-15)
+
+    # Test average reference without projection
+    reref, ref_data = set_eeg_reference(raw.copy(), ref_channels="average",
+                                        projection=False)
+    _test_reference(raw, reref, ref_data, eeg_chans)
+
+    # projection=True only works for ref_channels='average'
+    assert_raises(ValueError, set_eeg_reference, raw, [], True, True)
+    assert_raises(ValueError, set_eeg_reference, raw, ['EEG 001'], True, True)
+
 
 @testing.requires_testing_data
 def test_set_bipolar_reference():
     """Test bipolar referencing."""
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
+    raw.apply_proj()
+
     reref = set_bipolar_reference(raw, 'EEG 001', 'EEG 002', 'bipolar',
                                   {'kind': FIFF.FIFFV_EOG_CH,
                                    'extra': 'some extra value'})
@@ -181,6 +264,13 @@ def test_set_bipolar_reference():
     reref = set_bipolar_reference(raw, 'EEG 001', 'EEG 002')
     assert_true('EEG 001-EEG 002' in reref.ch_names)
 
+    # Minimalist call with twice the same anode
+    reref = set_bipolar_reference(raw,
+                                  ['EEG 001', 'EEG 001', 'EEG 002'],
+                                  ['EEG 002', 'EEG 003', 'EEG 003'])
+    assert_true('EEG 001-EEG 002' in reref.ch_names)
+    assert_true('EEG 001-EEG 003' in reref.ch_names)
+
     # Set multiple references at once
     reref = set_bipolar_reference(
         raw,
@@ -199,9 +289,10 @@ def test_set_bipolar_reference():
     # Test creating a bipolar reference that doesn't involve EEG channels:
     # it should not set the custom_ref_applied flag
     reref = set_bipolar_reference(raw, 'MEG 0111', 'MEG 0112',
-                                  ch_info={'kind': FIFF.FIFFV_MEG_CH})
+                                  ch_info={'kind': FIFF.FIFFV_MEG_CH},
+                                  verbose='error')
     assert_true(not reref.info['custom_ref_applied'])
-    assert_true('MEG 0111-MEG 0112' in reref.ch_names)
+    assert_true('MEG 0111-MEG 0112'[:15] in reref.ch_names)
 
     # Test a battery of invalid inputs
     assert_raises(ValueError, set_bipolar_reference, raw,
@@ -234,7 +325,7 @@ def _check_channel_names(inst, ref_names):
 @testing.requires_testing_data
 def test_add_reference():
     """Test adding a reference."""
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     # check if channel already exists
     assert_raises(ValueError, add_reference_channels,
@@ -261,7 +352,7 @@ def test_add_reference():
     assert_array_equal(ref_data, 0)
 
     # add reference channel to Raw when no digitization points exist
-    raw = read_raw_fif(fif_fname, add_eeg_ref=False).crop(0, 1).load_data()
+    raw = read_raw_fif(fif_fname).crop(0, 1).load_data()
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     del raw.info['dig']
 
@@ -296,21 +387,18 @@ def test_add_reference():
     assert_array_equal(ref_data, 0)
 
     # add reference channel to epochs
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     events = read_events(eve_fname)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, add_eeg_ref=False)
+                    picks=picks_eeg, preload=True)
     # default: proj=True, after which adding a Ref channel is prohibited
     assert_raises(RuntimeError, add_reference_channels, epochs, 'Ref')
 
     # create epochs in delayed mode, allowing removal of CAR when re-reffing
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, proj='delayed',
-                    add_eeg_ref=False)
+                    picks=picks_eeg, preload=True, proj='delayed')
     epochs_ref = add_reference_channels(epochs, 'Ref', copy=True)
-    # CAR after custom reference is an Error
-    assert_raises(RuntimeError, epochs_ref.set_eeg_reference)
 
     assert_equal(epochs_ref._data.shape[1], epochs._data.shape[1] + 1)
     _check_channel_names(epochs_ref, 'Ref')
@@ -322,13 +410,12 @@ def test_add_reference():
                        epochs_ref.get_data()[:, picks_eeg, :])
 
     # add two reference channels to epochs
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     events = read_events(eve_fname)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     # create epochs in delayed mode, allowing removal of CAR when re-reffing
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, proj='delayed',
-                    add_eeg_ref=False)
+                    picks=picks_eeg, preload=True, proj='delayed')
     with warnings.catch_warnings(record=True):  # multiple set zero
         epochs_ref = add_reference_channels(epochs, ['M1', 'M2'], copy=True)
     assert_equal(epochs_ref._data.shape[1], epochs._data.shape[1] + 2)
@@ -344,13 +431,12 @@ def test_add_reference():
                        epochs_ref.get_data()[:, picks_eeg, :])
 
     # add reference channel to evoked
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     events = read_events(eve_fname)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     # create epochs in delayed mode, allowing removal of CAR when re-reffing
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, proj='delayed',
-                    add_eeg_ref=False)
+                    picks=picks_eeg, preload=True, proj='delayed')
     evoked = epochs.average()
     evoked_ref = add_reference_channels(evoked, 'Ref', copy=True)
     assert_equal(evoked_ref.data.shape[0], evoked.data.shape[0] + 1)
@@ -363,13 +449,12 @@ def test_add_reference():
                        evoked_ref.data[picks_eeg, :])
 
     # add two reference channels to evoked
-    raw = read_raw_fif(fif_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(fif_fname, preload=True)
     events = read_events(eve_fname)
     picks_eeg = pick_types(raw.info, meg=False, eeg=True)
     # create epochs in delayed mode, allowing removal of CAR when re-reffing
     epochs = Epochs(raw, events=events, event_id=1, tmin=-0.2, tmax=0.5,
-                    picks=picks_eeg, preload=True, proj='delayed',
-                    add_eeg_ref=False)
+                    picks=picks_eeg, preload=True, proj='delayed')
     evoked = epochs.average()
     with warnings.catch_warnings(record=True):  # multiple set zero
         evoked_ref = add_reference_channels(evoked, ['M1', 'M2'], copy=True)
@@ -384,8 +469,9 @@ def test_add_reference():
                        evoked_ref.data[picks_eeg, :])
 
     # Test invalid inputs
-    raw_np = read_raw_fif(fif_fname, preload=False, add_eeg_ref=False)
+    raw_np = read_raw_fif(fif_fname, preload=False)
     assert_raises(RuntimeError, add_reference_channels, raw_np, ['Ref'])
     assert_raises(ValueError, add_reference_channels, raw, 1)
 
+
 run_tests_if_main()
