@@ -16,7 +16,7 @@ from .inverse import (combine_xyz, prepare_inverse_operator, _assemble_kernel,
                       _pick_channels_inverse_operator, _check_method,
                       _check_ori, _subject_from_inverse)
 from ..parallel import parallel_func
-from ..utils import logger, verbose, warn
+from ..utils import logger, verbose, warn, _freqs_dep
 from ..externals import six
 
 
@@ -24,7 +24,7 @@ def _prepare_source_params(inst, inverse_operator, label=None,
                            lambda2=1.0 / 9.0, method="dSPM", nave=1,
                            decim=1, pca=True, pick_ori="normal",
                            prepared=False, verbose=None):
-    """Prepare inverse operator and params for spectral / TFR analysis"""
+    """Prepare inverse operator and params for spectral / TFR analysis."""
     if not prepared:
         inv = prepare_inverse_operator(inverse_operator, nave, lambda2, method)
     else:
@@ -42,7 +42,7 @@ def _prepare_source_params(inst, inverse_operator, label=None,
     #   This does all the data transformations to compute the weights for the
     #   eigenleads
     #
-    K, noise_norm, vertno = _assemble_kernel(inv, label, method, pick_ori)
+    K, noise_norm, vertno, _ = _assemble_kernel(inv, label, method, pick_ori)
 
     if pca:
         U, s, Vh = linalg.svd(K, full_matrices=False)
@@ -64,7 +64,7 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
                               baseline=None, baseline_mode='logratio',
                               pca=True, n_jobs=1, prepared=False,
                               verbose=None):
-    """Compute source space induced power in given frequency bands
+    """Compute source space induced power in given frequency bands.
 
     Parameters
     ----------
@@ -96,11 +96,20 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
         If a is None the beginning of the data is used and if b is None then b
         is set to the end of the interval. If baseline is equal to (None, None)
         all the time interval is used.
-    baseline_mode : None | 'logratio' | 'zscore'
-        Do baseline correction with ratio (power is divided by mean
-        power during baseline) or zscore (power is divided by standard
-        deviation of power during baseline after subtracting the mean,
-        power = [power - mean(power_baseline)] / std(power_baseline)).
+    baseline_mode : 'mean' | 'ratio' | 'logratio' | 'percent' | 'zscore' | 'zlogratio' | None
+        Perform baseline correction by
+
+          - subtracting the mean baseline power ('mean')
+          - dividing by the mean baseline power ('ratio')
+          - dividing by the mean baseline power and taking the log ('logratio')
+          - subtracting the mean baseline power followed by dividing by the
+            mean baseline power ('percent')
+          - subtracting the mean baseline power and dividing by the standard
+            deviation of the baseline power ('zscore')
+          - dividing by the mean baseline power, taking the log, and dividing
+            by the standard deviation of the baseline power ('zlogratio')
+
+        If None no baseline correction is applied.
     pca : bool
         If True, the true dimension of data is estimated before running
         the time-frequency transforms. It reduces the computation times
@@ -110,20 +119,21 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
     prepared : bool
         If True, do not call `prepare_inverse_operator`.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
     stcs : dict with a SourceEstimate (or VolSourceEstimate) for each band
         The estimated source space induced power estimates.
-    """
-    method = _check_method(method)
+    """  # noqa: E501
+    _check_method(method)
 
-    frequencies = np.concatenate([np.arange(band[0], band[1] + df / 2.0, df)
-                                 for _, band in six.iteritems(bands)])
+    freqs = np.concatenate([np.arange(band[0], band[1] + df / 2.0, df)
+                            for _, band in six.iteritems(bands)])
 
     powers, _, vertno = _source_induced_power(
-        epochs, inverse_operator, frequencies, label=label, lambda2=lambda2,
+        epochs, inverse_operator, freqs, label=label, lambda2=lambda2,
         method=method, nave=nave, n_cycles=n_cycles, decim=decim,
         use_fft=use_fft, pca=pca, n_jobs=n_jobs, with_plv=False,
         prepared=prepared)
@@ -132,9 +142,9 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
     stcs = dict()
 
     subject = _subject_from_inverse(inverse_operator)
-    _log_rescale(baseline, baseline_mode)
+    _log_rescale(baseline, baseline_mode)  # for early failure
     for name, band in six.iteritems(bands):
-        idx = [k for k, f in enumerate(frequencies) if band[0] <= f <= band[1]]
+        idx = [k for k, f in enumerate(freqs) if band[0] <= f <= band[1]]
 
         # average power in band + mean over epochs
         power = np.mean(powers[:, idx, :], axis=1)
@@ -155,7 +165,7 @@ def source_band_induced_power(epochs, inverse_operator, bands, label=None,
 
 
 def _prepare_tfr(data, decim, pick_ori, Ws, K, source_ori):
-    """Aux function to prepare TFR source localization"""
+    """Prepare TFR source localization."""
     n_times = data[:, :, ::decim].shape[2]
     n_freqs = len(Ws)
     n_sources = K.shape[0]
@@ -171,7 +181,7 @@ def _prepare_tfr(data, decim, pick_ori, Ws, K, source_ori):
 @verbose
 def _compute_pow_plv(data, K, sel, Ws, source_ori, use_fft, Vh,
                      with_power, with_plv, pick_ori, decim, verbose=None):
-    """Aux function for induced power and PLV"""
+    """Aux function for induced power and PLV."""
     shape, is_free_ori = _prepare_tfr(data, decim, pick_ori, Ws, K, source_ori)
     n_sources, n_times = shape[:2]
     power = np.zeros(shape, dtype=np.float)  # power or raw TFR
@@ -197,7 +207,7 @@ def _compute_pow_plv(data, K, sel, Ws, source_ori, use_fft, Vh,
 
 def _single_epoch_tfr(data, is_free_ori, K, Ws, use_fft, decim, shape,
                       with_plv, with_power):
-    """Compute single trial TFRs, either ITC, power or raw TFR"""
+    """Compute single trial TFRs, either ITC, power or raw TFR."""
     tfr_e = np.zeros(shape, dtype=np.float)  # power or raw TFR
     # phase lock
     plv_e = np.zeros(shape, dtype=np.complex) if with_plv else None
@@ -245,12 +255,12 @@ def _single_epoch_tfr(data, is_free_ori, K, Ws, use_fft, decim, shape,
 
 
 @verbose
-def _source_induced_power(epochs, inverse_operator, frequencies, label=None,
+def _source_induced_power(epochs, inverse_operator, freqs, label=None,
                           lambda2=1.0 / 9.0, method="dSPM", nave=1, n_cycles=5,
                           decim=1, use_fft=False, pca=True, pick_ori="normal",
                           n_jobs=1, with_plv=True, zero_mean=False,
                           prepared=False, verbose=None):
-    """Aux function for source induced power"""
+    """Aux function for source induced power."""
     epochs_data = epochs.get_data()
     K, sel, Vh, vertno, is_free_ori, noise_norm = _prepare_source_params(
         inst=epochs, inverse_operator=inverse_operator, label=label,
@@ -264,7 +274,7 @@ def _source_induced_power(epochs, inverse_operator, frequencies, label=None,
 
     logger.info('Computing source power ...')
 
-    Ws = morlet(Fs, frequencies, n_cycles=n_cycles, zero_mean=zero_mean)
+    Ws = morlet(Fs, freqs, n_cycles=n_cycles, zero_mean=zero_mean)
 
     n_jobs = min(n_jobs, len(epochs_data))
     out = parallel(my_compute_source_tfrs(data=data, K=K, sel=sel, Ws=Ws,
@@ -290,13 +300,13 @@ def _source_induced_power(epochs, inverse_operator, frequencies, label=None,
 
 
 @verbose
-def source_induced_power(epochs, inverse_operator, frequencies, label=None,
+def source_induced_power(epochs, inverse_operator, freqs, label=None,
                          lambda2=1.0 / 9.0, method="dSPM", nave=1, n_cycles=5,
                          decim=1, use_fft=False, pick_ori=None,
                          baseline=None, baseline_mode='logratio', pca=True,
                          n_jobs=1, zero_mean=False, prepared=False,
-                         verbose=None):
-    """Compute induced power and phase lock
+                         frequencies=None, verbose=None):
+    """Compute induced power and phase lock.
 
     Computation can optionaly be restricted in a label.
 
@@ -332,13 +342,22 @@ def source_induced_power(epochs, inverse_operator, frequencies, label=None,
         the interval is between "a (s)" and "b (s)".
         If a is None the beginning of the data is used
         and if b is None then b is set to the end of the interval.
-        If baseline is equal ot (None, None) all the time
+        If baseline is equal to (None, None) all the time
         interval is used.
-    baseline_mode : None | 'logratio' | 'zscore'
-        Do baseline correction with ratio (power is divided by mean
-        power during baseline) or zscore (power is divided by standard
-        deviation of power during baseline after subtracting the mean,
-        power = [power - mean(power_baseline)] / std(power_baseline)).
+    baseline_mode : 'mean' | 'ratio' | 'logratio' | 'percent' | 'zscore' | 'zlogratio' | None
+        Perform baseline correction by
+
+          - subtracting the mean baseline power ('mean')
+          - dividing by the mean baseline power ('ratio')
+          - dividing by the mean baseline power and taking the log ('logratio')
+          - subtracting the mean baseline power followed by dividing by the
+            mean baseline power ('percent')
+          - subtracting the mean baseline power and dividing by the standard
+            deviation of the baseline power ('zscore')
+          - dividing by the mean baseline power, taking the log, and dividing
+            by the standard deviation of the baseline power ('zlogratio')
+
+        If None no baseline correction is applied.
     pca : bool
         If True, the true dimension of data is estimated before running
         the time-frequency transforms. It reduces the computation times
@@ -350,20 +369,18 @@ def source_induced_power(epochs, inverse_operator, frequencies, label=None,
     prepared : bool
         If True, do not call `prepare_inverse_operator`.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
-    """
-    method = _check_method(method)
-    pick_ori = _check_ori(pick_ori)
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
+    """  # noqa: E501
+    freqs = _freqs_dep(freqs, frequencies)
+    _check_method(method)
+    _check_ori(pick_ori, inverse_operator['source_ori'])
 
-    power, plv, vertno = _source_induced_power(epochs,
-                                               inverse_operator, frequencies,
-                                               label=label, lambda2=lambda2,
-                                               method=method, nave=nave,
-                                               n_cycles=n_cycles, decim=decim,
-                                               use_fft=use_fft,
-                                               pick_ori=pick_ori,
-                                               pca=pca, n_jobs=n_jobs,
-                                               prepared=False)
+    power, plv, vertno = _source_induced_power(
+        epochs, inverse_operator, freqs, label=label, lambda2=lambda2,
+        method=method, nave=nave, n_cycles=n_cycles, decim=decim,
+        use_fft=use_fft, pick_ori=pick_ori, pca=pca, n_jobs=n_jobs,
+        prepared=False)
 
     # Run baseline correction
     power = rescale(power, epochs.times[::decim], baseline, baseline_mode,
@@ -376,7 +393,7 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
                        tmin=None, tmax=None, fmin=0., fmax=200.,
                        n_fft=2048, overlap=0.5, pick_ori=None, label=None,
                        nave=1, pca=True, prepared=False, verbose=None):
-    """Compute source power spectrum density (PSD)
+    """Compute source power spectrum density (PSD).
 
     Parameters
     ----------
@@ -418,7 +435,8 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
     prepared : bool
         If True, do not call `prepare_inverse_operator`.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -426,7 +444,7 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
         The PSD (in dB) of each of the sources.
     """
     from scipy.signal import hanning
-    pick_ori = _check_ori(pick_ori)
+    _check_ori(pick_ori, inverse_operator['source_ori'])
 
     logger.info('Considering frequencies %g ... %g Hz' % (fmin, fmax))
 
@@ -447,7 +465,10 @@ def compute_source_psd(raw, inverse_operator, lambda2=1. / 9., method="dSPM",
     freqs_mask = (freqs >= 0) & (freqs >= fmin) & (freqs <= fmax)
     freqs = freqs[freqs_mask]
     fstep = np.mean(np.diff(freqs))
-    psd = np.zeros((K.shape[0], np.sum(freqs_mask)))
+    if is_free_ori and pick_ori is None:
+        psd = np.zeros((K.shape[0] // 3, np.sum(freqs_mask)))
+    else:
+        psd = np.zeros((K.shape[0], np.sum(freqs_mask)))
     n_windows = 0
 
     for this_start in np.arange(start, stop, int(n_fft * (1. - overlap))):
@@ -492,8 +513,7 @@ def _compute_source_psd_epochs(epochs, inverse_operator, lambda2=1. / 9.,
                                pca=True, inv_split=None, bandwidth=4.,
                                adaptive=False, low_bias=True, n_jobs=1,
                                prepared=False, verbose=None):
-    """ Generator for compute_source_psd_epochs """
-
+    """Generate compute_source_psd_epochs."""
     logger.info('Considering frequencies %g ... %g Hz' % (fmin, fmax))
 
     K, sel, Vh, vertno, is_free_ori, noise_norm = _prepare_source_params(
@@ -603,8 +623,9 @@ def compute_source_psd_epochs(epochs, inverse_operator, lambda2=1. / 9.,
                               adaptive=False, low_bias=True,
                               return_generator=False, n_jobs=1,
                               prepared=False, verbose=None):
-    """Compute source power spectrum density (PSD) from Epochs using
-       multi-taper method
+    """Compute source power spectrum density (PSD) from Epochs.
+
+    This uses the multi-taper method to compute the PSD.
 
     Parameters
     ----------
@@ -650,14 +671,14 @@ def compute_source_psd_epochs(epochs, inverse_operator, lambda2=1. / 9.,
     prepared : bool
         If True, do not call `prepare_inverse_operator`.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
     stcs : list (or generator object) of SourceEstimate | VolSourceEstimate
         The source space PSDs for each epoch.
     """
-
     # use an auxiliary function so we can either return a generator or a list
     stcs_gen = _compute_source_psd_epochs(epochs, inverse_operator,
                                           lambda2=lambda2, method=method,
