@@ -1,5 +1,4 @@
-"""Parallel util function
-"""
+"""Parallel util function."""
 
 # Author: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #
@@ -20,8 +19,9 @@ else:
 
 
 @verbose
-def parallel_func(func, n_jobs, verbose=None, max_nbytes='auto'):
-    """Return parallel instance with delayed function
+def parallel_func(func, n_jobs, verbose=None, max_nbytes='auto',
+                  pre_dispatch='2 * n_jobs'):
+    """Return parallel instance with delayed function.
 
     Util function to use joblib only if available
 
@@ -32,14 +32,31 @@ def parallel_func(func, n_jobs, verbose=None, max_nbytes='auto'):
     n_jobs: int
         Number of jobs to run in parallel
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
-        INFO or DEBUG will print parallel status, others will not.
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more). INFO or DEBUG
+        will print parallel status, others will not.
     max_nbytes : int, str, or None
         Threshold on the minimum size of arrays passed to the workers that
         triggers automated memmory mapping. Can be an int in Bytes,
         or a human-readable string, e.g., '1M' for 1 megabyte.
         Use None to disable memmaping of large arrays. Use 'auto' to
         use the value set using mne.set_memmap_min_size.
+    pre_dispatch : int, or string, optional
+        Controls the number of jobs that get dispatched during parallel
+        execution. Reducing this number can be useful to avoid an
+        explosion of memory consumption when more jobs get dispatched
+        than CPUs can process. This parameter can be:
+
+            - None, in which case all the jobs are immediately
+              created and spawned. Use this for lightweight and
+              fast-running jobs, to avoid delays due to on-demand
+              spawning of the jobs
+
+            - An int, giving the exact number of total jobs that are
+              spawned
+
+            - A string, giving an expression as a function of n_jobs,
+              as in '2*n_jobs'
 
     Returns
     -------
@@ -90,6 +107,7 @@ def parallel_func(func, n_jobs, verbose=None, max_nbytes='auto'):
 
     # create keyword arguments for Parallel
     kwargs = {'verbose': 5 if logger.level <= logging.INFO else 0}
+    kwargs['pre_dispatch'] = pre_dispatch
 
     if joblib_mmap:
         if cache_dir is None:
@@ -104,7 +122,7 @@ def parallel_func(func, n_jobs, verbose=None, max_nbytes='auto'):
 
 
 def check_n_jobs(n_jobs, allow_cuda=False):
-    """Check n_jobs in particular for negative values
+    """Check n_jobs in particular for negative values.
 
     Parameters
     ----------
