@@ -1,4 +1,4 @@
-"""Preprocessing with artifact detection, SSP, and ICA"""
+"""Preprocessing with artifact detection, SSP, and ICA."""
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Matti Hamalainen <msh@nmr.mgh.harvard.edu>
@@ -14,6 +14,7 @@ from .ecg import find_ecg_events, create_ecg_epochs
 from .ica import (ICA, ica_find_eog_events, ica_find_ecg_events,
                   get_score_funcs, read_ica, run_ica, corrmap)
 from .bads import find_outliers
+from .infomax_ import infomax
 from .stim import fix_stim_artifact
 from .maxwell import maxwell_filter
 from .xdawn import Xdawn
