@@ -8,8 +8,8 @@ import numpy as np
 
 from .peak_finder import peak_finder
 from .. import pick_types, pick_channels
-from ..utils import logger, verbose
-from ..filter import band_pass_filter
+from ..utils import logger, verbose, _pl
+from ..filter import filter_data
 from ..epochs import Epochs
 from ..externals.six import string_types
 
@@ -17,8 +17,8 @@ from ..externals.six import string_types
 @verbose
 def find_eog_events(raw, event_id=998, l_freq=1, h_freq=10,
                     filter_length='10s', ch_name=None, tstart=0,
-                    verbose=None):
-    """Locate EOG artifacts
+                    reject_by_annotation=False, verbose=None):
+    """Locate EOG artifacts.
 
     Parameters
     ----------
@@ -36,20 +36,27 @@ def find_eog_events(raw, event_id=998, l_freq=1, h_freq=10,
         If not None, use specified channel(s) for EOG
     tstart : float
         Start detection after tstart seconds.
+    reject_by_annotation : bool
+        Whether to omit data that is annotated as bad.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
     eog_events : array
         Events.
     """
-
     # Getting EOG Channel
     eog_inds = _get_eog_channel_index(ch_name, raw)
     logger.info('EOG channel index for this subject is: %s' % eog_inds)
 
-    eog, _ = raw[eog_inds, :]
+    # Reject bad segments.
+    reject_by_annotation = 'omit' if reject_by_annotation else None
+    eog, times = raw.get_data(picks=eog_inds,
+                              reject_by_annotation=reject_by_annotation,
+                              return_times=True)
+    times = times * raw.info['sfreq'] + raw.first_samp
 
     eog_events = _find_eog_events(eog, event_id=event_id, l_freq=l_freq,
                                   h_freq=h_freq,
@@ -57,29 +64,33 @@ def find_eog_events(raw, event_id=998, l_freq=1, h_freq=10,
                                   first_samp=raw.first_samp,
                                   filter_length=filter_length,
                                   tstart=tstart)
-
+    # Map times to corresponding samples.
+    eog_events[:, 0] = np.round(times[eog_events[:, 0] -
+                                      raw.first_samp]).astype(int)
     return eog_events
 
 
 def _find_eog_events(eog, event_id, l_freq, h_freq, sampling_rate, first_samp,
                      filter_length='10s', tstart=0.):
-    """Helper function"""
-
+    """Find EOG events."""
     logger.info('Filtering the data to remove DC offset to help '
                 'distinguish blinks from saccades')
 
     # filtering to remove dc offset so that we know which is blink and saccades
     fmax = np.minimum(45, sampling_rate / 2.0 - 0.75)  # protect Nyquist
-    filteog = np.array([band_pass_filter(x, sampling_rate, 2, fmax,
-                                         filter_length=filter_length)
-                        for x in eog])
+    filteog = np.array([filter_data(
+        x, sampling_rate, 2, fmax, None, filter_length, 0.5, 0.5,
+        phase='zero-double', fir_window='hann', fir_design='firwin2')
+        for x in eog])
     temp = np.sqrt(np.sum(filteog ** 2, axis=1))
 
     indexmax = np.argmax(temp)
 
     # easier to detect peaks with filtering.
-    filteog = band_pass_filter(eog[indexmax], sampling_rate, l_freq, h_freq,
-                               filter_length=filter_length)
+    filteog = filter_data(
+        eog[indexmax], sampling_rate, l_freq, h_freq, None,
+        filter_length, 0.5, 0.5, phase='zero-double', fir_window='hann',
+        fir_design='firwin2')
 
     # detecting eog blinks and generating event file
 
@@ -103,6 +114,7 @@ def _find_eog_events(eog, event_id, l_freq, h_freq, sampling_rate, first_samp,
 
 
 def _get_eog_channel_index(ch_name, inst):
+    """Get EOG channel index."""
     if isinstance(ch_name, string_types):
         # Check if multiple EOG Channels
         if ',' in ch_name:
@@ -116,8 +128,7 @@ def _get_eog_channel_index(ch_name, inst):
             raise ValueError('%s not in channel list' % ch_name)
         else:
             logger.info('Using channel %s as EOG channel%s' % (
-                        " and ".join(ch_name),
-                        '' if len(eog_inds) < 2 else 's'))
+                        " and ".join(ch_name), _pl(eog_inds)))
     elif ch_name is None:
 
         eog_inds = pick_types(inst.info, meg=False, eeg=False, stim=False,
@@ -138,11 +149,11 @@ def _get_eog_channel_index(ch_name, inst):
 
 
 @verbose
-def create_eog_epochs(raw, ch_name=None, event_id=998, picks=None,
-                      tmin=-0.5, tmax=0.5, l_freq=1, h_freq=10,
-                      reject=None, flat=None, baseline=None,
-                      preload=True, verbose=None):
-    """Conveniently generate epochs around EOG artifact events
+def create_eog_epochs(raw, ch_name=None, event_id=998, picks=None, tmin=-0.5,
+                      tmax=0.5, l_freq=1, h_freq=10, reject=None, flat=None,
+                      baseline=None, preload=True, reject_by_annotation=True,
+                      verbose=None):
+    """Conveniently generate epochs around EOG artifact events.
 
     Parameters
     ----------
@@ -190,8 +201,17 @@ def create_eog_epochs(raw, ch_name=None, event_id=998, picks=None,
         interval is used. If None, no correction is applied.
     preload : bool
         Preload epochs or not.
+    reject_by_annotation : bool
+        Whether to reject based on annotations. If True (default), segments
+        whose description begins with ``'bad'`` are not used for finding
+        artifacts and epochs overlapping with them are rejected. If False, no
+        rejection based on annotations is performed.
+
+        .. versionadded:: 0.14.0
+
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -199,11 +219,12 @@ def create_eog_epochs(raw, ch_name=None, event_id=998, picks=None,
         Data epoched around EOG events.
     """
     events = find_eog_events(raw, ch_name=ch_name, event_id=event_id,
-                             l_freq=l_freq, h_freq=h_freq)
+                             l_freq=l_freq, h_freq=h_freq,
+                             reject_by_annotation=reject_by_annotation)
 
     # create epochs around EOG events
-    eog_epochs = Epochs(raw, events=events, event_id=event_id,
-                        tmin=tmin, tmax=tmax, proj=False, reject=reject,
-                        flat=flat, picks=picks, baseline=baseline,
-                        preload=preload)
+    eog_epochs = Epochs(raw, events=events, event_id=event_id, tmin=tmin,
+                        tmax=tmax, proj=False, reject=reject, flat=flat,
+                        picks=picks, baseline=baseline, preload=preload,
+                        reject_by_annotation=reject_by_annotation)
     return eog_epochs
