@@ -6,7 +6,7 @@ from .. utils import logger, verbose
 
 @verbose
 def peak_finder(x0, thresh=None, extrema=1, verbose=None):
-    """Noise tolerant fast peak finding algorithm
+    """Noise-tolerant fast peak-finding algorithm.
 
     Parameters
     ----------
@@ -20,7 +20,8 @@ def peak_finder(x0, thresh=None, extrema=1, verbose=None):
         1 if maxima are desired, -1 if minima are desired
         (default = maxima, 1).
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -42,7 +43,6 @@ def peak_finder(x0, thresh=None, extrema=1, verbose=None):
     x(1250:1255) = max(x);
     peak_finder(x)
     """
-
     x0 = np.asanyarray(x0)
 
     if x0.ndim >= 2:
