@@ -9,30 +9,32 @@ import os
 import os.path as op
 import warnings
 
-from nose.tools import assert_true, assert_raises, assert_equal, assert_false
+from nose.tools import (assert_true, assert_raises, assert_equal, assert_false,
+                        assert_not_equal, assert_is_none)
+import pytest
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
                            assert_allclose)
 from scipy import stats
 from itertools import product
 
-from mne import Epochs, read_events, pick_types, create_info, EpochsArray
+from mne import (Epochs, read_events, pick_types, create_info, EpochsArray,
+                 EvokedArray, Annotations)
 from mne.cov import read_cov
 from mne.preprocessing import (ICA, ica_find_ecg_events, ica_find_eog_events,
                                read_ica, run_ica)
-from mne.preprocessing.ica import (get_score_funcs, corrmap, _get_ica_map,
-                                   _ica_explained_variance, _sort_components)
+from mne.preprocessing.ica import (get_score_funcs, corrmap, _sort_components,
+                                   _ica_explained_variance)
 from mne.io import read_raw_fif, Info, RawArray
 from mne.io.meas_info import _kind_dict
 from mne.io.pick import _DATA_CH_TYPES_SPLIT
 from mne.tests.common import assert_naming
-from mne.utils import (catch_logging, _TempDir, requires_sklearn, slow_test,
+from mne.utils import (catch_logging, _TempDir, requires_sklearn,
                        run_tests_if_main)
 
 # Set our plotters to test mode
 import matplotlib
 matplotlib.use('Agg')  # for testing don't use X server
-import matplotlib.pyplot as plt  # noqa
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
@@ -48,7 +50,7 @@ score_funcs_unsuited = ['pointbiserialr', 'ansari']
 try:
     from sklearn.utils.validation import NonBLASDotWarning
     warnings.simplefilter('error', NonBLASDotWarning)
-except:
+except Exception:
     pass
 
 
@@ -56,19 +58,19 @@ except:
 def test_ica_full_data_recovery():
     """Test recovery of full data when no source is rejected."""
     # Most basic recovery
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(0.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(0.5, stop).load_data()
     events = read_events(event_name)
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')[:10]
     with warnings.catch_warnings(record=True):  # bad proj
         epochs = Epochs(raw, events[:4], event_id, tmin, tmax, picks=picks,
-                        baseline=(None, 0), preload=True, add_eeg_ref=False)
+                        baseline=(None, 0), preload=True)
     evoked = epochs.average()
     n_channels = 5
     data = raw._data[:n_channels].copy()
     data_epochs = epochs.get_data()
     data_evoked = evoked.data
+    raw.annotations = Annotations([0.5], [0.5], ['BAD'])
     for method in ['fastica']:
         stuff = [(2, n_channels, True), (2, n_channels // 2, False)]
         for n_components, n_pca_components, ok in stuff:
@@ -115,8 +117,7 @@ def test_ica_full_data_recovery():
 def test_ica_rank_reduction():
     """Test recovery ICA rank reduction."""
     # Most basic recovery
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(0.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(0.5, stop).load_data()
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')[:10]
     n_components = 5
@@ -143,8 +144,7 @@ def test_ica_rank_reduction():
 @requires_sklearn
 def test_ica_reset():
     """Test ICA resetting."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(0.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(0.5, stop).load_data()
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')[:10]
 
@@ -158,30 +158,30 @@ def test_ica_reset():
         'pca_explained_variance_',
         'pca_mean_'
     )
-    with warnings.catch_warnings(record=True):
+    with warnings.catch_warnings(record=True):  # convergence
         ica = ICA(
             n_components=3, max_pca_components=3, n_pca_components=3,
             method='fastica', max_iter=1).fit(raw, picks=picks)
 
     assert_true(all(hasattr(ica, attr) for attr in run_time_attrs))
+    assert_not_equal(ica.labels_, None)
     ica._reset()
     assert_true(not any(hasattr(ica, attr) for attr in run_time_attrs))
+    assert_not_equal(ica.labels_, None)
 
 
 @requires_sklearn
 def test_ica_core():
     """Test ICA on raw and epochs."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(1.5, stop, copy=False).load_data()
-    picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
-                       eog=False, exclude='bads')
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
+
     # XXX. The None cases helped revealing bugs but are time consuming.
     test_cov = read_cov(test_cov_name)
     events = read_events(event_name)
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')
     epochs = Epochs(raw, events[:4], event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), preload=True, add_eeg_ref=False)
+                    baseline=(None, 0), preload=True)
     noise_cov = [None, test_cov]
     # removed None cases to speed up...
     n_components = [2, 1.0]  # for future dbg add cases
@@ -210,7 +210,7 @@ def test_ica_core():
         assert_raises(RuntimeError, ica.get_sources, epochs)
 
         # test decomposition
-        with warnings.catch_warnings(record=True):
+        with warnings.catch_warnings(record=True):  # convergence
             ica.fit(raw, picks=pcks, start=start, stop=stop)
             repr(ica)  # to test repr
         assert_true('mag' in ica)  # should now work without error
@@ -221,7 +221,12 @@ def test_ica_core():
             ica.fit(raw, picks=pcks, start=start, stop=stop)
         assert_array_almost_equal(unmixing1, ica.unmixing_matrix_)
 
-        sources = ica.get_sources(raw)[:, :][0]
+        raw_sources = ica.get_sources(raw)
+        # test for #3804
+        assert_equal(raw_sources._filenames, [None])
+        print(raw_sources)
+
+        sources = raw_sources[:, :][0]
         assert_true(sources.shape[0] == ica.n_components_)
 
         # test preload filter
@@ -270,14 +275,15 @@ def test_ica_core():
     assert_raises(ValueError, ica.apply, offender)
 
 
-@slow_test
+@pytest.mark.slowtest
 @requires_sklearn
 def test_ica_additional():
     """Test additional ICA functionality."""
+    import matplotlib.pyplot as plt
     tempdir = _TempDir()
     stop2 = 500
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(1.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
+    raw.annotations = Annotations([0.5], [0.5], ['BAD'])
     # XXX This breaks the tests :(
     # raw.info['bads'] = [raw.ch_names[1]]
     test_cov = read_cov(test_cov_name)
@@ -285,7 +291,7 @@ def test_ica_additional():
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')
     epochs = Epochs(raw, events[:4], event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), preload=True, add_eeg_ref=False)
+                    baseline=(None, 0), preload=True)
     # test if n_components=None works
     with warnings.catch_warnings(record=True):
         ica = ICA(n_components=None,
@@ -296,7 +302,7 @@ def test_ica_additional():
     picks2 = pick_types(raw.info, meg=True, stim=False, ecg=False,
                         eog=True, exclude='bads')
     epochs_eog = Epochs(raw, events[:4], event_id, tmin, tmax, picks=picks2,
-                        baseline=(None, 0), preload=True, add_eeg_ref=False)
+                        baseline=(None, 0), preload=True)
 
     test_cov2 = test_cov.copy()
     ica = ICA(noise_cov=test_cov2, n_components=3, max_pca_components=4,
@@ -310,9 +316,16 @@ def test_ica_additional():
     ica = ICA(n_components=3, max_pca_components=4,
               n_pca_components=4)
     assert_raises(RuntimeError, ica.save, '')
+
     with warnings.catch_warnings(record=True):
         ica.fit(raw, picks=[1, 2, 3, 4, 5], start=start, stop=stop2)
 
+    # check passing a ch_name to find_bads_ecg
+    with warnings.catch_warnings(record=True):  # filter length
+        _, scores_1 = ica.find_bads_ecg(raw)
+        _, scores_2 = ica.find_bads_ecg(raw, raw.ch_names[1])
+    assert_false(scores_1[0] == scores_2[0])
+
     # test corrmap
     ica2 = ica.copy()
     ica3 = ica.copy()
@@ -321,10 +334,15 @@ def test_ica_additional():
     corrmap([ica, ica2], (0, 0), threshold=2, plot=False, show=False)
     assert_true(ica.labels_["blinks"] == ica2.labels_["blinks"])
     assert_true(0 in ica.labels_["blinks"])
-    template = _get_ica_map(ica)[0]
+    # test retrieval of component maps as arrays
+    components = ica.get_components()
+    template = components[:, 0]
+    EvokedArray(components, ica.info, tmin=0.).plot_topomap([0])
+
     corrmap([ica, ica3], template, threshold='auto', label='blinks', plot=True,
             ch_type="mag")
     assert_true(ica2.labels_["blinks"] == ica3.labels_["blinks"])
+
     plt.close('all')
 
     # test warnings on bad filenames
@@ -398,15 +416,12 @@ def test_ica_additional():
 
         # test filtering
         d1 = ica_raw._data[0].copy()
-        ica_raw.filter(4, 20, l_trans_bandwidth='auto',
-                       h_trans_bandwidth='auto', filter_length='auto',
-                       phase='zero', fir_window='hamming')
+        ica_raw.filter(4, 20, fir_design='firwin2')
         assert_equal(ica_raw.info['lowpass'], 20.)
         assert_equal(ica_raw.info['highpass'], 4.)
         assert_true((d1 != ica_raw._data[0]).any())
         d1 = ica_raw._data[0].copy()
-        ica_raw.notch_filter([10], filter_length='auto', trans_bandwidth=10,
-                             phase='zero', fir_window='hamming')
+        ica_raw.notch_filter([10], trans_bandwidth=10, fir_design='firwin')
         assert_true((d1 != ica_raw._data[0]).any())
 
         ica.n_pca_components = 2
@@ -451,7 +466,7 @@ def test_ica_additional():
         assert_array_almost_equal(_raw1[:, :][0], _raw2[:, :][0])
 
     os.remove(test_ica_fname)
-    # check scrore funcs
+    # check score funcs
     for name, func in get_score_funcs().items():
         if name in score_funcs_unsuited:
             continue
@@ -472,6 +487,11 @@ def test_ica_additional():
         ica.detect_artifacts(raw, start_find=0, stop_find=50, ecg_ch=ch_name,
                              eog_ch=ch_name, skew_criterion=idx,
                              var_criterion=idx, kurt_criterion=idx)
+
+    evoked = epochs.average()
+    evoked_data = evoked.data.copy()
+    raw_data = raw[:][0].copy()
+    epochs_data = epochs.get_data().copy()
     with warnings.catch_warnings(record=True):
         idx, scores = ica.find_bads_ecg(raw, method='ctps')
         assert_equal(len(scores), ica.n_components_)
@@ -481,19 +501,32 @@ def test_ica_additional():
         idx, scores = ica.find_bads_eog(raw)
         assert_equal(len(scores), ica.n_components_)
 
-        ica.labels_ = None
         idx, scores = ica.find_bads_ecg(epochs, method='ctps')
+
         assert_equal(len(scores), ica.n_components_)
         assert_raises(ValueError, ica.find_bads_ecg, epochs.average(),
                       method='ctps')
         assert_raises(ValueError, ica.find_bads_ecg, raw,
                       method='crazy-coupling')
 
+        idx, scores = ica.find_bads_eog(raw)
+        assert_equal(len(scores), ica.n_components_)
+
         raw.info['chs'][raw.ch_names.index('EOG 061') - 1]['kind'] = 202
         idx, scores = ica.find_bads_eog(raw)
         assert_true(isinstance(scores, list))
         assert_equal(len(scores[0]), ica.n_components_)
 
+        idx, scores = ica.find_bads_eog(evoked, ch_name='MEG 1441')
+        assert_equal(len(scores), ica.n_components_)
+
+        idx, scores = ica.find_bads_ecg(evoked, method='correlation')
+        assert_equal(len(scores), ica.n_components_)
+
+    assert_array_equal(raw_data, raw[:][0])
+    assert_array_equal(epochs_data, epochs.get_data())
+    assert_array_equal(evoked_data, evoked.data)
+
     # check score funcs
     for name, func in get_score_funcs().items():
         if name in score_funcs_unsuited:
@@ -531,13 +564,13 @@ def test_ica_additional():
     # Test ica fiff export
     ica_raw = ica.get_sources(raw, start=0, stop=100)
     assert_true(ica_raw.last_samp - ica_raw.first_samp == 100)
-    assert_true(len(ica_raw._filenames) == 0)  # API consistency
+    assert_equal(len(ica_raw._filenames), 1)  # API consistency
     ica_chans = [ch for ch in ica_raw.ch_names if 'ICA' in ch]
     assert_true(ica.n_components_ == len(ica_chans))
     test_ica_fname = op.join(op.abspath(op.curdir), 'test-ica_raw.fif')
     ica.n_components = np.int32(ica.n_components)
     ica_raw.save(test_ica_fname, overwrite=True)
-    ica_raw2 = read_raw_fif(test_ica_fname, preload=True, add_eeg_ref=False)
+    ica_raw2 = read_raw_fif(test_ica_fname, preload=True)
     assert_allclose(ica_raw._data, ica_raw2._data, rtol=1e-5, atol=1e-4)
     ica_raw2.close()
     os.remove(test_ica_fname)
@@ -558,12 +591,23 @@ def test_ica_additional():
         ncomps_ = ica._check_n_pca_components(ncomps)
         assert_true(ncomps_ == expected)
 
+    ica = ICA()
+    ica.fit(raw, picks=picks[:5])
+    with warnings.catch_warnings(record=True):  # filter length
+        ica.find_bads_ecg(raw)
+    ica.find_bads_eog(epochs, ch_name='MEG 0121')
+    assert_array_equal(raw_data, raw[:][0])
+
+    raw.drop_channels(['MEG 0122'])
+    with warnings.catch_warnings(record=True):  # filter length
+        assert_raises(RuntimeError, ica.find_bads_eog, raw)
+        assert_raises(RuntimeError, ica.find_bads_ecg, raw)
+
 
 @requires_sklearn
 def test_run_ica():
     """Test run_ica function."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(1.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
     params = []
     params += [(None, -1, slice(2), [0, 1])]  # varicance, kurtosis idx
     params += [(None, 'MEG 1531')]  # ECG / EOG channel params
@@ -578,8 +622,7 @@ def test_run_ica():
 @requires_sklearn
 def test_ica_reject_buffer():
     """Test ICA data raw buffer rejection."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(1.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
     picks = pick_types(raw.info, meg=True, stim=False, ecg=False,
                        eog=False, exclude='bads')
     ica = ICA(n_components=3, max_pca_components=4, n_pca_components=4)
@@ -587,7 +630,7 @@ def test_ica_reject_buffer():
     with catch_logging() as drop_log:
         with warnings.catch_warnings(record=True):
             ica.fit(raw, picks[:5], reject=dict(mag=2.5e-12), decim=2,
-                    tstep=0.01, verbose=True)
+                    tstep=0.01, verbose=True, reject_by_annotation=False)
         assert_true(raw._data[:5, ::2].shape[1] - 4 == ica.n_samples_)
     log = [l for l in drop_log.getvalue().split('\n') if 'detected' in l]
     assert_equal(len(log), 1)
@@ -596,8 +639,7 @@ def test_ica_reject_buffer():
 @requires_sklearn
 def test_ica_twice():
     """Test running ICA twice."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
-    raw.crop(1.5, stop, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
     picks = pick_types(raw.info, meg='grad', exclude='bads')
     n_components = 0.9
     max_pca_components = None
@@ -656,13 +698,12 @@ def test_bad_channels():
 @requires_sklearn
 def test_eog_channel():
     """Test that EOG channel is included when performing ICA."""
-    raw = read_raw_fif(raw_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname, preload=True)
     events = read_events(event_name)
     picks = pick_types(raw.info, meg=True, stim=True, ecg=False,
                        eog=True, exclude='bads')
     epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), preload=True,
-                    add_eeg_ref=False)
+                    baseline=(None, 0), preload=True)
     n_components = 0.9
     ica = ICA(n_components=n_components, method='fastica')
     # Test case for MEG and EOG data. Should have EOG channel
@@ -681,4 +722,95 @@ def test_eog_channel():
         ica.fit(inst, picks=picks1)
         assert_false(any('EOG' in ch for ch in ica.ch_names))
 
+
+@requires_sklearn
+def test_max_pca_components_none():
+    """Test max_pca_components=None."""
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
+    events = read_events(event_name)
+    picks = pick_types(raw.info, eeg=True, meg=False)
+    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
+                    baseline=(None, 0), preload=True)
+
+    max_pca_components = None
+    n_components = 10
+    random_state = 12345
+
+    tempdir = _TempDir()
+    output_fname = op.join(tempdir, 'test_ica-ica.fif')
+
+    ica = ICA(max_pca_components=max_pca_components,
+              n_components=n_components, random_state=random_state)
+    with warnings.catch_warnings(record=True):  # convergence
+        ica.fit(epochs)
+    ica.save(output_fname)
+
+    ica = read_ica(output_fname)
+
+    # ICA.fit() replaced max_pca_components, which was previously None,
+    # with the appropriate integer value.
+    assert_equal(ica.max_pca_components, epochs.info['nchan'])
+    assert_equal(ica.n_components, 10)
+
+
+@requires_sklearn
+def test_n_components_none():
+    """Test n_components=None."""
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
+    events = read_events(event_name)
+    picks = pick_types(raw.info, eeg=True, meg=False)
+    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
+                    baseline=(None, 0), preload=True)
+
+    max_pca_components = 10
+    n_components = None
+    random_state = 12345
+
+    tempdir = _TempDir()
+    output_fname = op.join(tempdir, 'test_ica-ica.fif')
+
+    ica = ICA(max_pca_components=max_pca_components,
+              n_components=n_components, random_state=random_state)
+    with warnings.catch_warnings(record=True):  # convergence
+        ica.fit(epochs)
+    ica.save(output_fname)
+
+    ica = read_ica(output_fname)
+
+    # ICA.fit() replaced max_pca_components, which was previously None,
+    # with the appropriate integer value.
+    assert_equal(ica.max_pca_components, 10)
+    assert_is_none(ica.n_components)
+
+
+@requires_sklearn
+def test_n_components_and_max_pca_components_none():
+    """Test n_components and max_pca_components=None."""
+    raw = read_raw_fif(raw_fname).crop(1.5, stop).load_data()
+    events = read_events(event_name)
+    picks = pick_types(raw.info, eeg=True, meg=False)
+    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
+                    baseline=(None, 0), preload=True)
+
+    max_pca_components = None
+    n_components = None
+    random_state = 12345
+
+    tempdir = _TempDir()
+    output_fname = op.join(tempdir, 'test_ica-ica.fif')
+
+    ica = ICA(max_pca_components=max_pca_components,
+              n_components=n_components, random_state=random_state)
+    with warnings.catch_warnings(record=True):  # convergence
+        ica.fit(epochs)
+    ica.save(output_fname)
+
+    ica = read_ica(output_fname)
+
+    # ICA.fit() replaced max_pca_components, which was previously None,
+    # with the appropriate integer value.
+    assert_equal(ica.max_pca_components, epochs.info['nchan'])
+    assert_is_none(ica.n_components)
+
+
 run_tests_if_main()
