@@ -5,12 +5,10 @@
 import os.path as op
 import warnings
 import numpy as np
-import sys
-import scipy
+
 from numpy.testing import assert_equal, assert_allclose
+import pytest
 from nose.tools import assert_true, assert_raises
-from nose.plugins.skip import SkipTest
-from distutils.version import LooseVersion
 
 from mne import compute_raw_covariance, pick_types
 from mne.chpi import read_head_pos, filter_chpi
@@ -18,15 +16,15 @@ from mne.forward import _prep_meg_channels
 from mne.cov import _estimate_rank_meeg_cov
 from mne.datasets import testing
 from mne.io import (read_raw_fif, proc_history, read_info, read_raw_bti,
-                    read_raw_kit, _BaseRaw)
+                    read_raw_kit, BaseRaw)
 from mne.preprocessing.maxwell import (
     maxwell_filter, _get_n_moments, _sss_basis_basic, _sh_complex_to_real,
     _sh_real_to_complex, _sh_negate, _bases_complex_to_real, _trans_sss_basis,
-    _bases_real_to_complex, _sph_harm, _prep_mf_coils)
+    _bases_real_to_complex, _prep_mf_coils)
+from mne.fixes import _get_sph_harm
 from mne.tests.common import assert_meg_snr
-from mne.utils import (_TempDir, run_tests_if_main, slow_test, catch_logging,
+from mne.utils import (_TempDir, run_tests_if_main, catch_logging,
                        requires_version, object_diff, buggy_mkl_svd)
-from mne.externals.six import PY3
 
 warnings.simplefilter('always')  # Always throw warnings
 
@@ -119,16 +117,15 @@ def _assert_n_free(raw_sss, lower, upper=None):
 
 def read_crop(fname, lims=(0, None)):
     """Read and crop."""
-    return read_raw_fif(fname, allow_maxshield='yes',
-                        add_eeg_ref=False).crop(*lims)
+    return read_raw_fif(fname, allow_maxshield='yes').crop(*lims)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_movement_compensation():
     """Test movement compensation."""
     temp_dir = _TempDir()
-    lims = (0, 4, False)
+    lims = (0, 4)
     raw = read_crop(raw_fname, lims).load_data()
     head_pos = read_head_pos(pos_fname)
 
@@ -196,18 +193,20 @@ def test_movement_compensation():
     head_pos_bad = head_pos.copy()
     head_pos_bad[0, 4] = 1.  # off by more than 1 m
     with warnings.catch_warnings(record=True) as w:
-        maxwell_filter(raw, head_pos=head_pos_bad, bad_condition='ignore')
+        maxwell_filter(raw.copy().crop(0, 0.1), head_pos=head_pos_bad,
+                       bad_condition='ignore')
     assert_true(any('greater than 1 m' in str(ww.message) for ww in w))
 
     # make sure numerical error doesn't screw it up, though
     head_pos_bad = head_pos.copy()
     head_pos_bad[0, 0] = raw.first_samp / raw.info['sfreq'] - 5e-4
-    raw_sss_tweak = maxwell_filter(raw, head_pos=head_pos_bad,
-                                   origin=mf_head_origin)
-    assert_meg_snr(raw_sss_tweak, raw_sss, 2., 10., chpi_med_tol=11)
+    raw_sss_tweak = maxwell_filter(
+        raw.copy().crop(0, 0.05), head_pos=head_pos_bad, origin=mf_head_origin)
+    assert_meg_snr(raw_sss_tweak, raw_sss.copy().crop(0, 0.05), 1.4, 8.,
+                   chpi_med_tol=5)
 
 
-@slow_test
+@pytest.mark.slowtest
 def test_other_systems():
     """Test Maxwell filtering on KIT, BTI, and CTF files."""
     # KIT
@@ -288,26 +287,6 @@ def test_other_systems():
     assert_allclose(raw_sss._data, raw_sss_auto._data)
 
 
-def test_spherical_harmonics():
-    """Test spherical harmonic functions."""
-    from scipy.special import sph_harm
-    az, pol = np.meshgrid(np.linspace(0, 2 * np.pi, 30),
-                          np.linspace(0, np.pi, 20))
-    # As of Oct 16, 2015, Anancoda has a bug in scipy due to old compilers (?):
-    # https://github.com/ContinuumIO/anaconda-issues/issues/479
-    if (PY3 and
-            LooseVersion(scipy.__version__) >= LooseVersion('0.15') and
-            'Continuum Analytics' in sys.version):
-        raise SkipTest('scipy sph_harm bad in Py3k on Anaconda')
-
-    # Test our basic spherical harmonics
-    for degree in range(1, int_order):
-        for order in range(0, degree + 1):
-            sph = _sph_harm(order, degree, az, pol)
-            sph_scipy = sph_harm(order, degree, az, pol)
-            assert_allclose(sph, sph_scipy, atol=1e-7)
-
-
 def test_spherical_conversions():
     """Test spherical harmonic conversions."""
     # Test our real<->complex conversion functions
@@ -315,10 +294,10 @@ def test_spherical_conversions():
                           np.linspace(0, np.pi, 20))
     for degree in range(1, int_order):
         for order in range(0, degree + 1):
-            sph = _sph_harm(order, degree, az, pol)
+            sph = _get_sph_harm()(order, degree, az, pol)
             # ensure that we satisfy the conjugation property
             assert_allclose(_sh_negate(sph, order),
-                            _sph_harm(-order, degree, az, pol))
+                            _get_sph_harm()(-order, degree, az, pol))
             # ensure our conversion functions work
             sph_real_pos = _sh_complex_to_real(sph, order)
             sph_real_neg = _sh_complex_to_real(sph, -order)
@@ -445,6 +424,14 @@ def test_basic():
     assert_raises(ValueError, maxwell_filter, raw, origin='foo')
     assert_raises(ValueError, maxwell_filter, raw, origin=[0] * 4)
     assert_raises(ValueError, maxwell_filter, raw, mag_scale='foo')
+    raw_missing = raw.copy().load_data()
+    raw_missing.info['bads'] = ['MEG0111']
+    raw_missing.pick_types(meg=True)  # will be missing the bad
+    maxwell_filter(raw_missing)
+    with warnings.catch_warnings(record=True) as w:
+        maxwell_filter(raw_missing, calibration=fine_cal_fname)
+    assert_equal(len(w), 1)
+    assert_true('not in data' in str(w[0].message))
 
 
 @testing.requires_testing_data
@@ -493,7 +480,7 @@ def test_maxwell_filter_additional():
     assert_equal(cov_sss_rank, _get_n_moments(int_order))
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_bads_reconstruction():
     """Test Maxwell filter reconstruction of bad channels."""
@@ -504,9 +491,10 @@ def test_bads_reconstruction():
     assert_meg_snr(raw_sss, read_crop(sss_bad_recon_fname), 300.)
 
 
+@buggy_mkl_svd
 @requires_svd_convergence
 @testing.requires_testing_data
-def test_spatiotemporal_maxwell():
+def test_spatiotemporal():
     """Test Maxwell filter (tSSS) spatiotemporal processing."""
     # Load raw testing data
     raw = read_crop(raw_fname)
@@ -514,10 +502,13 @@ def test_spatiotemporal_maxwell():
     # Test that window is less than length of data
     assert_raises(ValueError, maxwell_filter, raw, st_duration=1000.)
 
-    # Check both 4 and 10 seconds because Elekta handles them differently
-    # This is to ensure that std/non-std tSSS windows are correctly handled
-    st_durations = [4., 10.]
-    tols = [325., 200.]
+    # We could check both 4 and 10 seconds because Elekta handles them
+    # differently (to ensure that std/non-std tSSS windows are correctly
+    # handled), but the 4-sec case should hopefully be sufficient.
+    st_durations = [4.]  # , 10.]
+    tols = [325.]  # , 200.]
+    kwargs = dict(origin=mf_head_origin, regularize=None,
+                  bad_condition='ignore')
     for st_duration, tol in zip(st_durations, tols):
         # Load tSSS data depending on st_duration and get data
         tSSS_fname = op.join(sss_path,
@@ -526,27 +517,15 @@ def test_spatiotemporal_maxwell():
         # Because Elekta's tSSS sometimes(!) lumps the tail window of data
         # onto the previous buffer if it's shorter than st_duration, we have to
         # crop the data here to compensate for Elekta's tSSS behavior.
-        if st_duration == 10.:
-            tsss_bench.crop(0, st_duration, copy=False)
+        # if st_duration == 10.:
+        #     tsss_bench.crop(0, st_duration)
+        #     raw.crop(0, st_duration)
 
         # Test sss computation at the standard head origin. Same cropping issue
         # as mentioned above.
-        if st_duration == 10.:
-            raw_tsss = maxwell_filter(raw.crop(0, st_duration),
-                                      origin=mf_head_origin,
-                                      st_duration=st_duration, regularize=None,
-                                      bad_condition='ignore')
-        else:
-            raw_tsss = maxwell_filter(raw, st_duration=st_duration,
-                                      origin=mf_head_origin, regularize=None,
-                                      bad_condition='ignore', verbose=True)
-            raw_tsss_2 = maxwell_filter(raw, st_duration=st_duration,
-                                        origin=mf_head_origin, regularize=None,
-                                        bad_condition='ignore', st_fixed=False,
-                                        verbose=True)
-            assert_meg_snr(raw_tsss, raw_tsss_2, 100., 1000.)
-            assert_equal(raw_tsss.estimate_rank(), 140)
-            assert_equal(raw_tsss_2.estimate_rank(), 140)
+        raw_tsss = maxwell_filter(
+            raw, st_duration=st_duration, **kwargs)
+        assert_equal(raw_tsss.estimate_rank(), 140)
         assert_meg_snr(raw_tsss, tsss_bench, tol)
         py_st = raw_tsss.info['proc_history'][0]['max_info']['max_st']
         assert_true(len(py_st) > 0)
@@ -558,55 +537,57 @@ def test_spatiotemporal_maxwell():
                   st_correlation=0.)
 
 
+@pytest.mark.slowtest
 @requires_svd_convergence
 @testing.requires_testing_data
 def test_spatiotemporal_only():
     """Test tSSS-only processing."""
     # Load raw testing data
-    raw = read_crop(raw_fname, (0, 2)).load_data()
-    picks = pick_types(raw.info, meg='mag', exclude=())
-    power = np.sqrt(np.sum(raw[picks][0] ** 2))
+    tmax = 0.5
+    raw = read_crop(raw_fname, (0, tmax)).load_data()
+    picks = pick_types(raw.info, meg=True, exclude='bads')[::2]
+    raw.pick_channels([raw.ch_names[pick] for pick in picks])
+    mag_picks = pick_types(raw.info, meg='mag', exclude=())
+    power = np.sqrt(np.sum(raw[mag_picks][0] ** 2))
     # basics
-    raw_tsss = maxwell_filter(raw, st_duration=1., st_only=True)
-    assert_equal(raw_tsss.estimate_rank(), 366)
-    _assert_shielding(raw_tsss, power, 10)
-    # temporal proj will actually reduce spatial DOF with small windows!
-    raw_tsss = maxwell_filter(raw, st_duration=0.1, st_only=True)
-    assert_true(raw_tsss.estimate_rank() < 350)
-    _assert_shielding(raw_tsss, power, 40)
+    raw_tsss = maxwell_filter(raw, st_duration=tmax / 2., st_only=True)
+    assert_equal(len(raw.info['projs']), len(raw_tsss.info['projs']))
+    assert_equal(raw_tsss.estimate_rank(), len(picks))
+    _assert_shielding(raw_tsss, power, 9)
     # with movement
     head_pos = read_head_pos(pos_fname)
-    raw_tsss = maxwell_filter(raw, st_duration=1., st_only=True,
+    raw_tsss = maxwell_filter(raw, st_duration=tmax / 2., st_only=True,
                               head_pos=head_pos)
-    assert_equal(raw_tsss.estimate_rank(), 366)
-    _assert_shielding(raw_tsss, power, 12)
+    assert_equal(raw_tsss.estimate_rank(), len(picks))
+    _assert_shielding(raw_tsss, power, 9)
     with warnings.catch_warnings(record=True):  # st_fixed False
-        raw_tsss = maxwell_filter(raw, st_duration=1., st_only=True,
+        raw_tsss = maxwell_filter(raw, st_duration=tmax / 2., st_only=True,
                                   head_pos=head_pos, st_fixed=False)
-    assert_equal(raw_tsss.estimate_rank(), 366)
-    _assert_shielding(raw_tsss, power, 12)
+    assert_equal(raw_tsss.estimate_rank(), len(picks))
+    _assert_shielding(raw_tsss, power, 9)
     # should do nothing
-    raw_tsss = maxwell_filter(raw, st_duration=1., st_correlation=1.,
+    raw_tsss = maxwell_filter(raw, st_duration=tmax, st_correlation=1.,
                               st_only=True)
     assert_allclose(raw[:][0], raw_tsss[:][0])
     # degenerate
     assert_raises(ValueError, maxwell_filter, raw, st_only=True)  # no ST
     # two-step process equivalent to single-step process
-    raw_tsss = maxwell_filter(raw, st_duration=1., st_only=True)
+    raw_tsss = maxwell_filter(raw, st_duration=tmax, st_only=True)
     raw_tsss = maxwell_filter(raw_tsss)
-    raw_tsss_2 = maxwell_filter(raw, st_duration=1.)
+    raw_tsss_2 = maxwell_filter(raw, st_duration=tmax)
     assert_meg_snr(raw_tsss, raw_tsss_2, 1e5)
     # now also with head movement, and a bad MEG channel
     assert_equal(len(raw.info['bads']), 0)
-    raw.info['bads'] = ['EEG001', 'MEG2623']
-    raw_tsss = maxwell_filter(raw, st_duration=1., st_only=True,
+    bads = [raw.ch_names[0]]
+    raw.info['bads'] = list(bads)
+    raw_tsss = maxwell_filter(raw, st_duration=tmax, st_only=True,
                               head_pos=head_pos)
-    assert_equal(raw.info['bads'], ['EEG001', 'MEG2623'])
-    assert_equal(raw_tsss.info['bads'], ['EEG001', 'MEG2623'])  # don't reset
+    assert_equal(raw.info['bads'], bads)
+    assert_equal(raw_tsss.info['bads'], bads)  # don't reset
     raw_tsss = maxwell_filter(raw_tsss, head_pos=head_pos)
-    assert_equal(raw_tsss.info['bads'], ['EEG001'])  # do reset MEG bads
-    raw_tsss_2 = maxwell_filter(raw, st_duration=1., head_pos=head_pos)
-    assert_equal(raw_tsss_2.info['bads'], ['EEG001'])
+    assert_equal(raw_tsss.info['bads'], [])  # do reset MEG bads
+    raw_tsss_2 = maxwell_filter(raw, st_duration=tmax, head_pos=head_pos)
+    assert_equal(raw_tsss_2.info['bads'], [])
     assert_meg_snr(raw_tsss, raw_tsss_2, 1e5)
 
 
@@ -632,6 +613,20 @@ def test_fine_calibration():
     assert_allclose(py_cal['cal_chans'], mf_cal['cal_chans'])
     assert_allclose(py_cal['cal_corrs'], mf_cal['cal_corrs'],
                     rtol=1e-3, atol=1e-3)
+    # with missing channels
+    raw_missing = raw.copy().load_data()
+    raw_missing.info['bads'] = ['MEG0111', 'MEG0943']  # 1 mag, 1 grad
+    raw_missing.info._check_consistency()
+    raw_sss_bad = maxwell_filter(
+        raw_missing, calibration=fine_cal_fname, origin=mf_head_origin,
+        regularize=None, bad_condition='ignore')
+    raw_missing.pick_types()  # actually remove bads
+    raw_sss_bad.pick_channels(raw_missing.ch_names)  # remove them here, too
+    with warnings.catch_warnings(record=True):
+        raw_sss_missing = maxwell_filter(
+            raw_missing, calibration=fine_cal_fname, origin=mf_head_origin,
+            regularize=None, bad_condition='ignore')
+    assert_meg_snr(raw_sss_missing, raw_sss_bad, 1000., 10000.)
 
     # Test 3D SSS fine calibration (no equivalent func in MaxFilter yet!)
     # very low SNR as proc differs, eventually we should add a better test
@@ -644,7 +639,7 @@ def test_fine_calibration():
                   calibration=fine_cal_fname)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_regularization():
     """Test Maxwell filter regularization."""
@@ -765,7 +760,7 @@ def test_head_translation():
 def _assert_shielding(raw_sss, erm_power, shielding_factor, meg='mag'):
     """Helper to assert a minimum shielding factor using empty-room power."""
     picks = pick_types(raw_sss.info, meg=meg, ref_meg=False)
-    if isinstance(erm_power, _BaseRaw):
+    if isinstance(erm_power, BaseRaw):
         picks_erm = pick_types(raw_sss.info, meg=meg, ref_meg=False)
         assert_allclose(picks, picks_erm)
         erm_power = np.sqrt((erm_power[picks_erm][0] ** 2).sum())
@@ -777,14 +772,13 @@ def _assert_shielding(raw_sss, erm_power, shielding_factor, meg='mag'):
 
 
 @buggy_mkl_svd
-@slow_test
+@pytest.mark.slowtest
 @requires_svd_convergence
 @testing.requires_testing_data
 def test_shielding_factor():
     """Test Maxwell filter shielding factor using empty room."""
-    raw_erm = read_crop(erm_fname).load_data()
-    picks = pick_types(raw_erm.info, meg='mag')
-    erm_power = raw_erm[picks][0]
+    raw_erm = read_crop(erm_fname).load_data().pick_types(meg=True)
+    erm_power = raw_erm[pick_types(raw_erm.info, meg='mag')][0]
     erm_power = np.sqrt(np.sum(erm_power * erm_power))
     erm_power_grad = raw_erm[pick_types(raw_erm.info, meg='grad')][0]
     erm_power_grad = np.sqrt(np.sum(erm_power * erm_power))
@@ -895,7 +889,7 @@ def test_shielding_factor():
     _assert_shielding(raw_sss, erm_power, 44)
 
 
-@slow_test
+@pytest.mark.slowtest
 @requires_svd_convergence
 @testing.requires_testing_data
 def test_all():
@@ -930,7 +924,7 @@ def test_all():
         assert_meg_snr(sss_py, sss_mf, mins[ii], meds[ii], msg=rf)
 
 
-@slow_test
+@pytest.mark.slowtest
 @requires_svd_convergence
 @testing.requires_testing_data
 def test_triux():
@@ -971,4 +965,12 @@ def test_triux():
                             st_duration=4., verbose=True)
     assert_meg_snr(sss_py, read_crop(tri_sss_st4_fname), 700., 1600)
 
+
+@testing.requires_testing_data
+def test_MGH_cross_talk():
+    raw = read_crop(raw_fname, (0., 1.))
+    raw_sss = maxwell_filter(raw, cross_talk=ctc_mgh_fname)
+    py_ctc = raw_sss.info['proc_history'][0]['max_info']['sss_ctc']
+    assert_true(len(py_ctc) > 0)
+
 run_tests_if_main()
