@@ -23,17 +23,17 @@ def test_fix_stim_artifact():
     """Test fix stim artifact."""
     events = read_events(event_fname)
 
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     assert_raises(RuntimeError, fix_stim_artifact, raw)
 
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False, preload=True)
+    raw = read_raw_fif(raw_fname, preload=True)
 
     # use window before stimulus in epochs
     tmin, tmax, event_id = -0.2, 0.5, 1
     picks = pick_types(raw.info, meg=True, eeg=True,
                        eog=True, stim=False, exclude='bads')
     epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
-                    preload=True, reject=None, add_eeg_ref=False)
+                    preload=True, reject=None)
     e_start = int(np.ceil(epochs.info['sfreq'] * epochs.tmin))
     tmin, tmax = -0.045, -0.015
     tmin_samp = int(-0.035 * epochs.info['sfreq']) - e_start
@@ -72,8 +72,7 @@ def test_fix_stim_artifact():
     # get epochs from raw with fixed data
     tmin, tmax, event_id = -0.2, 0.5, 1
     epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
-                    preload=True, reject=None, baseline=None,
-                    add_eeg_ref=False)
+                    preload=True, reject=None, baseline=None)
     e_start = int(np.ceil(epochs.info['sfreq'] * epochs.tmin))
     tmin_samp = int(-0.035 * epochs.info['sfreq']) - e_start
     tmax_samp = int(-0.015 * epochs.info['sfreq']) - e_start
