@@ -74,6 +74,9 @@ def test_fieldtrip_client():
                 epoch2 = rt_client.get_data_as_epoch(n_samples=5, picks=picks)
                 n_channels2, n_samples2 = epoch2.get_data().shape[1:]
 
+                # case of picks=None
+                epoch = rt_client.get_data_as_epoch(n_samples=5)
+
         assert_true(tmin_samp2 > tmin_samp1)
         assert_true(len(w) >= 1)
         assert_equal(n_samples, 5)
