@@ -1,5 +1,4 @@
-"""Generate html report from MNE database
-"""
+"""Generate self-contained HTML reports from MNE objects."""
 
 # Authors: Alex Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Mainak Jas <mainak@neuro.hut.fi>
@@ -15,14 +14,14 @@ import codecs
 import time
 from glob import glob
 import base64
-from datetime import datetime as dt
 
 import numpy as np
 
 from . import read_evokeds, read_events, pick_types, read_cov
-from .io import Raw, read_info
-from .utils import _TempDir, logger, verbose, get_subjects_dir, warn
-from .viz import plot_events, plot_trans, plot_cov
+from .io import Raw, read_info, _stamp_to_dt
+from .utils import (_TempDir, logger, verbose, get_subjects_dir, warn,
+                    _import_mlab)
+from .viz import plot_events, plot_alignment, plot_cov
 from .viz._3d import _plot_mri_contours
 from .forward import read_forward_solution
 from .epochs import read_epochs
@@ -47,19 +46,19 @@ SECTION_ORDER = ['raw', 'events', 'epochs', 'evoked', 'covariance', 'trans',
 
 def _fig_to_img(function=None, fig=None, image_format='png',
                 scale=None, **kwargs):
-    """Wrapper function to plot figure and create a binary image"""
-
+    """Plot figure and create a binary image."""
     import matplotlib.pyplot as plt
     from matplotlib.figure import Figure
     if not isinstance(fig, Figure) and function is None:
         from scipy.misc import imread
         mlab = None
         try:
-            from mayavi import mlab  # noqa
-        except:  # on some systems importing Mayavi raises SystemExit (!)
-            warn('Could not import mayavi. Trying to render'
+            mlab = _import_mlab()
+        # on some systems importing Mayavi raises SystemExit (!)
+        except Exception as e:
+            warn('Could not import mayavi (%r). Trying to render'
                  '`mayavi.core.scene.Scene` figure instances'
-                 ' will throw an error.')
+                 ' will throw an error.' % (e,))
         tempdir = _TempDir()
         temp_fname = op.join(tempdir, 'test')
         if fig.scene is not None:
@@ -80,8 +79,11 @@ def _fig_to_img(function=None, fig=None, image_format='png',
     output = BytesIO()
     if scale is not None:
         _scale_mpl_figure(fig, scale)
-    fig.savefig(output, format=image_format, bbox_inches='tight',
-                dpi=fig.get_dpi())
+    logger.debug('Saving figure %s with dpi %s'
+                 % (fig.get_size_inches(), fig.get_dpi()))
+    # We don't use bbox_inches='tight' here because it can break
+    # newer matplotlib, and should only save a little bit of space
+    fig.savefig(output, format=image_format, dpi=fig.get_dpi())
     plt.close(fig)
     output = output.getvalue()
     return (output if image_format == 'svg' else
@@ -89,7 +91,7 @@ def _fig_to_img(function=None, fig=None, image_format='png',
 
 
 def _scale_mpl_figure(fig, scale):
-    """Magic scaling helper
+    """Magic scaling helper.
 
     Keeps font-size and artist sizes constant
     0.5 : current font - 4pt
@@ -128,8 +130,7 @@ def _figs_to_mrislices(sl, n_jobs, **kwargs):
 
 
 def _iterate_trans_views(function, **kwargs):
-    """Auxiliary function to iterate over views in trans fig.
-    """
+    """Auxiliary function to iterate over views in trans fig."""
     from scipy.misc import imread
     import matplotlib.pyplot as plt
     import mayavi
@@ -153,7 +154,7 @@ def _iterate_trans_views(function, **kwargs):
         ax.axis('off')
 
     mayavi.mlab.close(fig)
-    img = _fig_to_img(fig=fig2)
+    img = _fig_to_img(fig=fig2, image_format='png')
     return img
 
 ###############################################################################
@@ -161,9 +162,7 @@ def _iterate_trans_views(function, **kwargs):
 
 
 def _is_bad_fname(fname):
-    """Auxiliary function for identifying bad file naming patterns
-       and highlighting them in red in the TOC.
-    """
+    """Identify bad file naming patterns and highlight them in the TOC."""
     if fname.endswith('(whitened)'):
         fname = fname[:-11]
 
@@ -174,7 +173,7 @@ def _is_bad_fname(fname):
 
 
 def _get_fname(fname):
-    """Get fname without -#-"""
+    """Get fname without -#-."""
     if '-#-' in fname:
         fname = fname.split('-#-')[0]
     else:
@@ -184,9 +183,7 @@ def _get_fname(fname):
 
 
 def _get_toc_property(fname):
-    """Auxiliary function to assign class names to TOC
-       list elements to allow toggling with buttons.
-    """
+    """Assign class names to TOC elements to allow toggling with buttons."""
     if fname.endswith(('-eve.fif', '-eve.fif.gz')):
         div_klass = 'events'
         tooltip = fname
@@ -240,9 +237,9 @@ def _get_toc_property(fname):
     return div_klass, tooltip, text
 
 
-def _iterate_files(report, fnames, info, cov, baseline, sfreq, on_error):
-    """Auxiliary function to parallel process in batch mode.
-    """
+def _iterate_files(report, fnames, info, cov, baseline, sfreq, on_error,
+                   image_format):
+    """Parallel process in batch mode."""
     htmls, report_fnames, report_sectionlabels = [], [], []
 
     def _update_html(html, report_fname, report_sectionlabel):
@@ -271,25 +268,26 @@ def _iterate_files(report, fnames, info, cov, baseline, sfreq, on_error):
                 report_sectionlabel = 'inverse'
             elif fname.endswith(('-ave.fif', '-ave.fif.gz')):
                 if cov is not None:
-                    html = report._render_whitened_evoked(fname, cov, baseline)
+                    html = report._render_whitened_evoked(fname, cov, baseline,
+                                                          image_format)
                     report_fname = fname + ' (whitened)'
                     report_sectionlabel = 'evoked'
                     _update_html(html, report_fname, report_sectionlabel)
 
-                html = report._render_evoked(fname, baseline)
+                html = report._render_evoked(fname, baseline, image_format)
                 report_fname = fname
                 report_sectionlabel = 'evoked'
             elif fname.endswith(('-eve.fif', '-eve.fif.gz')):
-                html = report._render_eve(fname, sfreq)
+                html = report._render_eve(fname, sfreq, image_format)
                 report_fname = fname
                 report_sectionlabel = 'events'
             elif fname.endswith(('-epo.fif', '-epo.fif.gz')):
-                html = report._render_epochs(fname)
+                html = report._render_epochs(fname, image_format)
                 report_fname = fname
                 report_sectionlabel = 'epochs'
             elif (fname.endswith(('-cov.fif', '-cov.fif.gz')) and
                   report.info_fname is not None):
-                html = report._render_cov(fname, info)
+                html = report._render_cov(fname, info, image_format)
                 report_fname = fname
                 report_sectionlabel = 'covariance'
             elif (fname.endswith(('-trans.fif', '-trans.fif.gz')) and
@@ -320,10 +318,8 @@ def _iterate_files(report, fnames, info, cov, baseline, sfreq, on_error):
 # IMAGE FUNCTIONS
 
 
-def _build_image(data, cmap='gray'):
-    """Build an image encoded in base64.
-    """
-
+def _build_image_png(data, cmap='gray'):
+    """Build an image encoded in base64."""
     import matplotlib.pyplot as plt
     from matplotlib.figure import Figure
     from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
@@ -342,8 +338,7 @@ def _build_image(data, cmap='gray'):
 
 
 def _iterate_sagittal_slices(array, limits=None):
-    """Iterate sagittal slice.
-    """
+    """Iterate sagittal slices."""
     shape = array.shape[0]
     for ind in range(shape):
         if limits and ind not in limits:
@@ -352,8 +347,7 @@ def _iterate_sagittal_slices(array, limits=None):
 
 
 def _iterate_axial_slices(array, limits=None):
-    """Iterate axial slice.
-    """
+    """Iterate axial slices."""
     shape = array.shape[1]
     for ind in range(shape):
         if limits and ind not in limits:
@@ -362,8 +356,7 @@ def _iterate_axial_slices(array, limits=None):
 
 
 def _iterate_coronal_slices(array, limits=None):
-    """Iterate coronal slice.
-    """
+    """Iterate coronal slices."""
     shape = array.shape[2]
     for ind in range(shape):
         if limits and ind not in limits:
@@ -371,41 +364,44 @@ def _iterate_coronal_slices(array, limits=None):
         yield ind, np.flipud(np.rot90(array[:, :, ind]))
 
 
-def _iterate_mri_slices(name, ind, global_id, slides_klass, data, cmap,
-                        image_format='png'):
-    """Auxiliary function for parallel processing of mri slices.
-    """
+def _iterate_mri_slices(name, ind, global_id, slides_klass, data, cmap):
+    """Auxiliary function for parallel processing of mri slices."""
     img_klass = 'slideimg-%s' % name
 
     caption = u'Slice %s %s' % (name, ind)
     slice_id = '%s-%s-%s' % (name, global_id, ind)
     div_klass = 'span12 %s' % slides_klass
-    img = _build_image(data, cmap=cmap)
+    img = _build_image_png(data, cmap=cmap)
     first = True if ind == 0 else False
-    html = _build_html_image(img, slice_id, div_klass,
-                             img_klass, caption, first)
+    html = _build_html_image(img, slice_id, div_klass, img_klass, caption,
+                             first, image_format='png')
     return ind, html
 
 
 ###############################################################################
 # HTML functions
 
-def _build_html_image(img, id, div_klass, img_klass, caption=None, show=True):
-    """Build a html image from a slice array.
-    """
+def _build_html_image(img, id, div_klass, img_klass, caption=None,
+                      show=True, image_format='png'):
+    """Build a html image from a slice array."""
     html = []
     add_style = u'' if show else u'style="display: none"'
     html.append(u'<li class="%s" id="%s" %s>' % (div_klass, id, add_style))
     html.append(u'<div class="thumbnail">')
-    html.append(u'<img class="%s" alt="" style="width:90%%;" '
-                'src="data:image/png;base64,%s">'
-                % (img_klass, img))
+    if image_format == 'png':
+        html.append(u'<img class="%s" alt="" style="width:90%%;" '
+                    'src="data:image/png;base64,%s">'
+                    % (img_klass, img))
+    else:
+        html.append(u'<div style="text-align:center;" class="%s">%s</div>'
+                    % (img_klass, img))
     html.append(u'</div>')
     if caption:
         html.append(u'<h4>%s</h4>' % caption)
     html.append(u'</li>')
     return u'\n'.join(html)
 
+
 slider_template = HTMLTemplate(u"""
 <script>$("#{{slider_id}}").slider({
                        range: "min",
@@ -446,8 +442,7 @@ slider_full_template = Template(u"""
 
 def _build_html_slider(slices_range, slides_klass, slider_id,
                        start_value=None):
-    """Build an html slider for a given slices range and a slices klass.
-    """
+    """Build an html slider for a given slices range and a slices klass."""
     if start_value is None:
         start_value = slices_range[len(slices_range) // 2]
     return slider_template.substitute(slider_id=slider_id,
@@ -758,13 +753,27 @@ toc_list = Template(u"""
 
 
 def _check_scale(scale):
-    """Helper to ensure valid scale value is passed"""
+    """Ensure valid scale value is passed."""
     if np.isscalar(scale) and scale <= 0:
         raise ValueError('scale must be positive, not %s' % scale)
 
 
+def _check_image_format(rep, image_format):
+    """Ensure fmt is valid."""
+    if image_format not in ('png', 'svg'):
+        if rep is None:
+            raise ValueError('image_format must be "svg" or "png", got %s'
+                             % (image_format,))
+        elif image_format is not None:
+            raise ValueError('image_format must be one of "svg", "png", or '
+                             'None, got %s' % (image_format,))
+        else:  # rep is not None and image_format is None
+            image_format = rep.image_format
+    return image_format
+
+
 class Report(object):
-    """Object for rendering HTML
+    """Object for rendering HTML.
 
     Parameters
     ----------
@@ -789,8 +798,16 @@ class Report(object):
         interval is used.
         The baseline (a, b) includes both endpoints, i.e. all
         timepoints t such that a <= t <= b.
+    image_format : str
+        Default image format to use (default is 'png').
+        SVG uses vector graphics, so fidelity is higher but can increase
+        file size and browser image rendering time as well.
+
+        .. versionadded:: 0.15
+
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Notes
     -----
@@ -801,14 +818,14 @@ class Report(object):
 
     def __init__(self, info_fname=None, subjects_dir=None,
                  subject=None, title=None, cov_fname=None, baseline=None,
-                 verbose=None):
-
+                 image_format='png', verbose=None):  # noqa: D102
         self.info_fname = info_fname
         self.cov_fname = cov_fname
         self.baseline = baseline
         self.subjects_dir = get_subjects_dir(subjects_dir, raise_error=False)
         self.subject = subject
         self.title = title
+        self.image_format = _check_image_format(None, image_format)
         self.verbose = verbose
 
         self.initial_id = 0
@@ -839,18 +856,16 @@ class Report(object):
         return s
 
     def __len__(self):
-        """The number of items in report."""
+        """Return the number of items in report."""
         return len(self.fnames)
 
     def _get_id(self):
-        """Get id of plot.
-        """
+        """Get id of plot."""
         self.initial_id += 1
         return self.initial_id
 
     def _validate_input(self, items, captions, section, comments=None):
-        """Validate input.
-        """
+        """Validate input."""
         if not isinstance(items, (list, tuple)):
             items = [items]
         if not isinstance(captions, (list, tuple)):
@@ -876,9 +891,7 @@ class Report(object):
 
     def _add_figs_to_section(self, figs, captions, section='custom',
                              image_format='png', scale=None, comments=None):
-        """Auxiliary method for `add_section` and `add_figs_to_section`.
-        """
-
+        """Auxiliary method for `add_section` and `add_figs_to_section`."""
         figs, captions, comments = self._validate_input(figs, captions,
                                                         section, comments)
         _check_scale(scale)
@@ -903,7 +916,7 @@ class Report(object):
             self.html.append(html)
 
     def add_figs_to_section(self, figs, captions, section='custom',
-                            scale=None, image_format='png', comments=None):
+                            scale=None, image_format=None, comments=None):
         """Append custom user-defined figures.
 
         Parameters
@@ -923,12 +936,15 @@ class Report(object):
             the relative scaling (might not work for scale <= 1 depending on
             font sizes). If function, should take a figure object as input
             parameter. Defaults to None.
-        image_format : {'png', 'svg'}
-            The image format to be used for the report. Defaults to 'png'.
+        image_format : str | None
+            The image format to be used for the report, can be 'png' or 'svd'.
+            None (default) will use the default specified during Report
+            class construction.
         comments : None | str | list of str
             A string of text or a list of strings of text to be appended after
             the figure.
         """
+        image_format = _check_image_format(self, image_format)
         return self._add_figs_to_section(figs=figs, captions=captions,
                                          section=section, scale=scale,
                                          image_format=image_format,
@@ -977,10 +993,8 @@ class Report(object):
                                  "'svg' are supported. Got %s" % image_format)
 
             # Convert image to binary string.
-            output = BytesIO()
             with open(fname, 'rb') as f:
-                output.write(f.read())
-            img = base64.b64encode(output.getvalue()).decode('ascii')
+                img = base64.b64encode(f.read()).decode('ascii')
             html = image_template.substitute(img=img, id=global_id,
                                              image_format=image_format,
                                              div_klass=div_klass,
@@ -1025,7 +1039,7 @@ class Report(object):
 
     def add_bem_to_section(self, subject, caption='BEM', section='bem',
                            decim=2, n_jobs=1, subjects_dir=None):
-        """Renders a bem slider html str.
+        """Render a bem slider html str.
 
         Parameters
         ----------
@@ -1061,8 +1075,8 @@ class Report(object):
         self.html.extend(html)
 
     def add_slider_to_section(self, figs, captions=None, section='custom',
-                              title='Slider', scale=None, image_format='png'):
-        """Renders a slider of figs to the report.
+                              title='Slider', scale=None, image_format=None):
+        """Render a slider of figs to the report.
 
         Parameters
         ----------
@@ -1085,15 +1099,17 @@ class Report(object):
             the relative scaling (might not work for scale <= 1 depending on
             font sizes). If function, should take a figure object as input
             parameter. Defaults to None.
-        image_format : {'png', 'svg'}
-            The image format to be used for the report. Defaults to 'png'.
+        image_format : str | None
+            The image format to be used for the report, can be 'png' or 'svd'.
+            None (default) will use the default specified during Report
+            class construction.
 
         Notes
         -----
         .. versionadded:: 0.10.0
         """
-
         _check_scale(scale)
+        image_format = _check_image_format(self, image_format)
         if not isinstance(figs[0], list):
             figs = [figs]
         else:
@@ -1133,7 +1149,8 @@ class Report(object):
             slice_id = '%s-%s-%s' % (name, global_id, sl[ii])
             first = True if ii == 0 else False
             slices.append(_build_html_image(img, slice_id, div_klass,
-                          img_klass, caption, first))
+                          img_klass, caption, first,
+                          image_format=image_format))
         # Render the slider
         slider_id = 'select-%s-%s' % (name, global_id)
         # Render the slices
@@ -1155,8 +1172,7 @@ class Report(object):
     # HTML rendering
     def _render_one_axis(self, slices_iter, name, global_id, cmap,
                          n_elements, n_jobs):
-        """Render one axis of the array.
-        """
+        """Render one axis of the array."""
         global_id = global_id or name
         html = []
         slices, slices_range = [], []
@@ -1184,9 +1200,7 @@ class Report(object):
     # global rendering functions
     @verbose
     def _init_render(self, verbose=None):
-        """Initialize the renderer.
-        """
-
+        """Initialize the renderer."""
         inc_fnames = ['jquery-1.10.2.min.js', 'jquery-ui.min.js',
                       'bootstrap.min.js', 'jquery-ui.min.css',
                       'bootstrap.min.css']
@@ -1208,8 +1222,9 @@ class Report(object):
 
     @verbose
     def parse_folder(self, data_path, pattern='*.fif', n_jobs=1, mri_decim=2,
-                     sort_sections=True, on_error='warn', verbose=None):
-        """Renders all the files in the folder.
+                     sort_sections=True, on_error='warn', image_format=None,
+                     verbose=None):
+        r"""Render all the files in the folder.
 
         Parameters
         ----------
@@ -1231,9 +1246,18 @@ class Report(object):
         on_error : str
             What to do if a file cannot be rendered. Can be 'ignore',
             'warn' (default), or 'raise'.
+        image_format : str | None
+            The image format to be used for the report, can be 'png' or 'svd'.
+            None (default) will use the default specified during Report
+            class construction.
+
+            .. versionadded:: 0.15
         verbose : bool, str, int, or None
-            If not None, override default verbose level (see mne.verbose).
+            If not None, override default verbose level (see
+            :func:`mne.verbose` and :ref:`Logging documentation <tut_logging>`
+            for more).
         """
+        image_format = _check_image_format(self, image_format)
         valid_errors = ['ignore', 'warn', 'raise']
         if on_error not in valid_errors:
             raise ValueError('on_error must be one of %s, not %s'
@@ -1272,7 +1296,8 @@ class Report(object):
                     'time)' % len(fnames))
         use_jobs = min(n_jobs, max(1, len(fnames)))
         parallel, p_fun, _ = parallel_func(_iterate_files, use_jobs)
-        r = parallel(p_fun(self, fname, info, cov, baseline, sfreq, on_error)
+        r = parallel(p_fun(self, fname, info, cov, baseline, sfreq, on_error,
+                           image_format)
                      for fname in np.array_split(fnames, use_jobs))
         htmls, report_fnames, report_sectionlabels = zip(*r)
 
@@ -1311,7 +1336,6 @@ class Report(object):
         overwrite : bool
             If True, overwrite report if it already exists.
         """
-
         if fname is None:
             if not hasattr(self, 'data_path'):
                 self.data_path = op.dirname(__file__)
@@ -1354,9 +1378,7 @@ class Report(object):
 
     @verbose
     def _render_toc(self, verbose=None):
-        """Render the Table of Contents.
-        """
-
+        """Render the Table of Contents."""
         logger.info('Rendering : Table of Contents')
 
         html_toc = u'<div id="container">'
@@ -1434,8 +1456,7 @@ class Report(object):
 
     def _render_array(self, array, global_id=None, cmap='gray',
                       limits=None, n_jobs=1):
-        """Render mri without bem contours.
-        """
+        """Render mri without bem contours (only PNG)."""
         html = []
         html.append(u'<div class="row">')
         # Axial
@@ -1465,8 +1486,7 @@ class Report(object):
 
     def _render_one_bem_axis(self, mri_fname, surf_fnames, global_id,
                              shape, orientation='coronal', decim=2, n_jobs=1):
-        """Render one axis of bem contours.
-        """
+        """Render one axis of bem contours (only PNG)."""
         orientation_name2axis = dict(sagittal=0, axial=1, coronal=2)
         orientation_axis = orientation_name2axis[orientation]
         n_slices = shape[orientation_axis]
@@ -1489,7 +1509,8 @@ class Report(object):
             caption = u'Slice %s %s' % (name, sl[ii])
             first = True if ii == 0 else False
             slices.append(_build_html_image(img, slice_id, div_klass,
-                          img_klass, caption, first))
+                                            img_klass, caption, first,
+                                            image_format='png'))
 
         # Render the slider
         slider_id = 'select-%s-%s' % (name, global_id)
@@ -1502,9 +1523,8 @@ class Report(object):
         html.append(u'</div>')
         return '\n'.join(html)
 
-    def _render_image(self, image, cmap='gray', n_jobs=1):
-        """Render one slice of mri without bem.
-        """
+    def _render_image_png(self, image, cmap='gray', n_jobs=1):
+        """Render one slice of mri without bem as a PNG."""
         import nibabel as nib
 
         global_id = self._get_id()
@@ -1523,16 +1543,13 @@ class Report(object):
         html = u'<li class="mri" id="%d">\n' % global_id
         html += u'<h2>%s</h2>\n' % name
         html += self._render_array(data, global_id=global_id,
-                                   cmap=cmap, limits=limits,
-                                   n_jobs=n_jobs)
+                                   cmap=cmap, limits=limits, n_jobs=n_jobs)
         html += u'</li>\n'
         return html
 
     def _render_raw(self, raw_fname):
-        """Render raw.
-        """
+        """Render raw (only text)."""
         global_id = self._get_id()
-        div_klass = 'raw'
         caption = u'Raw : %s' % raw_fname
 
         raw = Raw(raw_fname)
@@ -1552,23 +1569,18 @@ class Report(object):
             ecg = 'Not available'
         meas_date = raw.info['meas_date']
         if meas_date is not None:
-            meas_date = dt.fromtimestamp(meas_date[0]).strftime("%B %d, %Y")
+            meas_date = _stamp_to_dt(meas_date).strftime("%B %d, %Y") + ' GMT'
         tmin = raw.first_samp / raw.info['sfreq']
         tmax = raw.last_samp / raw.info['sfreq']
 
-        html = raw_template.substitute(div_klass=div_klass,
-                                       id=global_id,
-                                       caption=caption,
-                                       info=raw.info,
-                                       meas_date=meas_date,
-                                       n_eeg=n_eeg, n_grad=n_grad,
-                                       n_mag=n_mag, eog=eog,
-                                       ecg=ecg, tmin=tmin, tmax=tmax)
+        html = raw_template.substitute(
+            div_klass='raw', id=global_id, caption=caption, info=raw.info,
+            meas_date=meas_date, n_eeg=n_eeg, n_grad=n_grad, n_mag=n_mag,
+            eog=eog, ecg=ecg, tmin=tmin, tmax=tmax)
         return html
 
     def _render_forward(self, fwd_fname):
-        """Render forward.
-        """
+        """Render forward."""
         div_klass = 'forward'
         caption = u'Forward: %s' % fwd_fname
         fwd = read_forward_solution(fwd_fname)
@@ -1581,8 +1593,7 @@ class Report(object):
         return html
 
     def _render_inverse(self, inv_fname):
-        """Render inverse.
-        """
+        """Render inverse."""
         div_klass = 'inverse'
         caption = u'Inverse: %s' % inv_fname
         inv = read_inverse_operator(inv_fname)
@@ -1594,27 +1605,24 @@ class Report(object):
                                         repr=repr_inv)
         return html
 
-    def _render_evoked(self, evoked_fname, baseline=None, figsize=None):
-        """Render evoked.
-        """
+    def _render_evoked(self, evoked_fname, baseline, image_format):
+        """Render evoked."""
+        logger.debug('Evoked: Reading %s' % evoked_fname)
         evokeds = read_evokeds(evoked_fname, baseline=baseline, verbose=False)
 
         html = []
-        for ev in evokeds:
+        for ei, ev in enumerate(evokeds):
             global_id = self._get_id()
-
             kwargs = dict(show=False)
-            img = _fig_to_img(ev.plot, **kwargs)
-
+            logger.debug('Evoked: Plotting instance %s/%s'
+                         % (ei + 1, len(evokeds)))
+            img = _fig_to_img(ev.plot, image_format=image_format,
+                              **kwargs)
             caption = u'Evoked : %s (%s)' % (evoked_fname, ev.comment)
-            div_klass = 'evoked'
-            img_klass = 'evoked'
-            show = True
-            html.append(image_template.substitute(img=img, id=global_id,
-                                                  div_klass=div_klass,
-                                                  img_klass=img_klass,
-                                                  caption=caption,
-                                                  show=show))
+            html.append(image_template.substitute(
+                img=img, id=global_id, div_klass='evoked',
+                img_klass='evoked', caption=caption, show=True,
+                image_format=image_format))
             has_types = []
             if len(pick_types(ev.info, meg=False, eeg=True)) > 0:
                 has_types.append('eeg')
@@ -1623,134 +1631,100 @@ class Report(object):
             if len(pick_types(ev.info, meg='mag', eeg=False)) > 0:
                 has_types.append('mag')
             for ch_type in has_types:
-                kwargs.update(ch_type=ch_type)
-                img = _fig_to_img(ev.plot_topomap, **kwargs)
+                logger.debug('    Topomap type %s' % ch_type)
+                img = _fig_to_img(ev.plot_topomap, ch_type=ch_type,
+                                  image_format=image_format,
+                                  **kwargs)
                 caption = u'Topomap (ch_type = %s)' % ch_type
-                html.append(image_template.substitute(img=img,
-                                                      div_klass=div_klass,
-                                                      img_klass=img_klass,
-                                                      caption=caption,
-                                                      show=show))
-
+                html.append(image_template.substitute(
+                    img=img, div_klass='evoked', img_klass='evoked',
+                    caption=caption, show=True, image_format=image_format))
+        logger.debug('Evoked: done')
         return '\n'.join(html)
 
-    def _render_eve(self, eve_fname, sfreq=None):
-        """Render events.
-        """
+    def _render_eve(self, eve_fname, sfreq, image_format):
+        """Render events."""
         global_id = self._get_id()
         events = read_events(eve_fname)
-
         kwargs = dict(events=events, sfreq=sfreq, show=False)
-        img = _fig_to_img(plot_events, **kwargs)
-
+        img = _fig_to_img(plot_events, image_format=image_format,
+                          **kwargs)
         caption = 'Events : ' + eve_fname
-        div_klass = 'events'
-        img_klass = 'events'
-        show = True
-
-        html = image_template.substitute(img=img, id=global_id,
-                                         div_klass=div_klass,
-                                         img_klass=img_klass,
-                                         caption=caption,
-                                         show=show)
+        html = image_template.substitute(
+            img=img, id=global_id, div_klass='events', img_klass='events',
+            caption=caption, show=True, image_format=image_format)
         return html
 
-    def _render_epochs(self, epo_fname):
-        """Render epochs.
-        """
+    def _render_epochs(self, epo_fname, image_format):
+        """Render epochs."""
         global_id = self._get_id()
-
         epochs = read_epochs(epo_fname)
         kwargs = dict(subject=self.subject, show=False)
-        img = _fig_to_img(epochs.plot_drop_log, **kwargs)
+        img = _fig_to_img(epochs.plot_drop_log, image_format=image_format,
+                          **kwargs)
         caption = 'Epochs : ' + epo_fname
-        div_klass = 'epochs'
-        img_klass = 'epochs'
         show = True
-        html = image_template.substitute(img=img, id=global_id,
-                                         div_klass=div_klass,
-                                         img_klass=img_klass,
-                                         caption=caption,
-                                         show=show)
+        html = image_template.substitute(
+            img=img, id=global_id, div_klass='epochs', img_klass='epochs',
+            caption=caption, show=show, image_format=image_format)
         return html
 
-    def _render_cov(self, cov_fname, info_fname):
-        """Render cov.
-        """
+    def _render_cov(self, cov_fname, info_fname, image_format):
+        """Render cov."""
         global_id = self._get_id()
         cov = read_cov(cov_fname)
         fig, _ = plot_cov(cov, info_fname, show=False)
-        img = _fig_to_img(fig=fig)
+        img = _fig_to_img(fig=fig, image_format=image_format)
         caption = 'Covariance : %s (n_samples: %s)' % (cov_fname, cov.nfree)
-        div_klass = 'covariance'
-        img_klass = 'covariance'
         show = True
-        html = image_template.substitute(img=img, id=global_id,
-                                         div_klass=div_klass,
-                                         img_klass=img_klass,
-                                         caption=caption,
-                                         show=show)
+        html = image_template.substitute(
+            img=img, id=global_id, div_klass='covariance',
+            img_klass='covariance', caption=caption, show=show,
+            image_format=image_format)
         return html
 
-    def _render_whitened_evoked(self, evoked_fname, noise_cov, baseline):
-        """Show whitened evoked.
-        """
-        global_id = self._get_id()
-
+    def _render_whitened_evoked(self, evoked_fname, noise_cov, baseline,
+                                image_format):
+        """Render whitened evoked."""
         evokeds = read_evokeds(evoked_fname, verbose=False)
-
         html = []
         for ev in evokeds:
-
             ev = read_evokeds(evoked_fname, ev.comment, baseline=baseline,
                               verbose=False)
-
             global_id = self._get_id()
-
             kwargs = dict(noise_cov=noise_cov, show=False)
-            img = _fig_to_img(ev.plot_white, **kwargs)
+            img = _fig_to_img(ev.plot_white, image_format=image_format,
+                              **kwargs)
 
             caption = u'Whitened evoked : %s (%s)' % (evoked_fname, ev.comment)
-            div_klass = 'evoked'
-            img_klass = 'evoked'
             show = True
-            html.append(image_template.substitute(img=img, id=global_id,
-                                                  div_klass=div_klass,
-                                                  img_klass=img_klass,
-                                                  caption=caption,
-                                                  show=show))
+            html.append(image_template.substitute(
+                img=img, id=global_id, div_klass='evoked',
+                img_klass='evoked', caption=caption, show=show,
+                image_format=image_format))
         return '\n'.join(html)
 
-    def _render_trans(self, trans, path, info, subject,
-                      subjects_dir, image_format='png'):
-        """Render trans.
-        """
+    def _render_trans(self, trans, path, info, subject, subjects_dir):
+        """Render trans (only PNG)."""
         kwargs = dict(info=info, trans=trans, subject=subject,
                       subjects_dir=subjects_dir)
         try:
-            img = _iterate_trans_views(function=plot_trans, **kwargs)
+            img = _iterate_trans_views(function=plot_alignment, **kwargs)
         except IOError:
-            img = _iterate_trans_views(function=plot_trans, source='head',
-                                       **kwargs)
+            img = _iterate_trans_views(function=plot_alignment,
+                                       surfaces=['head'], **kwargs)
 
         if img is not None:
             global_id = self._get_id()
-            caption = 'Trans : ' + trans
-            div_klass = 'trans'
-            img_klass = 'trans'
-            show = True
-            html = image_template.substitute(img=img, id=global_id,
-                                             div_klass=div_klass,
-                                             img_klass=img_klass,
-                                             caption=caption,
-                                             width=75,
-                                             show=show)
+            html = image_template.substitute(
+                img=img, id=global_id, div_klass='trans',
+                img_klass='trans', caption='Trans : ' + trans, width=75,
+                show=True, image_format='png')
             return html
 
     def _render_bem(self, subject, subjects_dir, decim, n_jobs,
                     section='mri', caption='BEM'):
-        """Render mri+bem.
-        """
+        """Render mri+bem (only PNG)."""
         import nibabel as nib
 
         subjects_dir = get_subjects_dir(subjects_dir, raise_error=True)
@@ -1765,7 +1739,8 @@ class Report(object):
 
         if not op.isdir(bem_path):
             warn('Subject bem directory "%s" does not exist' % bem_path)
-            return self._render_image(mri_fname, cmap='gray', n_jobs=n_jobs)
+            return self._render_image_png(mri_fname, cmap='gray',
+                                          n_jobs=n_jobs)
 
         surf_fnames = []
         for surf_name in ['*inner_skull', '*outer_skull', '*outer_skin']:
@@ -1777,7 +1752,7 @@ class Report(object):
                 continue
         if len(surf_fnames) == 0:
             warn('No surfaces found at all, rendering empty MRI')
-            return self._render_image(mri_fname, cmap='gray')
+            return self._render_image_png(mri_fname, cmap='gray')
         # XXX : find a better way to get max range of slices
         nim = nib.load(mri_fname)
         data = nim.get_data()
@@ -1810,7 +1785,6 @@ class Report(object):
 
 
 def _clean_varnames(s):
-
     # Remove invalid characters
     s = re.sub('[^0-9a-zA-Z_]', '', s)
 
@@ -1820,8 +1794,7 @@ def _clean_varnames(s):
 
 
 def _recursive_search(path, pattern):
-    """Auxiliary function for recursive_search of the directory.
-    """
+    """Auxiliary function for recursive_search of the directory."""
     filtered_files = list()
     for dirpath, dirnames, files in os.walk(path):
         for f in fnmatch.filter(files, pattern):
@@ -1834,10 +1807,8 @@ def _recursive_search(path, pattern):
 
 
 def _fix_global_ids(html):
-    """Auxiliary function for fixing the global_ids after reordering in
-       _render_toc().
-    """
-    html = re.sub('id="\d+"', 'id="###"', html)
+    """Fix the global_ids after reordering in _render_toc()."""
+    html = re.sub(r'id="\d+"', 'id="###"', html)
     global_id = 1
     while len(re.findall('id="###"', html)) > 0:
         html = re.sub('id="###"', 'id="%s"' % global_id, html, count=1)
