@@ -5,24 +5,25 @@
 # License: BSD (3-clause)
 import copy
 import warnings
+import math
 
 import numpy as np
 
 from ..io.pick import pick_channels_cov
 from ..forward import apply_forward
-from ..utils import check_random_state, verbose, _time_mask
+from ..utils import check_random_state, verbose, _time_mask, warn, deprecated
 
 
 @verbose
-def simulate_evoked(fwd, stc, info, cov, snr=3., tmin=None, tmax=None,
-                    iir_filter=None, random_state=None, verbose=None):
-    """Generate noisy evoked data
+def simulate_evoked(fwd, stc, info, cov, nave=30, tmin=None, tmax=None,
+                    iir_filter=None, random_state=None, snr=None, use_cps=None,
+                    verbose=None):
+    """Generate noisy evoked data.
 
     .. note:: No projections from ``info`` will be present in the
               output ``evoked``. You can use e.g.
               :func:`evoked.add_proj <mne.Evoked.add_proj>` or
-              :func:`evoked.add_eeg_average_proj
-              <mne.Evoked.add_eeg_average_proj>`
+              :func:`evoked.set_eeg_reference <mne.Evoked.set_eeg_reference>`
               to add them afterward as necessary.
 
     Parameters
@@ -35,21 +36,32 @@ def simulate_evoked(fwd, stc, info, cov, snr=3., tmin=None, tmax=None,
         Measurement info to generate the evoked.
     cov : Covariance object
         The noise covariance.
-    snr : float
-        signal to noise ratio in dB. It corresponds to
-        10 * log10( var(signal) / var(noise) ).
+    nave : int
+        Number of averaged epochs (defaults to 30).
+
+        .. versionadded:: 0.15.0
     tmin : float | None
         start of time interval to estimate SNR. If None first time point
-        is used.
+        is used. tmin is deprecated and will be removed in 0.16.
     tmax : float | None
         start of time interval to estimate SNR. If None last time point
-        is used.
+        is used. tmax is deprecated and will be removed in 0.16.
     iir_filter : None | array
         IIR filter coefficients (denominator) e.g. [1, -1, 0.2].
     random_state : None | int | np.random.RandomState
         To specify the random generator state.
+    snr : float
+        signal to noise ratio in dB. It corresponds to
+        ``10 * log10( var(signal) / var(noise) )``.
+        snr is deprecated and will be removed in 0.16.
+    use_cps : None | bool (default None)
+        Whether to use cortical patch statistics to define normal
+        orientations when converting to fixed orientation (if necessary).
+
+        .. versionadded:: 0.15
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -64,20 +76,52 @@ def simulate_evoked(fwd, stc, info, cov, snr=3., tmin=None, tmax=None,
 
     Notes
     -----
+    To make the equivalence between snr and nave, when the snr is given
+    instead of nave::
+
+        nave = (1 / 10 ** ((actual_snr - snr)) / 20) ** 2
+
+    where actual_snr is the snr to the generated noise before scaling.
+
     .. versionadded:: 0.10.0
     """
-    evoked = apply_forward(fwd, stc, info)
-    if snr < np.inf:
+    if snr is not None:
+        warn('snr is deprecated and will be removed in 0.16. Set it to '
+             'None to remove this warning. Set nave parameter instead.',
+             DeprecationWarning)
+
+    if tmin is not None:
+        warn('tmin is deprecated and will be removed in 0.16. Set it to '
+             'None to remove this warning.',
+             DeprecationWarning)
+
+    if tmax is not None:
+        warn('tmax is deprecated and will be removed in 0.16. Set it to '
+             'None to remove this warning.',
+             DeprecationWarning)
+
+    evoked = apply_forward(fwd, stc, info, use_cps=use_cps)
+    if nave < np.inf:
         noise = simulate_noise_evoked(evoked, cov, iir_filter, random_state)
-        evoked_noise = add_noise_evoked(evoked, noise, snr, tmin=tmin,
-                                        tmax=tmax)
-    else:
-        evoked_noise = evoked
-    return evoked_noise
+
+        # Convert snr to nave before deprecation
+        if snr == np.inf:
+            nave = snr
+        elif snr is not None:
+            tmask = _time_mask(evoked.times, tmin, tmax,
+                               sfreq=evoked.info['sfreq'])
+            tmp = \
+                10 * np.log10(np.mean((evoked.data[:, tmask] ** 2)) /
+                              np.mean((noise.data ** 2)))
+            nave = 1. / 10 ** ((tmp - float(snr)) / 10)
+
+        evoked.data += noise.data / math.sqrt(nave)
+        evoked.nave = np.int(nave)
+    return evoked
 
 
 def simulate_noise_evoked(evoked, cov, iir_filter=None, random_state=None):
-    """Creates noise as a multivariate Gaussian
+    """Create noise as a multivariate Gaussian.
 
     The spatial covariance of the noise is given from the cov matrix.
 
@@ -108,7 +152,7 @@ def simulate_noise_evoked(evoked, cov, iir_filter=None, random_state=None):
 
 
 def _generate_noise(info, cov, iir_filter, random_state, n_samples, zi=None):
-    """Helper to create spatially colored and temporally IIR-filtered noise"""
+    """Create spatially colored and temporally IIR-filtered noise."""
     from scipy.signal import lfilter
     noise_cov = pick_channels_cov(cov, include=info['ch_names'], exclude=[])
     if set(info['ch_names']) != set(noise_cov.ch_names):
@@ -132,8 +176,9 @@ def _generate_noise(info, cov, iir_filter, random_state, n_samples, zi=None):
     return noise, zf
 
 
+@deprecated('add_noise_evoked will be deprecated and removed in 0.16.')
 def add_noise_evoked(evoked, noise, snr, tmin=None, tmax=None):
-    """Adds noise to evoked object with specified SNR.
+    """Add noise to evoked object with specified SNR.
 
     SNR is computed in the interval from tmin to tmax.
 
@@ -158,8 +203,8 @@ def add_noise_evoked(evoked, noise, snr, tmin=None, tmax=None):
     """
     evoked = copy.deepcopy(evoked)
     tmask = _time_mask(evoked.times, tmin, tmax, sfreq=evoked.info['sfreq'])
-    tmp = 10 * np.log10(np.mean((evoked.data[:, tmask] ** 2).ravel()) /
-                        np.mean((noise.data ** 2).ravel()))
+    tmp = 10 * np.log10(np.mean((evoked.data[:, tmask] ** 2)) /
+                        np.mean((noise.data ** 2)))
     noise.data = 10 ** ((tmp - float(snr)) / 20) * noise.data
     evoked.data += noise.data
     return evoked
