@@ -16,8 +16,7 @@ from ..externals.six import string_types
 
 
 def _f_oneway(*args):
-    """
-    Performs a 1-way ANOVA.
+    """Perform a 1-way ANOVA.
 
     The one-way ANOVA tests the null hypothesis that 2 or more groups have
     the same population mean. The test is applied to samples from two or
@@ -89,12 +88,12 @@ def _f_oneway(*args):
 
 
 def f_oneway(*args):
-    """Call scipy.stats.f_oneway, but return only f-value"""
+    """Call scipy.stats.f_oneway, but return only f-value."""
     return _f_oneway(*args)[0]
 
 
 def _map_effects(n_factors, effects):
-    """Map effects to indices"""
+    """Map effects to indices."""
     if n_factors > len(ascii_uppercase):
         raise ValueError('Maximum number of factors supported is 26')
 
@@ -152,14 +151,14 @@ def _map_effects(n_factors, effects):
     return selection, names
 
 
-def _get_contrast_indices(effect_idx, n_factors):
-    """Henson's factor coding, see num2binvec"""
+def _get_contrast_indices(effect_idx, n_factors):  # noqa: D401
+    """Henson's factor coding, see num2binvec."""
     binrepr = np.binary_repr(effect_idx, n_factors)
     return np.array([int(i) for i in binrepr], dtype=int)
 
 
 def _iter_contrasts(n_subjects, factor_levels, effect_picks):
-    """ Aux Function: Setup contrasts """
+    """Set up contrasts."""
     from scipy.signal import detrend
     sc = []
     n_factors = len(factor_levels)
@@ -186,7 +185,7 @@ def _iter_contrasts(n_subjects, factor_levels, effect_picks):
 
 def f_threshold_mway_rm(n_subjects, factor_levels, effects='A*B',
                         pvalue=0.05):
-    """ Compute f-value thesholds for a two-way ANOVA
+    """Compute f-value thesholds for a two-way ANOVA.
 
     Parameters
     ----------
@@ -235,7 +234,7 @@ def f_threshold_mway_rm(n_subjects, factor_levels, effects='A*B',
 
 def f_mway_rm(data, factor_levels, effects='all', alpha=0.05,
               correction=False, return_pvals=True):
-    """M-way repeated measures ANOVA for fully balanced designs
+    """Compute M-way repeated measures ANOVA for fully balanced designs.
 
     Parameters
     ----------
@@ -337,3 +336,16 @@ def f_mway_rm(data, factor_levels, effects='all', alpha=0.05,
 
     # handle single effect returns
     return [np.squeeze(np.asarray(vv)) for vv in (fvalues, pvalues)]
+
+
+def _parametric_ci(arr, ci=.95):
+    """Calculate the `ci`% parametric confidence interval for `arr`."""
+    from scipy import stats
+    mean, sigma = arr.mean(0), stats.sem(arr, 0)
+    # This is highly convoluted to support 17th century Scipy
+    # XXX Fix when Scipy 0.12 support is dropped!
+    # then it becomes just:
+    # return stats.t.interval(ci, loc=mean, scale=sigma, df=arr.shape[0])
+    return np.asarray([stats.t.interval(ci, arr.shape[0],
+                       loc=mean_, scale=sigma_)
+                       for mean_, sigma_ in zip(mean, sigma)]).T
