@@ -1,5 +1,4 @@
-"""T-test with permutations
-"""
+"""T-test with permutations."""
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 #          Fernando Perez (bin_perm_rep function)
@@ -9,19 +8,20 @@
 from math import sqrt
 import numpy as np
 
+from ..utils import check_random_state
 from ..parallel import parallel_func
 from .. import verbose
 
 
 def bin_perm_rep(ndim, a=0, b=1):
-    """bin_perm_rep(ndim) -> ndim permutations with repetitions of (a,b).
+    """Ndim permutations with repetitions of (a,b).
 
     Returns an array with all the possible permutations with repetitions of
     (0,1) in ndim dimensions.  The array is shaped as (2**ndim,ndim), and is
     ordered with the last index changing fastest.  For examble, for ndim=3:
 
-    Examples:
-
+    Examples
+    --------
     >>> bin_perm_rep(3)
     array([[0, 0, 0],
            [0, 0, 1],
@@ -32,7 +32,6 @@ def bin_perm_rep(ndim, a=0, b=1):
            [1, 1, 0],
            [1, 1, 1]])
     """
-
     # Create the leftmost column as 0,0,...,1,1,...
     nperms = 2 ** ndim
     perms = np.empty((nperms, ndim), type(a))
@@ -49,7 +48,7 @@ def bin_perm_rep(ndim, a=0, b=1):
 
 
 def _max_stat(X, X2, perms, dof_scaling):
-    """Aux function for permutation_t_test (for parallel comp)"""
+    """Aux function for permutation_t_test (for parallel comp)."""
     n_samples = len(X)
     mus = np.dot(perms, X) / float(n_samples)
     stds = np.sqrt(X2[None, :] - mus ** 2) * dof_scaling  # std with splitting
@@ -90,7 +89,8 @@ def permutation_t_test(X, n_permutations=10000, tail=0, n_jobs=1,
     n_jobs : int
         Number of CPUs to use for computation.
     verbose : bool, str, int, or None
-        If not None, override default verbose level (see mne.verbose).
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -127,6 +127,7 @@ def permutation_t_test(X, n_permutations=10000, tail=0, n_jobs=1,
     std0 = np.sqrt(X2 - mu0 ** 2) * dof_scaling  # get std with var splitting
     T_obs = np.mean(X, axis=0) / (std0 / sqrt(n_samples))
 
+    # XXX Someday this should be refactored with the cluster code
     if do_exact:
         perms = bin_perm_rep(n_samples, a=1, b=-1)[1:, :]
     else:
@@ -149,4 +150,34 @@ def permutation_t_test(X, n_permutations=10000, tail=0, n_jobs=1,
 
     return T_obs, p_values, H0
 
-permutation_t_test.__test__ = False  # for nosetests
+
+def _bootstrap_ci(arr, ci=.95, n_bootstraps=2000, stat_fun='mean',
+                  random_state=None):
+    """Get confidence intervals from non-parametric bootstrap."""
+    if stat_fun == "mean":
+        def stat_fun(x):
+            return x.mean(axis=0)
+    elif stat_fun == 'median':
+        def stat_fun(x):
+            return np.median(x, axis=0)
+    elif not callable(stat_fun):
+        raise ValueError("stat_fun must be 'mean', 'median' or callable.")
+    n_trials = arr.shape[0]
+    indices = np.arange(n_trials, dtype=int)  # BCA would be cool to have too
+    rng = check_random_state(random_state)
+    boot_indices = rng.choice(indices, replace=True,
+                              size=(n_bootstraps, len(indices)))
+    stat = np.array([stat_fun(arr[inds]) for inds in boot_indices])
+    ci = (((1 - ci) / 2) * 100, ((1 - ((1 - ci) / 2))) * 100)
+    ci_low, ci_up = np.percentile(stat, ci, axis=0)
+    return np.array([ci_low, ci_up])
+
+
+def _ci(arr, ci=.95, method="bootstrap", n_bootstraps=2000, random_state=None):
+    """Calculate confidence interval. Aux function for plot_compare_evokeds."""
+    if method == "bootstrap":
+        return _bootstrap_ci(arr, ci=ci, n_bootstraps=n_bootstraps,
+                             random_state=random_state)
+    else:
+        from . import _parametric_ci
+        return _parametric_ci(arr, ci=ci)
