@@ -1,18 +1,25 @@
+# Authors: Eric Larson <larson.eric.d@gmail.com>
+#          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
+#
+# License: BSD (3-clause)
+
+from functools import partial
 import os
+import warnings
+
 import numpy as np
+from scipy import sparse, linalg, stats
 from numpy.testing import (assert_equal, assert_array_equal,
                            assert_array_almost_equal)
 from nose.tools import assert_true, assert_raises
-from scipy import sparse, linalg, stats
-from mne.fixes import partial
-import warnings
+
 from mne.parallel import _force_serial
 from mne.stats.cluster_level import (permutation_cluster_test,
                                      permutation_cluster_1samp_test,
                                      spatio_temporal_cluster_test,
                                      spatio_temporal_cluster_1samp_test,
                                      ttest_1samp_no_p, summarize_clusters_stc)
-from mne.utils import run_tests_if_main, slow_test, _TempDir, catch_logging
+from mne.utils import run_tests_if_main, _TempDir, catch_logging
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
@@ -44,8 +51,7 @@ def _get_conditions():
 
 
 def test_cache_dir():
-    """Test use of cache dir
-    """
+    """Test use of cache dir."""
     tempdir = _TempDir()
     orig_dir = os.getenv('MNE_CACHE_DIR', None)
     orig_size = os.getenv('MNE_MEMMAP_MIN_SIZE', None)
@@ -78,14 +84,25 @@ def test_cache_dir():
             del os.environ['MNE_MEMMAP_MIN_SIZE']
 
 
+def test_permutation_large_n_samples():
+    """Test that non-replacement works with large N."""
+    X = np.random.RandomState(0).randn(72, 1) + 1
+    for n_samples in (11, 72):
+        tails = (0, 1) if n_samples <= 20 else (0,)
+        for tail in tails:
+            H0 = permutation_cluster_1samp_test(
+                X[:n_samples], threshold=1e-4, tail=tail)[-1]
+            assert H0.shape == (1024,)
+            assert len(np.unique(H0)) >= 1024 - (H0 == 0).sum()
+
+
 def test_permutation_step_down_p():
-    """Test cluster level permutations with step_down_p
-    """
+    """Test cluster level permutations with step_down_p."""
     try:
         try:
             from sklearn.feature_extraction.image import grid_to_graph
         except ImportError:
-            from scikits.learn.feature_extraction.image import grid_to_graph  # noqa
+            from scikits.learn.feature_extraction.image import grid_to_graph  # noqa: F401,E501 analysis:ignore
     except ImportError:
         return
     rng = np.random.RandomState(0)
@@ -112,8 +129,7 @@ def test_permutation_step_down_p():
 
 
 def test_cluster_permutation_test():
-    """Test cluster level permutations tests
-    """
+    """Test cluster level permutations tests."""
     condition1_1d, condition2_1d, condition1_2d, condition2_2d = \
         _get_conditions()
     for condition1, condition2 in zip((condition1_1d, condition1_2d),
@@ -137,10 +153,8 @@ def test_cluster_permutation_test():
         assert_array_equal(cluster_p_values, cluster_p_values_buff)
 
 
-@slow_test
 def test_cluster_permutation_t_test():
-    """Test cluster level permutations T-test
-    """
+    """Test cluster level permutations T-test."""
     condition1_1d, condition2_1d, condition1_2d, condition2_2d = \
         _get_conditions()
 
@@ -186,8 +200,7 @@ def test_cluster_permutation_t_test():
 
 
 def test_cluster_permutation_with_connectivity():
-    """Test cluster level permutations with connectivity matrix
-    """
+    """Test cluster level permutations with connectivity matrix."""
     try:
         try:
             from sklearn.feature_extraction.image import grid_to_graph
@@ -312,6 +325,10 @@ def test_cluster_permutation_with_connectivity():
         assert_raises(ValueError, spatio_temporal_func, X1d_3,
                       connectivity=connectivity, tail=-1,
                       threshold=dict(start=-1, step=1))
+        # Make sure connectivity has to be sparse
+        assert_raises(ValueError, spatio_temporal_func, X1d_3,
+                      n_permutations=50, connectivity=connectivity.todense(),
+                      max_step=1, threshold=1.67)
 
         # wrong type for threshold
         assert_raises(TypeError, spatio_temporal_func, X1d_3,
@@ -330,10 +347,8 @@ def test_cluster_permutation_with_connectivity():
         assert_true(np.min(out_connectivity_6[2]) < 0.05)
 
 
-@slow_test
 def test_permutation_connectivity_equiv():
-    """Test cluster level permutations with and without connectivity
-    """
+    """Test cluster level permutations with and without connectivity."""
     try:
         try:
             from sklearn.feature_extraction.image import grid_to_graph
@@ -349,69 +364,66 @@ def test_permutation_connectivity_equiv():
     # add some significant points
     X[:, :, 0:2] += 10  # span two time points and two spatial points
     X[:, 1, 3] += 20  # span one time point
-    max_steps = [1, 1, 1, 2]
+    max_steps = [1, 1, 1, 2, 1]
     # This will run full algorithm in two ways, then the ST-algorithm in 2 ways
     # All of these should give the same results
-    conns = [None, grid_to_graph(n_time, n_space),
-             grid_to_graph(1, n_space), grid_to_graph(1, n_space)]
+    conns = [None,
+             grid_to_graph(n_time, n_space),
+             grid_to_graph(1, n_space),
+             grid_to_graph(1, n_space),
+             None]
     stat_map = None
-    thresholds = [2, dict(start=1.5, step=1.0)]
-    sig_counts = [2, 5]
-    sdps = [0, 0.05, 0.05]
-    ots = ['mask', 'mask', 'indices']
+    thresholds = [2, 2, 2, 2, dict(start=0.01, step=1.0)]
+    sig_counts = [2, 2, 2, 2, 5]
     stat_fun = partial(ttest_1samp_no_p, sigma=1e-3)
-    for thresh, count in zip(thresholds, sig_counts):
-        cs = None
-        ps = None
-        for max_step, conn in zip(max_steps, conns):
-            for sdp, ot in zip(sdps, ots):
-                t, clusters, p, H0 = \
-                    permutation_cluster_1samp_test(
-                        X, threshold=thresh, connectivity=conn, n_jobs=2,
-                        max_step=max_step, stat_fun=stat_fun,
-                        step_down_p=sdp, out_type=ot)
-                # make sure our output datatype is correct
-                if ot == 'mask':
-                    assert_true(isinstance(clusters[0], np.ndarray))
-                    assert_true(clusters[0].dtype == bool)
-                    assert_array_equal(clusters[0].shape, X.shape[1:])
-                else:  # ot == 'indices'
-                    assert_true(isinstance(clusters[0], tuple))
 
-                # make sure all comparisons were done; for TFCE, no perm
-                # should come up empty
-                if count == 8:
-                    assert_true(not np.any(H0 == 0))
-                inds = np.where(p < 0.05)[0]
-                assert_true(len(inds) == count)
-                this_cs = [clusters[ii] for ii in inds]
-                this_ps = p[inds]
-                this_stat_map = np.zeros((n_time, n_space), dtype=bool)
-                for ci, c in enumerate(this_cs):
-                    if isinstance(c, tuple):
-                        this_c = np.zeros((n_time, n_space), bool)
-                        for x, y in zip(c[0], c[1]):
-                            this_stat_map[x, y] = True
-                            this_c[x, y] = True
-                        this_cs[ci] = this_c
-                        c = this_c
-                    this_stat_map[c] = True
-                if cs is None:
-                    ps = this_ps
-                    cs = this_cs
-                if stat_map is None:
-                    stat_map = this_stat_map
-                assert_array_equal(ps, this_ps)
-                assert_true(len(cs) == len(this_cs))
-                for c1, c2 in zip(cs, this_cs):
-                    assert_array_equal(c1, c2)
-                assert_array_equal(stat_map, this_stat_map)
+    cs = None
+    ps = None
+    for thresh, count, max_step, conn in zip(thresholds, sig_counts,
+                                             max_steps, conns):
+        t, clusters, p, H0 = \
+            permutation_cluster_1samp_test(
+                X, threshold=thresh, connectivity=conn, n_jobs=2,
+                max_step=max_step, stat_fun=stat_fun)
+        # make sure our output datatype is correct
+        assert_true(isinstance(clusters[0], np.ndarray))
+        assert_true(clusters[0].dtype == bool)
+        assert_array_equal(clusters[0].shape, X.shape[1:])
+
+        # make sure all comparisons were done; for TFCE, no perm
+        # should come up empty
+        inds = np.where(p < 0.05)[0]
+        assert_equal(len(inds), count)
+        if isinstance(thresh, dict):
+            assert_equal(len(clusters), n_time * n_space)
+            assert_true(np.all(H0 != 0))
+            continue
+        this_cs = [clusters[ii] for ii in inds]
+        this_ps = p[inds]
+        this_stat_map = np.zeros((n_time, n_space), dtype=bool)
+        for ci, c in enumerate(this_cs):
+            if isinstance(c, tuple):
+                this_c = np.zeros((n_time, n_space), bool)
+                for x, y in zip(c[0], c[1]):
+                    this_stat_map[x, y] = True
+                    this_c[x, y] = True
+                this_cs[ci] = this_c
+                c = this_c
+            this_stat_map[c] = True
+        if cs is None:
+            ps = this_ps
+            cs = this_cs
+        if stat_map is None:
+            stat_map = this_stat_map
+        assert_array_equal(ps, this_ps)
+        assert_true(len(cs) == len(this_cs))
+        for c1, c2 in zip(cs, this_cs):
+            assert_array_equal(c1, c2)
+        assert_array_equal(stat_map, this_stat_map)
 
 
-@slow_test
-def spatio_temporal_cluster_test_connectivity():
-    """Test spatio-temporal cluster permutations
-    """
+def test_spatio_temporal_cluster_connectivity():
+    """Test spatio-temporal cluster permutations."""
     try:
         try:
             from sklearn.feature_extraction.image import grid_to_graph
@@ -453,17 +465,21 @@ def spatio_temporal_cluster_test_connectivity():
                                      threshold=threshold, n_jobs=2,
                                      buffer_size=None)
     assert_array_equal(p_values_no_conn, p_values2)
+    assert_raises(ValueError, spatio_temporal_cluster_test,
+                  [data1_2d, data2_2d], tail=1, threshold=-2.)
+    assert_raises(ValueError, spatio_temporal_cluster_test,
+                  [data1_2d, data2_2d], tail=-1, threshold=2.)
+    assert_raises(ValueError, spatio_temporal_cluster_test,
+                  [data1_2d, data2_2d], tail=0, threshold=-1)
 
 
 def ttest_1samp(X):
-    """Returns T-values
-    """
+    """Return T-values."""
     return stats.ttest_1samp(X, 0)[0]
 
 
 def test_summarize_clusters():
-    """Test cluster summary stcs
-    """
+    """Test cluster summary stcs."""
     clu = (np.random.random([1, 20484]),
            [(np.array([0]), np.array([0, 2, 4]))],
            np.array([0.02, 0.1]),
@@ -474,4 +490,30 @@ def test_summarize_clusters():
     assert_raises(RuntimeError, summarize_clusters_stc, clu)
 
 
+def test_permutation_test_H0():
+    """Test that H0 is populated properly during testing."""
+    rng = np.random.RandomState(0)
+    data = rng.rand(7, 10, 1) - 0.5
+    with warnings.catch_warnings(record=True) as w:
+        t, clust, p, h0 = spatio_temporal_cluster_1samp_test(
+            data, threshold=100, n_permutations=1024, seed=rng)
+    assert_equal(len(w), 1)
+    assert_true('No clusters found' in str(w[0].message))
+    assert_equal(len(h0), 0)
+
+    for n_permutations in (1024, 65, 64, 63):
+        t, clust, p, h0 = spatio_temporal_cluster_1samp_test(
+            data, threshold=0.1, n_permutations=n_permutations, seed=rng)
+        assert_equal(len(h0), min(n_permutations, 64))
+        assert_true(isinstance(clust[0], tuple))  # sets of indices
+    for tail, thresh in zip((-1, 0, 1), (-0.1, 0.1, 0.1)):
+        with warnings.catch_warnings(record=True) as w:
+            t, clust, p, h0 = spatio_temporal_cluster_1samp_test(
+                data, threshold=thresh, seed=rng, tail=tail, out_type='mask')
+        assert_equal(len(w), 0)
+        assert_true(isinstance(clust[0], np.ndarray))  # bool mask
+        # same as "128 if tail else 64"
+        assert_equal(len(h0), 2 ** (7 - (tail == 0)))  # exact test
+
+
 run_tests_if_main()
