@@ -8,14 +8,14 @@ from numpy.testing import assert_allclose, assert_equal, assert_array_equal
 from scipy import linalg
 
 from .. import pick_types, Evoked
-from ..io import _BaseRaw
+from ..io import BaseRaw
 from ..io.constants import FIFF
 from ..bem import fit_sphere_to_headshape
 
 
 def _get_data(x, ch_idx):
     """Helper to get the (n_ch, n_times) data array"""
-    if isinstance(x, _BaseRaw):
+    if isinstance(x, BaseRaw):
         return x[ch_idx][0]
     elif isinstance(x, Evoked):
         return x.data[ch_idx]
@@ -97,9 +97,10 @@ def assert_dig_allclose(info_py, info_bin):
                         err_msg='Failure on %s:\n%s\n%s'
                         % (ii, d_py['r'], d_bin['r']))
     if any(d['kind'] == FIFF.FIFFV_POINT_EXTRA for d in dig_py):
-        r_bin, o_head_bin, o_dev_bin = fit_sphere_to_headshape(info_bin,
-                                                               units='m')
-        r_py, o_head_py, o_dev_py = fit_sphere_to_headshape(info_py, units='m')
+        r_bin, o_head_bin, o_dev_bin = fit_sphere_to_headshape(
+            info_bin, units='m', verbose='error')
+        r_py, o_head_py, o_dev_py = fit_sphere_to_headshape(
+            info_py, units='m', verbose='error')
         assert_allclose(r_py, r_bin, atol=1e-6)
         assert_allclose(o_dev_py, o_dev_bin, rtol=1e-5, atol=1e-6)
         assert_allclose(o_head_py, o_head_bin, rtol=1e-5, atol=1e-6)
