@@ -38,7 +38,7 @@ raw_fname = op.join(base_dir, 'test_raw.fif')
 
 def test_fix_stim():
     """Test fixing stim STI016 for Neuromag."""
-    raw = read_raw_fif(raw_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname, preload=True)
     # 32768 (016) + 3 (002+001) bits gets incorrectly coded during acquisition
     raw._data[raw.ch_names.index('STI 014'), :3] = [0, -32765, 0]
     with warnings.catch_warnings(record=True) as w:
@@ -53,10 +53,10 @@ def test_fix_stim():
 def test_add_events():
     """Test adding events to a Raw file."""
     # need preload
-    raw = read_raw_fif(raw_fname, preload=False, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events = np.array([[raw.first_samp, 0, 1]])
     assert_raises(RuntimeError, raw.add_events, events, 'STI 014')
-    raw = read_raw_fif(raw_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname, preload=True)
     orig_events = find_events(raw, 'STI 014')
     # add some events
     events = np.array([raw.first_samp, 0, 1])
@@ -179,7 +179,7 @@ def test_io_events():
 def test_find_events():
     """Test find events in raw file."""
     events = read_events(fname)
-    raw = read_raw_fif(raw_fname, preload=True, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname, preload=True)
     # let's test the defaulting behavior while we're at it
     extra_ends = ['', '_1']
     orig_envs = [os.getenv('MNE_STIM_CHANNEL%s' % s) for s in extra_ends]
@@ -192,7 +192,7 @@ def test_find_events():
     events11 = find_events(raw, mask=3, mask_type='not_and')
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter('always')
-        events22 = read_events(fname, mask=3)
+        events22 = read_events(fname, mask=3, mask_type='not_and')
         assert_true(sum('events masked' in str(ww.message) for ww in w) == 1)
     assert_array_equal(events11, events22)
 
@@ -210,7 +210,7 @@ def test_find_events():
     raw._data[stim_channel_idx, 5:] = 0
     # 1 == '0b1', 2 == '0b10', 3 == '0b11', 4 == '0b100'
 
-    assert_raises(TypeError, find_events, raw, mask="0")
+    assert_raises(TypeError, find_events, raw, mask="0", mask_type='and')
     assert_raises(ValueError, find_events, raw, mask=0, mask_type='blah')
     # testing mask_type. default = 'not_and'
     assert_array_equal(find_events(raw, shortest_event=1, mask=1,
@@ -343,6 +343,22 @@ def test_find_events():
         if o is not None:
             os.environ['MNE_STIM_CHANNEL%s' % s] = o
 
+    # Test with list of stim channels
+    raw._data[stim_channel_idx, 1:101] = np.zeros(100)
+    raw._data[stim_channel_idx, 10:11] = 1
+    raw._data[stim_channel_idx, 30:31] = 3
+    stim_channel2 = 'STI 015'
+    stim_channel2_idx = pick_channels(raw.info['ch_names'],
+                                      include=[stim_channel2])
+    raw._data[stim_channel2_idx, :] = 0
+    raw._data[stim_channel2_idx, :100] = raw._data[stim_channel_idx, 5:105]
+    events1 = find_events(raw, stim_channel='STI 014')
+    events2 = events1.copy()
+    events2[:, 0] -= 5
+    events = find_events(raw, stim_channel=['STI 014', stim_channel2])
+    assert_array_equal(events[::2], events2)
+    assert_array_equal(events[1::2], events1)
+
 
 def test_pick_events():
     """Test pick events in a events ndarray."""
@@ -366,7 +382,7 @@ def test_pick_events():
 
 def test_make_fixed_length_events():
     """Test making events of a fixed length."""
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events = make_fixed_length_events(raw, id=1)
     assert_true(events.shape[1], 3)
     events_zero = make_fixed_length_events(raw, 1, first_samp=False)
@@ -390,7 +406,7 @@ def test_make_fixed_length_events():
 def test_define_events():
     """Test defining response events."""
     events = read_events(fname)
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events_, _ = define_target_events(events, 5, 32, raw.info['sfreq'],
                                       .2, 0.7, 42, 99)
     n_target = events[events[:, 2] == 5].shape[0]
@@ -442,7 +458,8 @@ def test_acqparser():
     assert_true(acqp['Surprise visual'])
     # test TRIUX file
     raw = read_raw_fif(fname_raw_elekta, preload=False)
-    acqp = AcqParserFIF(raw.info)
+    acqp = raw.acqparser
+    assert_true(acqp is raw.acqparser)  # same one, not regenerated
     # test __repr__()
     assert_true(repr(acqp))
     # this file should not be in compatibility mode
