@@ -14,6 +14,7 @@ from scipy import fftpack
 from numpy.testing import (assert_array_almost_equal, assert_equal,
                            assert_array_equal, assert_allclose)
 from nose.tools import assert_true, assert_raises, assert_not_equal
+import pytest
 
 from mne import (equalize_channels, pick_types, read_evokeds, write_evokeds,
                  grand_average, combine_evoked, create_info, read_events,
@@ -21,7 +22,7 @@ from mne import (equalize_channels, pick_types, read_evokeds, write_evokeds,
 from mne.evoked import _get_peak, Evoked, EvokedArray
 from mne.io import read_raw_fif
 from mne.tests.common import assert_naming
-from mne.utils import (_TempDir, requires_pandas, slow_test, requires_version,
+from mne.utils import (_TempDir, requires_pandas, requires_version,
                        run_tests_if_main)
 from mne.externals.six.moves import cPickle as pickle
 
@@ -55,13 +56,12 @@ def test_decim():
     assert_array_equal(data_epochs, data_epochs_3)
 
     # Now let's do it with some real data
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events = read_events(event_name)
     sfreq_new = raw.info['sfreq'] / decim
     raw.info['lowpass'] = sfreq_new / 4.  # suppress aliasing warnings
     picks = pick_types(raw.info, meg=True, eeg=True, exclude=())
-    epochs = Epochs(raw, events, 1, -0.2, 0.5, picks=picks, preload=True,
-                    add_eeg_ref=False)
+    epochs = Epochs(raw, events, 1, -0.2, 0.5, picks=picks, preload=True)
     for offset in (0, 1):
         ev_ep_decim = epochs.copy().decimate(decim, offset).average()
         ev_decim = epochs.average().decimate(decim, offset)
@@ -110,7 +110,7 @@ def test_hash_evoked():
     assert_not_equal(hash(ave), hash(ave_2))
 
 
-@slow_test
+@pytest.mark.slowtest
 def test_io_evoked():
     """Test IO for evoked data (fif + gz) with integer and str args."""
     tempdir = _TempDir()
@@ -226,7 +226,7 @@ def test_shift_time_evoked():
 
 
 def test_evoked_resample():
-    """Test for resampling of evoked data."""
+    """Test resampling evoked data."""
     tempdir = _TempDir()
     # upsample, write it out, read it in
     ave = read_evokeds(fname, 0)
@@ -256,6 +256,17 @@ def test_evoked_resample():
     assert_true(ave_up.data.shape[1] == 2 * ave_normal.data.shape[1])
 
 
+def test_evoked_filter():
+    """Test filtering evoked data."""
+    # this is mostly a smoke test as the Epochs and raw tests are more complete
+    ave = read_evokeds(fname, 0).pick_types('grad')
+    ave.data[:] = 1.
+    assert round(ave.info['lowpass']) == 172
+    ave_filt = ave.copy().filter(None, 40., fir_design='firwin')
+    assert ave_filt.info['lowpass'] == 40.
+    assert_allclose(ave.data, 1., atol=1e-6)
+
+
 def test_evoked_detrend():
     """Test for detrending evoked data."""
     ave = read_evokeds(fname, 0)
@@ -274,7 +285,7 @@ def test_to_data_frame():
     assert_raises(ValueError, ave.to_data_frame, picks=np.arange(400))
     df = ave.to_data_frame()
     assert_true((df.columns == ave.ch_names).all())
-    df = ave.to_data_frame(index=None).reset_index('time')
+    df = ave.to_data_frame(index=None).reset_index()
     assert_true('time' in df.columns)
     assert_array_equal(df.values[:, 1], ave.data[0] * 1e13)
     assert_array_equal(df.values[:, 3], ave.data[2] * 1e15)
@@ -328,6 +339,10 @@ def test_get_peak():
     assert_true(time_idx < len(evoked.times))
     assert_equal(ch_name, 'MEG 1421')
 
+    assert_raises(ValueError, evoked.get_peak, ch_type='mag', merge_grads=True)
+    ch_name, time_idx = evoked.get_peak(ch_type='grad', merge_grads=True)
+    assert_equal(ch_name, 'MEG 244X')
+
     data = np.array([[0., 1.,  2.],
                      [0., -3.,  0]])
 
@@ -417,8 +432,7 @@ def test_arithmetic():
     # combine_evoked([ev1, ev2]) should be the same as ev1 + ev2:
     # data should be added according to their `nave` weights
     # nave = ev1.nave + ev2.nave
-    with warnings.catch_warnings(record=True):  # deprecation no weights
-        ev = combine_evoked([ev1, ev2])
+    ev = combine_evoked([ev1, ev2], weights='nave')
     assert_equal(ev.nave, ev1.nave + ev2.nave)
     assert_allclose(ev.data, 1. / 3. * np.ones_like(ev.data))
 
@@ -516,7 +530,7 @@ def test_array_epochs():
 
     # test kind check
     assert_raises(TypeError, EvokedArray, data1, info, tmin=0, kind=1)
-    assert_raises(ValueError, EvokedArray, data1, info, tmin=0, kind='mean')
+    assert_raises(ValueError, EvokedArray, data1, info, kind='mean')
 
     # test match between channels info and data
     ch_names = ['EEG %03d' % (i + 1) for i in range(19)]
