@@ -18,7 +18,8 @@ ok_scipy_submodules = set(['scipy', 'numpy',  # these appear in old scipy
                            'misc', 'sparse', 'version'])
 scipy_submodules = set(x.split('.')[1] for x in sys.modules.keys()
                        if x.startswith('scipy.') and '__' not in x and
-                       not x.split('.')[1].startswith('_'))
+                       not x.split('.')[1].startswith('_')
+                       and sys.modules[x] is not None)
 bad = scipy_submodules - ok_scipy_submodules
 if len(bad) > 0:
     out.append('Found un-nested scipy submodules: %s' % list(bad))
@@ -53,4 +54,5 @@ def test_module_nesting():
     if proc.returncode:
         raise AssertionError(stdout)
 
+
 run_tests_if_main()
