@@ -9,6 +9,7 @@ from scipy import sparse
 
 from numpy.testing import assert_array_equal, assert_array_almost_equal
 from nose.tools import assert_equal, assert_true, assert_false, assert_raises
+import pytest
 
 from mne.datasets import testing
 from mne import (read_label, stc_to_label, read_source_estimate,
@@ -17,7 +18,7 @@ from mne import (read_label, stc_to_label, read_source_estimate,
                  read_surface)
 from mne.label import Label, _blend_colors, label_sign_flip
 from mne.utils import (_TempDir, requires_sklearn, get_subjects_dir,
-                       run_tests_if_main, slow_test)
+                       run_tests_if_main)
 from mne.fixes import assert_is, assert_is_not
 from mne.label import _n_colors
 from mne.source_space import SourceSpaces
@@ -166,7 +167,7 @@ def assert_labels_equal(l0, l1, decimal=5, comment=True, color=True):
 
 
 def test_copy():
-    """Test label copying"""
+    """Test label copying."""
     label = read_label(label_fname)
     label_2 = label.copy()
     label_2.pos += 1
@@ -174,8 +175,7 @@ def test_copy():
 
 
 def test_label_subject():
-    """Test label subject name extraction
-    """
+    """Test label subject name extraction."""
     label = read_label(label_fname)
     assert_is(label.subject, None)
     assert_true('unknown' in repr(label))
@@ -185,8 +185,7 @@ def test_label_subject():
 
 
 def test_label_addition():
-    """Test label addition
-    """
+    """Test label addition."""
     pos = np.random.RandomState(0).rand(10, 3)
     values = np.arange(10.) / 10
     idx0 = list(range(7))
@@ -228,7 +227,6 @@ def test_label_addition():
 
     # adding lh and rh
     l2.hemi = 'rh'
-    # this now has deprecated behavior
     bhl = l0 + l2
     assert_equal(bhl.hemi, 'both')
     assert_equal(len(bhl), len(l0) + len(l2))
@@ -257,7 +255,7 @@ def test_label_addition():
 
 @testing.requires_testing_data
 def test_label_in_src():
-    """Test label in src"""
+    """Test label in src."""
     src = read_source_spaces(src_fname)
     label = read_label(v1_label_fname)
 
@@ -284,11 +282,15 @@ def test_label_in_src():
     vertices = np.append([-1], vert_in_src)
     assert_raises(ValueError, Label(vertices, hemi='lh').fill, src)
 
+    # test filling empty label
+    label = Label([], hemi='lh')
+    label.fill(src)
+    assert_array_equal(label.vertices, np.array([], int))
+
 
 @testing.requires_testing_data
 def test_label_io_and_time_course_estimates():
-    """Test IO for label + stc files
-    """
+    """Test IO for label + stc files."""
     stc = read_source_estimate(stc_fname)
     label = read_label(real_label_fname)
     stc_label = stc.in_label(label)
@@ -299,8 +301,7 @@ def test_label_io_and_time_course_estimates():
 
 @testing.requires_testing_data
 def test_label_io():
-    """Test IO of label files
-    """
+    """Test IO of label files."""
     tempdir = _TempDir()
     label = read_label(label_fname)
 
@@ -335,7 +336,7 @@ def _assert_labels_equal(labels_a, labels_b, ignore_pos=False):
 
 @testing.requires_testing_data
 def test_annot_io():
-    """Test I/O from and to *.annot files"""
+    """Test I/O from and to *.annot files."""
     # copy necessary files from fsaverage to tempdir
     tempdir = _TempDir()
     subject = 'fsaverage'
@@ -382,8 +383,7 @@ def test_annot_io():
 
 @testing.requires_testing_data
 def test_read_labels_from_annot():
-    """Test reading labels from FreeSurfer parcellation
-    """
+    """Test reading labels from FreeSurfer parcellation."""
     # test some invalid inputs
     assert_raises(ValueError, read_labels_from_annot, 'sample', hemi='bla',
                   subjects_dir=subjects_dir)
@@ -440,8 +440,7 @@ def test_read_labels_from_annot():
 
 @testing.requires_testing_data
 def test_read_labels_from_annot_annot2labels():
-    """Test reading labels from parc. by comparing with mne_annot2labels
-    """
+    """Test reading labels from parc. by comparing with mne_annot2labels."""
     label_fnames = glob.glob(label_dir + '/*.label')
     label_fnames.sort()
     labels_mne = [read_label(fname) for fname in label_fnames]
@@ -453,7 +452,7 @@ def test_read_labels_from_annot_annot2labels():
 
 @testing.requires_testing_data
 def test_write_labels_to_annot():
-    """Test writing FreeSurfer parcellation from labels"""
+    """Test writing FreeSurfer parcellation from labels."""
     tempdir = _TempDir()
 
     labels = read_labels_from_annot('sample', subjects_dir=subjects_dir)
@@ -580,7 +579,7 @@ def test_write_labels_to_annot():
 @requires_sklearn
 @testing.requires_testing_data
 def test_split_label():
-    """Test splitting labels"""
+    """Test splitting labels."""
     aparc = read_labels_from_annot('fsaverage', 'aparc', 'lh',
                                    regexp='lingual', subjects_dir=subjects_dir)
     lingual = aparc[0]
@@ -630,12 +629,11 @@ def test_split_label():
                  [16181, 7022, 5965, 5300, 823] + [1] * 23)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_sklearn
 def test_stc_to_label():
-    """Test stc_to_label
-    """
+    """Test stc_to_label."""
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter('always')
         src = read_source_spaces(fwd_fname)
@@ -693,11 +691,10 @@ def test_stc_to_label():
         assert_labels_equal(l1, l2, decimal=4)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_morph():
-    """Test inter-subject label morphing
-    """
+    """Test inter-subject label morphing."""
     label_orig = read_label(real_label_fname)
     label_orig.subject = 'sample'
     # should work for specifying vertices for both hemis, or just the
@@ -719,7 +716,8 @@ def test_morph():
     verts = [np.arange(10242), np.arange(10242)]
     for hemi in ['lh', 'rh']:
         label.hemi = hemi
-        label.morph(None, 'fsaverage', 5, verts, subjects_dir, 2)
+        with warnings.catch_warnings(record=True):  # morph map maybe missing
+            label.morph(None, 'fsaverage', 5, verts, subjects_dir, 2)
     assert_raises(TypeError, label.morph, None, 1, 5, verts,
                   subjects_dir, 2)
     assert_raises(TypeError, label.morph, None, 'fsaverage', 5.5, verts,
@@ -730,7 +728,7 @@ def test_morph():
 
 @testing.requires_testing_data
 def test_grow_labels():
-    """Test generation of circular source labels"""
+    """Test generation of circular source labels."""
     seeds = [0, 50000]
     # these were chosen manually in mne_analyze
     should_be_in = [[49, 227], [51207, 48794]]
@@ -772,7 +770,7 @@ def test_grow_labels():
 
 @testing.requires_testing_data
 def test_label_sign_flip():
-    """Test label sign flip computation"""
+    """Test label sign flip computation."""
     src = read_source_spaces(src_fname)
     label = Label(vertices=src[0]['vertno'][:5], hemi='lh')
     src[0]['nn'][label.vertices] = np.array(
@@ -784,14 +782,21 @@ def test_label_sign_flip():
     known_flips = np.array([1, 1, np.nan, 1, 1])
     idx = [0, 1, 3, 4]  # indices that are usable (third row is orthognoal)
     flip = label_sign_flip(label, src)
-    # Need the abs here because the direction is arbitrary
-    assert_array_almost_equal(np.abs(np.dot(flip[idx], known_flips[idx])),
-                              len(idx))
+    assert_array_almost_equal(np.dot(flip[idx], known_flips[idx]), len(idx))
+    bi_label = label + Label(vertices=src[1]['vertno'][:5], hemi='rh')
+    src[1]['nn'][src[1]['vertno'][:5]] = -src[0]['nn'][label.vertices]
+    flip = label_sign_flip(bi_label, src)
+    known_flips = np.array([1, 1, np.nan, 1, 1, 1, 1, np.nan, 1, 1])
+    idx = [0, 1, 3, 4, 5, 6, 8, 9]
+    assert_array_almost_equal(np.dot(flip[idx], known_flips[idx]), 0.)
+    src[1]['nn'][src[1]['vertno'][:5]] *= -1
+    flip = label_sign_flip(bi_label, src)
+    assert_array_almost_equal(np.dot(flip[idx], known_flips[idx]), len(idx))
 
 
 @testing.requires_testing_data
 def test_label_center_of_mass():
-    """Test computing the center of mass of a label"""
+    """Test computing the center of mass of a label."""
     stc = read_source_estimate(stc_fname)
     stc.lh_data[:] = 0
     vertex_stc = stc.center_of_mass('sample', subjects_dir=subjects_dir)[0]
@@ -809,6 +814,9 @@ def test_label_center_of_mass():
     for label, expected in zip([labels[2], labels[3], labels[-5]],
                                [141162, 145221, 55979]):
         label.values[:] = -1
+        assert_raises(ValueError, label.center_of_mass,
+                      subjects_dir=subjects_dir)
+        label.values[:] = 0
         assert_raises(ValueError, label.center_of_mass,
                       subjects_dir=subjects_dir)
         label.values[:] = 1
@@ -840,4 +848,5 @@ def test_label_center_of_mass():
     assert_raises(IOError, label.center_of_mass, subjects_dir=subjects_dir,
                   surf='foo')
 
+
 run_tests_if_main()
