@@ -5,7 +5,7 @@
 
 import os
 from nose.tools import assert_raises
-from nose.plugins.skip import SkipTest
+from unittest import SkipTest
 from os import path as op
 import sys
 
@@ -21,7 +21,8 @@ skip_files = (
     # part of testing compatibility with older BV formats is testing
     # the line endings and coding schemes used there
     'test_old_layout_latin1_software_filter.vhdr',
-    'test_old_layout_latin1_software_filter.vmrk'
+    'test_old_layout_latin1_software_filter.vmrk',
+    'searchindex.dat',
 )
 
 
@@ -69,4 +70,5 @@ def test_line_endings():
     # now check mne
     _assert_line_endings(_get_root_dir())
 
+
 run_tests_if_main()
