@@ -2,23 +2,23 @@
 #          Teon Brooks <teon.brooks@gmail.com>
 #
 # License: BSD (3-clause)
-import sys
+
+import glob
 import os
 import os.path as op
-import glob
-import warnings
 import shutil
+import warnings
 
 from nose.tools import assert_true, assert_equal, assert_raises
-from nose.plugins.skip import SkipTest
+import pytest
 
-from mne import Epochs, read_events, pick_types, read_evokeds
-from mne.io import Raw
+from mne import Epochs, read_events, read_evokeds
+from mne.io import read_raw_fif
 from mne.datasets import testing
 from mne.report import Report
 from mne.utils import (_TempDir, requires_mayavi, requires_nibabel,
-                       requires_PIL, run_tests_if_main, slow_test)
-from mne.viz import plot_trans
+                       requires_PIL, run_tests_if_main)
+from mne.viz import plot_alignment
 
 import matplotlib
 matplotlib.use('Agg')  # for testing don't use X server
@@ -44,12 +44,11 @@ evoked_fname = op.join(base_dir, 'test-ave.fif')
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_PIL
 def test_render_report():
-    """Test rendering -*.fif files for mne report.
-    """
+    """Test rendering -*.fif files for mne report."""
     tempdir = _TempDir()
     raw_fname_new = op.join(tempdir, 'temp_raw.fif')
     event_fname_new = op.join(tempdir, 'temp_raw-eve.fif')
@@ -66,19 +65,22 @@ def test_render_report():
     # create and add -epo.fif and -ave.fif files
     epochs_fname = op.join(tempdir, 'temp-epo.fif')
     evoked_fname = op.join(tempdir, 'temp-ave.fif')
-    raw = Raw(raw_fname_new)
-    picks = pick_types(raw.info, meg='mag', eeg=False)  # faster with one type
-    epochs = Epochs(raw, read_events(event_fname), 1, -0.2, 0.2, picks=picks)
+    # Speed it up by picking channels
+    raw = read_raw_fif(raw_fname_new, preload=True)
+    raw.pick_channels(['MEG 0111', 'MEG 0121'])
+    raw.del_proj()
+    epochs = Epochs(raw, read_events(event_fname), 1, -0.2, 0.2)
     epochs.save(epochs_fname)
-    epochs.average().save(evoked_fname)
+    # This can take forever (stall Travis), so let's make it fast
+    # Also, make sure crop range is wide enough to avoid rendering bug
+    epochs.average().crop(0.1, 0.2).save(evoked_fname)
 
     report = Report(info_fname=raw_fname_new, subjects_dir=subjects_dir)
-    if sys.version.startswith('3.5'):  # XXX Some strange MPL/3.5 error...
-        raise SkipTest('Python 3.5 and mpl have unresolved issues')
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter('always')
         report.parse_folder(data_path=tempdir, on_error='raise')
     assert_true(len(w) >= 1)
+    assert_true(repr(report))
 
     # Check correct paths and filenames
     fnames = glob.glob(op.join(tempdir, '*.fif'))
@@ -89,6 +91,7 @@ def test_render_report():
 
     assert_equal(len(report.fnames), len(fnames))
     assert_equal(len(report.html), len(report.fnames))
+    assert_equal(len(report.fnames), len(report))
 
     # Check saving functionality
     report.data_path = tempdir
@@ -113,6 +116,7 @@ def test_render_report():
         warnings.simplefilter('always')
         report.parse_folder(data_path=tempdir, pattern=pattern)
     assert_true(len(w) >= 1)
+    assert_true(repr(report))
 
     fnames = glob.glob(op.join(tempdir, '*.raw')) + \
         glob.glob(op.join(tempdir, '*.raw'))
@@ -121,13 +125,22 @@ def test_render_report():
                     [op.basename(x) for x in report.fnames])
         assert_true(''.join(report.html).find(op.basename(fname)) != -1)
 
+    assert_raises(ValueError, Report, image_format='foo')
+    assert_raises(ValueError, Report, image_format=None)
+
+    # SVG rendering
+    report = Report(info_fname=raw_fname_new, subjects_dir=subjects_dir,
+                    image_format='svg')
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        report.parse_folder(data_path=tempdir, on_error='raise')
+
 
 @testing.requires_testing_data
 @requires_mayavi
 @requires_PIL
 def test_render_add_sections():
-    """Test adding figures/images to section.
-    """
+    """Test adding figures/images to section."""
     from PIL import Image
     tempdir = _TempDir()
     import matplotlib.pyplot as plt
@@ -166,20 +179,20 @@ def test_render_add_sections():
 
     evoked = read_evokeds(evoked_fname, condition='Left Auditory',
                           baseline=(-0.2, 0.0))
-    fig = plot_trans(evoked.info, trans_fname, subject='sample',
-                     subjects_dir=subjects_dir)
+    fig = plot_alignment(evoked.info, trans_fname, subject='sample',
+                         subjects_dir=subjects_dir)
 
     report.add_figs_to_section(figs=fig,  # test non-list input
                                captions='random image', scale=1.2)
+    assert_true(repr(report))
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_mayavi
 @requires_nibabel()
 def test_render_mri():
-    """Test rendering MRI for mne report.
-    """
+    """Test rendering MRI for mne report."""
     tempdir = _TempDir()
     trans_fname_new = op.join(tempdir, 'temp-trans.fif')
     for a, b in [[trans_fname, trans_fname_new]]:
@@ -191,13 +204,13 @@ def test_render_mri():
         report.parse_folder(data_path=tempdir, mri_decim=30, pattern='*',
                             n_jobs=2)
     report.save(op.join(tempdir, 'report.html'), open_browser=False)
+    assert_true(repr(report))
 
 
 @testing.requires_testing_data
 @requires_nibabel()
 def test_render_mri_without_bem():
-    """Test rendering MRI without BEM for mne report.
-    """
+    """Test rendering MRI without BEM for mne report."""
     tempdir = _TempDir()
     os.mkdir(op.join(tempdir, 'sample'))
     os.mkdir(op.join(tempdir, 'sample', 'mri'))
@@ -214,8 +227,7 @@ def test_render_mri_without_bem():
 @testing.requires_testing_data
 @requires_nibabel()
 def test_add_htmls_to_section():
-    """Test adding html str to mne report.
-    """
+    """Test adding html str to mne report."""
     report = Report(info_fname=raw_fname,
                     subject='sample', subjects_dir=subjects_dir)
     html = '<b>MNE-Python is AWESOME</b>'
@@ -224,11 +236,11 @@ def test_add_htmls_to_section():
     idx = report._sectionlabels.index('report_' + section)
     html_compare = report.html[idx]
     assert_true(html in html_compare)
+    assert_true(repr(report))
 
 
 def test_add_slider_to_section():
-    """Test adding a slider with a series of images to mne report.
-    """
+    """Test adding a slider with a series of images to mne report."""
     tempdir = _TempDir()
     from matplotlib import pyplot as plt
     report = Report(info_fname=raw_fname,
@@ -251,6 +263,7 @@ def test_add_slider_to_section():
 
 
 def test_validate_input():
+    """Test Report input validation."""
     report = Report()
     items = ['a', 'b', 'c']
     captions = ['Letter A', 'Letter B', 'Letter C']
