@@ -12,7 +12,7 @@ raw_new_fname = op.join(test_path, 'test_chpi_raw_sss.fif')
 
 
 def test_read_selection():
-    """Test reading of selections"""
+    """Test reading of selections."""
     # test one channel for each selection
     ch_names = ['MEG 2211', 'MEG 0223', 'MEG 1312', 'MEG 0412', 'MEG 1043',
                 'MEG 2042', 'MEG 2032', 'MEG 0522', 'MEG 1031']
@@ -20,7 +20,7 @@ def test_read_selection():
                  'Right-parietal', 'Left-occipital', 'Right-occipital',
                  'Left-frontal', 'Right-frontal']
 
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     for i, name in enumerate(sel_names):
         sel = read_selection(name)
         assert_true(ch_names[i] in sel)
@@ -40,7 +40,7 @@ def test_read_selection():
     assert_true(len(set(frontal).intersection(set(occipital))) == 0)
 
     ch_names_new = [ch.replace(' ', '') for ch in ch_names]
-    raw_new = read_raw_fif(raw_new_fname, add_eeg_ref=False)
+    raw_new = read_raw_fif(raw_new_fname)
     for i, name in enumerate(sel_names):
         sel = read_selection(name, info=raw_new.info)
         assert_true(ch_names_new[i] in sel)
