@@ -7,23 +7,27 @@ from copy import deepcopy
 import numpy as np
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
                            assert_allclose, assert_equal)
-
+import pytest
 from scipy.fftpack import fft
 
 from mne.datasets import testing
-from mne import (stats, SourceEstimate, VolSourceEstimate, Label,
-                 read_source_spaces, MixedSourceEstimate, read_source_estimate,
-                 morph_data, extract_label_time_course,
+from mne import (stats, SourceEstimate, VectorSourceEstimate,
+                 VolSourceEstimate, Label, read_source_spaces,
+                 read_evokeds, MixedSourceEstimate, find_events, Epochs,
+                 read_source_estimate, morph_data, extract_label_time_course,
                  spatio_temporal_tris_connectivity,
                  spatio_temporal_src_connectivity,
-                 spatial_inter_hemi_connectivity)
+                 spatial_inter_hemi_connectivity,
+                 spatial_src_connectivity)
 from mne.source_estimate import (compute_morph_matrix, grade_to_vertices,
-                                 grade_to_tris)
+                                 grade_to_tris, _get_vol_mask)
 
-from mne.minimum_norm import read_inverse_operator
+from mne.minimum_norm import (read_inverse_operator, apply_inverse,
+                              apply_inverse_epochs)
 from mne.label import read_labels_from_annot, label_sign_flip
 from mne.utils import (_TempDir, requires_pandas, requires_sklearn,
-                       requires_h5py, run_tests_if_main, slow_test)
+                       requires_h5py, run_tests_if_main, requires_nibabel)
+from mne.io import read_raw_fif
 
 warnings.simplefilter('always')  # enable b/c these tests throw warnings
 
@@ -31,6 +35,9 @@ data_path = testing.data_path(download=False)
 subjects_dir = op.join(data_path, 'subjects')
 fname_inv = op.join(data_path, 'MEG', 'sample',
                     'sample_audvis_trunc-meg-eeg-oct-6-meg-inv.fif')
+fname_evoked = op.join(data_path, 'MEG', 'sample',
+                       'sample_audvis_trunc-ave.fif')
+fname_raw = op.join(data_path, 'MEG', 'sample', 'sample_audvis_trunc_raw.fif')
 fname_t1 = op.join(data_path, 'subjects', 'sample', 'mri', 'T1.mgz')
 fname_src = op.join(data_path, 'MEG', 'sample',
                     'sample_audvis_trunc-meg-eeg-oct-6-fwd.fif')
@@ -49,8 +56,8 @@ rng = np.random.RandomState(0)
 
 
 @testing.requires_testing_data
-def test_aaspatial_inter_hemi_connectivity():
-    """Test spatial connectivity between hemispheres"""
+def test_spatial_inter_hemi_connectivity():
+    """Test spatial connectivity between hemispheres."""
     # trivial cases
     conn = spatial_inter_hemi_connectivity(fname_src_3, 5e-6)
     assert_equal(conn.data.size, 0)
@@ -82,11 +89,10 @@ def test_aaspatial_inter_hemi_connectivity():
         assert_true(set(use_labels) - set(good_labels) == set())
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_volume_stc():
-    """Test volume STCs
-    """
+    """Test volume STCs."""
     tempdir = _TempDir()
     N = 100
     data = np.arange(N)[:, np.newaxis]
@@ -140,14 +146,12 @@ def test_volume_stc():
         with warnings.catch_warnings(record=True):  # nib<->numpy
             img = nib.load(vol_fname)
         assert_true(img.shape == t1_img.shape + (len(stc.times),))
-        assert_array_almost_equal(img.get_affine(), t1_img.get_affine(),
-                                  decimal=5)
+        assert_array_almost_equal(img.affine, t1_img.affine, decimal=5)
 
         # export without saving
         img = stc.as_volume(src, dest='mri', mri_resolution=True)
         assert_true(img.shape == t1_img.shape + (len(stc.times),))
-        assert_array_almost_equal(img.get_affine(), t1_img.get_affine(),
-                                  decimal=5)
+        assert_array_almost_equal(img.affine, t1_img.affine, decimal=5)
 
     except ImportError:
         print('Save as nifti test skipped, needs NiBabel')
@@ -155,22 +159,27 @@ def test_volume_stc():
 
 @testing.requires_testing_data
 def test_expand():
-    """Test stc expansion
-    """
-    stc = read_source_estimate(fname_stc, 'sample')
-    assert_true('sample' in repr(stc))
-    labels_lh = read_labels_from_annot('sample', 'aparc', 'lh',
-                                       subjects_dir=subjects_dir)
-    new_label = labels_lh[0] + labels_lh[1]
-    stc_limited = stc.in_label(new_label)
-    stc_new = stc_limited.copy()
-    stc_new.data.fill(0)
-    for label in labels_lh[:2]:
-        stc_new += stc.in_label(label).expand(stc_limited.vertices)
-    assert_raises(TypeError, stc_new.expand, stc_limited.vertices[0])
-    assert_raises(ValueError, stc_new.expand, [stc_limited.vertices[0]])
-    # make sure we can't add unless vertno agree
-    assert_raises(ValueError, stc.__add__, stc.in_label(labels_lh[0]))
+    """Test stc expansion."""
+    stc_ = read_source_estimate(fname_stc, 'sample')
+    vec_stc_ = VectorSourceEstimate(np.zeros((stc_.data.shape[0], 3,
+                                              stc_.data.shape[1])),
+                                    stc_.vertices, stc_.tmin, stc_.tstep,
+                                    stc_.subject)
+
+    for stc in [stc_, vec_stc_]:
+        assert_true('sample' in repr(stc))
+        labels_lh = read_labels_from_annot('sample', 'aparc', 'lh',
+                                           subjects_dir=subjects_dir)
+        new_label = labels_lh[0] + labels_lh[1]
+        stc_limited = stc.in_label(new_label)
+        stc_new = stc_limited.copy()
+        stc_new.data.fill(0)
+        for label in labels_lh[:2]:
+            stc_new += stc.in_label(label).expand(stc_limited.vertices)
+        assert_raises(TypeError, stc_new.expand, stc_limited.vertices[0])
+        assert_raises(ValueError, stc_new.expand, [stc_limited.vertices[0]])
+        # make sure we can't add unless vertno agree
+        assert_raises(ValueError, stc.__add__, stc.in_label(labels_lh[0]))
 
 
 def _fake_stc(n_time=10):
@@ -178,9 +187,84 @@ def _fake_stc(n_time=10):
     return SourceEstimate(np.random.rand(100, n_time), verts, 0, 1e-1, 'foo')
 
 
+def _fake_vec_stc(n_time=10):
+    verts = [np.arange(10), np.arange(90)]
+    return VectorSourceEstimate(np.random.rand(100, 3, n_time), verts, 0, 1e-1,
+                                'foo')
+
+
+def _real_vec_stc():
+    inv = read_inverse_operator(fname_inv)
+    evoked = read_evokeds(fname_evoked, baseline=(None, 0))[0].crop(0, 0.01)
+    return apply_inverse(evoked, inv, pick_ori='vector')
+
+
+def _test_stc_integrety(stc):
+    """Test consistency of tmin, tstep, data.shape[-1] and times."""
+    n_times = len(stc.times)
+    assert_equal(stc._data.shape[-1], n_times)
+    assert_array_equal(stc.times, stc.tmin + np.arange(n_times) * stc.tstep)
+
+
+def test_stc_attributes():
+    """Test STC attributes."""
+    stc = _fake_stc(n_time=10)
+    vec_stc = _fake_vec_stc(n_time=10)
+
+    _test_stc_integrety(stc)
+    assert_array_almost_equal(
+        stc.times, [0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
+
+    def attempt_times_mutation(stc):
+        stc.times -= 1
+
+    def attempt_assignment(stc, attr, val):
+        setattr(stc, attr, val)
+
+    # .times is read-only
+    assert_raises(ValueError, attempt_times_mutation, stc)
+    assert_raises(ValueError, attempt_assignment, stc, 'times', [1])
+
+    # Changing .tmin or .tstep re-computes .times
+    stc.tmin = 1
+    assert_true(type(stc.tmin) == float)
+    assert_array_almost_equal(
+        stc.times, [1., 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9])
+
+    stc.tstep = 1
+    assert_true(type(stc.tstep) == float)
+    assert_array_almost_equal(
+        stc.times, [1., 2., 3., 4., 5., 6., 7., 8., 9., 10.])
+
+    # tstep <= 0 is not allowed
+    assert_raises(ValueError, attempt_assignment, stc, 'tstep', 0)
+    assert_raises(ValueError, attempt_assignment, stc, 'tstep', -1)
+
+    # Changing .data re-computes .times
+    stc.data = np.random.rand(100, 5)
+    assert_array_almost_equal(
+        stc.times, [1., 2., 3., 4., 5.])
+
+    # .data must match the number of vertices
+    assert_raises(ValueError, attempt_assignment, stc, 'data', [[1]])
+    assert_raises(ValueError, attempt_assignment, stc, 'data', None)
+
+    # .data much match number of dimensions
+    assert_raises(ValueError, attempt_assignment, stc, 'data', np.arange(100))
+    assert_raises(ValueError, attempt_assignment, vec_stc, 'data',
+                  [np.arange(100)])
+    assert_raises(ValueError, attempt_assignment, vec_stc, 'data',
+                  [[[np.arange(100)]]])
+
+    # .shape attribute must also work when ._data is None
+    stc._kernel = np.zeros((2, 2))
+    stc._sens_data = np.zeros((2, 3))
+    stc._data = None
+    assert_equal(stc.shape, (2, 3))
+
+
 def test_io_stc():
-    """Test IO for STC files
-    """
+    """Test IO for STC files."""
     tempdir = _TempDir()
     stc = _fake_stc()
     stc.save(op.join(tempdir, "tmp.stc"))
@@ -196,29 +280,31 @@ def test_io_stc():
 
 @requires_h5py
 def test_io_stc_h5():
-    """Test IO for STC files using HDF5
-    """
-    tempdir = _TempDir()
-    stc = _fake_stc()
-    assert_raises(ValueError, stc.save, op.join(tempdir, 'tmp'), ftype='foo')
-    out_name = op.join(tempdir, 'tmp')
-    stc.save(out_name, ftype='h5')
-    stc3 = read_source_estimate(out_name)
-    stc4 = read_source_estimate(out_name + '-stc.h5')
-    assert_raises(RuntimeError, read_source_estimate, out_name, subject='bar')
-    for stc_new in stc3, stc4:
-        assert_equal(stc_new.subject, stc.subject)
-        assert_array_equal(stc_new.data, stc.data)
-        assert_array_equal(stc_new.tmin, stc.tmin)
-        assert_array_equal(stc_new.tstep, stc.tstep)
-        assert_equal(len(stc_new.vertices), len(stc.vertices))
-        for v1, v2 in zip(stc_new.vertices, stc.vertices):
-            assert_array_equal(v1, v2)
+    """Test IO for STC files using HDF5."""
+    for stc in [_fake_stc(), _fake_vec_stc()]:
+        tempdir = _TempDir()
+        assert_raises(ValueError, stc.save, op.join(tempdir, 'tmp'),
+                      ftype='foo')
+        out_name = op.join(tempdir, 'tmp')
+        stc.save(out_name, ftype='h5')
+        stc.save(out_name, ftype='h5')  # test overwrite
+        stc3 = read_source_estimate(out_name)
+        stc4 = read_source_estimate(out_name + '-stc')
+        stc5 = read_source_estimate(out_name + '-stc.h5')
+        assert_raises(RuntimeError, read_source_estimate, out_name,
+                      subject='bar')
+        for stc_new in stc3, stc4, stc5:
+            assert_equal(stc_new.subject, stc.subject)
+            assert_array_equal(stc_new.data, stc.data)
+            assert_array_equal(stc_new.tmin, stc.tmin)
+            assert_array_equal(stc_new.tstep, stc.tstep)
+            assert_equal(len(stc_new.vertices), len(stc.vertices))
+            for v1, v2 in zip(stc_new.vertices, stc.vertices):
+                assert_array_equal(v1, v2)
 
 
 def test_io_w():
-    """Test IO for w files
-    """
+    """Test IO for w files."""
     tempdir = _TempDir()
     stc = _fake_stc(n_time=1)
     w_fname = op.join(tempdir, 'fake')
@@ -232,13 +318,14 @@ def test_io_w():
 
 
 def test_stc_arithmetic():
-    """Test arithmetic for STC files
-    """
+    """Test arithmetic for STC files."""
     stc = _fake_stc()
     data = stc.data.copy()
+    vec_stc = _fake_vec_stc()
+    vec_data = vec_stc.data.copy()
 
     out = list()
-    for a in [data, stc]:
+    for a in [data, stc, vec_data, vec_stc]:
         a = a + a * 3 + 3 * a - a ** 2 / 2
 
         a += a
@@ -262,75 +349,94 @@ def test_stc_arithmetic():
         out.append(a)
 
     assert_array_equal(out[0], out[1].data)
+    assert_array_equal(out[2], out[3].data)
     assert_array_equal(stc.sqrt().data, np.sqrt(stc.data))
+    assert_array_equal(vec_stc.sqrt().data, np.sqrt(vec_stc.data))
+    assert_array_equal(abs(stc).data, abs(stc.data))
+    assert_array_equal(abs(vec_stc).data, abs(vec_stc.data))
 
     stc_mean = stc.mean()
     assert_array_equal(stc_mean.data, np.mean(stc.data, 1)[:, None])
+    vec_stc_mean = vec_stc.mean()
+    assert_array_equal(vec_stc_mean.data,
+                       np.mean(vec_stc.data, 2)[:, :, None])
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_stc_methods():
-    """Test stc methods lh_data, rh_data, bin(), center_of_mass(), resample()
-    """
+    """Test stc methods lh_data, rh_data, bin(), resample()."""
+    stc_ = read_source_estimate(fname_stc)
+
+    # Make a vector version of the above source estimate
+    x = stc_.data[:, np.newaxis, :]
+    yz = np.zeros((x.shape[0], 2, x.shape[2]))
+    vec_stc_ = VectorSourceEstimate(
+        np.concatenate((x, yz), 1),
+        stc_.vertices, stc_.tmin, stc_.tstep, stc_.subject
+    )
+
+    for stc in [stc_, vec_stc_]:
+        # lh_data / rh_data
+        assert_array_equal(stc.lh_data, stc.data[:len(stc.lh_vertno)])
+        assert_array_equal(stc.rh_data, stc.data[len(stc.lh_vertno):])
+
+        # bin
+        binned = stc.bin(.12)
+        a = np.mean(stc.data[..., :np.searchsorted(stc.times, .12)], axis=-1)
+        assert_array_equal(a, binned.data[..., 0])
+
+        stc = read_source_estimate(fname_stc)
+        stc.subject = 'sample'
+        label_lh = read_labels_from_annot('sample', 'aparc', 'lh',
+                                          subjects_dir=subjects_dir)[0]
+        label_rh = read_labels_from_annot('sample', 'aparc', 'rh',
+                                          subjects_dir=subjects_dir)[0]
+        label_both = label_lh + label_rh
+        for label in (label_lh, label_rh, label_both):
+            assert_true(isinstance(stc.shape, tuple) and len(stc.shape) == 2)
+            stc_label = stc.in_label(label)
+            if label.hemi != 'both':
+                if label.hemi == 'lh':
+                    verts = stc_label.vertices[0]
+                else:  # label.hemi == 'rh':
+                    verts = stc_label.vertices[1]
+                n_vertices_used = len(label.get_vertices_used(verts))
+                assert_equal(len(stc_label.data), n_vertices_used)
+        stc_lh = stc.in_label(label_lh)
+        assert_raises(ValueError, stc_lh.in_label, label_rh)
+        label_lh.subject = 'foo'
+        assert_raises(RuntimeError, stc.in_label, label_lh)
+
+        stc_new = deepcopy(stc)
+        o_sfreq = 1.0 / stc.tstep
+        # note that using no padding for this STC reduces edge ringing...
+        stc_new.resample(2 * o_sfreq, npad=0, n_jobs=2)
+        assert_true(stc_new.data.shape[1] == 2 * stc.data.shape[1])
+        assert_true(stc_new.tstep == stc.tstep / 2)
+        stc_new.resample(o_sfreq, npad=0)
+        assert_true(stc_new.data.shape[1] == stc.data.shape[1])
+        assert_true(stc_new.tstep == stc.tstep)
+        assert_array_almost_equal(stc_new.data, stc.data, 5)
+
+
+@testing.requires_testing_data
+def test_center_of_mass():
+    """Test computing the center of mass on an stc."""
     stc = read_source_estimate(fname_stc)
-
-    # lh_data / rh_data
-    assert_array_equal(stc.lh_data, stc.data[:len(stc.lh_vertno)])
-    assert_array_equal(stc.rh_data, stc.data[len(stc.lh_vertno):])
-
-    # bin
-    bin = stc.bin(.12)
-    a = np.array((1,), dtype=stc.data.dtype)
-    a[0] = np.mean(stc.data[0, stc.times < .12])
-    assert a[0] == bin.data[0, 0]
-
     assert_raises(ValueError, stc.center_of_mass, 'sample')
     stc.lh_data[:] = 0
     vertex, hemi, t = stc.center_of_mass('sample', subjects_dir=subjects_dir)
     assert_true(hemi == 1)
-    # XXX Should design a fool-proof test case, but here were the results:
+    # XXX Should design a fool-proof test case, but here were the
+    # results:
     assert_equal(vertex, 124791)
     assert_equal(np.round(t, 2), 0.12)
 
-    stc = read_source_estimate(fname_stc)
-    stc.subject = 'sample'
-    label_lh = read_labels_from_annot('sample', 'aparc', 'lh',
-                                      subjects_dir=subjects_dir)[0]
-    label_rh = read_labels_from_annot('sample', 'aparc', 'rh',
-                                      subjects_dir=subjects_dir)[0]
-    label_both = label_lh + label_rh
-    for label in (label_lh, label_rh, label_both):
-        assert_true(isinstance(stc.shape, tuple) and len(stc.shape) == 2)
-        stc_label = stc.in_label(label)
-        if label.hemi != 'both':
-            if label.hemi == 'lh':
-                verts = stc_label.vertices[0]
-            else:  # label.hemi == 'rh':
-                verts = stc_label.vertices[1]
-            n_vertices_used = len(label.get_vertices_used(verts))
-            assert_equal(len(stc_label.data), n_vertices_used)
-    stc_lh = stc.in_label(label_lh)
-    assert_raises(ValueError, stc_lh.in_label, label_rh)
-    label_lh.subject = 'foo'
-    assert_raises(RuntimeError, stc.in_label, label_lh)
-
-    stc_new = deepcopy(stc)
-    o_sfreq = 1.0 / stc.tstep
-    # note that using no padding for this STC reduces edge ringing...
-    stc_new.resample(2 * o_sfreq, npad=0, n_jobs=2)
-    assert_true(stc_new.data.shape[1] == 2 * stc.data.shape[1])
-    assert_true(stc_new.tstep == stc.tstep / 2)
-    stc_new.resample(o_sfreq, npad=0)
-    assert_true(stc_new.data.shape[1] == stc.data.shape[1])
-    assert_true(stc_new.tstep == stc.tstep)
-    assert_array_almost_equal(stc_new.data, stc.data, 5)
-
 
 @testing.requires_testing_data
 def test_extract_label_time_course():
-    """Test extraction of label time courses from stc
-    """
+    """Test extraction of label time courses from stc."""
     n_stcs = 3
     n_times = 50
 
@@ -421,11 +527,10 @@ def test_extract_label_time_course():
     assert_true(x.size == 0)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_morph_data():
-    """Test morphing of data
-    """
+    """Test morphing of data."""
     tempdir = _TempDir()
     subject_from = 'sample'
     subject_to = 'fsaverage'
@@ -441,6 +546,11 @@ def test_morph_data():
     stc_to1 = stc_from.morph(subject_to, grade=3, smooth=12, buffer_size=1000,
                              subjects_dir=subjects_dir)
     stc_to1.save(op.join(tempdir, '%s_audvis-meg' % subject_to))
+    # Morphing to a density that is too high should raise an informative error
+    # (here we need to push to grade=6, but for some subjects even grade=5
+    # will break)
+    assert_raises(ValueError, stc_to1.morph, subject_from, grade=6,
+                  subjects_dir=subjects_dir)
     # make sure we can specify vertices
     vertices_to = grade_to_vertices(subject_to, grade=3,
                                     subjects_dir=subjects_dir)
@@ -526,16 +636,25 @@ def test_morph_data():
     assert_equal(stc_from.tmin, stc_from.tmin)
     assert_equal(stc_from.tstep, stc_from.tstep)
 
+    # Morph vector data
+    stc_vec = _real_vec_stc()
+
+    # Ignore warnings about number of steps
+    stc_vec_to1 = stc_vec.morph(subject_to, grade=3, smooth=12,
+                                buffer_size=1000, subjects_dir=subjects_dir)
+    stc_vec_to2 = stc_vec.morph_precomputed(subject_to, vertices_to, morph_mat)
+    assert_array_almost_equal(stc_vec_to1.data, stc_vec_to2.data)
+
 
 def _my_trans(data):
-    """FFT that adds an additional dimension by repeating result"""
+    """FFT that adds an additional dimension by repeating result."""
     data_t = fft(data)
     data_t = np.concatenate([data_t[:, :, None], data_t[:, :, None]], axis=2)
     return data_t, None
 
 
 def test_transform_data():
-    """Test applying linear (time) transform to data"""
+    """Test applying linear (time) transform to data."""
     # make up some data
     n_sensors, n_vertices, n_times = 10, 20, 4
     kernel = rng.randn(n_vertices, n_sensors)
@@ -565,7 +684,7 @@ def test_transform_data():
 
 
 def test_transform():
-    """Test applying linear (time) transform to data"""
+    """Test applying linear (time) transform to data."""
     # make up some data
     n_verts_lh, n_verts_rh, n_times = 10, 10, 10
     vertices = [np.arange(n_verts_lh), n_verts_lh + np.arange(n_verts_rh)]
@@ -597,27 +716,30 @@ def test_transform():
     verts = np.arange(len(stc.lh_vertno),
                       len(stc.lh_vertno) + len(stc.rh_vertno), 1)
     verts_rh = stc.rh_vertno
-    t_idx = [np.where(times >= -50)[0][0], np.where(times <= 500)[0][-1]]
-    data_t = stc.transform_data(np.abs, idx=verts, tmin_idx=t_idx[0],
-                                tmax_idx=t_idx[-1])
+    tmin_idx = np.searchsorted(times, 0)
+    tmax_idx = np.searchsorted(times, 501)  # Include 500ms in the range
+    data_t = stc.transform_data(np.abs, idx=verts, tmin_idx=tmin_idx,
+                                tmax_idx=tmax_idx)
     stc.transform(np.abs, idx=verts, tmin=-50, tmax=500, copy=False)
     assert_true(isinstance(stc, SourceEstimate))
-    assert_true((stc.tmin == 0.) & (stc.times[-1] == 0.5))
-    assert_true(len(stc.vertices[0]) == 0)
+    assert_equal(stc.tmin, 0.)
+    assert_equal(stc.times[-1], 0.5)
+    assert_equal(len(stc.vertices[0]), 0)
     assert_equal(stc.vertices[1], verts_rh)
     assert_array_equal(stc.data, data_t)
 
     times = np.round(1000 * stc.times)
-    t_idx = [np.where(times >= 0)[0][0], np.where(times <= 250)[0][-1]]
-    data_t = stc.transform_data(np.abs, tmin_idx=t_idx[0], tmax_idx=t_idx[-1])
+    tmin_idx, tmax_idx = np.searchsorted(times, 0), np.searchsorted(times, 250)
+    data_t = stc.transform_data(np.abs, tmin_idx=tmin_idx, tmax_idx=tmax_idx)
     stc.transform(np.abs, tmin=0, tmax=250, copy=False)
-    assert_true((stc.tmin == 0.) & (stc.times[-1] == 0.2))
+    assert_equal(stc.tmin, 0.)
+    assert_equal(stc.times[-1], 0.2)
     assert_array_equal(stc.data, data_t)
 
 
 @requires_sklearn
 def test_spatio_temporal_tris_connectivity():
-    """Test spatio-temporal connectivity from triangles"""
+    """Test spatio-temporal connectivity from triangles."""
     tris = np.array([[0, 1, 2], [3, 4, 5]])
     connectivity = spatio_temporal_tris_connectivity(tris, 2)
     x = [1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]
@@ -634,7 +756,7 @@ def test_spatio_temporal_tris_connectivity():
 
 @testing.requires_testing_data
 def test_spatio_temporal_src_connectivity():
-    """Test spatio-temporal connectivity from source spaces"""
+    """Test spatio-temporal connectivity from source spaces."""
     tris = np.array([[0, 1, 2], [3, 4, 5]])
     src = [dict(), dict()]
     connectivity = spatio_temporal_tris_connectivity(tris, 2)
@@ -642,6 +764,8 @@ def test_spatio_temporal_src_connectivity():
     src[1]['use_tris'] = np.array([[0, 1, 2]])
     src[0]['vertno'] = np.array([0, 1, 2])
     src[1]['vertno'] = np.array([0, 1, 2])
+    src[0]['type'] = 'surf'
+    src[1]['type'] = 'surf'
     connectivity2 = spatio_temporal_src_connectivity(src, 2)
     assert_array_equal(connectivity.todense(), connectivity2.todense())
     # add test for dist connectivity
@@ -649,6 +773,8 @@ def test_spatio_temporal_src_connectivity():
     src[1]['dist'] = np.ones((3, 3)) - np.eye(3)
     src[0]['vertno'] = [0, 1, 2]
     src[1]['vertno'] = [0, 1, 2]
+    src[0]['type'] = 'surf'
+    src[1]['type'] = 'surf'
     connectivity3 = spatio_temporal_src_connectivity(src, 2, dist=2)
     assert_array_equal(connectivity.todense(), connectivity3.todense())
     # add test for source space connectivity with omitted vertices
@@ -657,7 +783,7 @@ def test_spatio_temporal_src_connectivity():
         warnings.simplefilter('always')
         src_ = inverse_operator['src']
         connectivity = spatio_temporal_src_connectivity(src_, n_times=2)
-        assert len(w) == 1
+    assert_equal(len(w), 1)
     a = connectivity.shape[0] / 2
     b = sum([s['nuse'] for s in inverse_operator['src']])
     assert_true(a == b)
@@ -667,7 +793,7 @@ def test_spatio_temporal_src_connectivity():
 
 @requires_pandas
 def test_to_data_frame():
-    """Test stc Pandas exporter"""
+    """Test stc Pandas exporter."""
     n_vert, n_times = 10, 5
     vertices = [np.arange(n_vert, dtype=np.int), np.empty(0, dtype=np.int)]
     data = rng.randn(n_vert, n_times)
@@ -688,8 +814,7 @@ def test_to_data_frame():
 
 
 def test_get_peak():
-    """Test peak getter
-    """
+    """Test peak getter."""
     n_vert, n_times = 10, 5
     vertices = [np.arange(n_vert, dtype=np.int), np.empty(0, dtype=np.int)]
     data = rng.randn(n_vert, n_times)
@@ -717,8 +842,7 @@ def test_get_peak():
 
 @testing.requires_testing_data
 def test_mixed_stc():
-    """Test source estimate from mixed source space
-    """
+    """Test source estimate from mixed source space."""
     N = 90  # number of sources
     T = 2  # number of time points
     S = 3  # number of source spaces
@@ -738,4 +862,103 @@ def test_mixed_stc():
     assert_raises(ValueError, stc.plot_surface, src=vol)
 
 
+def test_vec_stc():
+    """Test vector source estimate."""
+    nn = np.array([
+        [1, 0, 0],
+        [0, 1, 0],
+        [0, 0, 1],
+        [np.sqrt(1 / 3.)] * 3
+    ])
+    src = [dict(nn=nn[:2]), dict(nn=nn[2:])]
+
+    verts = [np.array([0, 1]), np.array([0, 1])]
+    data = np.array([
+        [1, 0, 0],
+        [0, 2, 0],
+        [3, 0, 0],
+        [1, 1, 1],
+    ])[:, :, np.newaxis]
+    stc = VectorSourceEstimate(data, verts, 0, 1, 'foo')
+
+    # Magnitude of the vectors
+    assert_array_equal(stc.magnitude().data[:, 0], [1, 2, 3, np.sqrt(3)])
+
+    # Vector components projected onto the vertex normals
+    normal = stc.normal(src)
+    assert_array_equal(normal.data[:, 0], [1, 2, 0, np.sqrt(3)])
+
+
+@testing.requires_testing_data
+def test_epochs_vector_inverse():
+    """Test vector inverse consistency between evoked and epochs"""
+
+    raw = read_raw_fif(fname_raw)
+    events = find_events(raw, stim_channel='STI 014')[:2]
+    reject = dict(grad=2000e-13, mag=4e-12, eog=150e-6)
+
+    epochs = Epochs(raw, events, None, 0, 0.01, baseline=None,
+                    reject=reject, preload=True)
+
+    assert_equal(len(epochs), 2)
+
+    evoked = epochs.average(picks=range(len(epochs.ch_names)))
+
+    inv = read_inverse_operator(fname_inv)
+
+    method = "MNE"
+    snr = 3.
+    lambda2 = 1. / snr ** 2
+
+    stcs_epo = apply_inverse_epochs(epochs, inv, lambda2, method=method,
+                                    pick_ori='vector', return_generator=False)
+    stc_epo = np.mean(stcs_epo)
+
+    stc_evo = apply_inverse(evoked, inv, lambda2, method=method,
+                            pick_ori='vector')
+
+    assert_allclose(stc_epo.data, stc_evo.data, rtol=1e-9, atol=0)
+
+
+@requires_sklearn
+@testing.requires_testing_data
+def test_vol_connectivity():
+    """Test volume connectivity."""
+    from scipy import sparse
+    vol = read_source_spaces(fname_vsrc)
+
+    assert_raises(ValueError, spatial_src_connectivity, vol, dist=1.)
+
+    connectivity = spatial_src_connectivity(vol)
+    n_vertices = vol[0]['inuse'].sum()
+    assert_equal(connectivity.shape, (n_vertices, n_vertices))
+    assert_true(np.all(connectivity.data == 1))
+    assert_true(isinstance(connectivity, sparse.coo_matrix))
+
+    connectivity2 = spatio_temporal_src_connectivity(vol, n_times=2)
+    assert_equal(connectivity2.shape, (2 * n_vertices, 2 * n_vertices))
+    assert_true(np.all(connectivity2.data == 1))
+
+
+@requires_sklearn
+@requires_nibabel()
+@testing.requires_testing_data
+def test_vol_mask():
+    """Test extraction of volume mask."""
+    src = read_source_spaces(fname_vsrc)
+    mask = _get_vol_mask(src)
+    # Let's use an alternative way that should be equivalent
+    vertices = src[0]['vertno']
+    n_vertices = len(vertices)
+    data = (1 + np.arange(n_vertices))[:, np.newaxis]
+    stc_tmp = VolSourceEstimate(data, vertices, tmin=0., tstep=1.)
+    img = stc_tmp.as_volume(src, mri_resolution=False)
+    img_data = img.get_data()[:, :, :, 0].T
+    mask_nib = (img_data != 0)
+    assert_array_equal(img_data[mask_nib], data[:, 0])
+    assert_array_equal(np.where(mask_nib.ravel())[0], src[0]['vertno'])
+    assert_array_equal(mask, mask_nib)
+    assert_array_equal(img_data.shape, mask.shape)
+
+
 run_tests_if_main()
