@@ -2,8 +2,9 @@ from __future__ import print_function
 
 import os
 import os.path as op
+from unittest import SkipTest
 from nose.tools import assert_true, assert_raises
-from nose.plugins.skip import SkipTest
+import pytest
 import numpy as np
 from numpy.testing import assert_array_equal, assert_allclose, assert_equal
 import warnings
@@ -14,12 +15,13 @@ from mne import (read_source_spaces, vertex_to_mni, write_source_spaces,
                  add_source_space_distances, read_bem_surfaces,
                  morph_source_spaces, SourceEstimate)
 from mne.utils import (_TempDir, requires_fs_or_nibabel, requires_nibabel,
-                       requires_freesurfer, run_subprocess, slow_test,
+                       requires_freesurfer, run_subprocess,
                        requires_mne, requires_version, run_tests_if_main)
 from mne.surface import _accumulate_normals, _triangle_neighbors
 from mne.source_space import _get_mri_header, _get_mgz_header
 from mne.externals.six.moves import zip
 from mne.source_space import (get_volume_labels_from_aseg, SourceSpaces,
+                              get_volume_labels_from_src,
                               _compare_source_spaces)
 from mne.tests.common import assert_naming
 from mne.io.constants import FIFF
@@ -46,7 +48,7 @@ rng = np.random.RandomState(0)
 @testing.requires_testing_data
 @requires_nibabel(vox2ras_tkr=True)
 def test_mgz_header():
-    """Test MGZ header reading"""
+    """Test MGZ header reading."""
     header = _get_mgz_header(fname_mri)
     mri_hdr = _get_mri_header(fname_mri)
     assert_allclose(mri_hdr.get_data_shape(), header['dims'])
@@ -56,7 +58,7 @@ def test_mgz_header():
 
 @requires_version('scipy', '0.11')
 def test_add_patch_info():
-    """Test adding patch info to source space"""
+    """Test adding patch info to source space."""
     # let's setup a small source space
     src = read_source_spaces(fname_small)
     src_new = read_source_spaces(fname_small)
@@ -89,7 +91,7 @@ def test_add_patch_info():
 @testing.requires_testing_data
 @requires_version('scipy', '0.11')
 def test_add_source_space_distances_limited():
-    """Test adding distances to source space with a dist_limit"""
+    """Test adding distances to source space with a dist_limit."""
     tempdir = _TempDir()
     src = read_source_spaces(fname)
     src_new = read_source_spaces(fname)
@@ -124,11 +126,11 @@ def test_add_source_space_distances_limited():
         assert_allclose(np.zeros_like(d.data), d.data, rtol=0, atol=1e-6)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 @requires_version('scipy', '0.11')
 def test_add_source_space_distances():
-    """Test adding distances to source space"""
+    """Test adding distances to source space."""
     tempdir = _TempDir()
     src = read_source_spaces(fname)
     src_new = read_source_spaces(fname)
@@ -171,8 +173,7 @@ def test_add_source_space_distances():
 @testing.requires_testing_data
 @requires_mne
 def test_discrete_source_space():
-    """Test setting up (and reading/writing) discrete source spaces
-    """
+    """Test setting up (and reading/writing) discrete source spaces."""
     tempdir = _TempDir()
     src = read_source_spaces(fname)
     v = src[0]['vertno']
@@ -188,9 +189,7 @@ def test_discrete_source_space():
                         '--pos', temp_pos, '--src', temp_name])
         src_c = read_source_spaces(temp_name)
         pos_dict = dict(rr=src[0]['rr'][v], nn=src[0]['nn'][v])
-        src_new = setup_volume_source_space('sample', None,
-                                            pos=pos_dict,
-                                            subjects_dir=subjects_dir)
+        src_new = setup_volume_source_space(None, pos=pos_dict)
         _compare_source_spaces(src_c, src_new, mode='approx')
         assert_allclose(src[0]['rr'][v], src_new[0]['rr'],
                         rtol=1e-3, atol=1e-6)
@@ -198,23 +197,24 @@ def test_discrete_source_space():
                         rtol=1e-3, atol=1e-6)
 
         # now do writing
-        write_source_spaces(temp_name, src_c)
+        write_source_spaces(temp_name, src_c, overwrite=True)
         src_c2 = read_source_spaces(temp_name)
         _compare_source_spaces(src_c, src_c2)
 
         # now do MRI
         assert_raises(ValueError, setup_volume_source_space, 'sample',
                       pos=pos_dict, mri=fname_mri)
+        assert_equal(repr(src_new), repr(src_c))
+        assert_equal(src_new.kind, 'discrete')
     finally:
         if op.isfile(temp_name):
             os.remove(temp_name)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_volume_source_space():
-    """Test setting up volume source spaces
-    """
+    """Test setting up volume source spaces."""
     tempdir = _TempDir()
     src = read_source_spaces(fname_vol)
     temp_name = op.join(tempdir, 'temp-src.fif')
@@ -222,23 +222,26 @@ def test_volume_source_space():
     surf['rr'] *= 1e3  # convert to mm
     # The one in the testing dataset (uses bem as bounds)
     for bem, surf in zip((fname_bem, None), (None, surf)):
-        src_new = setup_volume_source_space('sample', temp_name, pos=7.0,
-                                            bem=bem, surface=surf,
-                                            mri=fname_mri,
-                                            subjects_dir=subjects_dir)
+        src_new = setup_volume_source_space(
+            'sample', pos=7.0, bem=bem, surface=surf, mri='T1.mgz',
+            subjects_dir=subjects_dir)
+        write_source_spaces(temp_name, src_new, overwrite=True)
+        src[0]['subject_his_id'] = 'sample'  # XXX: to make comparison pass
         _compare_source_spaces(src, src_new, mode='approx')
         del src_new
         src_new = read_source_spaces(temp_name)
         _compare_source_spaces(src, src_new, mode='approx')
-    assert_raises(IOError, setup_volume_source_space, 'sample', temp_name,
+    assert_raises(IOError, setup_volume_source_space, 'sample',
                   pos=7.0, bem=None, surface='foo',  # bad surf
                   mri=fname_mri, subjects_dir=subjects_dir)
+    assert_equal(repr(src), repr(src_new))
+    assert_equal(src.kind, 'volume')
 
 
 @testing.requires_testing_data
 @requires_mne
 def test_other_volume_source_spaces():
-    """Test setting up other volume source spaces"""
+    """Test setting up other volume source spaces."""
     # these are split off because they require the MNE tools, and
     # Travis doesn't seem to like them
 
@@ -250,14 +253,26 @@ def test_other_volume_source_spaces():
                     '--src', temp_name,
                     '--mri', fname_mri])
     src = read_source_spaces(temp_name)
-    src_new = setup_volume_source_space('sample', temp_name, pos=7.0,
-                                        mri=fname_mri,
+    src_new = setup_volume_source_space(None, pos=7.0, mri=fname_mri,
                                         subjects_dir=subjects_dir)
-    _compare_source_spaces(src, src_new, mode='approx')
+    # we use a more accurate elimination criteria, so let's fix the MNE-C
+    # source space
+    assert_equal(len(src_new[0]['vertno']), 7497)
+    assert_equal(len(src), 1)
+    assert_equal(len(src_new), 1)
+    good_mask = np.in1d(src[0]['vertno'], src_new[0]['vertno'])
+    src[0]['inuse'][src[0]['vertno'][~good_mask]] = 0
+    assert_equal(src[0]['inuse'].sum(), 7497)
+    src[0]['vertno'] = src[0]['vertno'][good_mask]
+    assert_equal(len(src[0]['vertno']), 7497)
+    src[0]['nuse'] = len(src[0]['vertno'])
+    assert_equal(src[0]['nuse'], 7497)
+    _compare_source_spaces(src_new, src, mode='approx')
+    assert_true('volume, shape' in repr(src))
     del src
     del src_new
-    assert_raises(ValueError, setup_volume_source_space, 'sample', temp_name,
-                  pos=7.0, sphere=[1., 1.], mri=fname_mri,  # bad sphere
+    assert_raises(ValueError, setup_volume_source_space, 'sample', pos=7.0,
+                  sphere=[1., 1.], mri=fname_mri,  # bad sphere
                   subjects_dir=subjects_dir)
 
     # now without MRI argument, it should give an error when we try
@@ -270,7 +285,7 @@ def test_other_volume_source_spaces():
 
 @testing.requires_testing_data
 def test_triangle_neighbors():
-    """Test efficient vertex neighboring triangles for surfaces"""
+    """Test efficient vertex neighboring triangles for surfaces."""
     this = read_source_spaces(fname)[0]
     this['neighbor_tri'] = [list() for _ in range(this['np'])]
     for p in range(this['ntri']):
@@ -286,7 +301,7 @@ def test_triangle_neighbors():
 
 
 def test_accumulate_normals():
-    """Test efficient normal accumulation for surfaces"""
+    """Test efficient normal accumulation for surfaces."""
     # set up comparison
     n_pts = int(1.6e5)  # approx number in sample source space
     n_tris = int(3.2e5)
@@ -310,37 +325,34 @@ def test_accumulate_normals():
     assert_allclose(nn, this['nn'], rtol=1e-7, atol=1e-7)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_setup_source_space():
-    """Test setting up ico, oct, and all source spaces
-    """
+    """Test setting up ico, oct, and all source spaces."""
     tempdir = _TempDir()
     fname_ico = op.join(data_path, 'subjects', 'fsaverage', 'bem',
                         'fsaverage-ico-5-src.fif')
     # first lets test some input params
     assert_raises(ValueError, setup_source_space, 'sample', spacing='oct',
-                  add_dist=False)
+                  add_dist=False, subjects_dir=subjects_dir)
     assert_raises(ValueError, setup_source_space, 'sample', spacing='octo',
-                  add_dist=False)
+                  add_dist=False, subjects_dir=subjects_dir)
     assert_raises(ValueError, setup_source_space, 'sample', spacing='oct6e',
-                  add_dist=False)
+                  add_dist=False, subjects_dir=subjects_dir)
     assert_raises(ValueError, setup_source_space, 'sample', spacing='7emm',
-                  add_dist=False)
+                  add_dist=False, subjects_dir=subjects_dir)
     assert_raises(ValueError, setup_source_space, 'sample', spacing='alls',
-                  add_dist=False)
-    assert_raises(IOError, setup_source_space, 'sample', spacing='oct6',
-                  subjects_dir=subjects_dir, add_dist=False)
+                  add_dist=False, subjects_dir=subjects_dir)
 
     # ico 5 (fsaverage) - write to temp file
     src = read_source_spaces(fname_ico)
-    temp_name = op.join(tempdir, 'temp-src.fif')
     with warnings.catch_warnings(record=True):  # sklearn equiv neighbors
         warnings.simplefilter('always')
-        src_new = setup_source_space('fsaverage', temp_name, spacing='ico5',
-                                     subjects_dir=subjects_dir, add_dist=False,
-                                     overwrite=True)
+        src_new = setup_source_space('fsaverage', spacing='ico5',
+                                     subjects_dir=subjects_dir, add_dist=False)
     _compare_source_spaces(src, src_new, mode='approx')
+    assert_equal(repr(src), repr(src_new))
+    assert_equal(repr(src).count('surface ('), 2)
     assert_array_equal(src[0]['vertno'], np.arange(10242))
     assert_array_equal(src[1]['vertno'], np.arange(10242))
 
@@ -349,28 +361,28 @@ def test_setup_source_space():
     temp_name = op.join(tempdir, 'temp-src.fif')
     with warnings.catch_warnings(record=True):  # sklearn equiv neighbors
         warnings.simplefilter('always')
-        src_new = setup_source_space('sample', temp_name, spacing='oct6',
-                                     subjects_dir=subjects_dir,
-                                     overwrite=True, add_dist=False)
+        src_new = setup_source_space('sample', spacing='oct6',
+                                     subjects_dir=subjects_dir, add_dist=False)
+        write_source_spaces(temp_name, src_new, overwrite=True)
+    assert_equal(src_new[0]['nuse'], 4098)
     _compare_source_spaces(src, src_new, mode='approx', nearest=False)
     src_new = read_source_spaces(temp_name)
     _compare_source_spaces(src, src_new, mode='approx', nearest=False)
 
     # all source points - no file writing
-    src_new = setup_source_space('sample', None, spacing='all',
+    src_new = setup_source_space('sample', spacing='all',
                                  subjects_dir=subjects_dir, add_dist=False)
     assert_true(src_new[0]['nuse'] == len(src_new[0]['rr']))
     assert_true(src_new[1]['nuse'] == len(src_new[1]['rr']))
 
     # dense source space to hit surf['inuse'] lines of _create_surf_spacing
-    assert_raises(RuntimeError, setup_source_space, 'sample', None,
+    assert_raises(RuntimeError, setup_source_space, 'sample',
                   spacing='ico6', subjects_dir=subjects_dir, add_dist=False)
 
 
 @testing.requires_testing_data
 def test_read_source_spaces():
-    """Test reading of source space meshes
-    """
+    """Test reading of source space meshes."""
     src = read_source_spaces(fname, patch_stats=True)
 
     # 3D source space
@@ -390,11 +402,10 @@ def test_read_source_spaces():
     assert_true(rh_use_faces.max() <= rh_points.shape[0] - 1)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_write_source_space():
-    """Test reading and writing of source spaces
-    """
+    """Test reading and writing of source spaces."""
     tempdir = _TempDir()
     src0 = read_source_spaces(fname, patch_stats=False)
     write_source_spaces(op.join(tempdir, 'tmp-src.fif'), src0)
@@ -414,8 +425,7 @@ def test_write_source_space():
 @testing.requires_testing_data
 @requires_fs_or_nibabel
 def test_vertex_to_mni():
-    """Test conversion of vertices to MNI coordinates
-    """
+    """Test conversion of vertices to MNI coordinates."""
     # obtained using "tksurfer (sample) (l/r)h white"
     vertices = [100960, 7620, 150549, 96761]
     coords = np.array([[-60.86, -11.18, -3.19], [-36.46, -93.18, -2.36],
@@ -430,8 +440,7 @@ def test_vertex_to_mni():
 @requires_freesurfer
 @requires_nibabel()
 def test_vertex_to_mni_fs_nibabel():
-    """Test equivalence of vert_to_mni for nibabel and freesurfer
-    """
+    """Test equivalence of vert_to_mni for nibabel and freesurfer."""
     n_check = 1000
     subject = 'sample'
     vertices = rng.randint(0, 100000, n_check)
@@ -448,19 +457,20 @@ def test_vertex_to_mni_fs_nibabel():
 @requires_freesurfer
 @requires_nibabel()
 def test_get_volume_label_names():
-    """Test reading volume label names
-    """
+    """Test reading volume label names."""
     aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
-    label_names = get_volume_labels_from_aseg(aseg_fname)
+    label_names, label_colors = get_volume_labels_from_aseg(aseg_fname,
+                                                            return_colors=True)
     assert_equal(label_names.count('Brain-Stem'), 1)
 
+    assert_equal(len(label_colors), len(label_names))
+
 
 @testing.requires_testing_data
 @requires_freesurfer
 @requires_nibabel()
 def test_source_space_from_label():
-    """Test generating a source space from volume label
-    """
+    """Test generating a source space from volume label."""
     tempdir = _TempDir()
     aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
     label_names = get_volume_labels_from_aseg(aseg_fname)
@@ -491,12 +501,42 @@ def test_source_space_from_label():
     _compare_source_spaces(src, src_from_file, mode='approx')
 
 
+@testing.requires_testing_data
+@requires_freesurfer
+@requires_nibabel()
+def test_read_volume_from_src():
+    """Test reading volumes from a mixed source space."""
+    aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
+    labels_vol = ['Left-Amygdala',
+                  'Brain-Stem',
+                  'Right-Amygdala']
+
+    src = read_source_spaces(fname)
+
+    # Setup a volume source space
+    vol_src = setup_volume_source_space('sample', mri=aseg_fname,
+                                        pos=5.0,
+                                        bem=fname_bem,
+                                        volume_label=labels_vol,
+                                        subjects_dir=subjects_dir)
+    # Generate the mixed source space
+    src += vol_src
+
+    volume_src = get_volume_labels_from_src(src, 'sample', subjects_dir)
+    volume_label = volume_src[0].name
+    volume_label = 'Left-' + volume_label.replace('-lh', '')
+
+    # Test
+    assert_equal(volume_label, src[2]['seg_name'])
+
+    assert_equal(src[2]['type'], 'vol')
+
+
 @testing.requires_testing_data
 @requires_freesurfer
 @requires_nibabel()
 def test_combine_source_spaces():
-    """Test combining source spaces
-    """
+    """Test combining source spaces."""
     tempdir = _TempDir()
     aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
     label_names = get_volume_labels_from_aseg(aseg_fname)
@@ -531,6 +571,8 @@ def test_combine_source_spaces():
     src.save(src_out_name)
     src_from_file = read_source_spaces(src_out_name)
     _compare_source_spaces(src, src_from_file, mode='approx')
+    assert_equal(repr(src), repr(src_from_file))
+    assert_equal(src.kind, 'combined')
 
     # test that all source spaces are in MRI coordinates
     coord_frames = np.array([s['coord_frame'] for s in src])
@@ -551,9 +593,9 @@ def test_combine_source_spaces():
 
     # unrecognized file type
     bad_image_fname = op.join(tempdir, 'temp-image.png')
-    with warnings.catch_warnings(record=True):  # vertices outside vol space
-        assert_raises(ValueError, src.export_volume, bad_image_fname,
-                      verbose='error')
+    # vertices outside vol space warning
+    assert_raises(ValueError, src.export_volume, bad_image_fname,
+                  verbose='error')
 
     # mixed coordinate frames
     disc3 = disc.copy()
@@ -565,8 +607,7 @@ def test_combine_source_spaces():
 
 @testing.requires_testing_data
 def test_morph_source_spaces():
-    """Test morphing of source spaces
-    """
+    """Test morphing of source spaces."""
     src = read_source_spaces(fname_fs)
     src_morph = read_source_spaces(fname_morph)
     src_morph_py = morph_source_spaces(src, 'sample',
@@ -574,10 +615,10 @@ def test_morph_source_spaces():
     _compare_source_spaces(src_morph, src_morph_py, mode='approx')
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_morphed_source_space_return():
-    """Test returning a morphed source space to the original subject"""
+    """Test returning a morphed source space to the original subject."""
     # let's create some random data on fsaverage
     data = rng.randn(20484, 1)
     tmin, tstep = 0, 1.
@@ -609,8 +650,7 @@ def test_morphed_source_space_return():
 
     # Compare to the original data
     stc_morph_morph = stc_morph.morph('fsaverage', stc_morph_return.vertices,
-                                      smooth=1,
-                                      subjects_dir=subjects_dir)
+                                      smooth=1, subjects_dir=subjects_dir)
     assert_equal(stc_morph_return.subject, stc_morph_morph.subject)
     for ii in range(2):
         assert_array_equal(stc_morph_return.vertices[ii],
@@ -620,6 +660,14 @@ def test_morphed_source_space_return():
                        stc_morph_morph.data[:, 0])[0, 1]
     assert_true(corr > 0.99, corr)
 
+    # Explicitly test having two vertices map to the same target vertex. We
+    # simulate this by having two vertices be at the same position.
+    src_fs2 = src_fs.copy()
+    vert1, vert2 = src_fs2[0]['vertno'][:2]
+    src_fs2[0]['rr'][vert1] = src_fs2[0]['rr'][vert2]
+    stc_morph_return = stc_morph.to_original_src(
+        src_fs2, subjects_dir=subjects_dir)
+
     # Degenerate cases
     stc_morph.subject = None  # no .subject provided
     assert_raises(ValueError, stc_morph.to_original_src,
@@ -636,6 +684,7 @@ def test_morphed_source_space_return():
     assert_raises(RuntimeError, stc_morph.to_original_src,
                   src, subjects_dir=subjects_dir)
 
+
 run_tests_if_main()
 
 # The following code was used to generate small-src.fif.gz.
