@@ -6,6 +6,7 @@ import warnings
 from shutil import copyfile
 from scipy import sparse
 from nose.tools import assert_true, assert_raises
+import pytest
 from numpy.testing import assert_array_equal, assert_allclose, assert_equal
 
 from mne.datasets import testing
@@ -14,10 +15,9 @@ from mne.surface import (read_morph_map, _compute_nearest,
                          fast_cross_3d, get_head_surf, read_curvature,
                          get_meg_helmet_surf)
 from mne.utils import (_TempDir, requires_mayavi, requires_tvtk,
-                       run_tests_if_main, slow_test)
+                       run_tests_if_main, object_diff)
 from mne.io import read_info
 from mne.transforms import _get_trans
-from mne.io.meas_info import _is_equal_dict
 
 data_path = testing.data_path(download=False)
 subjects_dir = op.join(data_path, 'subjects')
@@ -87,7 +87,7 @@ def test_compute_nearest():
         assert_array_equal(nn1, nn2)
 
 
-@slow_test
+@pytest.mark.slowtest
 @testing.requires_testing_data
 def test_make_morph_maps():
     """Test reading and creating morph maps."""
@@ -96,20 +96,27 @@ def test_make_morph_maps():
     for subject in ('sample', 'sample_ds', 'fsaverage_ds'):
         os.mkdir(op.join(tempdir, subject))
         os.mkdir(op.join(tempdir, subject, 'surf'))
+        regs = ('reg', 'left_right') if subject == 'fsaverage_ds' else ('reg',)
         for hemi in ['lh', 'rh']:
-            args = [subject, 'surf', hemi + '.sphere.reg']
-            copyfile(op.join(subjects_dir, *args),
-                     op.join(tempdir, *args))
+            for reg in regs:
+                args = [subject, 'surf', hemi + '.sphere.' + reg]
+                copyfile(op.join(subjects_dir, *args),
+                         op.join(tempdir, *args))
 
-    # this should trigger the creation of morph-maps dir and create the map
-    with warnings.catch_warnings(record=True):
-        mmap = read_morph_map('fsaverage_ds', 'sample_ds', tempdir)
-    mmap2 = read_morph_map('fsaverage_ds', 'sample_ds', subjects_dir)
-    assert_equal(len(mmap), len(mmap2))
-    for m1, m2 in zip(mmap, mmap2):
-        # deal with sparse matrix stuff
-        diff = (m1 - m2).data
-        assert_allclose(diff, np.zeros_like(diff), atol=1e-3, rtol=0)
+    for subject_from, subject_to, xhemi in (
+            ('fsaverage_ds', 'sample_ds', False),
+            ('fsaverage_ds', 'fsaverage_ds', True)):
+        # trigger the creation of morph-maps dir and create the map
+        with warnings.catch_warnings(record=True):
+            mmap = read_morph_map(subject_from, subject_to, tempdir,
+                                  xhemi=xhemi)
+        mmap2 = read_morph_map(subject_from, subject_to, subjects_dir,
+                               xhemi=xhemi)
+        assert_equal(len(mmap), len(mmap2))
+        for m1, m2 in zip(mmap, mmap2):
+            # deal with sparse matrix stuff
+            diff = (m1 - m2).data
+            assert_allclose(diff, np.zeros_like(diff), atol=1e-3, rtol=0)
 
     # This will also trigger creation, but it's trivial
     with warnings.catch_warnings(record=True):
@@ -136,7 +143,7 @@ def test_io_surface():
                                                     read_metadata=True)
         assert_array_equal(pts, c_pts)
         assert_array_equal(tri, c_tri)
-        assert_true(_is_equal_dict([vol_info, c_vol_info]))
+        assert_equal(object_diff(vol_info, c_vol_info), '')
 
 
 @testing.requires_testing_data
