@@ -4,21 +4,19 @@
 # Parts of this code were copied from NiTime http://nipy.sourceforge.net/nitime
 
 import numpy as np
-from scipy import fftpack, linalg
+from scipy import linalg
 
 from ..parallel import parallel_func
-from ..utils import sum_squared, warn
+from ..utils import sum_squared, warn, verbose
 
 
 def tridisolve(d, e, b, overwrite_b=True):
-    """
-    Symmetric tridiagonal system solver, from Golub and Van Loan pg 157
+    """Symmetric tridiagonal system solver, from Golub and Van Loan p157.
 
-    Note: Copied from NiTime
+    .. note:: Copied from NiTime.
 
     Parameters
     ----------
-
     d : ndarray
       main diagonal stored in d[:]
     e : ndarray
@@ -28,7 +26,6 @@ def tridisolve(d, e, b, overwrite_b=True):
 
     Returns
     -------
-
     x : ndarray
       Solution to Ax = b (if overwrite_b is False). Otherwise solution is
       stored in previous RHS vector b
@@ -59,14 +56,15 @@ def tridisolve(d, e, b, overwrite_b=True):
 
 
 def tridi_inverse_iteration(d, e, w, x0=None, rtol=1e-8):
-    """Perform an inverse iteration to find the eigenvector corresponding
-    to the given eigenvalue in a symmetric tridiagonal system.
+    """Perform an inverse iteration.
 
-    Note: Copied from NiTime
+    This will find the eigenvector corresponding to the given eigenvalue
+    in a symmetric tridiagonal system.
+
+    ..note:: Copied from NiTime.
 
     Parameters
     ----------
-
     d : ndarray
       main diagonal of the tridiagonal system
     e : ndarray
@@ -80,10 +78,8 @@ def tridi_inverse_iteration(d, e, w, x0=None, rtol=1e-8):
 
     Returns
     -------
-
     e: ndarray
       The converged eigenvector
-
     """
     eig_diag = d - w
     if x0 is None:
@@ -103,11 +99,12 @@ def tridi_inverse_iteration(d, e, w, x0=None, rtol=1e-8):
 
 def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
                  interp_kind='linear'):
-    """
-    Returns the Discrete Prolate Spheroidal Sequences of orders [0,Kmax-1]
-    for a given frequency-spacing multiple NW and sequence length N.
+    """Compute Discrete Prolate Spheroidal Sequences.
 
-    Note: Copied from NiTime
+    Will give of orders [0,Kmax-1] for a given frequency-spacing multiple
+    NW and sequence length N.
+
+    .. note:: Copied from NiTime.
 
     Parameters
     ----------
@@ -146,6 +143,7 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
     Volume 57 (1978), 1371430
     """
     from scipy import interpolate
+    from ..filter import next_fast_len
     Kmax = int(Kmax)
     W = float(half_nbw) / N
     nidx = np.arange(N, dtype='d')
@@ -229,9 +227,9 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
 
     # compute autocorr using FFT (same as nitime.utils.autocorr(dpss) * N)
     rxx_size = 2 * N - 1
-    n_fft = 2 ** int(np.ceil(np.log2(rxx_size)))
-    dpss_fft = fftpack.fft(dpss, n_fft)
-    dpss_rxx = np.real(fftpack.ifft(dpss_fft * dpss_fft.conj()))
+    n_fft = next_fast_len(rxx_size)
+    dpss_fft = np.fft.rfft(dpss, n_fft)
+    dpss_rxx = np.fft.irfft(dpss_fft * dpss_fft.conj(), n_fft)
     dpss_rxx = dpss_rxx[:, :N]
 
     r = 4 * W * np.sinc(2 * W * nidx)
@@ -251,15 +249,12 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
 
 def _psd_from_mt_adaptive(x_mt, eigvals, freq_mask, max_iter=150,
                           return_weights=False):
-    """
-    Perform an iterative procedure to compute the PSD from tapered spectra
-    using the optimal weights.
+    r"""Use iterative procedure to compute the PSD from tapered spectra.
 
-    Note: Modified from NiTime
+    .. note:: Modified from NiTime.
 
     Parameters
     ----------
-
     x_mt : array, shape=(n_signals, n_tapers, n_freqs)
        The DFTs of the tapered sequences (only positive frequencies)
     eigvals : array, length n_tapers
@@ -280,7 +275,6 @@ def _psd_from_mt_adaptive(x_mt, eigvals, freq_mask, max_iter=150,
 
     Notes
     -----
-
     The weights to use for making the multitaper estimate, such that
     :math:`S_{mt} = \sum_{k} |w_k|^2S_k^{mt} / \sum_{k} |w_k|^2`
     """
@@ -363,7 +357,7 @@ def _psd_from_mt_adaptive(x_mt, eigvals, freq_mask, max_iter=150,
 
 
 def _psd_from_mt(x_mt, weights):
-    """ compute PSD from tapered spectra
+    """Compute PSD from tapered spectra.
 
     Parameters
     ----------
@@ -385,7 +379,7 @@ def _psd_from_mt(x_mt, weights):
 
 
 def _csd_from_mt(x_mt, y_mt, weights_x, weights_y):
-    """ Compute CSD from tapered spectra
+    """Compute CSD from tapered spectra.
 
     Parameters
     ----------
@@ -411,7 +405,7 @@ def _csd_from_mt(x_mt, y_mt, weights_x, weights_y):
 
 
 def _mt_spectra(x, dpss, sfreq, n_fft=None):
-    """ Compute tapered spectra
+    """Compute tapered spectra.
 
     Parameters
     ----------
@@ -432,7 +426,6 @@ def _mt_spectra(x, dpss, sfreq, n_fft=None):
     freqs : array
         The frequency points in Hz of the spectra
     """
-
     if n_fft is None:
         n_fft = x.shape[1]
 
@@ -440,24 +433,26 @@ def _mt_spectra(x, dpss, sfreq, n_fft=None):
     x = x - np.mean(x, axis=-1)[:, np.newaxis]
 
     # only keep positive frequencies
-    freqs = fftpack.fftfreq(n_fft, 1. / sfreq)
-    freq_mask = (freqs >= 0)
-    freqs = freqs[freq_mask]
+    freqs = np.fft.rfftfreq(n_fft, 1. / sfreq)
 
     # The following is equivalent to this, but uses less memory:
     # x_mt = fftpack.fft(x[:, np.newaxis, :] * dpss, n=n_fft)
     n_tapers = dpss.shape[0] if dpss.ndim > 1 else 1
-    x_mt = np.zeros((len(x), n_tapers, freq_mask.sum()), dtype=np.complex128)
+    x_mt = np.zeros((len(x), n_tapers, len(freqs)), dtype=np.complex128)
     for idx, sig in enumerate(x):
-        x_mt[idx] = fftpack.fft(sig[np.newaxis, :] * dpss,
-                                n=n_fft)[:, freq_mask]
+        x_mt[idx] = np.fft.rfft(sig[np.newaxis, :] * dpss, n=n_fft)
+    # Adjust DC and maybe Nyquist, depending on one-sided transform
+    x_mt[:, :, 0] /= np.sqrt(2.)
+    if x.shape[1] % 2 == 0:
+        x_mt[:, :, -1] /= np.sqrt(2.)
     return x_mt, freqs
 
 
-def _psd_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
-                    adaptive=False, low_bias=True, normalization='length',
-                    n_jobs=1):
-    """Compute power spectrum density (PSD) using a multi-taper method
+@verbose
+def psd_array_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
+                         adaptive=False, low_bias=True, normalization='length',
+                         n_jobs=1, verbose=None):
+    """Compute power spectrum density (PSD) using a multi-taper method.
 
     Parameters
     ----------
@@ -483,6 +478,9 @@ def _psd_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
         the signal (as in nitime).
     n_jobs : int
         Number of parallel jobs to use (only used if adaptive=True).
+    verbose : bool, str, int, or None
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
 
     Returns
     -------
@@ -494,11 +492,11 @@ def _psd_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
 
     See Also
     --------
-    mne.io.Raw.plot_psd, mne.Epochs.plot_psd, csd_epochs
+    mne.io.Raw.plot_psd, mne.Epochs.plot_psd, csd_epochs, psd_multitaper
 
     Notes
     -----
-    .. versionadded:: 0.12.0
+    .. versionadded:: 0.14.0
     """
     if normalization not in ('length', 'full'):
         raise ValueError('Normalization must be "length" or "full", not %s'
@@ -522,9 +520,8 @@ def _psd_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
     dpss, eigvals = dpss_windows(n_times, half_nbw, n_tapers_max,
                                  low_bias=low_bias)
 
-    # descide which frequencies to keep
-    freqs = fftpack.fftfreq(x.shape[1], 1. / sfreq)
-    freqs = freqs[(freqs >= 0)]  # what we get from _mt_spectra
+    # decide which frequencies to keep
+    freqs = np.fft.rfftfreq(x.shape[1], 1. / sfreq)
     freq_mask = (freqs >= fmin) & (freqs <= fmax)
     freqs = freqs[freq_mask]
 
@@ -559,3 +556,88 @@ def _psd_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
     if ndim_in == 1:
         psd = psd[0]
     return psd, freqs
+
+
+@verbose
+def tfr_array_multitaper(epoch_data, sfreq, freqs, n_cycles=7.0,
+                         zero_mean=True, time_bandwidth=None, use_fft=True,
+                         decim=1, output='complex', n_jobs=1,
+                         frequencies=None, verbose=None):
+    """Compute time-frequency transforms using wavelets and multitaper windows.
+
+    Uses Morlet wavelets windowed with multiple DPSS tapers.
+
+    Parameters
+    ----------
+    epoch_data : array of shape (n_epochs, n_channels, n_times)
+        The epochs.
+    sfreq : float | int
+        Sampling frequency of the data.
+    freqs : array-like of floats, shape (n_freqs)
+        The frequencies.
+    n_cycles : float | array of float
+        Number of cycles  in the Morlet wavelet. Fixed number or one per
+        frequency. Defaults to 7.0.
+    zero_mean : bool
+        If True, make sure the wavelets have a mean of zero. Defaults to True.
+    time_bandwidth : float
+        If None, will be set to 4.0 (3 tapers). Time x (Full) Bandwidth
+        product. The number of good tapers (low-bias) is chosen automatically
+        based on this to equal floor(time_bandwidth - 1). Defaults to None
+    use_fft : bool
+        Use the FFT for convolutions or not. Defaults to True.
+    decim : int | slice
+        To reduce memory usage, decimation factor after time-frequency
+        decomposition. Defaults to 1.
+        If `int`, returns tfr[..., ::decim].
+        If `slice`, returns tfr[..., decim].
+
+        .. note::
+            Decimation may create aliasing artifacts, yet decimation
+            is done after the convolutions.
+
+    output : str, defaults to 'complex'
+
+        * 'complex' : single trial complex.
+        * 'power' : single trial power.
+        * 'phase' : single trial phase.
+        * 'avg_power' : average of single trial power.
+        * 'itc' : inter-trial coherence.
+        * 'avg_power_itc' : average of single trial power and inter-trial
+          coherence across trials.
+
+    n_jobs : int
+        The number of epochs to process at the same time. The parallelization
+        is implemented across channels. Defaults to 1.
+    verbose : bool, str, int, or None, defaults to None
+        If not None, override default verbose level (see :func:`mne.verbose`
+        and :ref:`Logging documentation <tut_logging>` for more).
+
+    Returns
+    -------
+    out : array
+        Time frequency transform of epoch_data. If output is in ['complex',
+        'phase', 'power'], then shape of out is (n_epochs, n_chans, n_freqs,
+        n_times), else it is (n_chans, n_freqs, n_times). If output is
+        'avg_power_itc', the real values code for 'avg_power' and the
+        imaginary values code for the 'itc': out = avg_power + i * itc
+
+    See Also
+    --------
+    mne.time_frequency.tfr_multitaper
+    mne.time_frequency.tfr_morlet
+    mne.time_frequency.tfr_array_morlet
+    mne.time_frequency.tfr_stockwell
+    mne.time_frequency.tfr_array_stockwell
+
+    Notes
+    -----
+    .. versionadded:: 0.14.0
+    """
+    from .tfr import _compute_tfr
+    return _compute_tfr(epoch_data, freqs, sfreq=sfreq,
+                        method='multitaper', n_cycles=n_cycles,
+                        zero_mean=zero_mean, time_bandwidth=time_bandwidth,
+                        use_fft=use_fft, decim=decim, output=output,
+                        n_jobs=n_jobs, frequencies=frequencies,
+                        verbose=verbose)
