@@ -1,18 +1,20 @@
 import numpy as np
 import os.path as op
+import warnings
+
 from numpy.testing import assert_array_almost_equal, assert_array_equal
 from nose.tools import assert_true, assert_false, assert_equal, assert_raises
+import pytest
 
 import mne
 from mne import Epochs, read_events, pick_types, create_info, EpochsArray
 from mne.io import read_raw_fif
-from mne.utils import (_TempDir, run_tests_if_main, slow_test, requires_h5py,
-                       grand_average)
-from mne.time_frequency import single_trial_power
-from mne.time_frequency.tfr import (cwt_morlet, morlet, tfr_morlet,
-                                    _make_dpss, tfr_multitaper, rescale,
-                                    AverageTFR, read_tfrs, write_tfrs,
-                                    combine_tfr, cwt, _compute_tfr)
+from mne.utils import _TempDir, run_tests_if_main, requires_h5py, grand_average
+from mne.time_frequency.tfr import (morlet, tfr_morlet, _make_dpss,
+                                    tfr_multitaper, AverageTFR, read_tfrs,
+                                    write_tfrs, combine_tfr, cwt, _compute_tfr,
+                                    EpochsTFR)
+from mne.time_frequency import tfr_array_multitaper, tfr_array_morlet
 from mne.viz.utils import _fake_click
 from itertools import product
 import matplotlib
@@ -41,7 +43,7 @@ def test_time_frequency():
     tmax = 0.498  # Allows exhaustive decimation testing
 
     # Setup for reading the raw data
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events = read_events(event_fname)
 
     include = []
@@ -52,14 +54,12 @@ def test_time_frequency():
                        stim=False, include=include, exclude=exclude)
 
     picks = picks[:2]
-    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), add_eeg_ref=False)
+    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks)
     data = epochs.get_data()
     times = epochs.times
     nave = len(data)
 
-    epochs_nopicks = Epochs(raw, events, event_id, tmin, tmax,
-                            baseline=(None, 0), add_eeg_ref=False)
+    epochs_nopicks = Epochs(raw, events, event_id, tmin, tmax)
 
     freqs = np.arange(6, 20, 5)  # define frequencies of interest
     n_cycles = freqs / 4.
@@ -95,6 +95,20 @@ def test_time_frequency():
     assert_array_almost_equal(power.data, power_picks_avg.data)
     assert_array_almost_equal(itc.data, itc_picks.data)
     assert_array_almost_equal(power.data, power_evoked.data)
+    # complex output
+    assert_raises(ValueError, tfr_morlet, epochs, freqs, n_cycles,
+                  return_itc=False, average=True, output="complex")
+    assert_raises(ValueError, tfr_morlet, epochs, freqs, n_cycles,
+                  output="complex", average=False, return_itc=True)
+    epochs_power_complex = tfr_morlet(epochs, freqs, n_cycles,
+                                      output="complex", average=False,
+                                      return_itc=False)
+    epochs_power_2 = abs(epochs_power_complex)
+    epochs_power_3 = epochs_power_2.copy()
+    epochs_power_3.data[:] = np.inf  # test that it's actually copied
+    assert_array_almost_equal(epochs_power_2.data, epochs_power_picks.data)
+    power_2 = epochs_power_2.average()
+    assert_array_almost_equal(power_2.data, power.data)
 
     print(itc)  # test repr
     print(itc.ch_names)  # test property
@@ -146,21 +160,22 @@ def test_time_frequency():
     assert_true(np.sum(itc.data >= 1) == 0)
     assert_true(np.sum(itc.data <= 0) == 0)
 
-    Fs = raw.info['sfreq']  # sampling in Hz
-    tfr = cwt_morlet(data[0], Fs, freqs, use_fft=True, n_cycles=2)
+    tfr = tfr_morlet(epochs[0], freqs, use_fft=True, n_cycles=2, average=False,
+                     return_itc=False).data[0]
     assert_true(tfr.shape == (len(picks), len(freqs), len(times)))
-    tfr2 = cwt_morlet(data[0], Fs, freqs, use_fft=True, n_cycles=2,
-                      decim=slice(0, 2))
+    tfr2 = tfr_morlet(epochs[0], freqs, use_fft=True, n_cycles=2,
+                      decim=slice(0, 2), average=False,
+                      return_itc=False).data[0]
     assert_true(tfr2.shape == (len(picks), len(freqs), 2))
 
-    single_power = single_trial_power(data, Fs, freqs, use_fft=False,
-                                      n_cycles=2)
-    single_power2 = single_trial_power(data, Fs, freqs, use_fft=False,
-                                       n_cycles=2, decim=slice(0, 2))
-    single_power3 = single_trial_power(data, Fs, freqs, use_fft=False,
-                                       n_cycles=2, decim=slice(1, 3))
-    single_power4 = single_trial_power(data, Fs, freqs, use_fft=False,
-                                       n_cycles=2, decim=slice(2, 4))
+    single_power = tfr_morlet(epochs, freqs, 2, average=False,
+                              return_itc=False).data
+    single_power2 = tfr_morlet(epochs, freqs, 2, decim=slice(0, 2),
+                               average=False, return_itc=False).data
+    single_power3 = tfr_morlet(epochs, freqs, 2, decim=slice(1, 3),
+                               average=False, return_itc=False).data
+    single_power4 = tfr_morlet(epochs, freqs, 2, decim=slice(2, 4),
+                               average=False, return_itc=False).data
 
     assert_array_almost_equal(np.mean(single_power, axis=0), power.data)
     assert_array_almost_equal(np.mean(single_power2, axis=0),
@@ -193,11 +208,11 @@ def test_time_frequency():
                                     decim=decim)
             assert_equal(power.data.shape[2],
                          np.ceil(float(len(times)) / decim))
-    freqs = range(50, 55)
+    freqs = list(range(50, 55))
     decim = 2
     _, n_chan, n_time = data.shape
-    tfr = cwt_morlet(data[0, :, :], sfreq=epochs.info['sfreq'],
-                     freqs=freqs, decim=decim)
+    tfr = tfr_morlet(epochs[0], freqs, 2., decim=decim, average=False,
+                     return_itc=False).data[0]
     assert_equal(tfr.shape, (n_chan, len(freqs), n_time // decim))
 
     # Test cwt modes
@@ -205,20 +220,25 @@ def test_time_frequency():
     assert_raises(ValueError, cwt, data[0, :, :], Ws, mode='foo')
     for use_fft in [True, False]:
         for mode in ['same', 'valid', 'full']:
-            # XXX JRK: full wavelet decomposition needs to be implemented
-            if (not use_fft) and mode == 'full':
-                assert_raises(ValueError, cwt, data[0, :, :], Ws,
-                              use_fft=use_fft, mode=mode)
-                continue
-            cwt(data[0, :, :], Ws, use_fft=use_fft, mode=mode)
+            cwt(data[0], Ws, use_fft=use_fft, mode=mode)
 
     # Test decim parameter checks
-    assert_raises(TypeError, single_trial_power, data, Fs, freqs,
-                  use_fft=False, n_cycles=2, decim=None)
     assert_raises(TypeError, tfr_morlet, epochs, freqs=freqs,
                   n_cycles=n_cycles, use_fft=True, return_itc=True,
                   decim='decim')
 
+    # When convolving in time, wavelets must not be longer than the data
+    assert_raises(ValueError, cwt, data[0, :, :Ws[0].size - 1], Ws,
+                  use_fft=False)
+    with warnings.catch_warnings(record=True) as w:
+        cwt(data[0, :, :Ws[0].size - 1], Ws, use_fft=True)
+    assert_equal(len(w), 1)
+
+    # Check for off-by-one errors when using wavelets with an even number of
+    # samples
+    psd = cwt(data[0], [Ws[0][:-1]], use_fft=False, mode='full')
+    assert_equal(psd.shape, (2, 1, 420))
+
 
 def test_dpsswavelet():
     """Test DPSS tapers."""
@@ -234,7 +254,7 @@ def test_dpsswavelet():
     assert_true(len(Ws[0]) == len(freqs))  # As many wavelets as asked for
 
 
-@slow_test
+@pytest.mark.slowtest
 def test_tfr_multitaper():
     """Test tfr_multitaper."""
     sfreq = 200.0
@@ -284,6 +304,11 @@ def test_tfr_multitaper():
 
     print(power_evoked)  # test repr for EpochsTFR
 
+    # Test channel picking
+    power_epochs_picked = power_epochs.copy().drop_channels(['SIM0002'])
+    assert_equal(power_epochs_picked.data.shape, (3, 1, 7, 200))
+    assert_equal(power_epochs_picked.ch_names, ['SIM0001'])
+
     assert_raises(ValueError, tfr_multitaper, epochs,
                   freqs=freqs, n_cycles=freqs / 2.,
                   return_itc=True, average=False)
@@ -378,6 +403,14 @@ def test_io():
 
     assert_raises(ValueError, read_tfrs, fname, condition='nonono')
 
+    # Test save of EpochsTFR.
+    data = np.zeros((5, 3, 2, 3))
+    tfr = EpochsTFR(info, data=data, times=times, freqs=freqs,
+                    comment='test', method='crazy-tfr')
+    tfr.save(fname, True)
+    read_tfr = read_tfrs(fname)[0]
+    assert_array_equal(tfr.data, read_tfr.data)
+
 
 def test_plot():
     """Test TFR plotting."""
@@ -390,7 +423,7 @@ def test_plot():
                            ['mag', 'mag', 'mag'])
     tfr = AverageTFR(info, data=data, times=times, freqs=freqs,
                      nave=20, comment='test', method='crazy-tfr')
-    tfr.plot([1, 2], title='title')
+    tfr.plot([1, 2], title='title', colorbar=False)
     plt.close('all')
     ax = plt.subplot2grid((2, 2), (0, 0))
     ax2 = plt.subplot2grid((2, 2), (1, 1))
@@ -468,7 +501,7 @@ def test_compute_tfr():
     tmax = 0.498  # Allows exhaustive decimation testing
 
     # Setup for reading the raw data
-    raw = read_raw_fif(raw_fname, add_eeg_ref=False)
+    raw = read_raw_fif(raw_fname)
     events = read_events(event_fname)
 
     exclude = raw.info['bads'] + ['MEG 2443', 'EEG 053']  # bads + 2 more
@@ -478,27 +511,25 @@ def test_compute_tfr():
                        stim=False, include=[], exclude=exclude)
 
     picks = picks[:2]
-    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), add_eeg_ref=False)
+    epochs = Epochs(raw, events, event_id, tmin, tmax, picks=picks)
     data = epochs.get_data()
     sfreq = epochs.info['sfreq']
     freqs = np.arange(10, 20, 3).astype(float)
 
     # Check all combination of options
-    for method, use_fft, zero_mean, output in product(
-        ('multitaper', 'morlet'), (False, True), (False, True),
+    for func, use_fft, zero_mean, output in product(
+        (tfr_array_multitaper, tfr_array_morlet), (False, True), (False, True),
         ('complex', 'power', 'phase',
          'avg_power_itc', 'avg_power', 'itc')):
         # Check exception
-        if (method == 'multitaper') and (output == 'phase'):
-            assert_raises(NotImplementedError, _compute_tfr, data, freqs,
-                          sfreq, method=method, output=output)
+        if (func == tfr_array_multitaper) and (output == 'phase'):
+            assert_raises(NotImplementedError, func, data, sfreq=sfreq,
+                          freqs=freqs, output=output)
             continue
 
         # Check runs
-        out = _compute_tfr(data, freqs, sfreq, method=method,
-                           use_fft=use_fft, zero_mean=zero_mean,
-                           n_cycles=2., output=output)
+        out = func(data, sfreq=sfreq, freqs=freqs, use_fft=use_fft,
+                   zero_mean=zero_mean, n_cycles=2., output=output)
         # Check shapes
         shape = np.r_[data.shape[:2], len(freqs), data.shape[2]]
         if ('avg' in output) or ('itc' in output):
@@ -513,23 +544,6 @@ def test_compute_tfr():
             assert_equal(np.float, out.dtype)
         assert_true(np.all(np.isfinite(out)))
 
-    # Check that functions are equivalent to
-    # i) single_trial_power: X, shape (n_signals, n_chans, n_times)
-    old_power = single_trial_power(data, sfreq, freqs, n_cycles=2.)
-    new_power = _compute_tfr(data, freqs, sfreq, n_cycles=2.,
-                             method='morlet', output='power')
-    assert_array_almost_equal(old_power, new_power)
-    old_power = single_trial_power(data, sfreq, freqs, n_cycles=2.,
-                                   times=epochs.times, baseline=(-.100, 0),
-                                   baseline_mode='ratio')
-    new_power = rescale(new_power, epochs.times, (-.100, 0), 'ratio')
-
-    # ii) cwt_morlet: X, shape (n_signals, n_times)
-    old_complex = cwt_morlet(data[0], sfreq, freqs, n_cycles=2.)
-    new_complex = _compute_tfr(data[[0]], freqs, sfreq, n_cycles=2.,
-                               method='morlet', output='complex')
-    assert_array_almost_equal(old_complex, new_complex[0])
-
     # Check errors params
     for _data in (None, 'foo', data[0]):
         assert_raises(ValueError, _compute_tfr, _data, freqs, sfreq)
@@ -566,13 +580,12 @@ def test_compute_tfr():
         shape = np.r_[data.shape[:2], len(freqs), n_time]
         for method in ('multitaper', 'morlet'):
             # Single trials
-            out = _compute_tfr(data, freqs, sfreq, method=method,
-                               decim=decim, n_cycles=2.)
+            out = _compute_tfr(data, freqs, sfreq, method=method, decim=decim,
+                               n_cycles=2.)
             assert_array_equal(shape, out.shape)
             # Averages
-            out = _compute_tfr(data, freqs, sfreq, method=method,
-                               decim=decim, output='avg_power',
-                               n_cycles=2.)
+            out = _compute_tfr(data, freqs, sfreq, method=method, decim=decim,
+                               output='avg_power', n_cycles=2.)
             assert_array_equal(shape[1:], out.shape)
 
 run_tests_if_main()
