@@ -13,14 +13,16 @@ import warnings
 
 import numpy as np
 from numpy.testing import assert_raises
-
+from nose.tools import assert_true
+import pytest
 
 from mne import read_events, Epochs, pick_types, read_cov
 from mne.channels import read_layout
 from mne.io import read_raw_fif
-from mne.utils import slow_test, run_tests_if_main
-from mne.viz.evoked import _butterfly_onselect, plot_compare_evokeds
+from mne.utils import run_tests_if_main, catch_logging
+from mne.viz.evoked import _line_plot_onselect, plot_compare_evokeds
 from mne.viz.utils import _fake_click
+from mne.stats import _parametric_ci
 
 # Set our plotters to test mode
 import matplotlib
@@ -39,16 +41,6 @@ n_chan = 6
 layout = read_layout('Vectorview-all')
 
 
-def _get_raw():
-    """Get raw data."""
-    return read_raw_fif(raw_fname, preload=False, add_eeg_ref=False)
-
-
-def _get_events():
-    """Get events."""
-    return read_events(event_name)
-
-
 def _get_picks(raw):
     """Get picks."""
     return pick_types(raw.info, meg=True, eeg=False, stim=False,
@@ -57,33 +49,31 @@ def _get_picks(raw):
 
 def _get_epochs():
     """Get epochs."""
-    raw = _get_raw()
+    raw = read_raw_fif(raw_fname)
     raw.add_proj([], remove_existing=True)
-    events = _get_events()
+    events = read_events(event_name)
     picks = _get_picks(raw)
     # Use a subset of channels for plotting speed
     picks = picks[np.round(np.linspace(0, len(picks) - 1, n_chan)).astype(int)]
-    picks[0] = 2  # make sure we have a magnetometer
-    epochs = Epochs(raw, events[:5], event_id, tmin, tmax, picks=picks,
-                    baseline=(None, 0), add_eeg_ref=False)
+    # make sure we have a magnetometer and a pair of grad pairs for topomap.
+    picks = np.concatenate([[2, 3, 4, 6, 7], picks])
+    epochs = Epochs(raw, events[:5], event_id, tmin, tmax, picks=picks)
     epochs.info['bads'] = [epochs.ch_names[-1]]
     return epochs
 
 
 def _get_epochs_delayed_ssp():
     """Get epochs with delayed SSP."""
-    raw = _get_raw()
-    events = _get_events()
+    raw = read_raw_fif(raw_fname)
+    events = read_events(event_name)
     picks = _get_picks(raw)
     reject = dict(mag=4e-12)
     epochs_delayed_ssp = Epochs(raw, events[:10], event_id, tmin, tmax,
-                                picks=picks, baseline=(None, 0),
-                                proj='delayed', reject=reject,
-                                add_eeg_ref=False)
+                                picks=picks, proj='delayed', reject=reject)
     return epochs_delayed_ssp
 
 
-@slow_test
+@pytest.mark.slowtest
 def test_plot_evoked():
     """Test plotting of evoked."""
     import matplotlib.pyplot as plt
@@ -99,8 +89,6 @@ def test_plot_evoked():
                     [ax.get_xlim()[0], ax.get_ylim()[1]], 'data')
         # plot with bad channels excluded & spatial_colors & zorder
         evoked.plot(exclude='bads')
-        evoked.plot(exclude=evoked.info['bads'], spatial_colors=True, gfp=True,
-                    zorder='std')
 
         # test selective updating of dict keys is working.
         evoked.plot(hline=[1], units=dict(mag='femto foo'))
@@ -127,16 +115,28 @@ def test_plot_evoked():
         plt.close('all')
 
         evoked.plot_topo()  # should auto-find layout
-        _butterfly_onselect(0, 200, ['mag', 'grad'], evoked)
+        _line_plot_onselect(0, 200, ['mag', 'grad'], evoked.info, evoked.data,
+                            evoked.times)
         plt.close('all')
 
         cov = read_cov(cov_fname)
         cov['method'] = 'empirical'
-        evoked.plot_white(cov)
+        # test rank param.
+        evoked.plot_white(cov, rank={'mag': 101, 'grad': 201})
+        evoked.plot_white(cov, rank={'mag': 101})  # test rank param.
+        evoked.plot_white(cov, rank={'grad': 201})  # test rank param.
+        assert_raises(
+            ValueError, evoked.plot_white, cov,
+            rank={'mag': 101, 'grad': 201, 'meg': 306})
+        assert_raises(
+            ValueError, evoked.plot_white, cov, rank={'meg': 306})
+
         evoked.plot_white([cov, cov])
 
         # plot_compare_evokeds: test condition contrast, CI, color assignment
         plot_compare_evokeds(evoked.copy().pick_types(meg='mag'))
+        plot_compare_evokeds(evoked.copy().pick_types(meg='grad'),
+                             picks=[1, 2])
         evoked.rename_channels({'MEG 2142': "MEG 1642"})
         assert len(plot_compare_evokeds(evoked)) == 2
         colors = dict(red='r', blue='b')
@@ -145,6 +145,7 @@ def test_plot_evoked():
         red.data *= 1.1
         blue.data *= 0.9
         plot_compare_evokeds([red, blue], picks=3)  # list of evokeds
+        plot_compare_evokeds([red, blue], picks=3, truncate_yaxis=True)
         plot_compare_evokeds([[red, evoked], [blue, evoked]],
                              picks=3)  # list of lists
         # test picking & plotting grads
@@ -155,7 +156,8 @@ def test_plot_evoked():
         plot_compare_evokeds(contrast, colors=colors, linestyles=linestyles,
                              picks=[0, 2], vlines=[.01, -.04], invert_y=True,
                              truncate_yaxis=False, ylim=dict(mag=(-10, 10)),
-                             styles={"red/stim": {"linewidth": 1}})
+                             styles={"red/stim": {"linewidth": 1}},
+                             show_sensors=True)
         assert_raises(ValueError, plot_compare_evokeds,
                       contrast, picks='str')  # bad picks: not int
         assert_raises(ValueError, plot_compare_evokeds, evoked, picks=3,
@@ -170,18 +172,37 @@ def test_plot_evoked():
                       gfp=True)  # no single-channel GFP
         assert_raises(TypeError, plot_compare_evokeds, evoked, picks=3,
                       ci='fake')  # ci must be float or None
+        assert_raises(TypeError, plot_compare_evokeds, evoked, picks=3,
+                      show_sensors='a')  # show_sensors must be int or bool
         contrast["red/stim"] = red
         contrast["blue/stim"] = blue
         plot_compare_evokeds(contrast, picks=[0], colors=['r', 'b'],
-                             ylim=dict(mag=(1, 10)))
+                             ylim=dict(mag=(1, 10)), ci=_parametric_ci,
+                             truncate_yaxis='max_ticks')
 
         # Hack to test plotting of maxfiltered data
         evoked_sss = evoked.copy()
-        evoked_sss.info['proc_history'] = [dict(max_info=None)]
-        evoked_sss.plot_white(cov)
+        sss = dict(sss_info=dict(in_order=80, components=np.arange(80)))
+        evoked_sss.info['proc_history'] = [dict(max_info=sss)]
+        evoked_sss.plot_white(cov, rank={'meg': 64})
+        assert_raises(
+            ValueError, evoked_sss.plot_white, cov, rank={'grad': 201})
         evoked_sss.plot_white(cov_fname)
+
+        # plot with bad channels excluded, spatial_colors, zorder & pos. layout
+        evoked.rename_channels({'MEG 0133': 'MEG 0000'})
+        evoked.plot(exclude=evoked.info['bads'], spatial_colors=True, gfp=True,
+                    zorder='std')
+        evoked.plot(exclude=[], spatial_colors=True, zorder='unsorted')
+        assert_raises(TypeError, evoked.plot, zorder='asdf')
         plt.close('all')
     evoked.plot_sensors()  # Test plot_sensors
     plt.close('all')
 
+    evoked.pick_channels(evoked.ch_names[:4])
+    with catch_logging() as log_file:
+        evoked.plot(verbose=True)
+    assert_true('Need more than one' in log_file.getvalue())
+
+
 run_tests_if_main()
