@@ -32,7 +32,7 @@ event_id, tmin, tmax = 1, -0.1, 0.2
 
 def _get_raw(preload=False):
     """Get raw data."""
-    return read_raw_fif(raw_fname, preload=preload, add_eeg_ref=False)
+    return read_raw_fif(raw_fname, preload=preload)
 
 
 def _get_events():
@@ -51,8 +51,7 @@ def _get_epochs():
     events = _get_events()
     picks = _get_picks(raw)
     with warnings.catch_warnings(record=True):  # bad proj
-        epochs = Epochs(raw, events[:10], event_id, tmin, tmax, picks=picks,
-                        baseline=(None, 0), add_eeg_ref=False)
+        epochs = Epochs(raw, events[:10], event_id, tmin, tmax, picks=picks)
     return epochs
 
 
@@ -60,6 +59,8 @@ def _get_epochs():
 def test_plot_ica_components():
     """Test plotting of ICA solutions."""
     import matplotlib.pyplot as plt
+    res = 8
+    fast_test = {"res": res, "contours": 0, "sensors": False}
     raw = _get_raw()
     ica = ICA(noise_cov=read_cov(cov_fname), n_components=2,
               max_pca_components=3, n_pca_components=3)
@@ -69,12 +70,12 @@ def test_plot_ica_components():
     warnings.simplefilter('always', UserWarning)
     with warnings.catch_warnings(record=True):
         for components in [0, [0], [0, 1], [0, 1] * 2, None]:
-            ica.plot_components(components, image_interp='bilinear', res=16,
-                                colorbar=True)
+            ica.plot_components(components, image_interp='bilinear',
+                                colorbar=True, **fast_test)
 
         # test interactive mode (passing 'inst' arg)
         plt.close('all')
-        ica.plot_components([0, 1], image_interp='bilinear', res=16, inst=raw)
+        ica.plot_components([0, 1], image_interp='bilinear', inst=raw, res=16)
 
         fig = plt.gcf()
         ax = [a for a in fig.get_children() if isinstance(a, plt.Axes)]
@@ -103,6 +104,7 @@ def test_plot_ica_properties():
     """Test plotting of ICA properties."""
     import matplotlib.pyplot as plt
 
+    res = 8
     raw = _get_raw(preload=True)
     raw.add_proj([], remove_existing=True)
     events = _get_events()
@@ -123,7 +125,7 @@ def test_plot_ica_properties():
     fig, ax = _create_properties_layout()
     assert_equal(len(ax), 5)
 
-    topoargs = dict(topomap_args={'res': 10})
+    topoargs = dict(topomap_args={'res': res, 'contours': 0, "sensors": False})
     ica.plot_properties(raw, picks=0, **topoargs)
     ica.plot_properties(epochs, picks=1, dB=False, plot_std=1.5, **topoargs)
     ica.plot_properties(epochs, picks=1, image_args={'sigma': 1.5},
@@ -142,7 +144,7 @@ def test_plot_ica_properties():
 
     fig, ax = plt.subplots(2, 3)
     ax = ax.ravel()[:-1]
-    ica.plot_properties(epochs, picks=1, axes=ax)
+    ica.plot_properties(epochs, picks=1, axes=ax, **topoargs)
     fig = ica.plot_properties(raw, picks=[0, 1], **topoargs)
     assert_equal(len(fig), 2)
     assert_raises(ValueError, plot_ica_properties, epochs, ica, picks=[0, 1],
@@ -150,13 +152,20 @@ def test_plot_ica_properties():
     assert_raises(ValueError, ica.plot_properties, epochs, axes='not axes')
     plt.close('all')
 
+    # Test merging grads.
+    raw = _get_raw(preload=True)
+    picks = pick_types(raw.info, meg='grad')[:10]
+    ica = ICA(n_components=2)
+    ica.fit(raw, picks=picks)
+    ica.plot_properties(raw)
+    plt.close('all')
+
 
 @requires_sklearn
 def test_plot_ica_sources():
     """Test plotting of ICA panel."""
     import matplotlib.pyplot as plt
-    raw = read_raw_fif(raw_fname, preload=False, add_eeg_ref=False)
-    raw.crop(0, 1, copy=False).load_data()
+    raw = read_raw_fif(raw_fname).crop(0, 1).load_data()
     picks = _get_picks(raw)
     epochs = _get_epochs()
     raw.pick_channels([raw.ch_names[k] for k in picks])
@@ -259,19 +268,9 @@ def test_plot_instance_components():
     with warnings.catch_warnings(record=True):  # bad proj
         ica.fit(raw, picks=picks)
     fig = ica.plot_sources(raw, exclude=[0], title='Components')
-    fig.canvas.key_press_event('down')
-    fig.canvas.key_press_event('up')
-    fig.canvas.key_press_event('right')
-    fig.canvas.key_press_event('left')
-    fig.canvas.key_press_event('o')
-    fig.canvas.key_press_event('-')
-    fig.canvas.key_press_event('+')
-    fig.canvas.key_press_event('=')
-    fig.canvas.key_press_event('pageup')
-    fig.canvas.key_press_event('pagedown')
-    fig.canvas.key_press_event('home')
-    fig.canvas.key_press_event('end')
-    fig.canvas.key_press_event('f11')
+    for key in ['down', 'up', 'right', 'left', 'o', '-', '+', '=', 'pageup',
+                'pagedown', 'home', 'end', 'f11', 'b']:
+        fig.canvas.key_press_event(key)
     ax = fig.get_axes()[0]
     line = ax.lines[0]
     _fake_click(fig, ax, [line.get_xdata()[0], line.get_ydata()[0]], 'data')
@@ -280,19 +279,9 @@ def test_plot_instance_components():
     plt.close('all')
     epochs = _get_epochs()
     fig = ica.plot_sources(epochs, exclude=[0], title='Components')
-    fig.canvas.key_press_event('down')
-    fig.canvas.key_press_event('up')
-    fig.canvas.key_press_event('right')
-    fig.canvas.key_press_event('left')
-    fig.canvas.key_press_event('o')
-    fig.canvas.key_press_event('-')
-    fig.canvas.key_press_event('+')
-    fig.canvas.key_press_event('=')
-    fig.canvas.key_press_event('pageup')
-    fig.canvas.key_press_event('pagedown')
-    fig.canvas.key_press_event('home')
-    fig.canvas.key_press_event('end')
-    fig.canvas.key_press_event('f11')
+    for key in ['down', 'up', 'right', 'left', 'o', '-', '+', '=', 'pageup',
+                'pagedown', 'home', 'end', 'f11', 'b']:
+        fig.canvas.key_press_event(key)
     # Test a click
     ax = fig.get_axes()[0]
     line = ax.lines[0]
