@@ -1,5 +1,4 @@
-"""Functions to plot M/EEG data on topo (one axes per channel)
-"""
+"""Functions to plot M/EEG data on topo (one axes per channel)."""
 from __future__ import print_function
 
 # Authors: Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
@@ -11,7 +10,8 @@ from __future__ import print_function
 
 from functools import partial
 from itertools import cycle
-
+from copy import deepcopy
+from matplotlib.colors import colorConverter
 import numpy as np
 
 from ..io.constants import Bunch
@@ -21,13 +21,13 @@ from ..channels.layout import _merge_grad_data, _pair_grad_sensors, find_layout
 from ..defaults import _handle_default
 from .utils import (_check_delayed_ssp, COLORS, _draw_proj_checkbox,
                     add_background_image, plt_show, _setup_vmin_vmax,
-                    DraggableColorbar)
+                    DraggableColorbar, _set_ax_facecolor, _setup_ax_spines)
 
 
 def iter_topography(info, layout=None, on_pick=None, fig=None,
                     fig_facecolor='k', axis_facecolor='k',
                     axis_spinecolor='k', layout_scale=None):
-    """ Create iterator over channel positions
+    """Create iterator over channel positions.
 
     This function returns a generator that unpacks into
     a series of matplotlib axis objects and data / channel
@@ -76,8 +76,8 @@ def iter_topography(info, layout=None, on_pick=None, fig=None,
 
 def _iter_topography(info, layout, on_pick, fig, fig_facecolor='k',
                      axis_facecolor='k', axis_spinecolor='k',
-                     layout_scale=None, unified=False, img=False):
-    """Private helper to iterate over topography
+                     layout_scale=None, unified=False, img=False, axes=None):
+    """Iterate over topography.
 
     Has the same parameters as iter_topography, plus:
 
@@ -106,9 +106,12 @@ def _iter_topography(info, layout, on_pick, fig, fig_facecolor='k',
     ch_names = _clean_names(info['ch_names'])
     iter_ch = [(x, y) for x, y in enumerate(layout.names) if y in ch_names]
     if unified:
-        under_ax = plt.axes([0, 0, 1, 1])
+        if axes is None:
+            under_ax = plt.axes([0, 0, 1, 1])
+            under_ax.axis('off')
+        else:
+            under_ax = axes
         under_ax.set(xlim=[0, 1], ylim=[0, 1])
-        under_ax.axis('off')
         axs = list()
     for idx, name in iter_ch:
         ch_idx = ch_names.index(name)
@@ -146,13 +149,18 @@ def _iter_topography(info, layout, on_pick, fig, fig_facecolor='k',
 
 
 def _plot_topo(info, times, show_func, click_func=None, layout=None,
-               vmin=None, vmax=None, ylim=None, colorbar=None,
-               border='none', axis_facecolor='k', fig_facecolor='k',
-               cmap='RdBu_r', layout_scale=None, title=None, x_label=None,
-               y_label=None, font_color='w', unified=False, img=False):
-    """Helper function to plot on sensor layout"""
+               vmin=None, vmax=None, ylim=None, colorbar=None, border='none',
+               axis_facecolor='k', fig_facecolor='k', cmap='RdBu_r',
+               layout_scale=None, title=None, x_label=None, y_label=None,
+               font_color='w', unified=False, img=False, axes=None):
+    """Plot on sensor layout."""
     import matplotlib.pyplot as plt
 
+    if layout.kind == 'custom':
+        layout = deepcopy(layout)
+        layout.pos[:, :2] -= layout.pos[:, :2].min(0)
+        layout.pos[:, :2] /= layout.pos[:, :2].max(0)
+
     # prepare callbacks
     tmin, tmax = times[[0, -1]]
     click_func = show_func if click_func is None else click_func
@@ -160,22 +168,27 @@ def _plot_topo(info, times, show_func, click_func=None, layout=None,
                       vmax=vmax, ylim=ylim, x_label=x_label,
                       y_label=y_label, colorbar=colorbar)
 
-    fig = plt.figure()
+    if axes is None:
+        fig = plt.figure()
+        axes = plt.axes([0.015, 0.025, 0.97, 0.95])
+        _set_ax_facecolor(axes, fig_facecolor)
+    else:
+        fig = axes.figure
     if colorbar:
         sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin, vmax))
         sm.set_array(np.linspace(vmin, vmax))
-        ax = plt.axes([0.015, 0.025, 1.05, .8], axisbg=fig_facecolor)
-        cb = fig.colorbar(sm, ax=ax)
+        cb = fig.colorbar(sm, ax=axes, pad=0.025, fraction=0.075, shrink=0.5,
+                          anchor=(-1, 0.5))
         cb_yticks = plt.getp(cb.ax.axes, 'yticklabels')
         plt.setp(cb_yticks, color=font_color)
-        ax.axis('off')
+    axes.axis('off')
 
     my_topo_plot = _iter_topography(info, layout=layout, on_pick=on_pick,
                                     fig=fig, layout_scale=layout_scale,
                                     axis_spinecolor=border,
                                     axis_facecolor=axis_facecolor,
                                     fig_facecolor=fig_facecolor,
-                                    unified=unified, img=img)
+                                    unified=unified, img=img, axes=axes)
 
     for ax, ch_idx in my_topo_plot:
         if layout.kind == 'Vectorview-all' and ylim is not None:
@@ -188,20 +201,15 @@ def _plot_topo(info, times, show_func, click_func=None, layout=None,
                   vmax=vmax, ylim=ylim_)
 
     if title is not None:
-        plt.figtext(0.03, 0.9, title, color=font_color, fontsize=19)
+        plt.figtext(0.03, 0.95, title, color=font_color, fontsize=15, va='top')
 
     return fig
 
 
 def _plot_topo_onpick(event, show_func):
-    """Onpick callback that shows a single channel in a new figure"""
-
+    """Onpick callback that shows a single channel in a new figure."""
     # make sure that the swipe gesture in OS-X doesn't open many figures
     orig_ax = event.inaxes
-    if event.inaxes is None or (not hasattr(orig_ax, '_mne_ch_idx') and
-                                not hasattr(orig_ax, '_mne_axs')):
-        return
-
     import matplotlib.pyplot as plt
     try:
         if hasattr(orig_ax, '_mne_axs'):  # in unified, single-axes mode
@@ -213,16 +221,20 @@ def _plot_topo_onpick(event, show_func):
                     orig_ax = ax
                     break
             else:
+                # no axis found
                 return
+        elif not hasattr(orig_ax, '_mne_ch_idx'):
+            # neither old nor new mode
+            return
         ch_idx = orig_ax._mne_ch_idx
         face_color = orig_ax._mne_ax_face_color
         fig, ax = plt.subplots(1)
 
         plt.title(orig_ax._mne_ch_name)
-        ax.set_axis_bgcolor(face_color)
+        _set_ax_facecolor(ax, face_color)
 
         # allow custom function to override parameters
-        show_func(plt, ch_idx)
+        show_func(ax, ch_idx)
 
     except Exception as err:
         # matplotlib silently ignores exceptions in event handlers,
@@ -233,7 +245,7 @@ def _plot_topo_onpick(event, show_func):
 
 
 def _compute_scalings(bn, xlim, ylim):
-    """Compute scale factors for a unified plot"""
+    """Compute scale factors for a unified plot."""
     if isinstance(ylim[0], (tuple, list, np.ndarray)):
         ylim = (ylim[0][0], ylim[1][0])
     pos = bn.pos
@@ -244,33 +256,80 @@ def _compute_scalings(bn, xlim, ylim):
 
 
 def _check_vlim(vlim):
-    """AUX function"""
+    """Check the vlim."""
     return not np.isscalar(vlim) and vlim is not None
 
 
 def _imshow_tfr(ax, ch_idx, tmin, tmax, vmin, vmax, onselect, ylim=None,
-                tfr=None, freq=None, vline=None, x_label=None, y_label=None,
-                colorbar=False, picker=True, cmap=('RdBu_r', True), title=None,
-                hline=None):
-    """ Aux function to show time-freq map on topo """
-    import matplotlib.pyplot as plt
+                tfr=None, freq=None, x_label=None, y_label=None,
+                colorbar=False, cmap=('RdBu_r', True), yscale='auto'):
+    """Show time-frequency map as two-dimensional image."""
+    from matplotlib import pyplot as plt, ticker
     from matplotlib.widgets import RectangleSelector
 
-    extent = (tmin, tmax, freq[0], freq[-1])
-    cmap, interactive_cmap = cmap
+    if yscale not in ['auto', 'linear', 'log']:
+        raise ValueError("yscale should be either 'auto', 'linear', or 'log'"
+                         ", got {}".format(yscale))
 
-    img = ax.imshow(tfr[ch_idx], extent=extent, aspect="auto", origin="lower",
-                    vmin=vmin, vmax=vmax, picker=picker, cmap=cmap)
-    if isinstance(ax, plt.Axes):
-        if x_label is not None:
-            ax.set_xlabel(x_label)
-        if y_label is not None:
-            ax.set_ylabel(y_label)
+    cmap, interactive_cmap = cmap
+    times = np.linspace(tmin, tmax, num=tfr[ch_idx].shape[1])
+
+    # test yscale
+    if yscale == 'log' and not freq[0] > 0:
+        raise ValueError('Using log scale for frequency axis requires all your'
+                         ' frequencies to be positive (you cannot include'
+                         ' the DC component (0 Hz) in the TFR).')
+    if len(freq) < 2 or freq[0] == 0:
+        yscale = 'linear'
+    elif yscale != 'linear':
+        ratio = freq[1:] / freq[:-1]
+    if yscale == 'auto':
+        if freq[0] > 0 and np.allclose(ratio, ratio[0]):
+            yscale = 'log'
+        else:
+            yscale = 'linear'
+
+    # compute bounds between time samples
+    time_diff = np.diff(times) / 2. if len(times) > 1 else [0.0005]
+    time_lims = np.concatenate([[times[0] - time_diff[0]], times[:-1] +
+                               time_diff, [times[-1] + time_diff[-1]]])
+
+    # the same for frequency - depending on whether yscale is log
+    if yscale == 'linear':
+        freq_diff = np.diff(freq) / 2. if len(freq) > 1 else [0.5]
+        freq_lims = np.concatenate([[freq[0] - freq_diff[0]], freq[:-1] +
+                                   freq_diff, [freq[-1] + freq_diff[-1]]])
     else:
-        if x_label is not None:
-            plt.xlabel(x_label)
-        if y_label is not None:
-            plt.ylabel(y_label)
+        log_freqs = np.concatenate([[freq[0] / ratio[0]], freq,
+                                   [freq[-1] * ratio[0]]])
+        freq_lims = np.sqrt(log_freqs[:-1] * log_freqs[1:])
+
+    # construct a time-frequency bounds grid
+    time_mesh, freq_mesh = np.meshgrid(time_lims, freq_lims)
+
+    img = ax.pcolormesh(time_mesh, freq_mesh, tfr[ch_idx], cmap=cmap,
+                        vmin=vmin, vmax=vmax)
+
+    # limits, yscale and yticks
+    ax.set_xlim(time_lims[0], time_lims[-1])
+    if ylim is None:
+        ylim = (freq_lims[0], freq_lims[-1])
+    ax.set_ylim(ylim)
+
+    if yscale == 'log':
+        ax.set_yscale('log')
+        ax.get_yaxis().set_major_formatter(ticker.ScalarFormatter())
+
+    ax.yaxis.set_minor_formatter(ticker.NullFormatter())
+    ax.yaxis.set_minor_locator(ticker.NullLocator())  # get rid of minor ticks
+    tick_vals = freq[np.unique(np.linspace(
+        0, len(freq) - 1, 12).round().astype('int'))]
+    ax.set_yticks(tick_vals)
+
+    if x_label is not None:
+        ax.set_xlabel(x_label)
+    if y_label is not None:
+        ax.set_ylabel(y_label)
     if colorbar:
         if isinstance(colorbar, DraggableColorbar):
             cbar = colorbar.cbar  # this happens with multiaxes case
@@ -278,10 +337,6 @@ def _imshow_tfr(ax, ch_idx, tmin, tmax, vmin, vmax, onselect, ylim=None,
             cbar = plt.colorbar(mappable=img)
         if interactive_cmap:
             ax.CB = DraggableColorbar(cbar, img)
-    if title:
-        plt.title(title)
-    if not isinstance(ax, plt.Axes):
-        ax = plt.gca()
     ax.RS = RectangleSelector(ax, onselect=onselect)  # reference must be kept
 
 
@@ -289,7 +344,7 @@ def _imshow_tfr_unified(bn, ch_idx, tmin, tmax, vmin, vmax, onselect,
                         ylim=None, tfr=None, freq=None, vline=None,
                         x_label=None, y_label=None, colorbar=False,
                         picker=True, cmap='RdBu_r', title=None, hline=None):
-    """Aux function to show multiple tfrs on topo using a single axes"""
+    """Show multiple tfrs on topo using a single axes."""
     _compute_scalings(bn, (tmin, tmax), (freq[0], freq[-1]))
     ax = bn.ax
     data_lines = bn.data_lines
@@ -302,38 +357,104 @@ def _imshow_tfr_unified(bn, ch_idx, tmin, tmax, vmin, vmax, onselect,
 
 def _plot_timeseries(ax, ch_idx, tmin, tmax, vmin, vmax, ylim, data, color,
                      times, vline=None, x_label=None, y_label=None,
-                     colorbar=False, hline=None):
-    """Aux function to show time series on topo split across multiple axes"""
+                     colorbar=False, hline=None, hvline_color='w',
+                     labels=None):
+    """Show time series on topo split across multiple axes."""
     import matplotlib.pyplot as plt
     picker_flag = False
     for data_, color_ in zip(data, color):
         if not picker_flag:
             # use large tol for picker so we can click anywhere in the axes
-            ax.plot(times, data_[ch_idx], color_, picker=1e9)
+            ax.plot(times, data_[ch_idx], color=color_, picker=1e9)
             picker_flag = True
         else:
-            ax.plot(times, data_[ch_idx], color_)
-    if vline:
-        for x in vline:
-            plt.axvline(x, color='w', linewidth=0.5)
-    if hline:
-        for y in hline:
-            plt.axhline(y, color='w', linewidth=0.5)
+            ax.plot(times, data_[ch_idx], color=color_)
+
     if x_label is not None:
         plt.xlabel(x_label)
+
     if y_label is not None:
         if isinstance(y_label, list):
-            plt.ylabel(y_label[ch_idx])
+            ax.set_ylabel(y_label[ch_idx])
         else:
-            plt.ylabel(y_label)
+            ax.set_ylabel(y_label)
+
+    def _format_coord(x, y, labels, ax):
+        """Create status string based on cursor coordinates."""
+        idx = np.abs(times - x).argmin()
+        ylabel = ax.get_ylabel()
+        unit = (ylabel[ylabel.find('(') + 1:ylabel.find(')')]
+                if '(' in ylabel and ')' in ylabel else '')
+        labels = [''] * len(data) if labels is None else labels
+        # try to estimate whether to truncate condition labels
+        slen = 10 + sum([12 + len(unit) + len(label) for label in labels])
+        bar_width = (ax.figure.get_size_inches() * ax.figure.dpi)[0] / 5.5
+        trunc_labels = bar_width < slen
+        s = '%6.3f s: ' % times[idx]
+        for data_, label in zip(data, labels):
+            s += '%7.2f %s' % (data_[ch_idx, idx], unit)
+            if trunc_labels:
+                label = (label if len(label) <= 10 else
+                         '%s..%s' % (label[:6], label[-2:]))
+            s += ' [%s] ' % label if label else ' '
+        return s
+
+    ax.format_coord = lambda x, y: _format_coord(x, y, labels=labels, ax=ax)
+
+    def _cursor_vline(event):
+        """Draw cursor (vertical line)."""
+        ax = event.inaxes
+        if not ax:
+            return
+        if ax._cursorline is not None:
+            ax._cursorline.remove()
+        ax._cursorline = ax.axvline(event.xdata, color=ax._cursorcolor)
+        ax.figure.canvas.draw()
+
+    def _rm_cursor(event):
+        ax = event.inaxes
+        if ax._cursorline is not None:
+            ax._cursorline.remove()
+            ax._cursorline = None
+        ax.figure.canvas.draw()
+
+    ax._cursorline = None
+    # choose cursor color based on perceived brightness of background
+    try:
+        facecol = colorConverter.to_rgb(ax.get_facecolor())
+    except AttributeError:  # older MPL
+        facecol = colorConverter.to_rgb(ax.get_axis_bgcolor())
+    face_brightness = np.dot(facecol, np.array([299, 587, 114]))
+    ax._cursorcolor = 'white' if face_brightness < 150 else 'black'
+
+    plt.connect('motion_notify_event', _cursor_vline)
+    plt.connect('axes_leave_event', _rm_cursor)
+
+    _setup_ax_spines(ax, vline, tmin, tmax)
+    ax.figure.set_facecolor('k' if hvline_color is 'w' else 'w')
+    ax.spines['bottom'].set_color(hvline_color)
+    ax.spines['left'].set_color(hvline_color)
+    ax.tick_params(axis='x', colors=hvline_color, which='both')
+    ax.tick_params(axis='y', colors=hvline_color, which='both')
+    ax.title.set_color(hvline_color)
+    ax.xaxis.label.set_color(hvline_color)
+    ax.yaxis.label.set_color(hvline_color)
+
+    if vline:
+        plt.axvline(vline, color=hvline_color, linewidth=1.0,
+                    linestyle='--')
+    if hline:
+        plt.axhline(hline, color=hvline_color, linewidth=1.0, zorder=10)
+
     if colorbar:
         plt.colorbar()
 
 
 def _plot_timeseries_unified(bn, ch_idx, tmin, tmax, vmin, vmax, ylim, data,
                              color, times, vline=None, x_label=None,
-                             y_label=None, colorbar=False, hline=None):
-    """Aux function to show multiple time series on topo using a single axes"""
+                             y_label=None, colorbar=False, hline=None,
+                             hvline_color='w'):
+    """Show multiple time series on topo using a single axes."""
     import matplotlib.pyplot as plt
     if not (ylim and not any(v is None for v in ylim)):
         ylim = np.array([np.min(data), np.max(data)])
@@ -346,13 +467,15 @@ def _plot_timeseries_unified(bn, ch_idx, tmin, tmax, vmin, vmax, ylim, data,
     for data_, color_ in zip(data, color):
         data_lines.append(ax.plot(
             bn.x_t + bn.x_s * times, bn.y_t + bn.y_s * data_[ch_idx],
-            color_, clip_on=True, clip_box=pos)[0])
+            color=color_, clip_on=True, clip_box=pos)[0])
     if vline:
         vline = np.array(vline) * bn.x_s + bn.x_t
-        ax.vlines(vline, pos[1], pos[1] + pos[3], color='w', linewidth=0.5)
+        ax.vlines(vline, pos[1], pos[1] + pos[3], color=hvline_color,
+                  linewidth=0.5, linestyle='--')
     if hline:
         hline = np.array(hline) * bn.y_s + bn.y_t
-        ax.hlines(hline, pos[0], pos[0] + pos[2], color='w', linewidth=0.5)
+        ax.hlines(hline, pos[0], pos[0] + pos[2], color=hvline_color,
+                  linewidth=0.5)
     if x_label is not None:
         ax.text(pos[0] + pos[2] / 2., pos[1], x_label,
                 horizontalalignment='center', verticalalignment='top')
@@ -369,10 +492,10 @@ def _erfimage_imshow(ax, ch_idx, tmin, tmax, vmin, vmax, ylim=None, data=None,
                      epochs=None, sigma=None, order=None, scalings=None,
                      vline=None, x_label=None, y_label=None, colorbar=False,
                      cmap='RdBu_r'):
-    """Aux function to plot erfimage on sensor topography"""
+    """Plot erfimage on sensor topography."""
     from scipy import ndimage
     import matplotlib.pyplot as plt
-    this_data = data[:, ch_idx, :].copy()
+    this_data = data[:, ch_idx, :].copy() * scalings[ch_idx]
 
     if callable(order):
         order = order(epochs.times, this_data)
@@ -383,9 +506,9 @@ def _erfimage_imshow(ax, ch_idx, tmin, tmax, vmin, vmax, ylim=None, data=None,
     if sigma > 0.:
         this_data = ndimage.gaussian_filter1d(this_data, sigma=sigma, axis=0)
 
-    ax.imshow(this_data, extent=[tmin, tmax, 0, len(data)], aspect='auto',
-              origin='lower', vmin=vmin, vmax=vmax, picker=True, cmap=cmap,
-              interpolation='nearest')
+    img = ax.imshow(this_data, extent=[tmin, tmax, 0, len(data)],
+                    aspect='auto', origin='lower', vmin=vmin, vmax=vmax,
+                    picker=True, cmap=cmap, interpolation='nearest')
 
     ax = plt.gca()
     if x_label is not None:
@@ -393,21 +516,21 @@ def _erfimage_imshow(ax, ch_idx, tmin, tmax, vmin, vmax, ylim=None, data=None,
     if y_label is not None:
         ax.set_ylabel(y_label)
     if colorbar:
-        plt.colorbar()
+        plt.colorbar(mappable=img)
 
 
 def _erfimage_imshow_unified(bn, ch_idx, tmin, tmax, vmin, vmax, ylim=None,
                              data=None, epochs=None, sigma=None, order=None,
                              scalings=None, vline=None, x_label=None,
                              y_label=None, colorbar=False, cmap='RdBu_r'):
-    """Aux function to plot erfimage topography using a single axis"""
+    """Plot erfimage topography using a single axis."""
     from scipy import ndimage
     _compute_scalings(bn, (tmin, tmax), (0, len(epochs.events)))
     ax = bn.ax
     data_lines = bn.data_lines
     extent = (bn.x_t + bn.x_s * tmin, bn.x_t + bn.x_s * tmax, bn.y_t,
               bn.y_t + bn.y_s * len(epochs.events))
-    this_data = data[:, ch_idx, :].copy()
+    this_data = data[:, ch_idx, :].copy() * scalings[ch_idx]
 
     if callable(order):
         order = order(epochs.times, this_data)
@@ -428,7 +551,7 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
                       border='none', ylim=None, scalings=None, title=None,
                       proj=False, vline=(0.,), hline=(0.,), fig_facecolor='k',
                       fig_background=None, axis_facecolor='k', font_color='w',
-                      merge_grads=False, show=True):
+                      merge_grads=False, legend=True, axes=None, show=True):
     """Plot 2D topography of evoked responses.
 
     Clicking on the plot of an individual sensor opens a new figure showing
@@ -482,6 +605,16 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
     merge_grads : bool
         Whether to use RMS value of gradiometer pairs. Only works for Neuromag
         data. Defaults to False.
+    legend : bool | int | string | tuple
+        If True, create a legend based on evoked.comment. If False, disable the
+        legend. Otherwise, the legend is created and the parameter value is
+        passed as the location parameter to the matplotlib legend call. It can
+        be an integer (e.g. 0 corresponds to upper right corner of the plot),
+        a string (e.g. 'upper right'), or a tuple (x, y coordinates of the
+        lower left corner of the legend in the axes coordinate system).
+        See matplotlib documentation for more details.
+    axes : instance of matplotlib Axes | None
+        Axes to plot into. If None, axes will be created.
     show : bool
         Show figure if True.
 
@@ -490,6 +623,8 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
     fig : Instance of matplotlib.figure.Figure
         Images of evoked responses at sensor locations
     """
+    import matplotlib.pyplot as plt
+
     if not type(evoked) in (tuple, list):
         evoked = [evoked]
 
@@ -564,7 +699,7 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
 
         for e in evoked:
             for pick, ch_type in zip(picks, types_used):
-                e.data[pick] = e.data[pick] * scalings[ch_type]
+                e.data[pick] *= scalings[ch_type]
 
         if proj is True and all(e.proj is not True for e in evoked):
             evoked = [e.apply_proj() for e in evoked]
@@ -593,21 +728,34 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
         raise TypeError('ylim must be None or a dict. Got %s.' % type(ylim))
 
     data = [e.data for e in evoked]
+    comments = [e.comment for e in evoked]
     show_func = partial(_plot_timeseries_unified, data=data, color=color,
-                        times=times, vline=vline, hline=hline)
+                        times=times, vline=vline, hline=hline,
+                        hvline_color=font_color)
     click_func = partial(_plot_timeseries, data=data, color=color, times=times,
-                         vline=vline, hline=hline)
+                         vline=vline, hline=hline, hvline_color=font_color,
+                         labels=comments)
 
     fig = _plot_topo(info=info, times=times, show_func=show_func,
-                     click_func=click_func, layout=layout,
-                     colorbar=False, ylim=ylim_, cmap=None,
-                     layout_scale=layout_scale, border=border,
-                     fig_facecolor=fig_facecolor, font_color=font_color,
-                     axis_facecolor=axis_facecolor, title=title,
-                     x_label='Time (s)', y_label=y_label, unified=True)
+                     click_func=click_func, layout=layout, colorbar=False,
+                     ylim=ylim_, cmap=None, layout_scale=layout_scale,
+                     border=border, fig_facecolor=fig_facecolor,
+                     font_color=font_color, axis_facecolor=axis_facecolor,
+                     title=title, x_label='Time (s)', y_label=y_label,
+                     unified=True, axes=axes)
 
     add_background_image(fig, fig_background)
 
+    if legend is not False:
+        legend_loc = 0 if legend is True else legend
+        labels = [e.comment if e.comment else 'Unknown' for e in evoked]
+        legend = plt.legend(labels, loc=legend_loc,
+                            prop={'size': 10})
+        legend.get_frame().set_facecolor(axis_facecolor)
+        txts = legend.get_texts()
+        for txt, col in zip(txts, color):
+            txt.set_color(col)
+
     if proj == 'interactive':
         for e in evoked:
             _check_delayed_ssp(e)
@@ -621,7 +769,7 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945, color=None,
 
 
 def _plot_update_evoked_topo_proj(params, bools):
-    """Helper function to update topo sensor plots"""
+    """Update topo sensor plots."""
     evokeds = [e.copy() for e in params['evokeds']]
     fig = params['fig']
     projs = [proj for proj, b in zip(params['projs'], bools) if b]
@@ -643,7 +791,7 @@ def plot_topo_image_epochs(epochs, layout=None, sigma=0., vmin=None,
                            layout_scale=.95, title=None, scalings=None,
                            border='none', fig_facecolor='k',
                            fig_background=None, font_color='w', show=True):
-    """Plot Event Related Potential / Fields image on topographies
+    """Plot Event Related Potential / Fields image on topographies.
 
     Parameters
     ----------
@@ -701,17 +849,15 @@ def plot_topo_image_epochs(epochs, layout=None, sigma=0., vmin=None,
     for idx in range(epochs.info['nchan']):
         ch_type = channel_type(epochs.info, idx)
         scale_coeffs.append(scalings.get(ch_type, 1))
-    for epoch_data in data:
-        epoch_data *= np.asarray(scale_coeffs)[:, np.newaxis]
     vmin, vmax = _setup_vmin_vmax(data, vmin, vmax)
 
     if layout is None:
         layout = find_layout(epochs.info)
 
-    show_func = partial(_erfimage_imshow_unified, scalings=scalings,
+    show_func = partial(_erfimage_imshow_unified, scalings=scale_coeffs,
                         order=order, data=data, epochs=epochs, sigma=sigma,
                         cmap=cmap)
-    erf_imshow = partial(_erfimage_imshow, scalings=scalings, order=order,
+    erf_imshow = partial(_erfimage_imshow, scalings=scale_coeffs, order=order,
                          data=data, epochs=epochs, sigma=sigma, cmap=cmap)
 
     fig = _plot_topo(info=epochs.info, times=epochs.times,
