@@ -96,7 +96,7 @@ In MNE-Python we default to using FIR filtering. As noted in Widmann *et al.*
 
     Despite IIR filters often being considered as computationally more
     efficient, they are recommended only when high throughput and sharp
-    cutoffs are required (Ifeachor and Jervis, 2002[2]_, p. 321),
+    cutoffs are required (Ifeachor and Jervis, 2002 [2]_, p. 321),
     ...FIR filters are easier to control, are always stable, have a
     well-defined passband, can be corrected to zero-phase without
     additional computations, and can be converted to minimum-phase.
@@ -139,14 +139,13 @@ from scipy import signal, fftpack
 import matplotlib.pyplot as plt
 
 from mne.time_frequency.tfr import morlet
+from mne.viz import plot_filter, plot_ideal_filter
 
 import mne
 
 sfreq = 1000.
 f_p = 40.
-ylim = [-60, 10]  # for dB plots
-xlim = [2, sfreq / 2.]
-blue = '#1f77b4'
+flim = (1., sfreq / 2.)  # limits for plotting
 
 ###############################################################################
 # Take for example an ideal low-pass filter, which would give a value of 1 in
@@ -158,45 +157,9 @@ nyq = sfreq / 2.  # the Nyquist frequency is half our sample rate
 freq = [0, f_p, f_p, nyq]
 gain = [1, 1, 0, 0]
 
-
-def box_off(ax):
-    ax.grid(zorder=0)
-    for key in ('top', 'right'):
-        ax.spines[key].set_visible(False)
-
-
-def plot_ideal(freq, gain, ax):
-    freq = np.maximum(freq, xlim[0])
-    xs, ys = list(), list()
-    my_freq, my_gain = list(), list()
-    for ii in range(len(freq)):
-        xs.append(freq[ii])
-        ys.append(ylim[0])
-        if ii < len(freq) - 1 and gain[ii] != gain[ii + 1]:
-            xs += [freq[ii], freq[ii + 1]]
-            ys += [ylim[1]] * 2
-            my_freq += np.linspace(freq[ii], freq[ii + 1], 20,
-                                   endpoint=False).tolist()
-            my_gain += np.linspace(gain[ii], gain[ii + 1], 20,
-                                   endpoint=False).tolist()
-        else:
-            my_freq.append(freq[ii])
-            my_gain.append(gain[ii])
-    my_gain = 10 * np.log10(np.maximum(my_gain, 10 ** (ylim[0] / 10.)))
-    ax.fill_between(xs, ylim[0], ys, color='r', alpha=0.1)
-    ax.semilogx(my_freq, my_gain, 'r--', alpha=0.5, linewidth=4, zorder=3)
-    xticks = [1, 2, 4, 10, 20, 40, 100, 200, 400]
-    ax.set(xlim=xlim, ylim=ylim, xticks=xticks, xlabel='Frequency (Hz)',
-           ylabel='Amplitude (dB)')
-    ax.set(xticklabels=xticks)
-    box_off(ax)
-
-half_height = np.array(plt.rcParams['figure.figsize']) * [1, 0.5]
-ax = plt.subplots(1, figsize=half_height)[1]
-plot_ideal(freq, gain, ax)
-ax.set(title='Ideal %s Hz lowpass' % f_p)
-mne.viz.tight_layout()
-plt.show()
+third_height = np.array(plt.rcParams['figure.figsize']) * [1, 1. / 3.]
+ax = plt.subplots(1, figsize=third_height)[1]
+plot_ideal_filter(freq, gain, ax, title='Ideal %s Hz lowpass' % f_p, flim=flim)
 
 ###############################################################################
 # This filter hypothetically achieves zero ripple in the frequency domain,
@@ -214,46 +177,19 @@ plt.show()
 n = int(round(0.1 * sfreq)) + 1
 t = np.arange(-n // 2, n // 2) / sfreq  # center our sinc
 h = np.sinc(2 * f_p * t) / (4 * np.pi)
-
-
-def plot_filter(h, title, freq, gain, show=True):
-    if h.ndim == 2:  # second-order sections
-        sos = h
-        n = mne.filter.estimate_ringing_samples(sos)
-        h = np.zeros(n)
-        h[0] = 1
-        h = signal.sosfilt(sos, h)
-        H = np.ones(512, np.complex128)
-        for section in sos:
-            f, this_H = signal.freqz(section[:3], section[3:])
-            H *= this_H
-    else:
-        f, H = signal.freqz(h)
-    fig, axs = plt.subplots(2)
-    t = np.arange(len(h)) / sfreq
-    axs[0].plot(t, h, color=blue)
-    axs[0].set(xlim=t[[0, -1]], xlabel='Time (sec)',
-               ylabel='Amplitude h(n)', title=title)
-    box_off(axs[0])
-    f *= sfreq / (2 * np.pi)
-    axs[1].semilogx(f, 10 * np.log10((H * H.conj()).real), color=blue,
-                    linewidth=2, zorder=4)
-    plot_ideal(freq, gain, axs[1])
-    mne.viz.tight_layout()
-    if show:
-        plt.show()
-
-plot_filter(h, 'Sinc (0.1 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Sinc (0.1 sec)', flim=flim)
 
 ###############################################################################
 # This is not so good! Making the filter 10 times longer (1 sec) gets us a
 # bit better stop-band suppression, but still has a lot of ringing in
-# the time domain. Note the x-axis is an order of magnitude longer here:
+# the time domain. Note the x-axis is an order of magnitude longer here,
+# and the filter has a correspondingly much longer group delay (again equal
+# to half the filter length, or 0.5 seconds):
 
 n = int(round(1. * sfreq)) + 1
 t = np.arange(-n // 2, n // 2) / sfreq
 h = np.sinc(2 * f_p * t) / (4 * np.pi)
-plot_filter(h, 'Sinc (1.0 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Sinc (1.0 sec)', flim=flim)
 
 ###############################################################################
 # Let's make the stop-band tighter still with a longer filter (10 sec),
@@ -262,7 +198,7 @@ plot_filter(h, 'Sinc (1.0 sec)', freq, gain)
 n = int(round(10. * sfreq)) + 1
 t = np.arange(-n // 2, n // 2) / sfreq
 h = np.sinc(2 * f_p * t) / (4 * np.pi)
-plot_filter(h, 'Sinc (10.0 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Sinc (10.0 sec)', flim=flim)
 
 ###############################################################################
 # Now we have very sharp frequency suppression, but our filter rings for the
@@ -273,7 +209,8 @@ plot_filter(h, 'Sinc (10.0 sec)', freq, gain)
 # based on desired response characteristics. These include:
 #
 #     1. The Remez_ algorithm (:func:`scipy.signal.remez`, `MATLAB firpm`_)
-#     2. Windowed FIR design (:func:`scipy.signal.firwin2`, `MATLAB fir2`_)
+#     2. Windowed FIR design (:func:`scipy.signal.firwin2`, `MATLAB fir2`_
+#        and :func:`scipy.signal.firwin`)
 #     3. Least squares designs (:func:`scipy.signal.firls`, `MATLAB firls`_)
 #     4. Frequency-domain design (construct filter in Fourier
 #        domain and use an :func:`IFFT <scipy.fftpack.ifft>` to invert it)
@@ -299,11 +236,9 @@ f_s = f_p + trans_bandwidth  # = 50 Hz
 
 freq = [0., f_p, f_s, nyq]
 gain = [1., 1., 0., 0.]
-ax = plt.subplots(1, figsize=half_height)[1]
-plot_ideal(freq, gain, ax)
-ax.set(title='%s Hz lowpass with a %s Hz transition' % (f_p, trans_bandwidth))
-mne.viz.tight_layout()
-plt.show()
+ax = plt.subplots(1, figsize=third_height)[1]
+title = '%s Hz lowpass with a %s Hz transition' % (f_p, trans_bandwidth)
+plot_ideal_filter(freq, gain, ax, title=title, flim=flim)
 
 ###############################################################################
 # Accepting a shallower roll-off of the filter in the frequency domain makes
@@ -312,7 +247,8 @@ plt.show()
 # domain signal. Here again for the 1 sec filter:
 
 h = signal.firwin2(n, freq, gain, nyq=nyq)
-plot_filter(h, 'Windowed 10-Hz transition (1.0 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Windowed 10-Hz transition (1.0 sec)',
+            flim=flim)
 
 ###############################################################################
 # Since our lowpass is around 40 Hz with a 10 Hz transition, we can actually
@@ -321,7 +257,8 @@ plot_filter(h, 'Windowed 10-Hz transition (1.0 sec)', freq, gain)
 
 n = int(round(sfreq * 0.5)) + 1
 h = signal.firwin2(n, freq, gain, nyq=nyq)
-plot_filter(h, 'Windowed 10-Hz transition (0.5 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Windowed 10-Hz transition (0.5 sec)',
+            flim=flim)
 
 ###############################################################################
 # But then if we shorten the filter too much (2 cycles of 10 Hz = 0.2 sec),
@@ -329,7 +266,8 @@ plot_filter(h, 'Windowed 10-Hz transition (0.5 sec)', freq, gain)
 
 n = int(round(sfreq * 0.2)) + 1
 h = signal.firwin2(n, freq, gain, nyq=nyq)
-plot_filter(h, 'Windowed 10-Hz transition (0.2 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Windowed 10-Hz transition (0.2 sec)',
+            flim=flim)
 
 ###############################################################################
 # If we want a filter that is only 0.1 seconds long, we should probably use
@@ -339,7 +277,35 @@ trans_bandwidth = 25
 f_s = f_p + trans_bandwidth
 freq = [0, f_p, f_s, nyq]
 h = signal.firwin2(n, freq, gain, nyq=nyq)
-plot_filter(h, 'Windowed 50-Hz transition (0.2 sec)', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'Windowed 50-Hz transition (0.2 sec)',
+            flim=flim)
+
+###############################################################################
+# So far we have only discussed *acausal* filtering, which means that each
+# sample at each time point :math:`t` is filtered using samples that come
+# after (:math:`t + \Delta t`) *and* before (:math:`t - \Delta t`) :math:`t`.
+# In this sense, each sample is influenced by samples that come both before
+# and after it. This is useful in many cases, espcially because it does not
+# delay the timing of events.
+#
+# However, sometimes it can be beneficial to use *causal* filtering,
+# whereby each sample :math:`t` is filtered only using time points that came
+# after it.
+#
+# Note that the delay is variable (whereas for linear/zero-phase filters it
+# is constant) but small in the pass-band. Unlike zero-phase filters, which
+# require time-shifting backward the output of a linear-phase filtering stage
+# (and thus becoming acausal), minimum-phase filters do not require any
+# compensation to achieve small delays in the passband. Note that as an
+# artifact of the minimum phase filter construction step, the filter does
+# not end up being as steep as the linear/zero-phase version.
+#
+# We can construct a minimum-phase filter from our existing linear-phase
+# filter with the ``minimum_phase`` function (that will be in SciPy 0.19's
+# :mod:`scipy.signal`), and note that the falloff is not as steep:
+
+h_min = mne.fixes.minimum_phase(h)
+plot_filter(h_min, sfreq, freq, gain, 'Minimum-phase', flim=flim)
 
 ###############################################################################
 # .. _tut_effect_on_signals:
@@ -362,7 +328,7 @@ tlim = [center - 0.2, center + 0.2]
 tticks = [tlim[0], center, tlim[1]]
 flim = [20, 70]
 
-x = np.zeros(int(sfreq * dur))
+x = np.zeros(int(sfreq * dur) + 1)
 blip = morlet(sfreq, [morlet_freq], n_cycles=7)[0].imag / 20.
 n_onset = int(center * sfreq) - len(blip) // 2
 x[n_onset:n_onset + len(blip)] += blip
@@ -373,8 +339,27 @@ x += rng.randn(len(x)) / 1000.
 x += np.sin(2. * np.pi * 60. * np.arange(len(x)) / sfreq) / 2000.
 
 ###############################################################################
-# Filter it with a shallow cutoff, linear-phase FIR and compensate for
-# the delay:
+# Filter it with a shallow cutoff, linear-phase FIR (which allows us to
+# compensate for the constant filter delay):
+
+transition_band = 0.25 * f_p
+f_s = f_p + transition_band
+filter_dur = 6.6 / transition_band / 2.  # sec
+n = int(sfreq * filter_dur)
+freq = [0., f_p, f_s, sfreq / 2.]
+gain = [1., 1., 0., 0.]
+# This would be equivalent:
+h = mne.filter.create_filter(x, sfreq, l_freq=None, h_freq=f_p,
+                             fir_design='firwin')
+x_v16 = np.convolve(h, x)[len(h) // 2:]
+
+plot_filter(h, sfreq, freq, gain, 'MNE-Python 0.16 default', flim=flim)
+
+###############################################################################
+# Filter it with a different design mode ``fir_design="firwin2"``, and also
+# compensate for the constant filter delay. This method does not produce
+# quite as sharp a transition compared to ``fir_design="firwin"``, despite
+# being twice as long:
 
 transition_band = 0.25 * f_p
 f_s = f_p + transition_band
@@ -382,10 +367,13 @@ filter_dur = 6.6 / transition_band  # sec
 n = int(sfreq * filter_dur)
 freq = [0., f_p, f_s, sfreq / 2.]
 gain = [1., 1., 0., 0.]
-h = signal.firwin2(n, freq, gain, nyq=sfreq / 2.)
-x_shallow = np.convolve(h, x)[len(h) // 2:]
+# This would be equivalent:
+# h = signal.firwin2(n, freq, gain, nyq=sfreq / 2.)
+h = mne.filter.create_filter(x, sfreq, l_freq=None, h_freq=f_p,
+                             fir_design='firwin2')
+x_v14 = np.convolve(h, x)[len(h) // 2:]
 
-plot_filter(h, 'MNE-Python 0.14 default', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'MNE-Python 0.14 default', flim=flim)
 
 ###############################################################################
 # This is actually set to become the default type of filter used in MNE-Python
@@ -400,15 +388,19 @@ filter_dur = 10.  # sec
 n = int(sfreq * filter_dur)
 freq = [0., f_p, f_s, sfreq / 2.]
 gain = [1., 1., 0., 0.]
-h = signal.firwin2(n, freq, gain, nyq=sfreq / 2.)
-x_steep = np.convolve(np.convolve(h, x)[::-1], h)[::-1][len(h) - 1:-len(h) - 1]
+# This would be equivalent
+# h = signal.firwin2(n, freq, gain, nyq=sfreq / 2.)
+h = mne.filter.create_filter(x, sfreq, l_freq=None, h_freq=f_p,
+                             h_trans_bandwidth=transition_band,
+                             filter_length='%ss' % filter_dur,
+                             fir_design='firwin2')
+x_v13 = np.convolve(np.convolve(h, x)[::-1], h)[::-1][len(h) - 1:-len(h) - 1]
 
-plot_filter(h, 'MNE-Python 0.13 default', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'MNE-Python 0.13 default', flim=flim)
 
 ###############################################################################
-# Finally, Let's also filter it with the
-# MNE-C default, which is a long-duration steep-slope FIR filter designed
-# using frequency-domain techniques:
+# Let's also filter it with the MNE-C default, which is a long-duration
+# steep-slope FIR filter designed using frequency-domain techniques:
 
 h = mne.filter.design_mne_c_filter(sfreq, l_freq=None, h_freq=f_p + 2.5)
 x_mne_c = np.convolve(h, x)[len(h) // 2:]
@@ -417,7 +409,22 @@ transition_band = 5  # Hz (default in MNE-C)
 f_s = f_p + transition_band
 freq = [0., f_p, f_s, sfreq / 2.]
 gain = [1., 1., 0., 0.]
-plot_filter(h, 'MNE-C default', freq, gain)
+plot_filter(h, sfreq, freq, gain, 'MNE-C default', flim=flim)
+
+###############################################################################
+# And now an example of a minimum-phase filter:
+
+h = mne.filter.create_filter(x, sfreq, l_freq=None, h_freq=f_p,
+                             phase='minimum', fir_design='firwin')
+x_min = np.convolve(h, x)
+transition_band = 0.25 * f_p
+f_s = f_p + transition_band
+filter_dur = 6.6 / transition_band  # sec
+n = int(sfreq * filter_dur)
+freq = [0., f_p, f_s, sfreq / 2.]
+gain = [1., 1., 0., 0.]
+plot_filter(h, sfreq, freq, gain, 'Minimum-phase filter', flim=flim)
+
 
 ###############################################################################
 # Both the MNE-Python 0.13 and MNE-C filhters have excellent frequency
@@ -428,38 +435,37 @@ plot_filter(h, 'MNE-C default', freq, gain)
 # and the time-domain ringing is thus more pronounced for the steep-slope,
 # long-duration filter than the shorter, shallower-slope filter:
 
-axs = plt.subplots(1, 2)[1]
+axes = plt.subplots(1, 2)[1]
 
 
 def plot_signal(x, offset):
     t = np.arange(len(x)) / sfreq
-    axs[0].plot(t, x + offset)
-    axs[0].set(xlabel='Time (sec)', xlim=t[[0, -1]])
-    box_off(axs[0])
+    axes[0].plot(t, x + offset)
+    axes[0].set(xlabel='Time (sec)', xlim=t[[0, -1]])
     X = fftpack.fft(x)
     freqs = fftpack.fftfreq(len(x), 1. / sfreq)
     mask = freqs >= 0
     X = X[mask]
     freqs = freqs[mask]
-    axs[1].plot(freqs, 20 * np.log10(np.abs(X)))
-    axs[1].set(xlim=xlim)
+    axes[1].plot(freqs, 20 * np.log10(np.abs(X)))
+    axes[1].set(xlim=flim)
 
-yticks = np.arange(5) / -30.
-yticklabels = ['Original', 'Noisy', 'FIR-shallow (0.14)', 'FIR-steep (0.13)',
-               'FIR-steep (MNE-C)']
+yticks = np.arange(7) / -30.
+yticklabels = ['Original', 'Noisy', 'FIR-firwin (0.16)', 'FIR-firwin2 (0.14)',
+               'FIR-steep (0.13)', 'FIR-steep (MNE-C)', 'Minimum-phase']
 plot_signal(x_orig, offset=yticks[0])
 plot_signal(x, offset=yticks[1])
-plot_signal(x_shallow, offset=yticks[2])
-plot_signal(x_steep, offset=yticks[3])
-plot_signal(x_mne_c, offset=yticks[4])
-axs[0].set(xlim=tlim, title='FIR, Lowpass=%d Hz' % f_p, xticks=tticks,
-           ylim=[-0.150, 0.025], yticks=yticks, yticklabels=yticklabels,)
-for text in axs[0].get_yticklabels():
+plot_signal(x_v16, offset=yticks[2])
+plot_signal(x_v14, offset=yticks[3])
+plot_signal(x_v13, offset=yticks[4])
+plot_signal(x_mne_c, offset=yticks[5])
+plot_signal(x_min, offset=yticks[6])
+axes[0].set(xlim=tlim, title='FIR, Lowpass=%d Hz' % f_p, xticks=tticks,
+            ylim=[-0.200, 0.025], yticks=yticks, yticklabels=yticklabels,)
+for text in axes[0].get_yticklabels():
     text.set(rotation=45, size=8)
-axs[1].set(xlim=flim, ylim=ylim, xlabel='Frequency (Hz)',
-           ylabel='Magnitude (dB)')
-box_off(axs[0])
-box_off(axs[1])
+axes[1].set(xlim=flim, ylim=(-60, 10), xlabel='Frequency (Hz)',
+            ylabel='Magnitude (dB)')
 mne.viz.tight_layout()
 plt.show()
 
@@ -482,9 +488,20 @@ plt.show()
 # to have a *maximally flat pass-band*. Let's look at a few orders of filter,
 # i.e., a few different number of coefficients used and therefore steepness
 # of the filter:
+#
+# .. note:: Notice that the group delay (which is related to the phase) of
+#           the IIR filters below are not constant. In the FIR case, we can
+#           design so-called linear-phase filters that have a constant group
+#           delay, and thus compensate for the delay (making the filter
+#           acausal) if necessary. This cannot be done with IIR filters, as
+#           they have a non-linear phase (non-constant group delay). As the
+#           filter order increases, the phase distortion near and in the
+#           transition band worsens. However, if acausal (forward-backward)
+#           filtering can be used, e.g. with :func:`scipy.signal.filtfilt`,
+#           these phase issues can theoretically be mitigated.
 
 sos = signal.iirfilter(2, f_p / nyq, btype='low', ftype='butter', output='sos')
-plot_filter(sos, 'Butterworth order=2', freq, gain)
+plot_filter(dict(sos=sos), sfreq, freq, gain, 'Butterworth order=2', flim=flim)
 
 # Eventually this will just be from scipy signal.sosfiltfilt, but 0.18 is
 # not widely adopted yet (as of June 2016), so we use our wrapper...
@@ -494,17 +511,6 @@ x_shallow = sosfiltfilt(sos, x)
 ###############################################################################
 # The falloff of this filter is not very steep.
 #
-# .. warning:: For brevity, we do not show the phase of these filters here.
-#              In the FIR case, we can design linear-phase filters, and
-#              compensate for the delay (making the filter acausal) if
-#              necessary. This cannot be done
-#              with IIR filters, as they have a non-linear phase.
-#              As the filter order increases, the
-#              phase distortion near and in the transition band worsens.
-#              However, if acausal (forward-backward) filtering can be used,
-#              e.g. with :func:`scipy.signal.filtfilt`, these phase issues
-#              can be mitigated.
-#
 # .. note:: Here we have made use of second-order sections (SOS)
 #           by using :func:`scipy.signal.sosfilt` and, under the
 #           hood, :func:`scipy.signal.zpk2sos` when passing the
@@ -522,7 +528,7 @@ x_shallow = sosfiltfilt(sos, x)
 # with a longer impulse response:
 
 sos = signal.iirfilter(8, f_p / nyq, btype='low', ftype='butter', output='sos')
-plot_filter(sos, 'Butterworth order=8', freq, gain)
+plot_filter(dict(sos=sos), sfreq, freq, gain, 'Butterworth order=8', flim=flim)
 x_steep = sosfiltfilt(sos, x)
 
 ###############################################################################
@@ -533,7 +539,8 @@ x_steep = sosfiltfilt(sos, x)
 
 sos = signal.iirfilter(8, f_p / nyq, btype='low', ftype='cheby1', output='sos',
                        rp=1)  # dB of acceptable pass-band ripple
-plot_filter(sos, 'Chebychev-1 order=8, ripple=1 dB', freq, gain)
+plot_filter(dict(sos=sos), sfreq, freq, gain,
+            'Chebychev-1 order=8, ripple=1 dB', flim=flim)
 
 ###############################################################################
 # And if we can live with even more ripple, we can get it slightly steeper,
@@ -542,7 +549,8 @@ plot_filter(sos, 'Chebychev-1 order=8, ripple=1 dB', freq, gain)
 
 sos = signal.iirfilter(8, f_p / nyq, btype='low', ftype='cheby1', output='sos',
                        rp=6)
-plot_filter(sos, 'Chebychev-1 order=8, ripple=6 dB', freq, gain)
+plot_filter(dict(sos=sos), sfreq, freq, gain,
+            'Chebychev-1 order=8, ripple=6 dB', flim=flim)
 
 ###############################################################################
 # Applying IIR filters
@@ -551,21 +559,20 @@ plot_filter(sos, 'Chebychev-1 order=8, ripple=6 dB', freq, gain)
 # Now let's look at how our shallow and steep Butterworth IIR filters
 # perform on our Morlet signal from before:
 
-axs = plt.subplots(1, 2)[1]
+axes = plt.subplots(1, 2)[1]
 yticks = np.arange(4) / -30.
 yticklabels = ['Original', 'Noisy', 'Butterworth-2', 'Butterworth-8']
 plot_signal(x_orig, offset=yticks[0])
 plot_signal(x, offset=yticks[1])
 plot_signal(x_shallow, offset=yticks[2])
 plot_signal(x_steep, offset=yticks[3])
-axs[0].set(xlim=tlim, title='IIR, Lowpass=%d Hz' % f_p, xticks=tticks,
-           ylim=[-0.125, 0.025], yticks=yticks, yticklabels=yticklabels,)
-for text in axs[0].get_yticklabels():
+axes[0].set(xlim=tlim, title='IIR, Lowpass=%d Hz' % f_p, xticks=tticks,
+            ylim=[-0.125, 0.025], yticks=yticks, yticklabels=yticklabels,)
+for text in axes[0].get_yticklabels():
     text.set(rotation=45, size=8)
-axs[1].set(xlim=flim, ylim=ylim, xlabel='Frequency (Hz)',
-           ylabel='Magnitude (dB)')
-box_off(axs[0])
-box_off(axs[1])
+axes[1].set(xlim=flim, ylim=(-60, 10), xlabel='Frequency (Hz)',
+            ylabel='Magnitude (dB)')
+mne.viz.adjust_axes(axes)
 mne.viz.tight_layout()
 plt.show()
 
@@ -658,14 +665,14 @@ ylim = [-2, 6]
 xlabel = 'Time (sec)'
 ylabel = 'Amplitude ($\mu$V)'
 tticks = [0, 0.5, 1.3, t[-1]]
-axs = plt.subplots(2, 2)[1].ravel()
-for ax, x_f, title in zip(axs, [x_lp_2, x_lp_30, x_hp_2, x_hp_p1],
+axes = plt.subplots(2, 2)[1].ravel()
+for ax, x_f, title in zip(axes, [x_lp_2, x_lp_30, x_hp_2, x_hp_p1],
                           ['LP$_2$', 'LP$_{30}$', 'HP$_2$', 'LP$_{0.1}$']):
     ax.plot(t, x, color='0.5')
     ax.plot(t, x_f, color='k', linestyle='--')
     ax.set(ylim=ylim, xlim=xlim, xticks=tticks,
            title=title, xlabel=xlabel, ylabel=ylabel)
-    box_off(ax)
+mne.viz.adjust_axes(axes)
 mne.viz.tight_layout()
 plt.show()
 
@@ -703,9 +710,9 @@ plt.show()
 
 
 def baseline_plot(x):
-    all_axs = plt.subplots(3, 2)[1]
-    for ri, (axs, freq) in enumerate(zip(all_axs, [0.1, 0.3, 0.5])):
-        for ci, ax in enumerate(axs):
+    all_axes = plt.subplots(3, 2)[1]
+    for ri, (axes, freq) in enumerate(zip(all_axes, [0.1, 0.3, 0.5])):
+        for ci, ax in enumerate(axes):
             if ci == 0:
                 iir_hp = signal.iirfilter(4, freq / sfreq, btype='highpass',
                                           output='sos')
@@ -717,10 +724,10 @@ def baseline_plot(x):
             if ri == 0:
                 ax.set(title=('No ' if ci == 0 else '') +
                        'Baseline Correction')
-            box_off(ax)
             ax.set(xticks=tticks, ylim=ylim, xlim=xlim, xlabel=xlabel)
             ax.set_ylabel('%0.1f Hz' % freq, rotation=0,
                           horizontalalignment='right')
+        mne.viz.adjust_axes(axes)
     mne.viz.tight_layout()
     plt.suptitle(title)
     plt.show()
@@ -775,7 +782,7 @@ baseline_plot(x)
 # and thus :func:`mne.io.Raw.filter` is used. This function under the hood
 # (among other things) calls :func:`mne.filter.filter_data` to actually
 # filter the data, which by default applies a zero-phase FIR filter designed
-# using :func:`scipy.signal.firwin2`. In Widmann *et al.* 2015 [7]_, they
+# using :func:`scipy.signal.firwin`. In Widmann *et al.* 2015 [7]_, they
 # suggest a specific set of parameters to use for high-pass filtering,
 # including:
 #
@@ -821,27 +828,28 @@ baseline_plot(x)
 # To choose the filter length automatically with ``filter_length='auto'``,
 # the reciprocal of the shortest transition bandwidth is used to ensure
 # decent attenuation at the stop frequency. Specifically, the reciprocal
-# (in samples) is multiplied by 6.2, 6.6, or 11.0 for the Hann, Hamming,
+# (in samples) is multiplied by 3.1, 3.3, or 5.0 for the Hann, Hamming,
 # or Blackman windows, respectively as selected by the ``fir_window``
-# argument.
+# argument for ``fir_design='firwin'``, and double these for
+# ``fir_design='firwin2'`` mode.
 #
-# .. note:: These multiplicative factors are double what is given in
-#           Ifeachor and Jervis [2]_ (p. 357). The window functions have a
-#           smearing effect on the frequency response; I&J thus take the
-#           approach of setting the stop frequency as
-#           :math:`f_s = f_p + f_{trans} / 2.`, but our stated definitions of
-#           :math:`f_s` and :math:`f_{trans}` do not
-#           allow us to do this in a nice way. Instead, we increase our filter
-#           length to achieve acceptable (20+ dB) attenuation by
-#           :math:`f_s = f_p + f_{trans}`, and excellent (50+ dB)
-#           attenuation by :math:`f_s + f_{trans}` (and usually earlier).
+# .. note:: For ``fir_design='firwin2'``, the multiplicative factors are
+#           doubled compared to what is given in Ifeachor and Jervis [2]_
+#           (p. 357), as :func:`scipy.signal.firwin2` has a smearing effect
+#           on the frequency response, which we compensate for by
+#           increasing the filter length. This is why
+#           ``fir_desgin='firwin'`` is preferred to ``fir_design='firwin2'``.
 #
 # In 0.14, we default to using a Hamming window in filter design, as it
 # provides up to 53 dB of stop-band attenuation with small pass-band ripple.
 #
 # .. note:: In band-pass applications, often a low-pass filter can operate
 #           effectively with fewer samples than the high-pass filter, so
-#           it is advisable to apply the high-pass and low-pass separately.
+#           it is advisable to apply the high-pass and low-pass separately
+#           when using ``fir_design='firwin2'``. For design mode
+#           ``fir_design='firwin'``, there is no need to separate the
+#           operations, as the lowpass and highpass elements are constructed
+#           separately to meet the transition band requirements.
 #
 # For more information on how to use the
 # MNE-Python filtering functions with real data, consult the preprocessing
