@@ -22,8 +22,8 @@ from mne.datasets import sample
 data_path = sample.data_path()
 raw_fname = data_path + '/MEG/sample/sample_audvis_filt-0-40_raw.fif'
 event_fname = data_path + '/MEG/sample/sample_audvis_filt-0-40_raw-eve.fif'
-raw = mne.io.read_raw_fif(raw_fname, add_eeg_ref=False, preload=True)
-raw.set_eeg_reference()  # set EEG average reference
+raw = mne.io.read_raw_fif(raw_fname, preload=True)
+raw.set_eeg_reference('average', projection=True)  # set EEG average reference
 
 ###############################################################################
 # Let's restrict the data to the EEG channels
@@ -59,7 +59,7 @@ print(raw.info['chs'][0]['loc'])
 
 ###############################################################################
 # And it's actually possible to plot the channel locations using
-# the :func:`mne.io.Raw.plot_sensors` method
+# :func:`mne.io.Raw.plot_sensors`.
 
 raw.plot_sensors()
 raw.plot_sensors('3d')  # in 3D
@@ -69,14 +69,14 @@ raw.plot_sensors('3d')  # in 3D
 # -------------------
 #
 # In the case where your data don't have locations you can set them
-# using a :func:`mne.channels.Montage`. MNE comes with a set of default
+# using a :class:`mne.channels.Montage`. MNE comes with a set of default
 # montages. To read one of them do:
 
 montage = mne.channels.read_montage('standard_1020')
 print(montage)
 
 ###############################################################################
-# To apply a montage on your data use the :func:`mne.io.set_montage`
+# To apply a montage on your data use the ``set_montage`` method.
 # function. Here don't actually call this function as our demo dataset
 # already contains good EEG channel locations.
 #
@@ -91,7 +91,7 @@ print(montage)
 # This explicitly prevents MNE from adding a default EEG average reference
 # required for source localization.
 
-raw_no_ref, _ = mne.io.set_eeg_reference(raw, [])
+raw_no_ref, _ = mne.set_eeg_reference(raw, [])
 
 ###############################################################################
 # We next define Epochs and compute an ERP for the left auditory condition.
@@ -99,7 +99,7 @@ reject = dict(eeg=180e-6, eog=150e-6)
 event_id, tmin, tmax = {'left/auditory': 1}, -0.2, 0.5
 events = mne.read_events(event_fname)
 epochs_params = dict(events=events, event_id=event_id, tmin=tmin, tmax=tmax,
-                     reject=reject, add_eeg_ref=False)
+                     reject=reject)
 
 evoked_no_ref = mne.Epochs(raw_no_ref, **epochs_params).average()
 del raw_no_ref  # save memory
@@ -111,7 +111,7 @@ evoked_no_ref.plot_topomap(times=[0.1], size=3., title=title)
 ###############################################################################
 # **Average reference**: This is normally added by default, but can also
 # be added explicitly.
-raw_car, _ = mne.io.set_eeg_reference(raw)
+raw_car, _ = mne.set_eeg_reference(raw, 'average', projection=True)
 evoked_car = mne.Epochs(raw_car, **epochs_params).average()
 del raw_car  # save memory
 
@@ -122,7 +122,7 @@ evoked_car.plot_topomap(times=[0.1], size=3., title=title)
 ###############################################################################
 # **Custom reference**: Use the mean of channels EEG 001 and EEG 002 as
 # a reference
-raw_custom, _ = mne.io.set_eeg_reference(raw, ['EEG 001', 'EEG 002'])
+raw_custom, _ = mne.set_eeg_reference(raw, ['EEG 001', 'EEG 002'])
 evoked_custom = mne.Epochs(raw_custom, **epochs_params).average()
 del raw_custom  # save memory
 
@@ -160,7 +160,7 @@ mne.combine_evoked([left, -right], weights='equal').plot_joint()
 # This is an equal-weighting difference. If you have imbalanced trial numbers,
 # you could also consider either equalizing the number of events per
 # condition (using
-# :meth:`epochs.equalize_epochs_counts <mne.Epochs.equalize_event_counts`).
+# :meth:`epochs.equalize_event_counts <mne.Epochs.equalize_event_counts>`).
 # As an example, first, we create individual ERPs for each condition.
 
 aud_l = epochs["auditory", "left"].average()
