@@ -17,8 +17,8 @@ import mne
 # First let's read in the raw sample data.
 data_path = mne.datasets.sample.data_path()
 fname = op.join(data_path, 'MEG', 'sample', 'sample_audvis_raw.fif')
-raw = mne.io.read_raw_fif(fname, add_eeg_ref=False)
-raw.set_eeg_reference()  # set EEG average reference
+raw = mne.io.read_raw_fif(fname)
+raw.set_eeg_reference('average', projection=True)  # set EEG average reference
 
 ###############################################################################
 # To create time locked epochs, we first need a set of events that contain the
@@ -41,7 +41,8 @@ raw.plot(n_channels=10, order=order, block=True)
 # from an outside source (like a separate file of events), pay special
 # attention in aligning the events correctly with the raw data.
 events = mne.find_events(raw)
-print(events)
+print('Found %s events, first five:' % len(events))
+print(events[:5])
 
 # Plot the events to get an idea of the paradigm
 # Specify colors and an event_id dictionary for the legend.
@@ -105,12 +106,12 @@ picks = mne.pick_types(raw.info, meg=True, eeg=False, eog=True)
 # for EEG and EOG electrodes.
 #
 # .. note:: In this tutorial, we don't preprocess the data. This is not
-#           something you would normally do. See our :ref:`tutorials` on
+#           something you would normally do. See our :ref:`documentation` on
 #           preprocessing for more.
 baseline = (None, 0.0)
 reject = {'mag': 4e-12, 'eog': 200e-6}
 epochs = mne.Epochs(raw, events=events, event_id=event_id, tmin=tmin,
-                    tmax=tmax, reject=reject, picks=picks, add_eeg_ref=False)
+                    tmax=tmax, baseline=baseline, reject=reject, picks=picks)
 
 ###############################################################################
 # Let's plot the epochs to see the results. The number at the top refers to the
