@@ -21,17 +21,25 @@ from mne.preprocessing import create_ecg_epochs, create_eog_epochs
 from mne.datasets import sample
 
 ###############################################################################
-# Setup paths and prepare raw data
+# Setup paths and prepare raw data.
 
 data_path = sample.data_path()
 raw_fname = data_path + '/MEG/sample/sample_audvis_filt-0-40_raw.fif'
 
-raw = mne.io.read_raw_fif(raw_fname, preload=True, add_eeg_ref=False)
+raw = mne.io.read_raw_fif(raw_fname, preload=True)
 raw.filter(1, 45, n_jobs=1, l_trans_bandwidth=0.5, h_trans_bandwidth=0.5,
-           filter_length='10s', phase='zero-double')
+           filter_length='10s', phase='zero-double', fir_design='firwin2')
+raw.annotations = mne.Annotations([1], [10], 'BAD')
+raw.plot(block=True)
+
+# For the sake of example we annotate first 10 seconds of the recording as
+# 'BAD'. This part of data is excluded from the ICA decomposition by default.
+# To turn this behavior off, pass ``reject_by_annotation=False`` to
+# :meth:`mne.preprocessing.ICA.fit`.
+raw.annotations = mne.Annotations([0], [10], 'BAD')
 
 ###############################################################################
-# 1) Fit ICA model using the FastICA algorithm
+# 1) Fit ICA model using the FastICA algorithm.
 
 # Other available choices are `infomax` or `extended-infomax`
 # We pass a float value between 0 and 1 to select n_components based on the
@@ -81,7 +89,7 @@ eog_inds = eog_inds[:n_max_eog]
 ica.exclude += eog_inds
 
 ###############################################################################
-# 3) Assess component selection and unmixing quality
+# 3) Assess component selection and unmixing quality.
 
 # estimate average artifact
 ecg_evoked = ecg_epochs.average()
