@@ -68,9 +68,9 @@ evoked = epochs.average()
 # operations such as nonparametric statistics) if you don't need high
 # spectrotemporal resolution.
 decim = 5
-frequencies = np.arange(8, 40, 2)  # define frequencies of interest
+freqs = np.arange(8, 40, 2)  # define frequencies of interest
 sfreq = raw.info['sfreq']  # sampling in Hz
-tfr_epochs = tfr_morlet(epochs, frequencies, n_cycles=4., decim=decim,
+tfr_epochs = tfr_morlet(epochs, freqs, n_cycles=4., decim=decim,
                         average=False, return_itc=False, n_jobs=1)
 
 # Baseline power
@@ -110,10 +110,10 @@ vmax = np.max(np.abs(T_obs))
 vmin = -vmax
 plt.subplot(2, 1, 1)
 plt.imshow(T_obs, cmap=plt.cm.gray,
-           extent=[times[0], times[-1], frequencies[0], frequencies[-1]],
+           extent=[times[0], times[-1], freqs[0], freqs[-1]],
            aspect='auto', origin='lower', vmin=vmin, vmax=vmax)
 plt.imshow(T_obs_plot, cmap=plt.cm.RdBu_r,
-           extent=[times[0], times[-1], frequencies[0], frequencies[-1]],
+           extent=[times[0], times[-1], freqs[0], freqs[-1]],
            aspect='auto', origin='lower', vmin=vmin, vmax=vmax)
 plt.colorbar()
 plt.xlabel('Time (ms)')
